/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "skills.h"

/* extern variables */
extern struct room_data *world;
extern struct descriptor_data *descriptor_list;
extern struct room_data *world;
extern char *fighting_styles[];
extern char *cmd_door[];
extern char *attack_forms[];

/* extern functions */
void raw_kill(struct char_data *ch, struct char_data *killer);
byte ProximityCheck(struct char_data *ch, struct obj_data *obj, struct char_data *target, int door);
void AttackItem(struct char_data *ch, struct obj_data *obj);
void AttackDoor(struct char_data *ch, int door);
int find_door(struct char_data *ch, char *type, char *dir, char *cmdname);
void kick_messages(struct char_data *ch, struct char_data *victim, int dam);
void improve_skill(struct char_data *ch, int Skill);
extern void mobact_rescueHandle(struct char_data *mob, struct char_data *attacker);

byte can_hit(struct char_data *ch, struct char_data *vict)
{
  byte ret_val = 0;

  if (!vict)
    send_to_char("It/they don't seem to be here.\r\n", ch);

  else if (vict == ch)
    send_to_char("You hit yourself...OUCH!.\r\n", ch);

  else if (IS_AFFECTED(ch, AFF_CHARM) && (ch->master == vict))
    act("$N is just such a good friend, you simply can't hit $M.", FALSE, ch, 0, vict, TO_CHAR);

/* you can't order a charmed pet to attack a player */
//  else if (IS_AFFECTED(ch, AFF_CHARM) &&
  //         !IS_NPC(ch->master) && !IS_NPC(vict));

/* sanctuary */
  else if (IS_AFFECTED4(vict, AFF4_SANCTUARY) && !makes_save(ch, SAVING_SPELL, 0))
    act("Your mind feels fuzzy. What were you trying to do?", FALSE, ch, 0, vict, TO_CHAR);

  else
    ret_val = 1;

  return ret_val;
}

ACMD(do_assist)
{
  struct char_data *helpee, *opponent;

  if (FIGHTING(ch)) {
    send_to_char("You're already fighting!  How can you assist someone else?\r\n", ch);
    return;
  }
  one_argument(argument, arg);

  if (!*arg)
    send_to_char("Whom do you wish to assist?\r\n", ch);
  else if (!(helpee = get_char_room_vis(ch, arg)))
    send_to_char(NOPERSON, ch);
  else if (helpee == ch)
    send_to_char("You can't help yourself any more than this!\r\n", ch);
  else {
    for (opponent = world[ch->in_room].people;
	 opponent && (FIGHTING(opponent) != helpee);
	 opponent = opponent->next_in_room);

    if (!opponent)
      act("But nobody is fighting $M!", FALSE, ch, 0, helpee, TO_CHAR);

    else if (!CAN_SEE(ch, opponent))
      act("You can't see who is fighting $M!", FALSE, ch, 0, helpee, TO_CHAR);
    else {
      send_to_char("You join the fight!\r\n", ch);
      act("$N assists you!", 0, helpee, 0, ch, TO_CHAR);
      act("$n assists $N.", FALSE, ch, 0, helpee, TO_NOTVICT);
      hit(ch, opponent, TYPE_UNDEFINED);
    }
  }
}


ACMD(do_hit)
{
  struct char_data *vict;
  struct obj_data *obj;
  int door, skl;
  one_argument(argument, arg);

  if (!*arg)
    send_to_char("Hit who?\r\n", ch);
  if (subcmd != SCMD_MURDER && (obj = get_obj_in_list_vis(ch, arg, world[ch->in_room].contents)) != NULL) {
    AttackItem(ch, obj);
    WAIT_STATE(ch, PULSE_VIOLENCE * 2);
    return;
  } else if (subcmd != SCMD_MURDER && (door = find_door(ch, arg, "", cmd_door[subcmd])) >= 0) {
    AttackDoor(ch, door);
    WAIT_STATE(ch, PULSE_VIOLENCE * 2);
    return;
  } else {
    if (!(vict = get_char_room_vis(ch, arg))) {
      send_to_char("Hit who?\r\n", ch);
      return;
    }
    if (!can_hit(ch, vict)) {
      act("You cant hit $N.", TRUE, ch, NULL, vict, TO_CHAR);
      return;
    }
    if (!IS_NPC(vict) && !IS_NPC(ch)) {
      if (subcmd != SCMD_MURDER) {
	send_to_char("Use 'murder' to hit another player.\r\n", ch);
	return;
      }
    }
    if ((GET_POS(ch) == POS_STANDING) && (vict != FIGHTING(ch))) {
      hit(ch, vict, TYPE_UNDEFINED);
      WAIT_STATE(ch, PULSE_VIOLENCE * 2);
#if 0
    } else
      send_to_char("You do the best you can!\r\n", ch);
#else
    } else if (vict == FIGHTING(ch)) {
      send_to_char("Okay, we continue fighting...\r\n", ch);
      return;
    } else {
      skl = GET_LEVEL(ch) * (IS_WARRIOR(ch) ? 3 : 1) + number(-20, 20);
      skl = BOUNDED(1, skl, (IS_WARRIOR(ch) ? 95 : 50));
      if ((GET_POS(vict) < POS_STANDING))
	skl += GET_LEVEL(ch);

      if (skl >= number(1, 101)) {
	stop_fighting(ch);
	set_fighting(ch, vict);
	send_to_char("You switch opponents!\r\n", ch);
	act("$n switches targets..", FALSE, ch, 0, 0, TO_ROOM);
	WAIT_STATE(ch, PULSE_VIOLENCE + 2);
      } else {
	send_to_char("You try to switch opponents, but you become confused!\r\n", ch);
	stop_fighting(ch);
	WAIT_STATE(ch, PULSE_VIOLENCE * 2);
      }
    }
#endif
  }
  mobact_rescueHandle(vict, ch);
}



ACMD(do_kill)
{
  struct char_data *vict;

  if ((GET_LEVEL(ch) < LVL_IMPL) || IS_NPC(ch)) {
    do_hit(ch, argument, cmd, subcmd);
    return;
  }
  one_argument(argument, arg);

  if (!*arg) {
    send_to_char("Kill who?\r\n", ch);
  } else {
    if (!(vict = get_char_room_vis(ch, arg)))
      send_to_char("They aren't here.\r\n", ch);
    else if (ch == vict)
      send_to_char("Your mother would be so sad.. :(\r\n", ch);
    else {
      act("You chop $M to pieces!  Ah!  The blood!", FALSE, ch, 0, vict, TO_CHAR);
      act("$N chops you to pieces!", FALSE, vict, 0, ch, TO_CHAR);
      act("$n brutally slays $N!", FALSE, ch, 0, vict, TO_NOTVICT);
      raw_kill(vict, ch);
    }
  }
}


#define TESTCHAR_TEST
ACMD(do_backstab)
{
  struct char_data *vict;
  int percent, prob;
  int prop_pos;
#ifdef TESTCHAR_TEST
  int istus = 0;
  char buf[500];
  if (isname("testchar", ch->player.name) || IS_TRUSTED(ch))
    istus = 1;
#endif
  if (!THIEVING_CLASS(ch) && !IS_TRUSTED(ch)) {
    send_to_char("You really wouldn't be very good at that.\r\n", ch);
    return;
  }
  if (IS_FIGHTING(ch) && IS_PC(ch)) {	// mobs are forced too sometimes

    send_to_char("You're a bit too busy for that right now.\r\n", ch);
    return;
  }
  one_argument(argument, buf);

  if (!(vict = get_char_room_vis(ch, buf))) {
    send_to_char("Backstab who?\r\n", ch);
    return;
  }
  if (vict == ch) {
    send_to_char("How can you sneak up on yourself?\r\n", ch);
    return;
  }
  if (!GET_EQ(ch, WEAR_WIELD)) {
    // do this properly with acts...
    act("You stiffen your finger, and jab $N in the back.", FALSE,
	ch, 0, vict, TO_CHAR);
    send_to_char("You feel someone poking you in the back.\r\n",
		 vict);
    act("$n stretches a finger, and pokes $N in the back", TRUE, ch,
	0, vict, TO_OTHERS);
    return;
  }
  if ((GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_PIERCE - TYPE_HIT) &&
      GET_FORM(ch) != FORM_THRUST) {
    send_to_char("Try thrusting that item.\r\n", ch);
    return;
  }
  // okay.. figure where I'm facing, and see if the victim is in that
  // 'step'.

  prop_pos = WithinAngle(vict, NULL, ch, NULL, 95, 95) ? 0 : 1;

  // position doesn't matter if the vict is blind or sleeping.
  // Otherwise, the vict needs to be facing directly away from the
  // thief.
  if (AWAKE(vict) && !IS_AFFECTED(vict, AFF_BLIND) && !prop_pos) {
    send_to_char("But they aren't facing away from you!\r\n", ch);
    return;
  }
  percent = number(1, 101);
  prob = 0;

  switch (GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) + TYPE_HIT) {
  case TYPE_STAB:
  case TYPE_PIERCE:
    prob = 100;
    break;
  case TYPE_SLASH:
  case TYPE_CLAW:
    prob = 80;
    break;
  default:
    send_to_char("You're going to stab with THAT weapon?  Not very smart for a thief.\r\n", ch);
    return;
  }
#ifdef TESTCHAR_TEST
  if (istus)
    sprintf(buf, "Base chance: %d", prob);
#endif

  if (FIGHTING(vict)) {
    // fighting victims are generally much more alert then peaceful
    // ones, so compensate
    prob -= 50;
#ifdef TESTCHAR_TEST
    if (istus)
      strcat(buf, " - 80");
#endif
  }
  // let the person try even if the vict hears 'em, but don't give
  // them any real chance at success
  if (ac_can_hear(vict, ch)) {
    prob -= 60;
#ifdef TESTCHAR_TEST
    strcat(buf, " - 70");
#endif
  }
  // what to do if the mob is "aware"?  blah, nothing at all for now.
  // Eventually, after being stabbed, a character should be 'aware'
  // for a few RL seconds.  I know I would be!
  //   MOB_FLAGGED(vict, MOB_AWARE))

#ifdef TESTCHAR_TEST
  if (istus) {
    send_to_char(buf, ch);
    sprintf(buf, " = %d.  Dice: %d\r\n", prob, percent);
    send_to_char(buf, ch);
  }
#endif

  if (percent > prob) {
    act("$n tries to backstab $N but misses.", FALSE, ch, NULL, vict, TO_OTHERS);
    act("You try to backstab $N but miss.", FALSE, ch, NULL, vict, TO_CHAR);
    act("$n tries to backstab you, but misses.", FALSE, ch, NULL, vict, TO_VICT);
    damage(ch, vict, 0, SKILL_BACKSTAB, WEAR_WIELD, LOCATION_VITALS);
  } else {
    act("$n sinks $s $o into $N's back.", FALSE, ch, GET_EQ(ch, WEAR_WIELD), vict, TO_OTHERS);
    act("You sink your $o into $N's back.", FALSE, ch, GET_EQ(ch, WEAR_WIELD), vict, TO_CHAR);
    act("$n sinks $s $o into your back.", FALSE, ch, GET_EQ(ch, WEAR_WIELD), vict, TO_VICT);
    hit(ch, vict, SKILL_BACKSTAB);
  }
}



ACMD(do_order)
{
  char name[100], message[256];
  char buf[256];
  bool found = FALSE;
  int org_room;
  struct char_data *vict;
  struct follow_type *k;

  half_chop(argument, name, message);

  if (!*name || !*message)
    send_to_char("Order who to do what?\r\n", ch);
  else if (!(vict = get_char_room_vis(ch, name)) && !is_abbrev(name, "followers"))
    send_to_char("That person isn't here.\r\n", ch);
  else if (ch == vict)
    send_to_char("You obviously suffer from skitzofrenia.\r\n", ch);

  else {
    if (IS_AFFECTED(ch, AFF_CHARM)) {
      send_to_char("Your superior would not aprove of you giving orders.\r\n", ch);
      return;
    }
    if (vict) {
      sprintf(buf, "$N orders you to '%s'", message);
      act(buf, FALSE, vict, 0, ch, TO_CHAR);
      act("$n gives $N an order.", FALSE, ch, 0, vict, PROXIMITY);

      if ((vict->master != ch) || !IS_AFFECTED(vict, AFF_CHARM))
	act("$n has an indifferent look.", FALSE, vict, 0, 0, PROXIMITY);
      else {
	send_to_char(OK, ch);
	command_interpreter(vict, message);
      }
    } else {			/* This is order "followers" */
      sprintf(buf, "$n issues the order '%s'.", message);
      act(buf, FALSE, ch, 0, vict, PROXIMITY);

      org_room = ch->in_room;

      for (k = ch->followers; k; k = k->next) {
	if (org_room == k->follower->in_room)
	  if (IS_AFFECTED(k->follower, AFF_CHARM)) {
	    found = TRUE;
	    command_interpreter(k->follower, message);
	  }
      }
      if (found)
	send_to_char(OK, ch);
      else
	send_to_char("Nobody here is a loyal subject of yours!\r\n", ch);
    }
  }
}


ACMD(do_flee)
{
  int i;

  if (GET_POS(ch) < POS_SITTING || GET_STAT(ch) < STAT_RESTING) {
    send_to_char("You are in pretty bad shape, unable to flee!\r\n", ch);
    return;
  }
#if 0
  /* reset battle fancies */
  GET_FORM(ch) = FORM_SWING;
  GET_AIMING(ch) = LOCATION_TORSO;
#endif

  /* just move them around the room a bit. If they were in same square,
   * they'll still be in range another round. Tough, flee again.
   */
  i = number(-1, 1);
  if (number(0, 1)) {
    if (GET_X_CORD(ch) + i > 0 && GET_X_CORD(ch) + i < GET_X_SIZE(ch->in_room) - 1) {
      GET_X_CORD(ch) += i;
      if (FIGHTING(ch)) {
	act("$n retreats from the battle!", TRUE, ch, 0, 0, PROXIMITY);
	send_to_char("You attempt to flee your battle!\r\n", ch);
#if 1
	if (FIGHTING(FIGHTING(ch)) == ch)
	  stop_fighting(FIGHTING(ch));
#endif
	stop_fighting(ch);
      } else {
	act("$n jumps a few feet to the side, as if startled.", TRUE, ch, 0, 0, PROXIMITY);
	send_to_char("With a fearful step, you quickly move.\r\n", ch);
      }
    }
    return;
  } else {
    if (GET_Y_CORD(ch) + i > 0 && GET_Y_CORD(ch) + i < GET_Y_SIZE(ch->in_room) - 1) {
      GET_Y_CORD(ch) += i;
      if (FIGHTING(ch)) {
	act("$n retreats from the battle!", TRUE, ch, 0, 0, PROXIMITY);
	send_to_char("You attempt to flee your battle!\r\n", ch);
#if 0
	if (FIGHTING(FIGHTING(ch)) == ch)
	  stop_fighting(FIGHTING(ch));
#endif
	stop_fighting(ch);
      } else {
	act("$n jumps a few feet to the side, as if startled.", TRUE, ch, 0, 0, PROXIMITY);
	send_to_char("With a fearful step, you quickly move.\r\n", ch);
      }
    }
    return;
  }
  send_to_char("You can't leave the battle just yet.\r\n", ch);
}

bool perform_bash(struct char_data * ch, struct char_data * vict)
{
  int percent, prob;

  percent = number(1, 101);	/* 101% is a complete failure */
  prob = GET_SKILL(ch, SKILL_BASH);

  if (MOB_FLAGGED(vict, MOB_NOBASH))
    percent = 101;

  if (percent > prob) {
    damage(ch, vict, 0, SKILL_BASH, -1, -1);
    SET_POS(ch, POS_SITTING + GET_STAT(ch));	/*??? both cases do about the same? */
    return FALSE;
  } else {
    damage(ch, vict, 1, SKILL_BASH, -1, -1);
    SET_POS(ch, POS_SITTING + GET_STAT(ch));
    WAIT_STATE(vict, PULSE_VIOLENCE);
    return TRUE;
  }
  WAIT_STATE(ch, PULSE_VIOLENCE * 2);
}

ACMD(do_bash)
{
  struct char_data *vict;

  one_argument(argument, arg);

  if (GET_CLASS(ch) != CLASS_FIGHTER) {
    send_to_char("You'd better leave all the martial arts to fighters.\r\n", ch);
    return;
  }
  if (!(vict = get_char_room_vis(ch, arg))) {
    if (FIGHTING(ch)) {
      vict = FIGHTING(ch);
    } else {
      send_to_char("Bash who?\r\n", ch);
      return;
    }
  }
  if (vict == ch) {
    send_to_char("Aren't we funny today...\r\n", ch);
    return;
  }
  if (!GET_EQ(ch, WEAR_WIELD)) {
    send_to_char("You need to wield a weapon to make it a success.\r\n", ch);
    return;
  }
  perform_bash(ch, vict);
}


ACMD(do_rescue)
{
  struct char_data *vict, *tmp_ch;
  int percent, prob;

  one_argument(argument, arg);

  if (!(vict = get_char_room_vis(ch, arg)))
    send_to_char("Whom do you want to rescue?\r\n", ch);
  else if (vict == ch)
    send_to_char("What about fleeing instead?\r\n", ch);
  else if (FIGHTING(ch) == vict)
    send_to_char("How can you rescue someone you are trying to kill?\r\n", ch);
  else {
    for (tmp_ch = world[ch->in_room].people; tmp_ch; tmp_ch = tmp_ch->next_in_room)
      if (FIGHTING(tmp_ch) == vict)
	break;

    if (!tmp_ch)
      act("But nobody is fighting $M!", FALSE, ch, 0, vict, TO_CHAR);
    else if (!IS_WARRIOR(ch))
      send_to_char("But only true warriors can do this!\r\n", ch);
    else {
      percent = number(1, 101);	/* 101% is a complete failure */
      prob = GET_SKILL(ch, SKILL_RESCUE);

      if (percent > prob) {
	send_to_char("You fail the rescue!\r\n", ch);
	return;
      }
      send_to_char("Banzai!  To the rescue...\r\n", ch);
      act("You are rescued by $N, you are confused!", FALSE, vict, 0, ch, TO_CHAR);
      act("$n heroically rescues $N!", FALSE, ch, 0, vict, TO_NOTVICT);

      if (FIGHTING(vict) == tmp_ch)
	stop_fighting(vict);
      if (FIGHTING(tmp_ch))
	stop_fighting(tmp_ch);
      if (FIGHTING(ch))
	stop_fighting(ch);

      set_fighting(ch, tmp_ch);
      set_fighting(tmp_ch, ch);

      WAIT_STATE(vict, 2 * PULSE_VIOLENCE);
    }
  }
}


bool perform_kick(struct char_data *ch, struct char_data *vict)
{
  int percent, prob, dam = 0;
  int skill_imp;

  percent = ((10 - (GET_AC(vict))) << 1) + number(1, 101);	/* 101% is a complete
								   * failure */

  prob = GET_SKILL(ch, SKILL_KICK);

  if (percent <= prob)
    dam = GET_LEVEL(ch) >> 1;

  if (dam)
    skill_imp = number(0, 1000);
  else
    skill_imp = number(0, 10000);

  //skill improvement
  if (IS_PC(ch))
    switch (GET_CLASS(ch)) {
    case CLASS_FIGHTER:
      if (prob <= 99 && skill_imp < 15)
	improve_skill(ch, SKILL_KICK);
      if (prob > 99)
	GET_SKILL(ch, SKILL_KICK) = 99;
      break;
    case CLASS_PALADIN:
    case CLASS_RANGER:
    case CLASS_THIEF:
      if (prob <= 80 && skill_imp < 10)
	improve_skill(ch, SKILL_KICK);
      if (prob > 80)
	GET_SKILL(ch, SKILL_KICK) = 80;
      break;
    case CLASS_CLERIC:
    case CLASS_MULTI_C_F:
    case CLASS_MULTI_F_T:
      if (prob <= 70 && skill_imp < 5)
	improve_skill(ch, SKILL_KICK);
      if (prob > 70)
	GET_SKILL(ch, SKILL_KICK) = 70;
      break;
    case CLASS_MULTI_F_M:
    case CLASS_MULTI_C_R:
    case CLASS_MULTI_F_I:
      if (prob <= 60 && skill_imp < 5)
	improve_skill(ch, SKILL_KICK);
      if (prob > 60)
	GET_SKILL(ch, SKILL_KICK) = 60;
      break;
    case CLASS_MULTI_C_F_M:
    case CLASS_MULTI_F_M_T:
      if (prob <= 50 && skill_imp < 5)
	improve_skill(ch, SKILL_KICK);
      if (prob > 50)
	GET_SKILL(ch, SKILL_KICK) = 50;
      break;
    default:
      send_to_char("You'd better leave all the martial arts to fighters.\r\n", ch);
      return FALSE;
    }				//switch

  kick_messages(ch, vict, dam);
  damage(ch, vict, dam, SKILL_KICK, -1, -1);

  if (dam > 0)
    do_knockBack(ch, vict, dam);

  WAIT_STATE(ch, PULSE_VIOLENCE * 3);
  if (dam > 0)
    return TRUE;
  else
    return FALSE;
}

ACMD(do_kick)
{
  struct char_data *vict;

  one_argument(argument, arg);

  if (!(vict = get_char_room_vis(ch, arg))) {
    if (FIGHTING(ch)) {
      vict = FIGHTING(ch);
    }				//if

    else {
      send_to_char("Kick who?\r\n", ch);
      return;
    }				//else

  }				//if

  if (!can_hit(ch, vict))
    return;
  if (vict == ch) {
    send_to_char("Aren't we funny today...\r\n", ch);
    return;
  }				//if

  perform_kick(ch, vict);
}				//do_kick


ACMD(do_style)
{
  one_argument(argument, buf);

  if (!*buf) {
    sprintf(buf, "Style: %s\n", fighting_styles[ch->char_specials.style]);
    send_to_char(buf, ch);
    return;
  }
  if (!str_cmp(buf, "aggressive")) {
    ch->char_specials.style = 0;
    send_to_char("You are now fighting aggressively.\n", ch);
    return;
  }
  if (!str_cmp(buf, "normal")) {
    ch->char_specials.style = 1;
    send_to_char("You are now fighting normally.\n", ch);
    return;
  }
  if (!str_cmp(buf, "defensive")) {
    ch->char_specials.style = 2;
    send_to_char("You are now fighting defensively.\n", ch);
    return;
  }
  send_to_char("That is not a style.\n", ch);
  return;
}

ACMD(do_feint)
{
  if (!IS_SET(AFF4_FLAGS(ch), AFF4_FEINT))
    SET_BIT(AFF4_FLAGS(ch), AFF4_FEINT);
  return;
}

ACMD(do_form)
{
  one_argument(argument, buf);

  if (!*buf) {
    sprintf(buf, "Form: %s\n", attack_forms[(int) GET_FORM(ch)]);
    send_to_char(buf, ch);
    return;
  }
  if (!str_cmp(buf, "thrust")) {
    ch->char_specials.s_o_t = FORM_THRUST;
    send_to_char("You are now thrusting with your weapon.\r\n", ch);
    return;
  }
  if (!str_cmp(buf, "swing")) {
    ch->char_specials.s_o_t = FORM_SWING;
    send_to_char("You are now swinging with your weapon.\r\n", ch);
    return;
  }
  send_to_char("That is not a form. Valid ones are: thrust, swing\r\n", ch);
  return;
}
