/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <signal.h>
#include <math.h>

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "handler.h"
#include "interpreter.h"
#include "db.h"
#include "spells.h"
#include "quest.h"
#include "skills.h"
#include "events.h"
#include "dg_scripts.h"
#include "sound.h"
#include "map.h"

/* External structures */
extern struct room_data *world;
extern struct index_data *mob_index;
extern struct message_list fight_messages[MAX_MESSAGES];
extern struct obj_data *object_list;
extern struct zone_data *zone_table;
extern int auto_save;		/* see config.c */
extern int max_exp_gain;	/* see config.c */
extern int max_exp_loss;	/* see config.c */
extern struct aq_data *aquest_table;
extern struct weapon_app_type weapon_app[];
extern const char rev_dir[];
extern struct str_app_type str_app[];
extern int aim_mod[];
extern char *dirs[];
extern struct index_data *obj_index;

/* External procedures */
ACMD(do_flee);
ACMD(do_return);
EVENTFUNC(fight_event);
EVENTFUNC(blink_event);
extern char *fread_action(FILE * fl, int nr);
extern char *fread_string(FILE * fl, char *error);
extern void hit(struct char_data *ch, struct char_data *victim, int type);
extern void forget(struct char_data *ch, struct char_data *victim);
extern void remember(struct char_data *ch, struct char_data *victim);
extern int ok_damage_shopkeeper(struct char_data *ch, struct char_data *victim);
extern byte ProximityCheck(struct char_data *ch, struct obj_data *obj, struct char_data *target, int door);
extern void kick_messages(struct char_data *ch, struct char_data *victim, int dam);
extern struct char_data *ForceReturn(struct char_data *ch);
extern char *item_condition(struct obj_data *obj);
extern int DamageItem(struct char_data *ch, struct obj_data *o, int num);
extern int DamageOneItem(struct char_data *ch, int dam_type, struct obj_data *obj, int dam);
extern void MakeScrap(struct char_data *ch, struct obj_data *obj, int dam_type);
extern void DamageStuff(struct char_data *v, int type, int dam);
extern byte IsShopKeeper(struct char_data *keeper);
extern int spelltype_to_damtype(int spelltype);
extern void smallmap_look(struct char_data *ch, sh_int size, int dif_room);
extern int special(struct char_data *ch, int cmd, char *command, char *arg, int why_called);
extern void MobCombat(struct char_data *ch, struct char_data *new_vict);

/* Weapon attack texts */
struct attack_hit_type attack_hit_text[] =
{
  {"hit", "hits", "hit"},	/* 0 */
  {"sting", "stings", "stung"},
  {"whip", "whips", "whipped"},
  {"slash", "slashes", "slashed"},
  {"bite", "bites", "bit"},
  {"bludgeon", "bludgeons", "bludgeoned"},	/* 5 */
  {"crush", "crushes", "crushed"},
  {"pound", "pounds", "pounded"},
  {"claw", "claws", "clawed"},
  {"maul", "mauls", "mauled"},
  {"thrash", "thrashes", "thrashed"},	/* 10 */
  {"pierce", "pierces", "pierced"},
  {"blast", "blasts", "blasted"},
  {"punch", "punches", "punched"},
  {"stab", "stabs", "stabbed"},
  {"chip", "chips", "chipped"}	/* 15 */
};


/* The Fight related routines */

void appear(struct char_data *ch)
{
  if (IS_AFFECTED(ch, AFF_HIDE)) {
    REMOVE_BIT(AFF_FLAGS(ch), AFF_HIDE);
    act("$n appears from the shadows.", FALSE, ch, 0, 0, PROXIMITY);
  }
  if (affected_by_spell(ch, SPELL_INVISIBILITY)) {
    affect_from_char(ch, SPELL_INVISIBILITY);
    if (GET_LEVEL(ch) < LVL_IMMORT) {
      act("$n slowly fades into existence.", FALSE, ch, 0, 0, PROXIMITY);
    } else
      act("You feel a strange presence as $n appears, seemingly from nowhere.",
	  FALSE, ch, 0, 0, PROXIMITY);
  }
}



void load_messages(void)
{
  FILE *fl;
  int i, type;
  struct message_type *messages;
  char chk[128];

  if (!(fl = fopen(MESS_FILE, "r"))) {
    sprintf(buf2, "SYSERR: Error reading combat message file %s", MESS_FILE);
    perror(buf2);
    raise(SIGSEGV);
  }
  for (i = 0; i < MAX_MESSAGES; i++) {
    fight_messages[i].a_type = 0;
    fight_messages[i].number_of_attacks = 0;
    fight_messages[i].msg = 0;
  }


  fgets(chk, 128, fl);
  while (!feof(fl) && (*chk == '\n' || *chk == '*'))
    fgets(chk, 128, fl);

  while (*chk == 'M') {
    fgets(chk, 128, fl);
    sscanf(chk, " %d\n", &type);
    for (i = 0; (i < MAX_MESSAGES) && (fight_messages[i].a_type != type) &&
	 (fight_messages[i].a_type); i++);
    if (i >= MAX_MESSAGES) {
      fprintf(stderr, "SYSERR: Too many combat messages.  Increase MAX_MESSAGES and recompile.");
      raise(SIGSEGV);
    }
    CREATE(messages, struct message_type, 1);
    fight_messages[i].number_of_attacks++;
    fight_messages[i].a_type = type;
    messages->next = fight_messages[i].msg;
    fight_messages[i].msg = messages;

    messages->die_msg.attacker_msg = fread_action(fl, i);
    messages->die_msg.victim_msg = fread_action(fl, i);
    messages->die_msg.room_msg = fread_action(fl, i);
    messages->miss_msg.attacker_msg = fread_action(fl, i);
    messages->miss_msg.victim_msg = fread_action(fl, i);
    messages->miss_msg.room_msg = fread_action(fl, i);
    messages->hit_msg.attacker_msg = fread_action(fl, i);
    messages->hit_msg.victim_msg = fread_action(fl, i);
    messages->hit_msg.room_msg = fread_action(fl, i);
    messages->god_msg.attacker_msg = fread_action(fl, i);
    messages->god_msg.victim_msg = fread_action(fl, i);
    messages->god_msg.room_msg = fread_action(fl, i);
    fgets(chk, 128, fl);
    while (!feof(fl) && (*chk == '\n' || *chk == '*'))
      fgets(chk, 128, fl);
  }

  fclose(fl);
}


void update_pos(struct char_data *victim)
{
  if ((GET_HIT(victim) > 0) && (GET_STAT(victim) > STAT_STUNNED)
      && (GET_HIT_HD(victim) > 0))
    return;
  else if ((GET_HIT(victim) > 0) && (GET_HIT_HD(victim) > 0))
    SET_POS(victim, GET_POS(victim) + STAT_NORMAL);
  else if ((GET_HIT(victim) <= HIT_DEAD) ||
	   (GET_HIT_HD(victim) <= HIT_DEAD))
    SET_POS(victim, POS_PRONE + STAT_DEAD);
  else if ((GET_HIT(victim) <= HIT_MORTALLYW) ||
	   (GET_HIT_HD(victim) <= HIT_MORTALLYW))
    SET_POS(victim, POS_PRONE + STAT_DYING);
  else if ((GET_HIT(victim) <= HIT_INCAP) ||
	   (GET_HIT_HD(victim) <= HIT_INCAP))
    SET_POS(victim, POS_PRONE + STAT_INCAP);
  else
    SET_POS(victim, POS_STANDING + STAT_NORMAL);
}


/* start one char fighting another (yes, it is horrible, I know... )  */
void set_fighting(struct char_data *ch, struct char_data *vict)
{
  struct fight_event_data *fed;
  int fight_delay(struct char_data *ch);
  char Gbuf[256];

  if (ch == vict)
    return;

  ch->char_specials.attacks_left = 0;

  assert(!FIGHTING(ch));
  SET_FACING(ch, face_angle(angle_offset(ch, NULL, vict, NULL)));

  if (IS_AFFECTED(ch, AFF_SLEEP))
    affect_from_char(ch, SPELL_SLEEP);

  if (IS_AFFECTED4(ch, AFF4_SLNT_MOVE))
    REMOVE_BIT(AFF4_FLAGS(ch), AFF4_SLNT_MOVE);

  // can't fight someone and watch at the same time...
  // Unless that is the person you are fighting (Boccob)
  if ((ch->watching) && (ch->watching->vict != vict))
    stop_watcher(ch);

  CREATE(fed, struct fight_event_data, 1);
  fed->ch = ch;
  fed->vict = vict;
  if (GET_ACTION(ch))
    event_cancel(GET_ACTION(ch));
  GET_ACTION(ch) = event_create(fight_event, fed, fight_delay(ch));
  FIGHTING(ch) = vict;
  HUNTING(ch) = vict;
  if (IS_NPC(ch))
    GET_AIMING(ch) = LOCATION_TORSO;
  if (IS_NPC(vict) && IS_CITIZEN(vict)) {
    sprintf(Gbuf, "shout *Guards! %s is trying to kill me!\r\n", fname(GET_NAME(ch)));
    //sprintf(buf1, "mcallforguards %s att_murder", fname(GET_NAME(ch)));
    command_interpreter(vict, Gbuf);
  }
}



/* remove a char from the list of fighting chars */


void make_corpse(struct char_data *ch)
{
  struct obj_data *corpse, *o;
  struct obj_data *money;
  int i;
  extern int max_npc_corpse_time, max_pc_corpse_time;

  struct obj_data *create_money(int amount, int type, int king);

  corpse = create_obj();

  corpse->item_number = 1;
  corpse->in_room = NOWHERE;
  corpse->name = str_dup("corpse");

  sprintf(buf2, "The corpse of %s is lying here.", GET_NAME(ch));
  corpse->description = str_dup(buf2);

  sprintf(buf2, "the corpse of %s", GET_NAME(ch));
  corpse->short_description = str_dup(buf2);

  GET_OBJ_TYPE(corpse) = ITEM_CORPSE;
  GET_OBJ_WEAR(corpse) = ITEM_WEAR_TAKE;
  GET_OBJ_VAL(corpse, 0) = 0;	/* You can't store stuff in a corpse */
//  GET_OBJ_VAL(corpse, 3) = 1; /* corpse identifier */
  GET_OBJ_VAL(corpse, 6) = GET_RACE(ch);	/* race identifier */
  /* identify owner */
  if (IS_NPC(ch))
    GET_OBJ_VAL(corpse, 7) = GET_MOB_VNUM(ch);
  else {
    GET_OBJ_VAL(corpse, 5) = 1;
    GET_OBJ_VAL(corpse, 7) = GET_ID(ch);
  }				//else

  GET_OBJ_WEIGHT(corpse) = GET_WEIGHT(ch) + IS_CARRYING_W(ch);
  if (IS_NPC(ch))
    GET_OBJ_TIMER(corpse) = max_npc_corpse_time;
  else
    GET_OBJ_TIMER(corpse) = max_pc_corpse_time;

  /* transfer character's inventory to the corpse */
  corpse->contains = ch->carrying;
  for (o = corpse->contains; o != NULL; o = o->next_content)
    o->in_obj = corpse;
  object_list_new_owner(corpse, NULL);

  /* transfer character's equipment to the corpse */
  for (i = 0; i < NUM_WEARS; i++)
    if (GET_EQ(ch, i)) {
      remove_otrigger(GET_EQ(ch, i), ch);
      obj_to_obj(unequip_char(ch, i), corpse);
    }
  /* transfer gold */
  if (GET_MONEY(ch) > 0) {
    /* following 'if' clause added to fix gold duplication loophole */
    if (IS_NPC(ch) || (!IS_NPC(ch) && ch->desc)) {
      money = create_money(1, 1, 0);
      obj_to_obj(money, corpse);
    }
    SUB_MONEY(ch, GET_MONEY(ch));
  }
  ch->carrying = NULL;
  IS_CARRYING_N(ch) = 0;
  IS_CARRYING_W(ch) = 0;

  put_obj_to_room(corpse, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
}

void stop_fighting(struct char_data *ch)
{
  if (IS_SET(AFF4_FLAGS(ch), AFF4_FEINT))
    REMOVE_BIT(AFF4_FLAGS(ch), AFF4_FEINT);
  while (affected_by_spell(ch, SKILL_FEINT))
    affect_from_char(ch, SKILL_FEINT);
  GET_ACTION(ch) = NULL;
  FIGHTING(ch) = NULL;
  ch->char_specials.attacks_left = 0;
  update_pos(ch);
}


/* When ch kills victim */
void change_alignment(struct char_data *ch, struct char_data *victim)
{
  return;
  /*
   * new alignment change algorithm: if you kill a monster with alignment A,
   * you move 1/16th of the way to having alignment -A.  Simple and fast.
   */
  /* Fix this for new system; Faf 5/20/99 */
  SET_ALIGNMENT(ch, (-GET_ALIGNMENT(victim) - GET_ALIGNMENT(ch)) >> 4);
}


void raw_kill(struct char_data *ch, struct char_data *killer)
{
  ch = ForceReturn(ch);

  if (FIGHTING(ch)) {
    stop_fighting(ch);
  }
  while (ch->affected)
    affect_remove(ch, ch->affected);

  if (killer) {
    autoquest_trigger_check(killer, ch, NULL, AQ_MOB_KILL);
    death_mtrigger(ch, killer);
  }
  make_corpse(ch);
  extract_char(ch);

  if (killer)
    autoquest_trigger_check(killer, NULL, NULL, AQ_MOB_SAVE);
}



void die(struct char_data *ch, struct char_data *killer)
{
  if (killer && FIGHTING(killer) && FIGHTING(killer) == ch)
    stop_fighting(killer);

  ch = ForceReturn(ch);

  if (IS_MOB(ch) && mob_index[GET_MOB_RNUM(ch)].func != NULL)
    (mob_index[GET_MOB_RNUM(ch)].func) (ch, killer, 0, "", "", PROC_DIE);

  gain_exp(ch, -(GET_EXP(ch) >> 1));
  raw_kill(ch, killer);
}



void perform_group_gain(struct char_data *ch, int base,
			struct char_data *victim)
{
  int share;

  share = MIN(max_exp_gain, MAX(1, base));

  if (share > 1) {
    sprintf(buf2, "You receive your share of experience -- %d points.\r\n", share);
    send_to_char(buf2, ch);
  } else
    send_to_char("You receive your share of experience -- one measly little point!\r\n", ch);

  gain_exp(ch, share);
  change_alignment(ch, victim);
}


void group_gain(struct char_data *ch, struct char_data *victim)
{
  int tot_members, base, tot_gain;
  struct char_data *k;
  struct follow_type *f;

  if (!(k = ch->master))
    k = ch;

  if (IS_AFFECTED(k, AFF_GROUP) && (k->in_room == ch->in_room))
    tot_members = 1;
  else
    tot_members = 0;

  for (f = k->followers; f; f = f->next)
    if (IS_AFFECTED(f->follower, AFF_GROUP) && f->follower->in_room == ch->in_room)
      tot_members++;

  /* round up to the next highest tot_members */
  tot_gain = (GET_EXP(victim)) + tot_members - 1;

  /* prevent illegal xp creation when killing players */
  if (!IS_NPC(victim))
    tot_gain = MIN(max_exp_loss * 2 / 3, tot_gain);

  if (tot_members >= 1)
    base = MAX(1, tot_gain / tot_members);
  else
    base = 0;

  if (IS_AFFECTED(k, AFF_GROUP) && k->in_room == ch->in_room)
    perform_group_gain(k, base, victim);

  for (f = k->followers; f; f = f->next)
    if (IS_AFFECTED(f->follower, AFF_GROUP) && f->follower->in_room == ch->in_room)
      perform_group_gain(f->follower, base, victim);
}


void solo_gain(struct char_data *ch, struct char_data *victim)
{
  int exp;

  exp = MIN(max_exp_gain, GET_EXP(victim));

  /* Calculate level-difference bonus */
  if (IS_NPC(ch))
    exp += MAX(0, (exp * MIN(4, (GET_LEVEL(victim) - GET_LEVEL(ch)))) / 8);
  else
    exp += MAX(0, (exp * MIN(8, (GET_LEVEL(victim) - GET_LEVEL(ch)))) / 8);

  exp = MAX(exp, 1);

  if (exp > 1) {
    sprintf(buf2, "You receive %d experience points.\r\n", exp);
    send_to_char(buf2, ch);
  } else
    send_to_char("You receive one lousy experience point.\r\n", ch);

  gain_exp(ch, exp);
  change_alignment(ch, victim);
}

/* We call this rather than massive damage, from instantkills of any type */
void SuddenDeath(struct char_data *ch, struct char_data *victim, char *kill_type)
{
  int XP;

  if (!ch || (ch == victim) || IS_TRUSTED(ch)) {
    if (IS_PC(victim)) {
      sprintf(buf, "%s died in %d to SuddenDeath from %s",
	      GET_NAME(victim),
	      ((victim->in_room == NOWHERE) ?
	       -1 : world[victim->in_room].number),
	      GET_NAME(ch));
      mudlog(buf, BRF, LVL_IMMORT, TRUE);
    }
    die(victim, ch);
    return;
  }
  if (ch->group) {
    group_gain(ch, victim);
  } else {
    send_to_char("&+YYou receive your share of experience.\r\n", ch);
    XP = (GET_EXP(victim));
    if (IsShopKeeper(victim))
      XP = 1;
    if (XP && IS_PC(ch))
      vlogf("%s: %d, instantkilled: %s", GET_NAME(ch), XP, GET_NAME(victim));
    gain_exp(ch, XP);
    change_alignment(ch, victim);
  }

  /* special death messages, assuming there is one */
#if 0
  if (kill_type)
    for (i = 0; i < MAX_MESSAGES; i++)
      if (fight_messages[i].a_type == kill_type) {
	k = dice(1, fight_messages[i].number_of_attacks);
	for (j = 1, messages = fight_messages[i].msg;
	     (j < k) && (messages); j++)
	  messages = messages->next;

	act(messages->die_msg.attacker_msg, FALSE, ch, ch->equipment[WEAR_WIELD], victim, TO_CHAR);
	act(messages->die_msg.victim_msg, FALSE, ch, ch->equipment[WEAR_WIELD], victim, TO_VICT);
	act(messages->die_msg.room_msg, FALSE, ch, ch->equipment[WEAR_WIELD], victim, TO_NOTVICT);
      }
#endif
  act("&+r$n is dead! R.I.P.", TRUE, victim, 0, 0, TO_ROOM);
  act("&=rlYou feel yourself falling to the ground.", FALSE, victim, 0, 0, TO_CHAR);
  act("&=rlYour soul leaves your body in the cold sleep of death...", FALSE, victim, 0, 0, TO_CHAR);

  if (IS_PC(victim)) {
    sprintf(buf, "%s died to SuddenDeath in %d",
	    GET_NAME(victim), ((victim->in_room == NOWHERE) ? -1 : world[victim->in_room].number));
    mudlog(buf, BRF, LVL_IMMORT, TRUE);

  }
  die(victim, ch);
}


 /* Location of attack texts */
struct attack_hit_type location_hit_text[NUMBER_LOCATIONS] =
{
  {
    " in $S vital area", " vitals", ""
  },
  {
    " in $S brain", "brain", ""
  },
  {
    " on $S head", " head", ""
  },
  {
    " on $S shield arm", " shield arm", ""
  },
  {
    " on $S shield hand", " shield hand", ""
  },
  {
    " on $S weapon hand", " weapon hand", ""
  },
  {
    " on $S weapon arm", " weapon arm", ""
  },
  {
    "", " torso", ""
  },				/* body/torso */
  {
    " on $S right leg", " right leg", ""
  },
  {
    " on $S left leg", " left leg", ""
  },
  {
    " on $S right foot", " right foot", ""
  },
  {
    " on $S left foot", " left foot", ""
  }
};

char *replace_string(const char *str, const char *weapon, const char *weapon_s,
		     const char *location_hit, const char *location_hit_s)
{
  static char buf[256];
  char *cp;

  cp = buf;

  for (; *str; str++) {
    if (*str == '#') {
      switch (*(++str)) {
      case 'w':
	for (; *weapon; *(cp++) = *(weapon++));
	break;
      case 'W':
	for (; *weapon_s; *(cp++) = *(weapon_s++));
	break;
      case 'l':
	for (; *location_hit; *(cp++) = *(location_hit++));
	break;
      case 'L':
	for (; *location_hit_s; *(cp++) = *(location_hit_s++));
	break;
      default:
	*(cp++) = '#';
	break;
      }
    } else {
      *(cp++) = *str;
    }
    *cp = 0;
  }
  return (buf);
}

void dam_message(int dam, struct char_data *ch, struct char_data *victim, int w_type)
{
  struct obj_data *wield = NULL;
  char *buf, buf_char[160], buf_vict[160], buf_notvict[160];
  int w_percent, h_percent, max_dam = 0, w_loop, h_loop;
  int hitloc;
  static int dam_ref[] =
  {
    0, 2, 7, 10, 15, 25, 40, 55, 70, 85, 9999
  };
  const char *weapon_damage[] =
  {
    "",
    " laughable",
    " feeble",
    " lame",
    " mediocre",
    " fine",
    " powerful",
    " mighty",
    " awesome",
    " devastating",
    " godly"
  };
#if 0
  /* these are with body parts. Appear to crash sometimes */
  const char *victim_damage[] =
  {
    "grazes $w$W#l.",
    "wounds $w$W#l.",
    "hits $w$W#l.",
    "hits $w hard#L.",
    "hits $w very hard#L.",
    "seriously wounds $w$W#l.",
    "enshrouds $w in a mist of blood!",		/* no hit_loc for last 4 */
    "beats the crap out of $w.",	/* Dead is dead, wherever it hits */
    "nearly slaughters $w!",
    "absolutely destroys $w!",
    "hits $w."
  };
#else
  const char *victim_damage[] =
  {
    "grazes $w.",
    "wounds $w.",
    "hits $w.",
    "hits $w hard.",
    "hits $w very hard.",
    "seriously wounds $w.",
    "enshrouds $w in a mist of blood!",
    "beats the crap out of $w.",
    "nearly slaughters $w!",
    "absolutely destroys $w!",
    "hits $w."
  };
#endif
  hitloc = GET_AIMING(ch);
  /* possibly hit a different spot */
  if (number(1, 10) > 9 - aim_mod[hitloc])
    hitloc = number(0, NUMBER_LOCATIONS);

  if (GET_EQ(ch, WEAR_WIELD)) {
    wield = GET_EQ(ch, WEAR_WIELD);
    max_dam = GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 1) * GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 2);
  }
  w_type -= TYPE_HIT;		/* Change to base of table with text */

/*** Temp section ***/
  /* following need some more work... armor, shield etc need variables */
  if (!dam) {
    sprintf(buf_notvict, "$n misses $N.");
    sprintf(buf_char, "You miss $N.");
    sprintf(buf_vict, "$n misses you.");
  } else if (dam == -1) {
    sprintf(buf_notvict, "$n tries to #w $N, who blocks with $S shield.");
    sprintf(buf_char, "You try to #w $N, but are blocked by $S shield.");
    sprintf(buf_vict, "$n tries to #w you, but you block with your shield.");
  } else if (dam == -2) {
    sprintf(buf_notvict, "$n tries to #w $N, who parrys.");
    sprintf(buf_char, "You try to #w $N, who parrys.");
    sprintf(buf_vict, "$n tries to #w you, but you parry.");
  } else if (dam == -3) {
    sprintf(buf_notvict, "$n tries to #w $N, who dodges.");
    sprintf(buf_char, "You try to #w $N, who dodges.");
    sprintf(buf_vict, "$n tries to #w you, but you dodge.");
  } else if (dam == -4) {
    sprintf(buf_notvict, "$n's weapon bounces off of $N's armor.");
    sprintf(buf_char, "Your weapon bounces off of $N's armor.");
    sprintf(buf_vict, "$n's weapon bounces off of your armor..");
  } else if (dam == -5) {
    sprintf(buf_notvict, "$n tries to #w $N, but $S armor absorbs the damage.");
    sprintf(buf_char, "You try to #w $N, but $S armor absorbs the damage.");
    sprintf(buf_vict, "$n tries to #w you, but your armor absorbs the damage.");
  } else if (dam == -6) {
    sprintf(buf_notvict, "$n #w $N, who doesn't seem to be affected.");
    sprintf(buf_char, "You #w $N, but $E doesn't seem to be affected.");
    sprintf(buf_vict, "$n #w you, but you aren't affected.");
  }
  /* more temp shit */
  if (dam <= 0) {
    buf = replace_string(buf_notvict, attack_hit_text[w_type].plural, attack_hit_text[w_type].singular,
    location_hit_text[hitloc].plural, location_hit_text[hitloc].singular);
    act(buf, FALSE, ch, wield, victim, TO_NOTVICT);
    buf = replace_string(buf_char, attack_hit_text[w_type].plural, attack_hit_text[w_type].singular,
    location_hit_text[hitloc].plural, location_hit_text[hitloc].singular);
    act(buf, FALSE, ch, wield, victim, TO_CHAR);
    buf = replace_string(buf_vict, attack_hit_text[w_type].plural, attack_hit_text[w_type].singular,
    location_hit_text[hitloc].plural, location_hit_text[hitloc].singular);
    act(buf, FALSE, ch, wield, victim, TO_VICT);
    return;
  }
/***** End of temporary section *****/
  h_percent =
      BOUNDED(0, (int) ((dam * 100) / (MAX(1, GET_HIT(victim)) + dam + 10)), 100);
  w_percent =
      BOUNDED(0, (int) ((dam * 100) / (max_dam + dam + 10)), 100);

  for (h_loop = 0; (h_percent > dam_ref[h_loop]); h_loop++);
  if (h_loop > 10)
    h_loop = 10;		/* h and w require reverse. dont ask why */
  for (w_loop = 0; (w_percent > dam_ref[w_loop]); w_loop++);
  if (w_loop > 10)
    w_loop = 0;

  if (!number(0, 3))
    w_loop = 0;

  sprintf(buf_char, "Your%s #W %s", weapon_damage[w_loop], victim_damage[h_loop]);
  sprintf(buf_vict, "$n's%s #W %s", weapon_damage[w_loop], victim_damage[h_loop]);
  sprintf(buf_notvict, "$n's%s #W %s", weapon_damage[w_loop], victim_damage[h_loop]);

  buf = replace_string(buf_notvict, attack_hit_text[w_type].plural, attack_hit_text[w_type].singular,
   location_hit_text[hitloc].plural, location_hit_text[hitloc].singular);
  act(buf, FALSE, ch, wield, victim, TO_NOTVICT);

  buf = replace_string(buf_char, attack_hit_text[w_type].plural, attack_hit_text[w_type].singular,
   location_hit_text[hitloc].plural, location_hit_text[hitloc].singular);
  act(buf, FALSE, ch, wield, victim, TO_CHAR);

  buf = replace_string(buf_vict, attack_hit_text[w_type].plural, attack_hit_text[w_type].singular,
   location_hit_text[hitloc].plural, location_hit_text[hitloc].singular);
  act(buf, FALSE, ch, wield, victim, TO_VICT);

//  play_sound("!!SOUND(battle* Q)", ch, ch->in_room, 0, 0, GET_Z_CORD(ch), TO_ROOM);
  //  play_sound("!!SOUND(battle* Q)", ch, ch->in_room, 0, 0, GET_Z_CORD(ch), TO_ROOM);
}

/*
 * message for doing damage with a spell or skill
 */
int skill_message(int dam, struct char_data *ch, struct char_data *vict,
		  int attacktype)
{
  int i, j, nr;
  struct message_type *msg;

  struct obj_data *weap = GET_EQ(ch, WEAR_WIELD);

  for (i = 0; i < MAX_MESSAGES; i++) {
    if (fight_messages[i].a_type == attacktype) {
      nr = dice(1, fight_messages[i].number_of_attacks);
      for (j = 1, msg = fight_messages[i].msg; (j < nr) && msg; j++)
	msg = msg->next;

      if (!IS_NPC(vict) && (GET_LEVEL(vict) >= LVL_IMMORT)) {
	act(msg->god_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
	act(msg->god_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT);
	act(msg->god_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
      } else if (dam != 0) {
	if (ch == vict) {
	} else {
	  if (GET_STAT(vict) == STAT_DEAD) {
	    act(msg->die_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
	    act(msg->die_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT | TO_SLEEP);
	    act(msg->die_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
	  } else {
	    act(msg->hit_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
	    act(msg->hit_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT | TO_SLEEP);
	    act(msg->hit_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
	  }
	}
      } else if (ch != vict) {	/* Dam == 0 */
	act(msg->miss_msg.attacker_msg, FALSE, ch, weap, vict, TO_CHAR);
	act(msg->miss_msg.victim_msg, FALSE, ch, weap, vict, TO_VICT | TO_SLEEP);
	act(msg->miss_msg.room_msg, FALSE, ch, weap, vict, TO_NOTVICT);
      }
      return 1;
    }
  }
  return 0;
}


void damage(struct char_data *ch, struct char_data *victim, int dam,
	    int attacktype, int which, int location)
{
  int orig_dam = dam;
  int dam2 = 0, exp_val = 0;
  struct affected_type *hjp;

  if (!victim || !ch)
    return;

  if (GET_STAT(victim) <= STAT_DEAD) {
    log("SYSERR: Attempt to damage a corpse.");
    return;
  }
  /* peaceful rooms */
  if (ch->nr != real_mobile(DG_CASTER_PROXY) &&
      ch != victim && ROOM_FLAGGED(ch->in_room, ROOM_PEACEFUL)) {
    send_to_char("This room just has such a peaceful, easy feeling...\r\n", ch);
    return;
  }
  /* shopkeeper protection */
  if (!ok_damage_shopkeeper(ch, victim))
    return;

  /* You can't damage an immortal! */
  if (!IS_NPC(victim) && IS_TRUSTED(victim) && (dam > 0))
    dam = 0;

  if (victim != ch) {
    /* Start the attacker fighting the victim */
    if (GET_STAT(ch) > STAT_STUNNED && (!FIGHTING(ch)))
      set_fighting(ch, victim);


    /* Start the victim fighting the attacker */
    if (GET_STAT(victim) > STAT_STUNNED && (!FIGHTING(victim))) {
      set_fighting(victim, ch);
      if ((MOB_FLAGGED(victim, MOB_MEMORY) || IS_GUARD(victim) || IS_CITIZEN(victim))
	  && !IS_NPC(ch)) {
	// set the flag if it wasnt, as this will aid checks in the future
	SET_BIT(MOB_FLAGS(victim), MOB_MEMORY);
	remember(victim, ch);
      }
    }
  }
  /* If you attack a pet, it hates your guts */
  if (victim->master == ch)
    stop_follower(victim);

  /* If the attacker is invisible, he becomes visible */
  if (IS_AFFECTED(ch, AFF_INVISIBLE | AFF_HIDE))
    appear(ch);


  if (IS_WEAPON(attacktype) && (dam > 0) && (ch != victim)) {
    if (stopped_by_passive_defense(ch, victim))
      dam = -4;
    else {
      dam -= get_damage_reduction(ch, victim);
      if (dam <= 0)
	dam = -5;
    }
    if (dam > 0) {
      if (NATURAL_ATTACK(attacktype) && (IS_AFFECTED3(victim, AFF3_PLUSONE)
				 || IS_AFFECTED3(victim, AFF3_PLUSTWO) ||
				  IS_AFFECTED3(victim, AFF3_PLUSTHREE) ||
				   IS_AFFECTED3(victim, AFF3_PLUSFOUR) ||
				    IS_AFFECTED3(victim, AFF3_PLUSFIVE)))
	dam = -6;
      else if ((IS_AFFECTED3(victim, AFF3_PLUSONE) && !CAN_HIT_ONE(ch, which)) ||
	(IS_AFFECTED3(victim, AFF3_PLUSTWO) && !CAN_HIT_TWO(ch, which)) ||
	       (IS_AFFECTED3(victim, AFF3_PLUSTHREE) && !CAN_HIT_THREE(ch, which)) ||
	       (IS_AFFECTED3(victim, AFF3_PLUSFOUR) && !CAN_HIT_FOUR(ch, which)) ||
       (IS_AFFECTED3(victim, AFF3_PLUSFIVE) && !CAN_HIT_FIVE(ch, which)))
	dam = -6;
    }
    if (orig_dam > 0) {
      if (IS_AFFECTED2(victim, AFF2_FIRESHIELD)) {
	act("$n is burned when $e hits $N.", FALSE, ch, NULL, victim, TO_OTHERS);
	act("You are burned when you hit $N.", TRUE, ch, NULL, victim, TO_CHAR);
	act("$n is burned when $e hits you.", FALSE, ch, NULL, victim, TO_VICT);
	damage(victim, ch, orig_dam, SPELL_FIRE_SHIELD, -1, -1);
      }
      if (IS_AFFECTED3(victim, AFF3_COLDSHIELD)) {
	act("$n shivers when $e hits $N.", FALSE, ch, NULL, victim, TO_OTHERS);
	act("You shiver when you hit $N.", TRUE, ch, NULL, victim, TO_CHAR);
	act("$n shivers when $e hits you.", FALSE, ch, NULL, victim, TO_VICT);
	damage(victim, ch, orig_dam, SPELL_COLD_SHIELD, -1, -1);
      }
    }
  } else {
    switch (spelltype_to_damtype(attacktype)) {
    case TYPE_FIRE:
      if (IS_AFFECTED3(victim, AFF3_COLDSHIELD))
	dam /= 2;
      break;
    case TYPE_COLD:
      if (IS_AFFECTED2(victim, AFF2_FIRESHIELD))
	dam /= 2;
      break;
    default:
    }
  }
  /* Set the maximum damage per round and subtract the hit points */
  dam2 = MAX(MIN(dam, 1000), 0);

  if (IS_SWINGING(ch) && IS_CUT(attacktype)) {
    dam2 *= 1.5;
  } else if (IS_IMPALE(attacktype)) {
    switch (location) {
    case LOCATION_TORSO:
      dam2 *= 2;
      break;
    case LOCATION_VITALS:
      dam2 *= 3;
      break;
    case LOCATION_FACE:
      dam2 *= 2;
      break;
    case LOCATION_BRAIN:
      dam2 *= 4;
      break;
    }
  }
  if (affected_by_spell(victim, SPELL_ARMOR)) {
    for (hjp = ch->affected; hjp; hjp = hjp->next) {
      if (hjp->type == SPELL_ARMOR)
	hjp->modifier -= dam2;

      if (hjp->modifier < 0)
	affect_from_char(victim, SPELL_ARMOR);
    }
  }
  /* pulled punches */
  if (!PRF_FLAGGED(ch, PRF_PUMMEL))
    alter_hit(victim, dam2, location, 0);

  /* Gain exp for the hit */
  if (IS_NPC(victim))
    exp_val = (GET_EXP(victim) * dam2) / GET_MAX_HIT(victim);
  else
    exp_val = GET_LEVEL(ch) * dam2;
  if (ch != victim)
    group_gain_exp(ch, exp_val);

  update_pos(victim);

  if (special(ch, 0, "", "", PROC_HIT))
    return;

  /*
   * skill_message sends a message from the messages file in lib/misc.
   * dam_message just sends a generic "You hit $n extremely hard.".
   * skill_message is preferable to dam_message because it is more
   * descriptive.
   * 
   * If we are _not_ attacking with a weapon (i.e. a spell), always use
   * skill_message. If we are attacking with a weapon: If this is a miss or a
   * death blow, send a skill_message if one exists; if not, default to a
   * dam_message. Otherwise, always send a dam_message.
   */
#if 0
  if (!IS_WEAPON(attacktype))
    if (attacktype == SKILL_KICK)
      kick_messages(ch, victim, dam);
    else
      skill_message(dam, ch, victim, attacktype);
  else {
    if (GET_STAT(victim) == STAT_DEAD || dam == 0) {
      if (!skill_message(dam, ch, victim, attacktype))
	dam_message(dam, ch, victim, attacktype);
    } else {
      dam_message(dam, ch, victim, attacktype);
    }
  }
#endif
  if (IS_WEAPON(attacktype))
    dam_message(dam, ch, victim, attacktype);

  if (IS_WEAPON(attacktype) && (orig_dam > 0)) {
    DamageOneItem(victim, attacktype,
		  GET_EQ(victim, parse_aiming(location)), orig_dam);
  } else if (orig_dam > 0)
    DamageStuff(victim, attacktype, orig_dam);

#if 0
  if (ch->equipment[WEAR_WIELD] && IS_WEAPON(attacktype))
    DamageOneItem(ch, TYPE_CHIP, ch->equipment[WEAR_WIELD], orig_dam);
#endif

  /* Use send_to_char -- act() doesn't send message if you are DEAD. */
  switch (GET_STAT(victim)) {
  case STAT_DYING:
    act("$n is mortally wounded, and will die soon, if not aided.", TRUE, victim, 0, 0, PROXIMITY);
    send_to_char("You are mortally wounded, and will die soon, if not aided.\r\n", victim);
    break;
/*
   case STAT_INCAP:
   act("$n is incapacitated and will slowly die, if not aided.", TRUE, victim, 0, 0, PROXIMITY);
   send_to_char("You are incapacitated an will slowly die, if not aided.\r\n", victim);
   break;
 */
  case STAT_STUNNED:
    act("$n is stunned, but will probably regain consciousness again.", TRUE, victim, 0, 0, PROXIMITY);
    send_to_char("You're stunned, but will probably regain consciousness again.\r\n", victim);
    break;
  case STAT_DEAD:
    act("$n is dead!  R.I.P.", FALSE, victim, 0, 0, PROXIMITY);
    send_to_char("You are dead!  Sorry...\r\n", victim);
    break;

  default:			/* >= POSITION SLEEPING */
    if (dam > (GET_MAX_HIT(victim) / 4))
      act("That really did HURT!", FALSE, victim, 0, 0, TO_CHAR);

    if (GET_HIT(victim) < (GET_MAX_HIT(victim) / 4)) {
      send_to_char("&+rYou are in SEVERE need of medical aid!&n\r\n", victim);
      if (MOB_FLAGGED(victim, MOB_WIMPY) && (ch != victim))
	do_flee(victim, "", 0, 0);
    }
    if (!IS_NPC(victim) && GET_WIMP_LEV(victim) && (victim != ch) &&
	GET_HIT(victim) < GET_WIMP_LEV(victim)) {
      send_to_char("Discretion gets the better of you, and you attempt to withdraw!\r\n", victim);
      do_flee(victim, "", 0, 0);
    }
    break;
  }

  /* Help out poor linkless people who are attacked */
  /* Gods are immune anyways, ignore them for now */
  if (!IS_NPC(victim) && !(victim->desc) && !IS_TRUSTED(victim)) {
    do_flee(victim, "", 0, 0);
    if (!FIGHTING(victim)) {
      act("$n is rescued by divine forces.", FALSE, victim, 0, 0, PROXIMITY);
      GET_WAS_IN(victim) = victim->in_room;
      char_from_room(victim);
      char_to_room(victim, 0, -1);
    }
  }
  /* stop someone from fighting if they're stunned or worse */
  if ((GET_STAT(victim) <= STAT_STUNNED) && (FIGHTING(victim)))
    stop_fighting(victim);

  /* Uh oh.  Victim died. */
  if (GET_STAT(victim) == STAT_DEAD) {
#if 0
// no more death exp
    if ((ch != victim) && (IS_NPC(victim) || victim->desc)) {
      if (IS_AFFECTED(ch, AFF_GROUP))
	group_gain(ch, victim);
      else
	solo_gain(ch, victim);
    }
#endif
    if (!IS_NPC(victim)) {
      sprintf(buf2, "%s killed by %s at %s", GET_NAME(victim), GET_NAME(ch),
	      world[victim->in_room].name);
      mudlog(buf2, BRF, LVL_IMMORT, TRUE);
      if (MOB_FLAGGED(ch, MOB_MEMORY))
	forget(ch, victim);
    }
    die(victim, ch);
  }
}


void do_knockBack(struct char_data *ch, struct char_data *vict, int dam)
{
  //damage required ranges from 2 (For tiny) to 128(for gargantuan)
  int dam_required = pow(2, GET_SIZE(vict));
  int direction = angle_offset(ch, NULL, vict, NULL);
  int distance = dam / dam_required;
  extern char *dirs[];
  float rad_dir = PI / 180.0 * direction;
  int d_x, d_y;

  if (!distance)
    return;

  sprintf(buf, "$n %s %s.", (distance > 2) ? "is sent flying" : "stumbles",
	  dirs[face_angle(direction)]);
  act(buf, FALSE, vict, NULL, ch, TO_PROXIMITY);
  sprintf(buf, "You %s %s.", (distance > 2) ? "are sent flying" : "stumble",
	  dirs[face_angle(direction)]);
  act(buf, FALSE, vict, NULL, NULL, TO_CHAR);

  d_x = sin(rad_dir) * distance;
  d_y = cos(rad_dir) * distance;

  GET_X_CORD(vict) = BOUNDED(1, GET_X_CORD(vict) - d_x,
			     GET_X_SIZE(vict->in_room) - 1);
  GET_Y_CORD(vict) = BOUNDED(1, GET_Y_CORD(vict) + d_y,
			     GET_Y_SIZE(vict->in_room) - 1);

  if ((GET_X_CORD(vict) == 1) || (GET_Y_CORD(vict) == 1) ||
      (GET_X_CORD(vict) == (GET_X_SIZE(vict->in_room) - 1)) ||
      (GET_Y_CORD(vict) == (GET_Y_SIZE(vict->in_room) - 1))) {
    sprintf(buf, "$N %s the wall.", (distance > 2) ? "crashes into" : "leans against");
    act(buf, FALSE, ch, NULL, vict, PROXIMITY);
    act(buf, FALSE, ch, NULL, vict, TO_CHAR);
    sprintf(buf, "You %s the wall.", (distance > 2) ? "crash into" : "lean against");
    act(buf, FALSE, ch, NULL, vict, TO_VICT);
  }
  if (FIGHTING(ch) && (FIGHTING(ch) == vict))
    stop_fighting(ch);
  if (IS_NPC(ch))
    HUNTING(ch) = vict;
  if (FIGHTING(vict) && (FIGHTING(vict) == ch))
    stop_fighting(vict);
  if (IS_NPC(vict))
    HUNTING(vict) = ch;
}


char can_block(struct char_data *ch, struct char_data *vict)
{
  struct obj_data *shield = GET_EQ(vict, WEAR_SHIELD);
  int angle;

  //if vict does not have shield
  if (!shield)
    return 0;

  // crippled arm
  if (IS_CRIPPLED(vict, LOCATION_SHIELD_ARM))
    return 0;

  //blocked once this round
  if (affected_by_spell(vict, SKILL_BLOCK))
    return 0;

  // character does not have shield skill
  if (!GET_SKILL(ch, SKILL_SHIELD))
    return 0;

  // if wielding and doesnt have sword_shield skill
  if (GET_EQ(ch, WEAR_WIELD) && !GET_SKILL(ch, SKILL_STYLE_WEAPON_SHIELD))
    return 0;

  // if ch is not in front of vict
  angle = positive_angle(angle_offset(vict, NULL, ch, NULL)
			 - angle_facing(GET_FACING(vict)));
  switch (GET_DOM_HAND(vict)) {
  case HAND_RIGHT:
    if ((angle > 135) && (angle < 315))
      return 0;
  case HAND_LEFT:
    if ((angle > 45) && (angle < 225))
      return 0;
  }

  // if vict is all out offense
  if (GET_STYLE(vict) == STYLE_AGGRESSIVE)
    return 0;

  //otherwise, should be ok to block
  return 1;
}

char should_block(struct char_data *ch, int location, int dam)
{
  if (dam > GET_DR(ch, location))
    return 1;
  return 0;
}

/* positive mod makes parrying easier */
char successful_block(struct char_data *ch, int location, int mod)
{
  struct affected_type *hjp, *next;
  int feint_mod = 0, roll = dice(3, 6);
  int skill_mod = 0;

  if (GET_EQ(ch, WEAR_WIELD))
    skill_mod += GET_SKILL(ch, SKILL_STYLE_WEAPON_SHIELD);
  skill_mod += GET_SKILL(ch, SKILL_SHIELD);
  skill_mod /= 2;

  if (affected_by_spell(ch, SKILL_FEINT)) {
    for (hjp = ch->affected; hjp; hjp = next) {
      next = hjp->next;
      if (hjp->type == SKILL_FEINT) {
	feint_mod = hjp->modifier;
	break;
      }
    }
  }
  affect_from_char(ch, SKILL_FEINT);

  //try block
  if (roll < (GET_DEX(ch) + thaco_to_skill(ch) - feint_mod + mod + skill_mod) / 2) {
    return 1;
  }
  return 0;
}

/* positive modifier makes parrying easier */
char successful_parry(struct char_data *ch, int modifier, struct obj_data *o)
{
  struct affected_type *hjp, *next;
  int feint_mod = 0, mod = 0;
  struct obj_data *obj;
  int success = 0, which = 0;
  struct obj_data *weapon, *weapon2;
  struct affected_type *af;
  extern const int perc_dam[NUM_MATERIALS];

  weapon = GET_EQ(ch, WEAR_WIELD);
  weapon2 = GET_EQ(ch, WEAR_WIELD2);

  if (!weapon && !weapon2)
    return 0;

  if (weapon && !affected_by_spell(ch, SKILL_PARRY)) {
    which = 1;
    CREATE(af, struct affected_type, 1);
    af->type = SKILL_PARRY;
    af->duration = 1;
    affect_to_char(ch, af);
  } else if (weapon2 && !affected_by_spell(ch, SKILL_PARRY2)) {
    which = 2;
    CREATE(af, struct affected_type, 1);
    af->type = SKILL_PARRY2;
    af->duration = 1;
    affect_to_char(ch, af);
  } else if (IS_FENCING(weapon) && !affected_by_spell(ch, SKILL_PARRY3)) {
    which = 1;
    CREATE(af, struct affected_type, 1);
    af->type = SKILL_PARRY3;
    af->duration = 1;
    affect_to_char(ch, af);
  } else
    return 0;

  if (affected_by_spell(ch, SKILL_FEINT)) {
    for (hjp = ch->affected; hjp; hjp = next) {
      next = hjp->next;
      if (hjp->type == SKILL_FEINT) {
	feint_mod = hjp->modifier;
	break;
      }
    }
  }
  affect_from_char(ch, SKILL_FEINT);

  obj = GET_EQ(ch, (which == 1) ? WEAR_WIELD : WEAR_WIELD2);

  if (!obj)
    return 0;

  if (o && IS_FENCING(obj) &&
      ((GET_OBJ_VAL(o, 7) == WEAPON_FLAIL)
       || (GET_OBJ_VAL(o, 7) == WEAPON_MORNING_STAR)))
    modifier -= 20;		//impossible for fencing weapons to parry flails

  //try parry
  mod = weapon_app[GET_OBJ_VAL(obj, 7)].parry;
  if (mod <= 1)
    success = (GET_DEX(ch) + thaco_to_skill(ch) - feint_mod + mod + modifier) / 2;
  if (mod == 2)
    success = ((GET_DEX(ch) + thaco_to_skill(ch) - feint_mod - modifier) * 2) / 3;
  if (dice(3, 6) < success)
    success = 1;
  else
    success = 0;

  if (success) {
    if (!DamageItem(ch, obj, 5) && o) {
      mod = GET_OBJ_CONDITION(obj);
      mod = (mod * 100) / perc_dam[GET_OBJ_MATERIAL(obj)];
      mod -= pow(5, (float) GET_OBJ_WEIGHT(o) / GET_OBJ_WEIGHT(obj));
      mod = MAX(0, mod);
      if (!number(0, mod))
	MakeScrap(ch, obj, 5);
    }
  }
  return success;
}

char can_parry(struct char_data *ch, struct char_data *vict)
{
  struct obj_data *weapon = GET_EQ(vict, WEAR_WIELD), *weapon2 = GET_EQ(vict, WEAR_WIELD2);

  //check for weapon to parry with
  if (!weapon && !weapon2)
    return 0;

  //aggressive
  if (GET_STYLE(vict) == STYLE_AGGRESSIVE)
    return 0;

  if (weapon && !affected_by_spell(vict, SKILL_PARRY) &&
      !IS_CRIPPLED(vict, LOCATION_WEAPON_ARM)) {
    return 1;
  }
  if (weapon2 && !affected_by_spell(vict, SKILL_PARRY2) &&
      !IS_CRIPPLED(vict, LOCATION_SHIELD_ARM)) {
    return 1;
  }
  if (weapon && IS_FENCING(weapon) && !affected_by_spell(vict, SKILL_PARRY3) &&
      !IS_CRIPPLED(vict, LOCATION_WEAPON_ARM)) {
    return 1;
  }
  return 0;
}

char should_parry(struct char_data *ch, int location, int dam)
{
  if (dam > GET_DR(ch, location))
    return 1;
  return 0;
}

char penetration_number(struct obj_data *shield)
{
  //varries by shield
  return 1;
}

char get_shield_damage_reduction(struct obj_data *shield)
{
  //varies by shield
  return 1;
}

char stopped_by_passive_defense(struct char_data *ch, struct char_data *vict)
{
  if (dice(3, 6) < GET_PD(vict, parse_aiming(GET_AIMING(ch))))
    return 1;
  return 0;
}

char get_damage_reduction(struct char_data *ch, struct char_data *vict)
{
  return GET_DR(vict, parse_aiming(GET_AIMING(ch)));
}

char makes_dodge(struct char_data *ch)
{
  //roll against dodge skill
  if (GET_POS(ch) != STAT_NORMAL)
    return 0;

  if (dice(3, 6) < ((GET_CON(ch) + GET_DEX(ch)) / 4 - get_encumberance(ch)))
    return 1;
  return 0;
}

void damage_to_object(struct obj_data *obj, int dam)
{
  //subtract hp from object
}

int modify_by_attack(struct char_data *ch, struct obj_data *obj)
{
  int mod = -20;

  if (!ch || !obj)
    return 0;

  if (IS_SWINGING(ch))
    mod = weapon_app[GET_OBJ_VAL(obj, 7)].swing;
  if (mod == -20)
    mod = weapon_app[GET_OBJ_VAL(obj, 7)].thrust;
  return mod;
}
int thaco_to_skill(struct char_data *ch)
{
  int calc_thaco;
  extern int thaco[NUM_CLASSES][LVL_IMPL + 1];

  /* Calculate the THAC0 of the attacker */
  if (!IS_NPC(ch))
    calc_thaco = thaco[(int) GET_CLASS(ch)][(int) GET_LEVEL(ch)];
  else				/* THAC0 for monsters is set in the HitRoll */
    calc_thaco = thaco[(int) CLASS_FIGHTER][(int) GET_LEVEL(ch)];
  calc_thaco -= (GET_HITROLL(ch) + 1) / 2;
  calc_thaco -= (int) ((GET_INT(ch) - 13) / 3);		/* Intelligence helps! */
  calc_thaco -= (int) ((GET_WIS(ch) - 13) / 3);		/* So does wisdom */

  calc_thaco = (14 - calc_thaco) / 2;

  return calc_thaco;
}

char get_encumberance(struct char_data *ch)
{
  int weight = IS_CARRYING_W(ch);
  extern struct str_app_type str_app[];

  int max = str_app[STRENGTH_APPLY_INDEX(ch)].carry_w;
  int percent = 100 * weight / max;

  if (percent == 0)
    return 0;
  if (percent < 20)
    return 1;
  if (percent < 40)
    return 2;
  if (percent < 60)
    return 3;
  if (percent < 80)
    return 4;
  if (percent < 100)
    return 5;
  else
    return 6;
}

void critical_miss(struct char_data *ch, int w_type)
{
  int roll = dice(3, 6);
  int dam, location, orig_loc;
  int num_die, i;
  struct obj_data *obj;
  extern struct str_app_type str_app[];

  location = (GET_EQ(ch, WEAR_WIELD)) ? WEAR_WIELD : GET_EQ(ch, WEAR_WIELD2) ? WEAR_WIELD2 : -1;
  if (location < 0)
    return;
  obj = GET_EQ(ch, location);

  if (obj && obj_index[GET_OBJ_RNUM(obj)].func != NULL)
    (obj_index[GET_OBJ_RNUM(obj)].func) (ch, obj, 0, "", "", PROC_CRIT_MISS);

  /* Start with the damage bonuses: the damroll and strength apply */
  if (IS_SWINGING(ch))
    dam = str_app[STRENGTH_APPLY_INDEX(ch)].swing;
  else
    dam = str_app[STRENGTH_APPLY_INDEX(ch)].thrust;

  //Apply weapon mods
  if (obj)
    dam += modify_by_attack(ch, obj);

  num_die = dam / 4;
  dam %= 4;
  if (dam == 3) {
    num_die++;
    dam = -1;
  }
  dam += GET_DAMROLL(ch);
  for (i = 3; i <= dam; i++) {
    dam -= i;
    num_die++;
  }

  if (obj && (weapon_app[GET_OBJ_VAL(obj, 7)].max_die))
    num_die = MIN(num_die, weapon_app[GET_OBJ_VAL(obj, 7)].max_die);

  dam = dice(num_die, 6) + dam;
  if (obj) {
    /* Add weapon-based damage if a weapon is being wielded */
    dam += dice(GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 2));
    // modify by thrust/swing
  } else {
    /* If no weapon, add bare hand damage instead */
    if (IS_NPC(ch)) {
      dam += dice(ch->mob_specials.damnodice, ch->mob_specials.damsizedice);
    } else {
      dam += number(0, 2);	/* Max 2 bare hand damage for players */
    }
  }

  switch (roll) {
  case 3:
  case 4:
  case 17:
  case 18:
    if (GET_OBJ_CRAFTSMANSHIP(obj) < 11)
      // break weapon
      break;
  case 5:
    location = number(0, 2);
    switch (location) {
    case 0:
      location = LOCATION_SHIELD_ARM;
      break;
    case 1:
      location = LOCATION_RIGHT_LEG;
      break;
    case 2:
      location = LOCATION_LEFT_LEG;
      break;
    }
    orig_loc = GET_AIMING(ch);
    GET_AIMING(ch) = location;
    damage(ch, ch, dam / 2, w_type, -1, location);
    GET_AIMING(ch) = orig_loc;
    break;
  case 6:
    orig_loc = GET_AIMING(ch);
    GET_AIMING(ch) = LOCATION_TORSO;
    damage(ch, ch, dam / 2, w_type, -1, location);
    GET_AIMING(ch) = orig_loc;
    break;
  case 7:
  case 8:
  case 9:
  case 10:
  case 11:
  case 12:
  case 13:
    //drop weapon
    put_obj_to_room(unequip_char(ch, location),
		    ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch),
		    GET_Z_CORD(ch));
    act("$n stumbles and drops $p.", FALSE, ch, obj, 0, PROXIMITY);
    act("You stumble, dropping $p.",
	FALSE, ch, obj, 0, TO_CHAR);
    break;
  case 14:
    //throw weapon;
    location = (GET_EQ(ch, WEAR_WIELD)) ? WEAR_WIELD : WEAR_WIELD2;
    obj = GET_EQ(ch, location);
    {				/* gld fix */
      int x = BOUNDED(1, GET_X_CORD(ch) + number(-3, 3),
		      GET_X_SIZE(ch->in_room));
      int y = BOUNDED(1, GET_Y_CORD(ch) + number(-3, 3),
		      GET_Y_SIZE(ch->in_room));
      if ((IS_WALL(world[ch->in_room].map->map_grid[x][y]) ||
	   world[ch->in_room].map->map_grid[x][y] == MAP_TREE ||
	   world[ch->in_room].map->map_grid[x][y] == MAP_RUBBLE ||
	   world[ch->in_room].map->map_grid[x][y] == MAP_NOTBLANK ||
	   UNPASSABLE_FURNITURE(ch, x, y) ||
      /* yank map_blank, since moving through a door always finds one */
	   world[ch->in_room].map->map_grid[x][y] == MAP_ERROR))
	x = GET_X_CORD(ch);
      y = GET_Y_CORD(ch);
      put_obj_to_room(unequip_char(ch, location),
		      ch->in_room, x, y,
		      GET_Z_CORD(ch));

      act("$n stumbles, throwing $p!", FALSE, ch, obj, 0, PROXIMITY);
      act("You stumble, throwing $p!", FALSE, ch, obj, 0, TO_CHAR);
    }
    break;
  case 15:
    //cripple arm
    cripple(ch, LOCATION_WEAPON_ARM, 10);
    break;
  case 16:
    //fall down, wait for 1 round
    //Stun(ch, 2);
    act("$n falls to the ground, stunning $self!",
	FALSE, ch, obj, 0, PROXIMITY);
    act("You fall to the groun, stunning yourself!",
	FALSE, ch, obj, 0, TO_CHAR);
    break;
  }
  return;
}

int critical_hit(struct char_data *ch, struct char_data *vict,
		 int damage)
{
  int location = GET_AIMING(ch);
  int roll = dice(3, 6);
  struct obj_data *weapon = NULL, *obj = NULL;

  obj = GET_EQ(ch, WEAR_WIELD);
  if (!obj)
    obj = GET_EQ(ch, WEAR_WIELD2);

//  if (obj && obj_index[GET_OBJ_RNUM(obj)].func != NULL)
  //    (obj_index[GET_OBJ_RNUM(obj)].func) (ch, obj, 0, "", "", PROC_CRIT_HIT);

  if (!has_location(vict, location))
    location = LOCATION_TORSO;
  switch (roll) {
  case 3:
  case 18:
    if (location == LOCATION_TORSO) {
      //KnockOut(vict, 10);
      return damage;
    } else
      return 3 * damage;
  case 4:
  case 17:
    return 3 * damage;
  case 5:
  case 16:
    return 2 * damage;
  case 6:
    return damage + 10;
  case 7:
  case 13:
    //Stun(vict, 5);
    return damage;
  case 8:
  case 12:
    if ((location >= LOCATION_SHIELD_ARM) &&
	(location <= LOCATION_LEFT_FOOT) &&
	(location != LOCATION_TORSO))
      cripple(vict, location, 6);
    return damage;
  case 9:
  case 10:
  case 11:
    return damage;
  case 14:
    if ((location >= LOCATION_SHIELD_ARM) &&
	(location <= LOCATION_LEFT_FOOT) &&
	(location != LOCATION_TORSO)) {
      cripple(vict, location, 6);
      return damage;
    } else
      return 2 * damage;
  case 15:
    if (!weapon)
      return damage;
    put_obj_to_room(unequip_char(vict, location),
		    vict->in_room, GET_X_CORD(vict), GET_Y_CORD(vict),
		    GET_Z_CORD(vict));
    act("$n drops $p in pain.", FALSE, vict, weapon, 0, PROXIMITY);
    act("Your hand explodes in pain, causing you to drop $p.",
	FALSE, vict, weapon, 0, TO_CHAR);
    return damage;
  default:
    return damage;
  }
}

void cripple(struct char_data *ch, int location, int duration)
{
  if (special(ch, 0, "", "", PROC_CRIPPLE))
    return;

  //cripple body part
  switch (location) {
  case LOCATION_WEAPON_ARM:
    GET_HIT_WA(ch) = HIT_INCAP;
    break;
  case LOCATION_WEAPON_HAND:
    GET_HIT_WH(ch) = HIT_INCAP;
    break;
  case LOCATION_SHIELD_ARM:
    GET_HIT_SA(ch) = HIT_INCAP;
    break;
  case LOCATION_SHIELD_HAND:
    GET_HIT_SH(ch) = HIT_INCAP;
    break;
  case LOCATION_LEFT_LEG:
    GET_HIT_LL(ch) = HIT_INCAP;
    break;
  case LOCATION_LEFT_FOOT:
    GET_HIT_LF(ch) = HIT_INCAP;
    break;
  case LOCATION_RIGHT_LEG:
    GET_HIT_RL(ch) = HIT_INCAP;
    break;
  case LOCATION_RIGHT_FOOT:
    GET_HIT_RF(ch) = HIT_INCAP;
    break;
  default:
  }
}

int has_location(struct char_data *ch, int location)
{
  return 1;
}

void hit(struct char_data *ch, struct char_data *victim, int type)
{
  struct obj_data *wielded = NULL;
  int w_type = 0, dam, diceroll, apr;
  struct affected_type *af, *hjp;
  char crit_hit = 0, mod = 0;
  struct mob_attack_data *Attack_type, *Next_attack;
  int backstab_mult(int level), chase_vict = 0;
  int min_distance = 0, max_distance = 1, distance = 0;
  int num_die, i, skill = GET_DEX(ch);
  int which = -1;
  int skill_style = SKILL_STYLE_ONE_HANDED_WEAPON, skill_style2 = SKILL_STYLE_ONE_HANDED_WEAPON;

  if (!ch || !victim)
    return;

  if (GET_STAT(ch) <= STAT_STUNNED || GET_STAT(victim) <= STAT_DEAD) {
    stop_fighting(ch);
    return;
  }
  update_pos(victim);

  SET_FACING(ch, angleBetweenCharAndVict(ch, victim));

  // if should use second weapon, use that as wielded
  wielded = ((ch->char_specials.attacks_left == 1) && GET_EQ(ch, WEAR_WIELD2)) ?
      GET_EQ(ch, WEAR_WIELD2) : GET_EQ(ch, WEAR_WIELD);
  if (wielded == GET_EQ(ch, WEAR_WIELD2))
    which = WEAR_WIELD2;
  if (wielded == GET_EQ(ch, WEAR_WIELD))
    which = WEAR_WIELD;

  //check for crippled arms.
  if ((!wielded || (which == WEAR_WIELD))
      && IS_CRIPPLED(ch, LOCATION_WEAPON_ARM)) {
    send_to_char("Your arm is too damaged to fight.\r\n", ch);
    return;
  }
  if ((which == WEAR_WIELD2) && IS_CRIPPLED(ch, LOCATION_SHIELD_ARM)) {
    send_to_char("Your off arm is too damaged to fight.\r\n", ch);
    return;
  }
  // get style, easier to put here than lower
  skill_style = GET_WIELDED_STYLE(ch);
  skill_style2 = GET_WIELDED_STYLE(victim);

  max_distance = BOUNDED(1, GET_SIZE(ch) - SIZE_MEDIUM + 1, 5);		/* might work */
  if (wielded)
    max_distance += weapon_app[GET_OBJ_VAL(wielded, 7)].range - 1;

  min_distance = (int) ((max_distance / 2));	/* round down */

  if (type == SKILL_BACKSTAB) {
    max_distance = 1;
    min_distance = 0;
  }
  if (ch->char_specials.attacks_left)
    ch->char_specials.attacks_left -= 1;
  else if (GET_STYLE(ch) == STYLE_AGGRESSIVE && IS_PC(ch))
    ch->char_specials.attacks_left = 1;

  distance = max_distance;
  /* Do some sanity checking, in case someone flees, etc. */
  if (!victWithinRangeofChar(ch, victim, distance)) {
    if (FIGHTING(ch) && (FIGHTING(ch) == victim))
      stop_fighting(ch);
    if (IS_NPC(ch) || (GET_STYLE(ch) == STYLE_AGGRESSIVE && IS_PC(ch))) {
      send_to_char("You advance upon your opponent.\r\n", ch);
      perform_move(ch, GET_FACING(ch), 0);
    }
  }
  /*In close combat with a weapon that you need space for */
  distance = min_distance;
  if ((distance > 0) && victWithinRangeofChar(ch, victim, distance - 1)) {
    if (FIGHTING(ch) && (FIGHTING(ch) == victim))
      stop_fighting(ch);
    if (FIGHTING(victim) && (FIGHTING(victim) == ch))
      stop_fighting(victim);
    /* hunting would be bad if we're too close */
    act("$N is much too close for you to hit.",
	TRUE, ch, NULL, victim, TO_CHAR);
    if (IS_NPC(ch) || (GET_STYLE(ch) == STYLE_AGGRESSIVE && IS_PC(ch)))
      perform_move(ch, angleBetweenCharAndVict(ch, victim), 1);
    return;
  }
  if (IS_AFFECTED4(ch, AFF4_FEINT)) {
    REMOVE_BIT(AFF4_FLAGS(ch), AFF4_FEINT);
    diceroll = GET_DEX(ch) - dice(3, 6) + GET_SKILL(ch, skill_style);
    dam = GET_DEX(victim) - dice(3, 6) + GET_SKILL(victim, skill_style2);
    if (diceroll > dam) {
      CREATE(af, struct affected_type, 1);
      af->type = SKILL_FEINT;
      af->modifier = diceroll;
      af->duration = -1;
      affect_to_char(victim, af);
    }
    return;
  }
  /* Find the weapon type (for display purposes only) */
  if (wielded && GET_OBJ_TYPE(wielded) == ITEM_WEAPON) {
    w_type = GET_OBJ_VAL(wielded, 3) + TYPE_HIT;
  } else if (IS_NPC(ch) && ch->mob_specials.mob_attack != NULL) {
    /* loop through mob attacks until we find the proper one */
    for (Attack_type = ch->mob_specials.mob_attack, apr = 0; Attack_type; Attack_type = Next_attack) {
      Next_attack = Attack_type->next;
      if (ch->char_specials.attacks_left == apr) {
	w_type = Attack_type->attack_type + TYPE_HIT;
	break;
      }
      apr++;
    }
  } else if (IS_HUMANOID(ch)) {
    w_type = TYPE_PUNCH;
  }
  if (!w_type)
    w_type = TYPE_HIT;

  /* set them now, as this will affect lots of things for mobs.
     it can/will always be cleared later if need be */
  if (IS_NPC(victim) && !FIGHTING(victim))
    set_fighting(victim, ch);

  /* roll the die and take your chances... */
  diceroll = dice(3, 6);

  if (GET_STAT(victim) <= STAT_STUNNED)
    diceroll += 5;
  diceroll = (BOUNDED(3, diceroll, 18));

  if ((diceroll != 3) || (diceroll != 18)) {

    //modify by prot good and prot evil
    if (IS_EVIL(ch) && IS_PROTECTED(victim, PROT_FROM_EVIL)) {
      skill -= 1;
    }
    if (IS_GOOD(ch) && IS_PROTECTED(victim, PROT_FROM_GOOD)) {
      skill -= 1;
    }
    //modify byvictim being blurred
    if (IS_AFFECTED(victim, AFF3_BLUR) && !IS_AFFECTED3(ch, AFF3_TRUE_SIGHT)) {
      skill -= 1;
    }
    //modify by called shot
    skill += aim_mod[GET_AIMING(ch)];

    //modify by weapon
    if (wielded && (IS_PC(ch) &&
		    !GET_SKILL(ch, GET_OBJ_VAL(wielded, 7) + 24)))
      skill += weapon_app[GET_OBJ_VAL(wielded, 7)].mod;

    // modify by levels of mastery
    if (wielded)
      skill += GET_SKILL(ch, GET_OBJ_VAL(wielded, 7)) - 1;

    // modify by weapon skill
    skill += thaco_to_skill(ch);

    // modify by style
    if (wielded && (IS_PC(ch) &&
		    !GET_SKILL(ch, skill_style)))
      skill -= 4;

    // modify by style proficiency
    if (wielded) {
      //fudging for warrior mobs
      if (IS_NPC(ch) && IS_WARRIOR(ch))
	skill += GET_LEVEL(ch) / 4;
      else if (IS_PC(ch))
	skill += GET_SKILL(ch, skill_style) - 1;
    }
    // off hand
    if (which == WEAR_WIELD2)
      skill -= 2;

    // sleeping or unconcious target
    if (IS_AFFECTED4(victim, AFF4_KNOCKED_OUT) || (GET_STAT(victim) <= STAT_SLEEPING))
      skill += 10;
  }
  /* decide whether this is a hit or a miss */
  if (((diceroll > skill) || (diceroll == 18)) &&
      (type != SKILL_BACKSTAB)) {
    /* the attacker missed the victim */
    damage(ch, victim, 0, w_type, which, -1);
    hitprcnt_mtrigger(victim);
    // check for critical miss
    if ((diceroll == 18) || ((diceroll > 16) && (skill != 17)))
      critical_miss(ch, w_type);
  } else if ((type == SKILL_BACKSTAB) &&
	     !number(0, 14 + (GET_LEVEL(ch) * 6))) {
    critical_miss(ch, w_type);
  } else {
    /* okay, we know the guy has been hit.  now calculate damage. */

    /* Start with the damage bonuses: the damroll and strength apply */
    if ((IS_SWINGING(ch) || IS_NPC(ch)) && (type != SKILL_BACKSTAB))
      dam = str_app[STRENGTH_APPLY_INDEX(ch)].swing;
    else
      dam = str_app[STRENGTH_APPLY_INDEX(ch)].thrust;
    if (wielded)
      dam += modify_by_attack(ch, wielded);

    num_die = dam / 4;
    dam %= 4;
    if (dam == 3) {
      num_die++;
      dam = -1;
    }
    dam += GET_DAMROLL(ch);
    for (i = 3; i <= dam; i++) {
      dam -= i;
      num_die++;
    }

    if (wielded && (weapon_app[GET_OBJ_VAL(wielded, 7)].max_die))
      num_die = MIN(num_die, weapon_app[GET_OBJ_VAL(wielded, 7)].max_die);

    dam = dice(num_die, 6) + dam;
    if (wielded) {
      /* Add weapon-based damage if a weapon is being wielded */
      dam += dice(GET_OBJ_VAL(wielded, 1), GET_OBJ_VAL(wielded, 2));
    } else {
      /* If no weapon, add bare hand damage instead */
      if (IS_NPC(ch) && ch->mob_specials.mob_attack != NULL) {
	/* loop through mob attacks until we find the proper one */
	for (Attack_type = ch->mob_specials.mob_attack, apr = 0; Attack_type; Attack_type = Next_attack) {
	  Next_attack = Attack_type->next;
	  if (ch->char_specials.attacks_left == apr) {
	    dam += number(Attack_type->min_damage, Attack_type->max_damage);
	    break;
	  }
	  apr++;
	}
      } else {
	dam += number(0, 2);	/* Max 2 bare hand damage for players */
	dam = MIN(dam, 6);
      }
      if (affected_by_spell(ch, SPELL_SHOCKING_GRASP)) {
	for (hjp = ch->affected; hjp; hjp = hjp->next)
	  if (hjp->type == SPELL_SHOCKING_GRASP) {
	    act("A surge of electricity shoots through $N when $n touches \
$M.", TRUE, ch, NULL, victim, TO_PROXIMITY);
	    act("A surge of electricity shoots through you when $n touches you.",
		TRUE, ch, NULL, victim, TO_VICT);
	    act("A surge of electricity passes from your hands when you \
touch $N.", TRUE, ch, NULL, victim, TO_CHAR);
	    damage(ch, victim, dice(1, 8) + hjp->level, SPELL_SHOCKING_GRASP,
		   -1, -1);
	  }
	affect_from_char(ch, SPELL_SHOCKING_GRASP);
      }
    }

    /*
     * Include a damage multiplier if victim isn't ready to fight:
     *
     * Position sitting  1.33 x normal
     * Position resting  1.66 x normal
     * Position sleeping 2.00 x normal
     * Position stunned  2.33 x normal
     * Position incap    2.66 x normal
     * Position mortally 3.00 x normal
     *
     * Note, this is a hack because it depends on the particular
     * values of the POSITION_XXX constants.
     */
    if (GET_POS(victim) <= POS_SITTING)
      dam *= 1 + (POS_STANDING - GET_POS(victim)) / 3;

    /* at least 1 hp damage min per hit */
    dam = MAX(1, dam);

    //critical hit?
    if ((diceroll < 5) ||
	(diceroll < MIN(7, (GET_DEX(ch) + thaco_to_skill(ch)
			    + aim_mod[GET_AIMING(ch)] - 10)))) {
      dam = critical_hit(ch, victim, dam);
      if (wielded && obj_index[GET_OBJ_RNUM(wielded)].func != NULL)
	(obj_index[GET_OBJ_RNUM(wielded)].func) (ch, wielded, dam, "", "", PROC_CRIT_HIT);
      crit_hit = 1;
    }
    // if backstab, cant block,parry or dodge
    if (type == SKILL_BACKSTAB) {
      dam *= backstab_mult(GET_LEVEL(ch));
      damage(ch, victim, dam, SKILL_BACKSTAB, which, LOCATION_VITALS);
      hitprcnt_mtrigger(victim);
      return;
    }
    //see if we should bock or parry
    if (!crit_hit && should_parry(victim, GET_AIMING(ch), dam)
	&& can_parry(ch, victim)) {
      mod = (CAN_SEE(victim, ch)) ? 0 : -4;
      if (successful_parry(victim, mod, wielded)) {
	dam = -2;
      }
    } else if (!crit_hit && should_block(victim, GET_AIMING(ch), dam) &&
	       can_block(ch, victim)) {
      if (successful_block(victim, GET_AIMING(ch), 0)) {
	dam = -1;
      }
    } else if (!crit_hit) {
      //try dodge
      if ((dam > 0) && makes_dodge(victim))
	dam = -3;
    }
    if ((GET_STYLE(victim) == STYLE_DEFENSIVE) && !crit_hit) {
      if ((dam > 0) && makes_dodge(victim))
	dam = -3;
    }
    damage(ch, victim, dam, w_type, which, GET_AIMING(ch));
    if (dam > 0)
      DamageOneItem(ch, w_type, wielded, dam >> 2);
    hitprcnt_mtrigger(victim);
  }
}


 /* Calculate delay before next hit */
int fight_delay(struct char_data *ch)
{
  int next_hit;
  int apr = 1;
  struct mob_attack_data *Attack_type, *Next_attack;

  next_hit = PASSES_PER_SEC * SECS_PER_ROUND;

  /* Weapon speed */

  /* Dexterity (Speed) */

  next_hit -= PASSES_PER_SEC * (GET_CON(ch) + GET_DEX(ch)) / 8;

  //spells
  if (AFF_FLAGGED(ch, AFF_HASTE))
    next_hit -= PASSES_PER_SEC * 5;

  if (AFF_FLAGGED(ch, AFF2_SLOW))
    next_hit += PASSES_PER_SEC * 5;

  //initiative
  next_hit += number(-2, 2) * PASSES_PER_SEC;

  /* Terrain */

  /* Class */
  if (IS_WARRIOR(ch) && (GET_LEVEL(ch) > 10))
    apr++;

  if (IS_NPC(ch)) {
    /* mobs have preset number of attacks */
    if (ch->mob_specials.mob_attack != NULL) {
      for (Attack_type = ch->mob_specials.mob_attack; Attack_type; Attack_type = Next_attack) {
	Next_attack = Attack_type->next;
	apr++;
      }
    }
    /* set attacks */
    if (ch->char_specials.attacks_left < 1) {
      ch->char_specials.attacks_left = apr - 1;		/* we count from 0 */
    }
  } else {
    /* pcs check fighting style and dual wield */
    if (GET_STYLE(ch) == STYLE_AGGRESSIVE)
      apr++;
    if (GET_EQ(ch, WEAR_WIELD2) && (GET_STYLE(ch) == STYLE_AGGRESSIVE))
      apr++;
    if (GET_STYLE(ch) == STYLE_DEFENSIVE)
      apr--;
  }
  apr = BOUNDED(0, apr, 10);

  if (!apr)
    return 3 * next_hit;

  return next_hit / apr;
}


EVENTFUNC(blink_event)
{
  struct char_data *ch = ((struct blink_event_data *) event_obj)->ch;
  int to_room = ch->in_room;

  if (!IS_AFFECTED3(ch, AFF3_BLINK))
    return 0;

  act("$n blinks out of existance!", FALSE, ch, 0, 0, PROXIMITY);
  act("You blink out of existance.", TRUE, ch, 0, 0, TO_CHAR);
  play_sound("!!SOUND(blink* Q)", ch, ch->in_room, 0, 0, GET_Z_CORD(ch), TO_ROOM);
  char_from_room(ch);
  char_to_room(ch, to_room, -1);
  act("$n blinks back into existance!", FALSE, ch, 0, 0, PROXIMITY);
  act("You blink back into existance.", TRUE, ch, 0, 0, TO_CHAR);
  smallmap_look(ch, view_distance(ch), -1);


  return PASSES_PER_SEC * SECS_PER_ROUND / 2;
}

 /*
    ** This is the actual event function called for each fighting event.
    ** The code is mostly ripped from perform_violence, except it's not
    ** looping, as it's called for each player's event as it happens (duh :)
  */
EVENTFUNC(fight_event)
{
  struct char_data *ch = ((struct fight_event_data *) event_obj)->ch;
  struct char_data *vict = ((struct fight_event_data *) event_obj)->vict;
  int next_hit;

  if (!vict || !ch || ((GET_STAT(vict) <= STAT_INCAP) &&
		   GET_STYLE(ch) != STYLE_AGGRESSIVE) || !FIGHTING(ch) ||
      (GET_STAT(ch) <= STAT_INCAP) || ch->in_room != vict->in_room) {
    if (ch && FIGHTING(ch))
      stop_fighting(ch);
    return 0;
  }
  if (GET_STAT(vict) == STAT_DEAD) {
    if (ch && FIGHTING(ch))
      stop_fighting(ch);
    return 0;
  }
  if (IS_NPC(ch)) {
    if (GET_MOB_WAIT(ch) > 0) {
      GET_MOB_WAIT(ch) -= PULSE_VIOLENCE;
      return PULSE_VIOLENCE;
    }
    GET_MOB_WAIT(ch) = 0;
    if (GET_POS(ch) < POS_STANDING) {
      SET_POS(ch, POS_STANDING + STAT_NORMAL);
      act("$n scrambles to $s feet!", TRUE, ch, 0, 0, TO_ROOM);
    }
  }
  /* spec_proc fighting actions */
  if (mob_index[GET_MOB_RNUM(ch)].func != NULL)
    (mob_index[GET_MOB_RNUM(ch)].func) (ch, ch, 0, "", "", PROC_NORMAL);

  if (IS_NPC(ch)) {
    /* trigger/script fighting actions */
    fight_mtrigger(ch);
    /* racial/class fighting actions */
    MobCombat(ch, NULL);	/* already fighting, so we dont set new_vict */
  } else if (FIGHTING(ch) && (GET_STAT(FIGHTING(ch)) > STAT_DEAD)) {
    hit(ch, FIGHTING(ch), TYPE_UNDEFINED);
  }
  if (FIGHTING(ch))
    next_hit = fight_delay(ch);
  else
    next_hit = 0;

//  play_sound("!!SOUND(battle* Q)", ch, ch->in_room, 0, 0, GET_Z_CORD(ch), TO_ROOM);

  return next_hit;
}


/*
 * this routine was repeated all over, made it a function, basically
 * handles switched gods and shapechangers, so that the right body dies.
 * Returns the right char, or NULL, if there is a problem.  JAB
 */
struct char_data *ForceReturn(struct char_data *ch)
{
  struct char_data *true_id, *t_ch = ch;

  if (!t_ch)
    return NULL;
/*
   if (IS_AFFECTED(ch, AFF_WRAITHFORM)) {
   BackToUsualForm(ch);
   return ch;
   }
 */
  /* morph'ed players go back...  */
  if (MORPH_MOB(t_ch)) {
    true_id = MORPH_MOB(t_ch);
    act("$n's dying body slowly changes back into $N.",
	FALSE, t_ch, 0, true_id, TO_ROOM);
    send_to_char("As the last of life leaves you, your body resumes its natural form.\r\n",
		 t_ch);
    if (FIGHTING(t_ch))
      stop_fighting(t_ch);
    un_morph(t_ch);
    return true_id;
  }
  /* switched god */
  if (t_ch->desc && t_ch->desc->original && IS_PC(t_ch->desc->original) &&
      t_ch->desc->original->switched) {
    do_return(t_ch, "", 0, 0);
  }
  return (t_ch);
}

char fends_off(struct char_data *defender, struct char_data *advancer)
{
  char def_roll, adv_roll;
  int style = GET_WIELDED_STYLE(defender);

  def_roll = dice(3, 6);
  adv_roll = dice(3, 6);

  adv_roll = GET_DEX(advancer) - adv_roll;
  def_roll = GET_DEX(defender) - def_roll + GET_SKILL(defender, style);

  if (adv_roll <= 0) {
    //  hit(defender, advancer, TYPE_UNDEFINED);
    return 1;
  } else if (adv_roll > def_roll) {
    return 0;
  }
  //hit(defender, advancer, TYPE_UNDEFINED);
  return 1;
}
