/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "dg_scripts.h"
#include "utils.h"
#include "comm.h"
#include "spells.h"
#include "handler.h"
#include "db.h"

extern struct room_data *world;
extern struct obj_data *object_list;
extern struct char_data *character_list;
extern struct index_data *obj_index;

extern struct descriptor_data *descriptor_list;
extern struct zone_data *zone_table;

extern int mini_mud;

extern struct default_mobile_stats *mob_defaults;
extern char weapon_verbs[];
extern int *max_ac_applys;
extern struct apply_mod_defaults *apmd;

void clearMemory(struct char_data *ch);
void act(char *str, int i, struct char_data *c, struct obj_data *o,
	 void *vict_obj, int j);

void damage(struct char_data *ch, struct char_data *victim,
	    int damage, int weapontype, int which, int location);

void weight_change_object(struct obj_data *obj, int weight);
void add_follower(struct char_data *ch, struct char_data *leader);
int dice(int number, int size);
extern struct spell_info_type spell_info[];
void raw_kill(struct char_data *ch, struct char_data *killer);

struct char_data *read_mobile(int, int);





// First Level Cleric Spells

ASPELL(spell_bless)
{
  struct char_data *target, *tar_next;

  act("You feel the precense of your deity infuse the area.", TRUE,
      ch, NULL, NULL, TO_CHAR);
  act("A powerful precense fills the area.",
      TRUE, ch, NULL, NULL, TO_PROXIMITY2);
  for (target = world[ch->in_room].people; target; target = tar_next) {
    tar_next = target->next_in_room;
    if (affectarea(ch, victim, loc, target, SPELL_BLESS)) {
      act("$N fills with a holy furvor, as your spell enters $S soul.",
	  TRUE, ch, NULL, target, TO_CHAR);
      act("$N fills with a holy furvor as $n's deity touches $S soul.",
	  TRUE, ch, NULL, target, TO_OTHERS);
      act("You feel the touch of $n's deity enter your soul.",
	  TRUE, ch, NULL, target, TO_VICT);
      ((*spell_info[SPELL_BLESS].routines) (level, ch, NULL, target, obj, loc, 0));
    }
  }
}

ASPELL(spell_cause_light)
{
  int dam;
  char mybuf1[MAX_STRING_LENGTH], mybuf2[100];

  dam = dice(1, 8);
  sprintf(mybuf2, "%s", (casttype == CAST_SPELL) ? "'s hands" :
	  (casttype == CAST_STAFF) ? "'s staff" :
	  (casttype == CAST_WAND) ? "'s wand" : "");

  sprintf(mybuf1, "A dark &+Llight&N surrounds $n%s as $e touches $N.", mybuf2);
  act(mybuf1, TRUE, ch, NULL, victim, TO_OTHERS);
  sprintf(mybuf1, "A dark &+Llight&N surrounds $n%s as $e touches you.", mybuf2);
  act(mybuf1, TRUE, ch, NULL, victim, TO_VICT);

  sprintf(mybuf2, "%s", (casttype == CAST_SPELL) ? "your hands" :
	  (casttype == CAST_STAFF) ? "your staff" :
	  (casttype == CAST_WAND) ? "your wand" : "you");
  sprintf(mybuf1, "A dark &+Llight&N surrounds %s as you touch $N.", mybuf2);
  act(mybuf1, TRUE, ch, NULL, victim, TO_CHAR);

  damage(ch, victim, dam, SPELL_CAUSE_LIGHT, -1, -1);
}

ASPELL(spell_create_water)
{
  int water;

  void name_to_drinkcon(struct obj_data *obj, int type);
  void name_from_drinkcon(struct obj_data *obj);

  if (ch == NULL || obj == NULL)
    return;
  level = MAX(MIN(level, LVL_IMPL), 1);

  if (GET_OBJ_TYPE(obj) == ITEM_DRINKCON) {
    if ((GET_OBJ_VAL(obj, 2) != LIQ_WATER) && (GET_OBJ_VAL(obj, 1) != 0)) {
      name_from_drinkcon(obj);
      GET_OBJ_VAL(obj, 2) = LIQ_SLIME;
      name_to_drinkcon(obj, LIQ_SLIME);
    } else {
      water = MAX(GET_OBJ_VAL(obj, 0) - GET_OBJ_VAL(obj, 1), 0);
      if (water > 0) {
	if (GET_OBJ_VAL(obj, 1) >= 0)
	  name_from_drinkcon(obj);
	GET_OBJ_VAL(obj, 2) = LIQ_WATER;
	GET_OBJ_VAL(obj, 1) += water;
	name_to_drinkcon(obj, LIQ_WATER);
	weight_change_object(obj, water);
	act("$p is filled.", FALSE, ch, obj, 0, TO_CHAR);
      }
    }
  }
}

ASPELL(spell_cure_light)
{
  int healpoints;

  healpoints = dice(1, 8);

  ((*spell_info[SPELL_CURE_LIGHT].routines) (level, ch, NULL, victim, obj, loc, healpoints));

  if (casttype == CAST_SPELL) {
    act("A soft &+Yglowing light&N surrounds $n's hands as $e touches $N.",
	TRUE, ch, NULL, victim, TO_OTHERS);
    act("A soft &+Yglowing light&N surrounds your hands as you touch $N.",
	TRUE, ch, NULL, victim, TO_CHAR);
    if (ch != victim)
      act("A soft &+Yglowing light&N surrounds $n's hands as $e touches you.",
	  TRUE, ch, NULL, victim, TO_VICT);
  } else {
    act("A soft &+Yglowing light&N surrounds $N.", TRUE, ch, NULL, victim,
	TO_OTHERS);
    act("A soft &+Yglowing light&N surrounds you.", TRUE, ch, NULL, victim,
	TO_VICT);
    if (ch != victim)
      act("A soft &+Yglowing light&N surrounds $N.", TRUE, ch, NULL,
	  victim, TO_CHAR);
  }
}

ASPELL(spell_curse)
{
  byte makes = 0;
  struct char_data *target, *tar_next;

  act("You feel the precense of your deity infuse the area.", TRUE,
      ch, NULL, NULL, TO_CHAR);
  act("A powerful precense fills the area.",
      TRUE, ch, NULL, NULL, TO_PROXIMITY2);

  for (target = world[ch->in_room].people; target; target = tar_next) {
    tar_next = target->next_in_room;
    makes = makes_save(target, savetype, 0);
    if (!makes) {
      if (affectarea(ch, victim, loc, target, SPELL_CURSE)) {
	act("$N is filled with doubt as your spell enters $S soul.",
	    TRUE, ch, NULL, target, TO_CHAR);
	act("$N shivers as $n's deity touches $S soul.",
	    TRUE, ch, NULL, target, TO_OTHERS);
	act("You feel a shiver of doubt as the touch of $n's \
 deity enters your soul.", TRUE, ch, NULL, target, TO_VICT);
	((*spell_info[SPELL_CURSE].routines) (level, ch, NULL, victim, obj, loc, 0));
      }
    }
  }
}

ASPELL(spell_detect_poison)
{
  if (victim) {
    if (victim == ch) {
      if (IS_AFFECTED(victim, AFF_POISON))
	send_to_char("You can sense poison in your blood.\r\n", ch);
      else
	send_to_char("You feel healthy.\r\n", ch);
    } else {
      if (IS_AFFECTED(victim, AFF_POISON))
	act("You sense that $E is poisoned.", FALSE, ch, 0, victim, TO_CHAR);
      else
	act("You sense that $E is healthy.", FALSE, ch, 0, victim, TO_CHAR);
    }
  }
  if (obj) {
    switch (GET_OBJ_TYPE(obj)) {
    case ITEM_DRINKCON:
    case ITEM_FOUNTAIN:
    case ITEM_FOOD:
      if (GET_OBJ_VAL(obj, 3))
	act("You sense that $p has been contaminated.", FALSE, ch, obj, 0, TO_CHAR);
      else
	act("You sense that $p is safe for consumption.", FALSE, ch, obj, 0,
	    TO_CHAR);
      break;
    default:
      send_to_char("You sense that it should not be consumed.\r\n", ch);
    }
  }
}

ASPELL(spell_faerie_fire)
{
  int a = level * 12;
  struct char_data *target, *tar_next;

  for (target = world[ch->in_room].people; target; target = tar_next) {
    tar_next = target->next_in_room;
    if (affectarea(ch, victim, loc, target, SPELL_FAERIE_FIRE) &&
	(a -= (GET_SIZE(target) + 1) * 6) && (a > 0)) {
      act("You are surrounded by a pale &+Wglowing&n light.", TRUE, ch, NULL,
	  victim, TO_VICT);
      act("$n is surrounded by a pale &+Wglowing&n light.", TRUE, ch, NULL,
	  victim, TO_OTHERS);
      act("$n is surrounded by a pale &+Wglowing&n light.", TRUE, ch, NULL,
	  victim, TO_CHAR);
      ((*spell_info[SPELL_FAERIE_FIRE].routines) (level, ch, NULL, victim, obj, loc, 0));
    }
  }
}

ASPELL(spell_sanctuary)
{
  act("$n's is surrounded by a soft pink glow.", TRUE, ch, NULL,
      victim, TO_OTHERS);
  act("You are surrounded by a pink glow.", TRUE, ch, NULL,
      victim, TO_VICT);
  act("Your touch causes $N to be surrounded by a soft pink glow.", TRUE, ch, NULL,
      victim, TO_CHAR);
  ((*spell_info[SPELL_SANCTUARY].routines) (level, ch, NULL, victim, obj, loc, 0));
}
// Second Level Cleric Spells

ASPELL(spell_aid)
{
  act("$N's is surrounded by a gentle glow as $n touches $m..", TRUE, ch, NULL,
      victim, TO_OTHERS);
  act("You are surrounded by a gentle glow as $n touches you.", TRUE, ch, NULL,
      victim, TO_VICT);
  act("$N is surrounded by a gentle glow as you touch $m.", TRUE, ch, NULL,
      victim, TO_CHAR);
  ((*spell_info[SPELL_AID].routines) (level, ch, NULL, victim, obj, loc, 0));
}

ASPELL(spell_barkskin)
{
  act("$n's skin takes on the consistency of bark.", TRUE, ch, NULL,
      victim, TO_OTHERS);
  act("Your skin feels like the bark of a tree.", TRUE, ch, NULL,
      victim, TO_VICT);
  act("Your touch turns $N's skin to bark.", TRUE, ch, NULL,
      victim, TO_CHAR);
  ((*spell_info[SPELL_BARKSKIN].routines) (level, ch, NULL, victim, obj, loc, 0));
}

ASPELL(spell_charm_mammal)
{
  if (victim == NULL || ch == NULL)
    return;

  if (victim == ch)
    send_to_char("You like yourself even better!\r\n", ch);
  else if (MOB_FLAGGED(victim, MOB_NOCHARM))
    send_to_char("Your victim resists!\r\n", ch);
  else if (IS_AFFECTED(ch, AFF_CHARM))
    send_to_char("You can't have any followers of your own!\r\n", ch);
  else if (IS_AFFECTED(victim, AFF_CHARM))
    send_to_char("You fail.\r\n", ch);
  else if (circle_follow(victim, ch))
    send_to_char("Sorry, following in circles can not be allowed.\r\n", ch);
  else if (!IS_HUMANOID(victim) && !IS_MAMMAL(victim))
    send_to_char("This will only work on mammals.\r\n", ch);
  else if (!makes_save(victim, savetype, 0)) {
    act("Isn't $n just such a nice fellow?", FALSE, ch, 0, victim, TO_VICT);
    ((*spell_info[SPELL_CHARM_MAMMAL].routines) (level, ch, NULL, victim, obj, loc, SPELL_CHARM_MAMMAL));
    if (IS_NPC(victim)) {
      REMOVE_BIT(MOB_FLAGS(victim), MOB_AGGRESSIVE);
    }
  }
}

// Third Level Cleric Spells

ASPELL(spell_cause_blindness)
{
  byte makes = makes_save(victim, savetype, 0);

  act("There is a &+Wbright flash of light&N as $n touches $N.", TRUE,
      ch, NULL, victim, TO_OTHERS);
  act("There is a &+Wflash of light&N as you grab $N.", TRUE,
      ch, NULL, victim, TO_CHAR);
  act("A &+Wbright flash of light&N fills your vision when &n touches you.",
      TRUE, ch, NULL, victim, TO_VICT);
  if (!makes) {
    act("All you can see is darkness.", TRUE, ch, NULL, victim, TO_VICT);
    ((*spell_info[SPELL_CAUSE_BLINDNESS].routines) (level, ch, NULL, victim, obj, loc, 0));
  }
}

ASPELL(spell_cause_deafness)
{
  byte makes = makes_save(victim, savetype, 0);

  act("There is a &+Wloud clap of thunder&N as $n touches $N.", TRUE,
      ch, NULL, victim, TO_OTHERS);
  act("There is a &+Wclap&N as you grab $N.", TRUE,
      ch, NULL, victim, TO_CHAR);
  act("A &+Wclap of thunder&N causes your ears to ring when &n touches you.",
      TRUE, ch, NULL, victim, TO_VICT);
  if (!makes) {
    act("You can't hear a thing", TRUE, ch, NULL, victim, TO_VICT);
    ((*spell_info[SPELL_CAUSE_DEAFNESS].routines) (level, ch, NULL, victim, obj, loc, 0));
  }
}

ASPELL(spell_cure_blindness)
{
  act("A &+Yglowing light&N surrounds $n's hand as $e touches $N.",
      TRUE, ch, NULL, victim, TO_OTHERS);
  act("A feeling of warmth runs down your arm, and enters $N's body.",
      TRUE, ch, NULL, victim, TO_CHAR);
  act("A feeling of warmth fills your body, and the darkness dissipates.",
      TRUE, ch, NULL, victim, TO_VICT);
  ((*spell_info[SPELL_CURE_BLINDNESS].routines) (level, ch, NULL, victim, obj, loc, 0));
}

ASPELL(spell_cure_deafness)
{
  act("A &+Yglowing light&N surrounds $n's hand as $e touches $N.",
      TRUE, ch, NULL, victim, TO_OTHERS);
  act("A feeling of warmth runs down your arm, and enters $N's body.",
      TRUE, ch, NULL, victim, TO_CHAR);
  act("A feeling of warmth fills your body, and the silence dissipates.",
      TRUE, ch, NULL, victim, TO_VICT);
  ((*spell_info[SPELL_CURE_DEAFNESS].routines) (level, ch, NULL, victim, obj, loc, 0));
}

// Fourth Level Cleric Spells

ASPELL(spell_cause_serious)
{
  int dam;

  dam = dice(2, 8) + 1;
  act("A dark &+Llight&N surrounds $n's hands as $e touches $N.",
      TRUE, ch, NULL, victim, TO_OTHERS);
  act("A dark &+Llight&N surrounds your hands as you touch $N.",
      TRUE, ch, NULL, victim, TO_CHAR);
  act("A dark &+Llight&N surrounds &n's hands as $e touches you.",
      TRUE, ch, NULL, victim, TO_VICT);
  damage(ch, victim, dam, SPELL_CAUSE_SERIOUS, -1, -1);
}

ASPELL(spell_cure_serious)
{
  int healpoints;


  healpoints = dice(2, 8) + 1;

  ((*spell_info[SPELL_CURE_SERIOUS].routines) (level, ch, NULL, victim, obj, loc, healpoints));

  if (casttype == CAST_SPELL) {
    act("A soft &+Yglowing light&N surrounds $n's hands as $e touches $N.",
	TRUE, ch, NULL, victim, TO_OTHERS);
    act("A soft &+Yglowing light&N surrounds your hands as you touch $N.",
	TRUE, ch, NULL, victim, TO_CHAR);
    if (ch != victim)
      act("A soft &+Yglowing light&N surrounds $n's hands as $e touches you.",
	  TRUE, ch, NULL, victim, TO_VICT);
  } else {
    act("A soft &+Yglowing light&N surrounds $N.", TRUE, ch, NULL, victim,
	TO_OTHERS);
    act("A soft &+Yglowing light&N surrounds you.", TRUE, ch, NULL, victim,
	TO_VICT);
    if (ch != victim)
      act("A soft &+Yglowing light&N surrounds $N.", TRUE, ch, NULL,
	  victim, TO_CHAR);
  }
}

// Fifth Level Cleric Spells

ASPELL(spell_cause_critic)
{
  int dam;

  dam = dice(3, 8) + 3;

  act("A dark &+Llight&N surrounds $n's hands as $e touches $N.",
      TRUE, ch, NULL, victim, TO_OTHERS);
  act("A dark &+Llight&N surrounds your hands as you touch $N.",
      TRUE, ch, NULL, victim, TO_CHAR);
  act("A dark &+Llight&N surrounds &n's hands as $e touches you.",
      TRUE, ch, NULL, victim, TO_VICT);
  damage(ch, victim, dam, SPELL_CAUSE_CRITIC, -1, -1);
}

ASPELL(spell_cure_critic)
{
  int healpoints;

  healpoints = dice(3, 8) + 3;

  ((*spell_info[SPELL_CURE_CRITIC].routines) (level, ch, NULL, victim, obj, loc, healpoints));

  if (casttype == CAST_SPELL) {
    act("A soft &+Yglowing light&N surrounds $n's hands as $e touches $N.",
	TRUE, ch, NULL, victim, TO_OTHERS);
    act("A soft &+Yglowing light&N surrounds your hands as you touch $N.",
	TRUE, ch, NULL, victim, TO_CHAR);
    if (ch != victim)
      act("A soft &+Yglowing light&N surrounds $n's hands as $e touches you.",
	  TRUE, ch, NULL, victim, TO_VICT);
  } else {
    act("A soft &+Yglowing light&N surrounds $N.", TRUE, ch, NULL, victim,
	TO_OTHERS);
    act("A soft &+Yglowing light&N surrounds you.", TRUE, ch, NULL, victim,
	TO_VICT);
    if (ch != victim)
      act("A soft &+Yglowing light&N surrounds $N.", TRUE, ch, NULL,
	  victim, TO_CHAR);
  }
}

// Sixth Level Cleric Spells

ASPELL(spell_heal)
{
  int healpoints = GET_MAX_HIT(victim) - number(1, 4);

  ((*spell_info[SPELL_HEAL].routines) (level, ch, obj, victim, NULL, loc, healpoints));
  if (casttype == CAST_SPELL) {
    act("A soft &+Yglowing light&N surrounds $n's hands as $e touches $N.",
	TRUE, ch, NULL, victim, TO_OTHERS);
    act("A soft &+Yglowing light&N surrounds your hands as you touch $N.",
	TRUE, ch, NULL, victim, TO_CHAR);
    if (ch != victim)
      act("A soft &+Yglowing light&N surrounds $n's hands as $e touches you.",
	  TRUE, ch, NULL, victim, TO_VICT);
  } else {
    act("A soft &+Yglowing light&N surrounds $N.", TRUE, ch, NULL, victim,
	TO_OTHERS);
    act("A soft &+Yglowing light&N surrounds you.", TRUE, ch, NULL, victim,
	TO_VICT);
    if (ch != victim)
      act("A soft &+Yglowing light&N surrounds $N.", TRUE, ch, NULL,
	  victim, TO_CHAR);
  }
}

ASPELL(spell_word_recall)
{
  extern sh_int r_mortal_start_room;

  if (victim == NULL || IS_NPC(victim))
    return;

  act("$n disappears.", TRUE, victim, 0, 0, TO_PROXIMITY2);
  char_from_room(victim);
  char_to_room(victim, r_mortal_start_room, -1);
  act("$n appears in the middle of the room.", TRUE, victim, 0, 0, TO_PROXIMITY2);
  look_at_room(victim, 0);
}

// Seventh Level Cleric Spells










/*
   ASPELL(spell_summon)
   {
   if (ch == NULL || victim == NULL)
   return;

   if (GET_LEVEL(victim) > MIN(LVL_IMMORT - 1, level + 3)) {
   send_to_char(SUMMON_FAIL, ch);
   return;
   }
   if (MOB_FLAGGED(victim, MOB_NOSUMMON) ||
   (IS_NPC(victim))) {
   send_to_char(SUMMON_FAIL, ch);
   return;
   }
   act("$n disappears suddenly.", TRUE, victim, 0, 0, TO_PROXIMITY2);

   char_from_room(victim);
   char_to_room(victim, ch->in_room, -1);

   act("$n arrives suddenly.", TRUE, victim, 0, 0, TO_PROXIMITY2);
   act("$n has summoned you!", FALSE, ch, 0, victim, TO_VICT);
   look_at_room(victim, 0);
   entry_memory_mtrigger(ch);
   greet_mtrigger(ch, -1); 
   greet_memory_mtrigger(ch);
   }
 */
