

/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"

#define LANGUAGE_CRYPT

char *language_names[LAST_TALK + 1] =
{
  "common",
  "elven",
  "dwarven",
  "goblin",
  "halfling",
  "hobgoblin",
  "kobold",
  "lizardman",
  "ogre",
  "orc",
  "gnome",
  "animal",
  "magic",
  "plant",
  "\n"
};

char *makedrunk(char *string, struct char_data *ch);
void keep_char(int);
void trans_char(int);


void language_show(struct char_data *ch)
{
  int a, lang;

  act("Proficiencies in languages:", TRUE, ch, 0, 0, TO_CHAR);
  for (a = 1; a < LAST_TALK; a++) {
    lang = GET_TALK(ch, a);
    if (lang > 0 && lang < 40)
      sprintf(buf, "You have a basic understanding of %s.",
	      language_names[a - 1]);
    else if (lang < 60)
      sprintf(buf, "You comprehend %s%s.",
	      language_names[a - 1], lang < 55 ? " somewhat" : "");
    else if (lang < 90)
      sprintf(buf, "You are quite fluent in %s.",
	      language_names[a - 1]);
    else
      sprintf(buf, "You are a master of %s.",
	      language_names[a - 1]);
    act(buf, TRUE, ch, 0, 0, TO_CHAR);
  }
  if (GET_TALK(ch, 0) >= 1 && GET_TALK(ch, 0) <= LAST_TALK)
    sprintf(buf, "Currently speaking: %s", language_names[GET_TALK(ch, 0) - 1]);
  else {
    sprintf(buf, "Currently speaking impossible language. Attempting to fix.\r\n");
    GET_TALK(ch, 0) = TALK_COMMON;
  }
  act(buf, TRUE, ch, 0, 0, TO_CHAR);
}

ACMD(do_speak)
{
  int a;

  if (!argument || !*argument) {
    language_show(ch);
    return;
  }
  one_argument(argument, buf);
  if ((a = search_block(buf, language_names, FALSE)) < 0) {
    send_to_char("Speak what language from now on?\r\n", ch);
    return;
  }
  if (GET_TALK(ch, a + 1) >= 20 || IS_TRUSTED(ch)) {
    GET_TALK(ch, 0) = a + 1;
    sprintf(buf, "Okay, now speaking %s.\r\n", language_names[a]);
    send_to_char(buf, ch);
  } else
    send_to_char("You don't know that language well enough to speak it!\r\n", ch);
}

void init_language(struct char_data *ch)
{
  int x;

  for (x = 0; x <= MAX_TONGUE; x++)
    GET_TALK(ch, x) = 0;

  switch (GET_RACE(ch)) {
  case RACE_ELF:
    GET_TALK(ch, TALK_ELF) = 100;
    GET_TALK(ch, TALK_COMMON) = (40 + (GET_INT(ch) * 3.5));
    GET_TALK(ch, TALK_USING) = TALK_ELF;
    GET_TALK(ch, LAST_HEARD) = TALK_ELF;
    break;
  case RACE_DWARF:
    GET_TALK(ch, TALK_DWARF) = 100;
    GET_TALK(ch, TALK_COMMON) = (40 + (GET_INT(ch) * 2.5));
    GET_TALK(ch, TALK_USING) = TALK_DWARF;
    GET_TALK(ch, LAST_HEARD) = TALK_DWARF;
    break;
  case RACE_GOBLIN:
    GET_TALK(ch, TALK_GOBLIN) = 100;
    GET_TALK(ch, TALK_COMMON) = (20 + (GET_INT(ch)));
    GET_TALK(ch, TALK_USING) = TALK_GOBLIN;
    GET_TALK(ch, LAST_HEARD) = TALK_GOBLIN;
    break;
  case RACE_HALFLING:
    GET_TALK(ch, TALK_HALFLING) = 100;
    GET_TALK(ch, TALK_COMMON) = (50 + (GET_INT(ch) * 3));
    GET_TALK(ch, TALK_USING) = TALK_HALFLING;
    GET_TALK(ch, LAST_HEARD) = TALK_HALFLING;
    break;
  case RACE_HOBGOBLIN:
    GET_TALK(ch, TALK_HOBGOBLIN) = 100;
    GET_TALK(ch, TALK_COMMON) = (30 + (GET_INT(ch)));
    GET_TALK(ch, TALK_USING) = TALK_HOBGOBLIN;
    GET_TALK(ch, LAST_HEARD) = TALK_HOBGOBLIN;
    break;
  case RACE_KOBOLD:
    GET_TALK(ch, TALK_KOBOLD) = 100;
    GET_TALK(ch, TALK_COMMON) = (20 + (GET_INT(ch)));
    GET_TALK(ch, TALK_USING) = TALK_KOBOLD;
    GET_TALK(ch, LAST_HEARD) = TALK_KOBOLD;
    break;
  case RACE_LIZARD_MAN:
    GET_TALK(ch, TALK_LIZARDMAN) = 100;
    GET_TALK(ch, TALK_COMMON) = (40 + (GET_INT(ch) * 3));
    GET_TALK(ch, TALK_USING) = TALK_LIZARDMAN;
    GET_TALK(ch, LAST_HEARD) = TALK_LIZARDMAN;
    break;
  case RACE_OGRE:
    GET_TALK(ch, TALK_OGRE) = 100;
    GET_TALK(ch, TALK_COMMON) = (20 + (GET_INT(ch) * 3));
    GET_TALK(ch, TALK_USING) = TALK_OGRE;
    GET_TALK(ch, LAST_HEARD) = TALK_OGRE;
    break;
  case RACE_ORC:
    GET_TALK(ch, TALK_ORC) = 100;
    GET_TALK(ch, TALK_COMMON) = (30 + (GET_INT(ch) * 3));
    GET_TALK(ch, TALK_USING) = TALK_ORC;
    GET_TALK(ch, LAST_HEARD) = TALK_ORC;
    break;
  case RACE_GNOME:
    GET_TALK(ch, TALK_GNOME) = 100;
    GET_TALK(ch, TALK_COMMON) = (50 + (GET_INT(ch) * 3));
    GET_TALK(ch, TALK_USING) = TALK_GNOME;
    GET_TALK(ch, LAST_HEARD) = TALK_GNOME;
    break;
  case RACE_HUMAN:
  case RACE_HALFELF:
  case RACE_HALFORC:
    GET_TALK(ch, TALK_COMMON) = 100;
    GET_TALK(ch, TALK_USING) = TALK_COMMON;
    GET_TALK(ch, LAST_HEARD) = TALK_COMMON;
    break;
  }

#if 1
  if (IS_ANIMAL(ch))
    GET_TALK(ch, TALK_ANIMAL) = 100;
#endif
}

char *language_known(struct char_data *ch, struct char_data *vict)
{
  static char speech[256];

#ifdef LANGUAGE_CRYPT
  int a;

  speech[0] = '\0';
  if (IS_TRUSTED(ch) || IS_NPC(ch) ||
  (IS_PC(ch) && ((GET_TALK(ch, 0) < 1) || (GET_TALK(ch, 0) > LAST_TALK))))
    return speech;
  a = GET_TALK(ch, 0);
  if ((a > 0) && (a <= LAST_TALK))
    if (			/* affected_by_spell(ch, SPELL_COMPREHEND_LANGUAGES) || */
	 ((IS_PC(vict) && (GET_TALK(vict, a) >= (20 + number(1, 20))))))
      sprintf(speech, "in %s ", language_names[a - 1]);
#else
  strcpy(speech, "");
#endif
  return speech;
}

void language_gain(struct char_data *ch, struct char_data *vict, int tongue)
{
#ifdef LANGUAGE_CRYPT

  if (IS_NPC(vict))
    return;

  if (!number(0, 19) && (number(50, 150) < (GET_INT(vict) - 15)) &&
      GET_TALK(vict, tongue) && (GET_TALK(vict, tongue) < 98) &&
      (GET_TALK(ch, tongue) > 75))
    GET_TALK(vict, tongue)++;

#endif
}

struct translation_table {
  const char *OrigString;
  const char *NewString;
};

/* fill this table with language transforms. */
struct translation_table language_table[] =
{
  {"mine", "myne"},
  {"Mine", "Myne"},
  {"that", "thaet"},
  {"That", "Thaet"},
  {"this", "thys"},
  {"This", "Thys"},
  {"the", "thea"},
  {"The", "Thea"},
  {"you", "you"},
  {"You", "You"},
  {"me", "me"},
  {"Me", "Me"},
  {"a", "e"},
  {"A", "E"},
  {"b", "c"},
  {"B", "C"},
  {"c", "d"},
  {"C", "D"},
  {"d", "f"},
  {"D", "F"},
  {"e", "i"},
  {"E", "I"},
  {"f", "g"},
  {"F", "G"},
  {"g", "h"},
  {"G", "H"},
  {"h", "j"},
  {"H", "J"},
  {"i", "o"},
  {"I", "O"},
  {"j", "k"},
  {"J", "K"},
  {"k", "l"},
  {"K", "L"},
  {"l", "m"},
  {"L", "M"},
  {"m", "n"},
  {"M", "N"},
  {"n", "p"},
  {"N", "P"},
  {"o", "u"},
  {"O", "U"},
  {"p", "q"},
  {"P", "Q"},
  {"q", "r"},
  {"Q", "R"},
  {"r", "s"},
  {"R", "S"},
  {"s", "t"},
  {"S", "T"},
  {"t", "v"},
  {"T", "V"},
  {"u", "y"},
  {"U", "Y"},
  {"v", "w"},
  {"V", "W"},
  {"w", "x"},
  {"W", "X"},
  {"x", "z"},
  {"X", "Z"},
  {"y", "a"},
  {"Y", "A"},
  {"z", "b"},
  {"Z", "B"},
  {"", ""}			/*table must end with empty string */
};

static char *ntstr;
static char *str;

void trans_char(int num)
{
  int i;

  for (i = 0; (i < num) && (*str != '\0'); i++) {
    struct translation_table *wijzer = language_table;

    while ((wijzer->OrigString)[0] != '\0') {
      if (!strncmp(wijzer->OrigString, str, strlen(wijzer->OrigString))) {
	strcpy(ntstr, wijzer->NewString);
	ntstr += strlen(wijzer->NewString);
	str += strlen(wijzer->OrigString);
	break;
      }
      wijzer++;
    }

/* if letter that occurs that isn't in table */
    if ((wijzer->OrigString)[0] == '\0') {
      ntstr[0] = str[0];
      ntstr++;
      str++;
    }
  }
  *ntstr = '\0';
}

void keep_char(int num)
{
  int i;

  for (i = 0; (i < num) && (*str != '\0'); i++) {
    *ntstr = *str;
    str++;
    ntstr++;
  }
}

/* transformation routine */
/* ch is the one who speaks.. victim the one who reads .. */
char *language_crypt(struct char_data *ch, struct char_data *victim, char *message)
{
  char translation[MAX_INPUT_LENGTH];
  static char string[512];
  int len, learned, i, ch_skill, vict_skill;
  ulong ttl;

  strcpy(string, message);
  strcpy(string, makedrunk(string, ch));

  ntstr = translation;
  str = string;

  if (string[0] == 0) {
    strcpy(string, "");
    return string;
  }
  if ((ch == NULL) || (victim == NULL)) {
    strcpy(string, "");
    return string;
  }
  if (IS_NPC(ch))
    return string;
  if (((IS_TRUSTED(ch))) || IS_TRUSTED(victim))
    return string;
  if (IS_PC(ch) && (GET_TALK(ch, 0) < 1 || GET_TALK(ch, 0) > LAST_TALK))
    return string;

  if (IS_PC(ch))
    ch_skill = GET_TALK(ch, (int) GET_TALK(ch, 0));
  else
    ch_skill = 100;
  /* listening is easier than speaking */
  if (IS_PC(victim))
    vict_skill = GET_TALK(victim, (int) GET_TALK(ch, 0)) + 10;
  else				/* npc's listen at 100% */
    vict_skill = 100;

  learned = (vict_skill + ch_skill) / 2;
  len = strlen(string);

  /* gain skill code */
  /* do a checksum and if the checksum is DIFFERENT from the last this
     vict heard, they have a chance to gain...  also require a minimum
     length of the phrase! */

  if (IS_PC(victim) && (len > 20)) {
    ttl = 0;
    for (i = 0; i < len; i++)
      ttl += string[i];
    if (((ubyte) ttl) != GET_TALK(victim, LAST_TALK)) {
      GET_TALK(victim, LAST_TALK) = ((ubyte) ttl);
      language_gain(ch, victim, GET_TALK(ch, 0));
    }
  }
  if (learned >= 90)
    return string;		/* no need for translation .. */
  if (learned == 0) {
    trans_char(len);
    strcpy(string, translation);
    return string;
  }
  while (*str != '\0') {

/* don't change all letters .. */
    if (learned < 30) {
      keep_char(1);
      trans_char(BOUNDED(1, 100 / learned - 1, 3));
    } else {
      keep_char(BOUNDED(2, (learned / (101 - learned) + 1), 5));
      trans_char(1);
    }
    ntstr[0] = '\0';
  }

  *ntstr = '\0';
  strcpy(string, translation);
  return string;
}

struct drunk_struct {
  int min_drunk_level;
  int number_of_rep;
  const char *replacement[11];
};


char *makedrunk(char *string, struct char_data *ch)
{
  char temp;
  int pos = 0, randomnum;
  struct drunk_struct drunk[] =
  {
    {3, 10,
     {"a", "a", "a", "A", "aa", "ah", "Ah", "ao", "aw", "oa", "ahhhh"}},
    {8, 5,
     {"b", "b", "b", "B", "B", "vb"}},
    {3, 5,
     {"c", "c", "C", "cj", "sj", "zj"}},
    {5, 2,
     {"d", "d", "D"}},
    {3, 3,
     {"e", "e", "eh", "E"}},
    {4, 5,
     {"f", "f", "ff", "fff", "fFf", "F"}},
    {8, 2,
     {"g", "g", "G"}},
    {9, 6,
     {"h", "h", "hh", "hhh", "Hhh", "HhH", "H"}},
    {7, 6,
     {"i", "i", "Iii", "ii", "iI", "Ii", "I"}},
    {9, 5,
     {"j", "j", "jj", "Jj", "jJ", "J"}},
    {7, 2,
     {"k", "k", "K"}},
    {3, 2,
     {"l", "l", "L"}},
    {5, 8,
     {"m", "m", "mm", "mmm", "mmmm", "mmmmm", "MmM", "mM", "M"}},
    {6, 6,
     {"n", "n", "nn", "Nn", "nnn", "nNn", "N"}},
    {3, 6,
     {"o", "o", "ooo", "ao", "aOoo", "Ooo", "ooOo"}},
    {3, 2,
     {"p", "p", "P"}},
    {5, 5,
     {"q", "q", "Q", "ku", "ququ", "kukeleku"}},
    {4, 2,
     {"r", "r", "R"}},
    {2, 5,
     {"s", "ss", "zzZzssZ", "ZSssS", "sSzzsss", "sSss"}},
    {5, 2,
     {"t", "t", "T"}},
    {3, 6,
     {"u", "u", "uh", "Uh", "Uhuhhuh", "uhU", "uhhu"}},
    {4, 2,
     {"v", "v", "V"}},
    {4, 2,
     {"w", "w", "W"}},
    {5, 6,
     {"x", "x", "X", "ks", "iks", "kz", "xz"}},
    {3, 2,
     {"y", "y", "Y"}},
    {2, 9,
  {"z", "z", "ZzzZz", "Zzz", "Zsszzsz", "szz", "sZZz", "ZSz", "zZ", "Z"}}
  };

  if (GET_COND(ch, DRUNK) > 0) {
    do {
      temp = toupper(*string);
      if ((temp >= 'A') && (temp <= 'Z')) {
	if (GET_COND(ch, DRUNK) > drunk[(temp - 'A')].min_drunk_level) {
	  randomnum = number(0, (drunk[(temp - 'A')].number_of_rep));
	  strcpy(&buf[pos], drunk[(temp - 'A')].replacement[randomnum]);
	  pos += strlen(drunk[(temp - 'A')].replacement[randomnum]);
	} else
	  buf[pos++] = *string;
      } else {
	if ((temp >= '0') && (temp <= '9')) {
	  temp = '0' + number(0, 9);
	  buf[pos++] = temp;
	} else
	  buf[pos++] = *string;
      }
    } while (*string++);

    buf[pos] = '\0';		/* Mark end of the string... */
    strcpy(string, buf);
    return (string);
  }
  return (string);
}
