/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "comm.h"
#include "spells.h"
#include "utils.h"
#include "db.h"
#include "shop.h"
#include "olc.h"
#include "handler.h"
#include "interpreter.h"
#include "dg_olc.h"

/*
 * Set this to 1 for debugging logs in medit_save_internally.
 */
#if 1
#define DEBUG
#endif

/*
 * Set this to 1 as a last resort to save mobiles.
 */
#if 1
#define I_CRASH
#endif

/*-------------------------------------------------------------------*/

/*
 * External variable declarations.
 */
extern char *kingdom_list[];
extern struct index_data *mob_index;
extern struct char_data *mob_proto;
extern struct char_data *character_list;
extern int top_of_mobt;
extern struct zone_data *zone_table;
extern int top_of_zone_table;
extern struct player_special_data dummy_mob;
extern struct attack_hit_type attack_hit_text[];
extern char *action_bits[];
extern char *affected_bits[];
extern char *affected2_bits[];
extern char *affected3_bits[];
extern char *int_ratings[];
extern char *affected4_bits[];
extern char *class_types[];
extern char *race_types[];
extern char *spells[];
extern char *position_types[];
extern char *genders[];
extern int top_shop;
extern struct shop_data *shop_index;
extern struct descriptor_data *descriptor_list;
extern const char *alignment_types[];
void paedit_disp_path_editor(struct descriptor_data *d);
char *paedit_parse_performance(struct path_perform *perform);
char *paedit_parse_action(struct path_info *path);
char *paedit_parse_type(struct path_info *path);
char *paedit_hrmin_disp(int hrmin);
void paedit_change_path(struct descriptor_data *d);
void paedit_list_days(struct descriptor_data *d);
void paedit_list_month(struct descriptor_data *d);
void paedit_time_edit_disp(struct descriptor_data *d);
void path_subfunctions(struct descriptor_data *d, char *arg);
extern const int thaco[NUM_CLASSES][LVL_IMPL + 1];
extern void trigedit_setup_new(struct descriptor_data *d);
extern void trigedit_setup_existing(struct descriptor_data *d, int rtrg_num);
extern void quest_save_to_file(FILE * fp, struct char_data *mob);
extern char *FathomMobIntelligence(struct char_data *mob, int num);
extern void string_write(struct descriptor_data *d, char **writeto, size_t len, long mailto, void *data);
extern void disp_kingdom_menu(struct descriptor_data *d);
extern void display_mob_spells(struct char_data *target, struct char_data *ch);

/*-------------------------------------------------------------------*/

/*
 * Handy internal macros.
 */

#define GET_ALIAS(mob) ((mob)->player.name)
#define GET_SDESC(mob) ((mob)->player.short_descr)
#define GET_LDESC(mob) ((mob)->player.long_descr)
#define GET_DDESC(mob) ((mob)->player.description)
#define S_KEEPER(shop) ((shop)->keeper)

/*-------------------------------------------------------------------*/

/*
 * Function prototypes.
 */
void medit_parse(struct descriptor_data *d, char *arg);
void medit_disp_menu(struct descriptor_data *d);
void medit_setup_new(struct descriptor_data *d);
void medit_setup_existing(struct descriptor_data *d, int rmob_num);
void medit_save_internally(struct descriptor_data *d);
void medit_save_to_disk(int zone_num);
void init_mobile(struct char_data *mob);
void copy_mobile(struct char_data *tmob, struct char_data *fmob);
void medit_disp_positions(struct descriptor_data *d);
void medit_disp_mob_flags(struct descriptor_data *d);
void medit_disp_aff_flags(struct descriptor_data *d);
void medit_disp_aff2_flags(struct descriptor_data *d);
void medit_disp_aff3_flags(struct descriptor_data *d);
void medit_disp_aff4_flags(struct descriptor_data *d);
void medit_disp_races(struct descriptor_data *d);
void medit_disp_class(struct descriptor_data *d);
void medit_disp_alignment(struct descriptor_data *d);
void medit_disp_attack_types(struct descriptor_data *d);
void medit_display_attack(struct descriptor_data *d);
void medit_delete_command(struct descriptor_data *d, int pos);
void medit_add_command(struct descriptor_data *d);
void medit_add_min(struct descriptor_data *d, int pos);
void medit_add_max(struct descriptor_data *d, int pos);
void medit_add_type(struct descriptor_data *d, int pos);
void medit_delete(struct descriptor_data *d);

/*-------------------------------------------------------------------*\
  utility functions 
\*-------------------------------------------------------------------*/

/*
 * Free a mobile structure that has been edited.
 * Take care of existing mobiles and their mob_proto!
 */

void medit_free_mobile(struct char_data *mob)
{
  int i;
  /*
   * Non-prototyped mobile.  Also known as new mobiles.
   */
  if (!mob)
    return;
  else if (GET_MOB_RNUM(mob) == -1) {
    if (mob->player.name)
      FREE(mob->player.name);
    if (mob->player.title)
      FREE(mob->player.title);
    if (mob->player.short_descr)
      FREE(mob->player.short_descr);
    if (mob->player.long_descr)
      FREE(mob->player.long_descr);
    if (mob->player.description)
      FREE(mob->player.description);
  } else if ((i = GET_MOB_RNUM(mob)) > -1) {	/* Prototyped mobile. */
    if (mob->player.name && mob->player.name != mob_proto[i].player.name)
      FREE(mob->player.name);
    if (mob->player.title && mob->player.title != mob_proto[i].player.title)
      FREE(mob->player.title);
    if (mob->player.short_descr && mob->player.short_descr != mob_proto[i].player.short_descr)
      FREE(mob->player.short_descr);
    if (mob->player.long_descr && mob->player.long_descr != mob_proto[i].player.long_descr)
      FREE(mob->player.long_descr);
    if (mob->player.description && mob->player.description != mob_proto[i].player.description)
      FREE(mob->player.description);
  }
  while (mob->affected)
    affect_remove(mob, mob->affected);

  FREE(mob);
}

void medit_setup_new(struct descriptor_data *d)
{
  struct char_data *mob;

  /*
   * Allocate a scratch mobile structure.  
   */
  CREATE(mob, struct char_data, 1);

  init_mobile(mob);

  GET_MOB_RNUM(mob) = -1;
  /*
   * Set up some default strings.
   */
  GET_ALIAS(mob) = str_dup("mob unfinished");
  GET_SDESC(mob) = str_dup("the unfinished mob");
  GET_LDESC(mob) = str_dup("An unfinished mob stands here.\r\n");
  GET_DDESC(mob) = str_dup("It looks unfinished.\r\n");
  OLC_MOB(d) = mob;
  OLC_VAL(d) = 0;		/* Has changed flag. (It hasn't so far, we just made it.) */
  OLC_PATH(d) = NULL;
  OLC_PATH_LIST(d) = NULL;
  OLC_PROTO(d) = OLC_NUM(d);
  GET_MOB_TREASURE(mob) = -1;
  medit_disp_menu(d);
}

/*-------------------------------------------------------------------*/

void medit_setup_existing(struct descriptor_data *d, int rmob_num)
{
  struct char_data *mob;
  struct path_info *ptemp, *phead;
  /*
   * Allocate a scratch mobile structure. 
   */
  CREATE(mob, struct char_data, 1);

  copy_mobile(mob, mob_proto + rmob_num);

  if (GET_PATH(mob))
    CREATE(OLC_PATH_LIST(d), struct path_info, 1);
  phead = OLC_PATH_LIST(d);
  for (ptemp = GET_PATH(mob); ptemp; ptemp = ptemp->next) {
    OLC_PATH_LIST(d)->type = ptemp->type;
    if (ptemp->arguments)
      OLC_PATH_LIST(d)->arguments = str_dup(ptemp->arguments);
    else
      OLC_PATH_LIST(d)->arguments = NULL;

    OLC_PATH_LIST(d)->target_room = ptemp->target_room;
    CREATE(OLC_PATH_LIST(d)->performance, struct path_perform, 1);
    if (ptemp->performance) {
      OLC_PATH_LIST(d)->performance->hrmin = ptemp->performance->hrmin;
      OLC_PATH_LIST(d)->performance->day = ptemp->performance->day;
      OLC_PATH_LIST(d)->performance->week = ptemp->performance->week;
      OLC_PATH_LIST(d)->performance->month = ptemp->performance->month;
    } else {
      OLC_PATH_LIST(d)->performance = NULL;
    }
    if (ptemp->next) {
      CREATE(OLC_PATH_LIST(d)->next, struct path_info, 1);
      OLC_PATH_LIST(d) = OLC_PATH_LIST(d)->next;
    }
  }
  OLC_PATH_LIST(d) = phead;
  OLC_PATH(d) = OLC_PATH_LIST(d);
  OLC_MOB(d) = mob;
  OLC_PROTO(d) = GET_MOB_ORIG_VNUM(OLC_MOB(d));
  OLC_ITEM_TYPE(d) = MOB_TRIGGER;
  dg_olc_script_copy(d);
  medit_disp_menu(d);
}

/*-------------------------------------------------------------------*/

/*
 * Copy one mob struct to another.
 */
void copy_mobile(struct char_data *tmob, struct char_data *fmob)
{
  struct trig_proto_list *proto, *fproto;
  /*
   * Free up any used strings.
   */
  if (GET_ALIAS(tmob))
    FREE(GET_ALIAS(tmob));
  if (GET_SDESC(tmob))
    FREE(GET_SDESC(tmob));
  if (GET_LDESC(tmob))
    FREE(GET_LDESC(tmob));
  if (GET_DDESC(tmob))
    FREE(GET_DDESC(tmob));

/* delete the old script list */
  proto = tmob->proto_script;
  while (proto) {
    fproto = proto;
    proto = proto->next;
    free(fproto);
  }
  /*
   * Copy mob over.
   */
  *tmob = *fmob;

  /*
   * Reallocate strings.
   */
  GET_ALIAS(tmob) = str_dup((GET_ALIAS(fmob) && *GET_ALIAS(fmob)) ? GET_ALIAS(fmob) : "undefined");
  GET_SDESC(tmob) = str_dup((GET_SDESC(fmob) && *GET_SDESC(fmob)) ? GET_SDESC(fmob) : "undefined");
  GET_LDESC(tmob) = str_dup((GET_LDESC(fmob) && *GET_LDESC(fmob)) ? GET_LDESC(fmob) : "undefined");
  GET_DDESC(tmob) = str_dup((GET_DDESC(fmob) && *GET_DDESC(fmob)) ? GET_DDESC(fmob) : "undefined");

  /* copy the new script list */
  if (fmob->proto_script) {
    fproto = fmob->proto_script;
    CREATE(proto, struct trig_proto_list, 1);
    tmob->proto_script = proto;
    do {
      proto->vnum = fproto->vnum;
      fproto = fproto->next;
      if (fproto) {
	CREATE(proto->next, struct trig_proto_list, 1);
	proto = proto->next;
      }
    } while (fproto);
  }
}

/*-------------------------------------------------------------------*/

/*
 * Ideally, this function should be in db.c, but I'll put it here for
 * portability.
 */
void init_mobile(struct char_data *mob)
{
  clear_char(mob);

  GET_HIT(mob) = 1;
  GET_WEIGHT(mob) = 200;
  GET_HEIGHT(mob) = 198;
  GET_MAGIC_RESISTANCE(mob) = 0;
  GET_HPDIE(mob) = 8;
  GET_HPPLUS(mob) = 0;
  GET_AC(mob) = 10;
  GET_MOB_THAC0(mob) = 20;
  GET_CLASS(mob) = -1;

  mob->real_abils.str = mob->real_abils.intel = mob->real_abils.wis = 11;
  mob->real_abils.dex = mob->real_abils.con = mob->real_abils.cha = 11;
  mob->aff_abils = mob->real_abils;

  SET_BIT(MOB_FLAGS(mob), MOB_ISNPC);
  mob->player_specials = &dummy_mob;
}

/*-------------------------------------------------------------------*/

#define ZCMD zone_table[zone].cmd[cmd_no]

/*
 * Save new/edited mob to memory.
 */
void medit_save_internally(struct descriptor_data *d)
{
  int rmob_num, found = 0, new_mob_num = 0, zone, cmd_no, shop;
  struct char_data *new_proto;
  struct index_data *new_index;
  struct char_data *live_mob;
  struct descriptor_data *dsc;

  /* put the script into proper position */
  OLC_MOB(d)->proto_script = OLC_SCRIPT(d);

  /*
   * Mob exists? Just update it.
   */
  if ((rmob_num = real_mobile(OLC_NUM(d))) != -1) {
    OLC_MOB(d)->proto_script = OLC_SCRIPT(d);
    copy_mobile((mob_proto + rmob_num), OLC_MOB(d));
    /*
     * Update live mobiles.
     */
    for (live_mob = character_list; live_mob; live_mob = live_mob->next)
      if (IS_MOB(live_mob) && GET_MOB_RNUM(live_mob) == rmob_num) {
	/*
	 * Only really need to update the strings, since these can
	 * cause protection faults.  The rest can wait till a reset/reboot.
	 */
	GET_ALIAS(live_mob) = GET_ALIAS(mob_proto + rmob_num);
	GET_SDESC(live_mob) = GET_SDESC(mob_proto + rmob_num);
	GET_LDESC(live_mob) = GET_LDESC(mob_proto + rmob_num);
	GET_DDESC(live_mob) = GET_DDESC(mob_proto + rmob_num);

      }
  }
  /*
   * Mob does not exist, we have to add it.
   */
  else {
#if defined(DEBUG)
    fprintf(stderr, "top_of_mobt: %d, new top_of_mobt: %d\n", top_of_mobt, top_of_mobt + 1);
#endif


    CREATE(new_proto, struct char_data, top_of_mobt + 2);
    CREATE(new_index, struct index_data, top_of_mobt + 2);

    for (rmob_num = 0; rmob_num <= top_of_mobt; rmob_num++) {
      if (!found) {		/* Is this the place? */
/*      if ((rmob_num > top_of_mobt) || (mob_index[rmob_num].virtual > OLC_NUM(d))) { */
	if (mob_index[rmob_num].virtual > OLC_NUM(d)) {
	  /*
	   * Yep, stick it here.
	   */
	  found = TRUE;
#if defined(DEBUG)
	  fprintf(stderr, "Inserted: rmob_num: %d\n", rmob_num);
#endif
	  new_index[rmob_num].virtual = OLC_NUM(d);
	  new_index[rmob_num].number = 0;
	  new_index[rmob_num].func = NULL;
	  new_mob_num = rmob_num;
	  GET_MOB_RNUM(OLC_MOB(d)) = rmob_num;
	  GET_MOB_ORIG_VNUM(OLC_MOB(d)) = OLC_PROTO(d);
	  copy_mobile((new_proto + rmob_num), OLC_MOB(d));
	  /*
	   * Copy the mob that should be here on top.
	   */
	  new_index[rmob_num + 1] = mob_index[rmob_num];
	  new_proto[rmob_num + 1] = mob_proto[rmob_num];
	  GET_MOB_RNUM(new_proto + rmob_num + 1) = rmob_num + 1;
	} else {		/* Nope, copy over as normal. */
	  new_index[rmob_num] = mob_index[rmob_num];
	  new_proto[rmob_num] = mob_proto[rmob_num];
	}
      } else {			/* We've already found it, copy the rest over. */
	new_index[rmob_num + 1] = mob_index[rmob_num];
	new_proto[rmob_num + 1] = mob_proto[rmob_num];
	GET_MOB_RNUM(new_proto + rmob_num + 1) = rmob_num + 1;
      }
    }
#if defined(DEBUG)
    fprintf(stderr, "rmob_num: %d, top_of_mobt: %d, array size: 0-%d (%d)\n",
	    rmob_num, top_of_mobt, top_of_mobt + 1, top_of_mobt + 2);
#endif
    if (!found) {		/* Still not found, must add it to the top of the table. */
#if defined(DEBUG)
      fprintf(stderr, "Append.\n");
#endif
      new_index[rmob_num].virtual = OLC_NUM(d);
      new_index[rmob_num].number = 0;
      new_index[rmob_num].func = NULL;
      new_mob_num = rmob_num;
      GET_MOB_RNUM(OLC_MOB(d)) = rmob_num;
      GET_MOB_ORIG_VNUM(OLC_MOB(d)) = OLC_PROTO(d);
      copy_mobile((new_proto + rmob_num), OLC_MOB(d));
    }
    /*
     * Replace tables.
     */
#if defined(DEBUG)
    fprintf(stderr, "Attempted free.\n");
#endif
#if !defined(I_CRASH)
    FREE(mob_index);
    FREE(mob_proto);
#endif
    mob_index = new_index;
    mob_proto = new_proto;
    top_of_mobt++;
#if defined(DEBUG)
    fprintf(stderr, "Free ok.\n");
#endif

    /*
     * Update live mobile rnums.
     */
    for (live_mob = character_list; live_mob; live_mob = live_mob->next)
      if (GET_MOB_RNUM(live_mob) > new_mob_num)
	GET_MOB_RNUM(live_mob)++;

    /*
     * Update zone table.
     */
    for (zone = 0; zone <= top_of_zone_table; zone++)
      for (cmd_no = 0; ZCMD.command != 'S'; cmd_no++)
	if (ZCMD.command == 'M')
	  if (ZCMD.arg1 >= new_mob_num)
	    ZCMD.arg1++;

    /*
     * Update shop keepers.
     */
    if (shop_index)
      for (shop = 0; shop <= top_shop; shop++)
	if (SHOP_KEEPER(shop) >= new_mob_num)
	  SHOP_KEEPER(shop)++;

    /*
     * Update keepers in shops being edited and other mobs being edited.
     */
    for (dsc = descriptor_list; dsc; dsc = dsc->next)
      if (dsc->connected == CON_SEDIT) {
	if (S_KEEPER(OLC_SHOP(dsc)) >= new_mob_num)
	  S_KEEPER(OLC_SHOP(dsc))++;
      } else if (dsc->connected == CON_MEDIT) {
	if (GET_MOB_RNUM(OLC_MOB(dsc)) >= new_mob_num)
	  GET_MOB_RNUM(OLC_MOB(dsc))++;
      }
  }

  GET_PATH(OLC_MOB(d)) = OLC_PATH_LIST(d);
  /* Update the path list */
  check_path();

  olc_add_to_save_list(zone_table[OLC_ZNUM(d)].number, OLC_SAVE_MOB);
}

/*-------------------------------------------------------------------*/

/*
 * Save ALL mobiles for a zone to their .mob file, mobs are all 
 * saved in Extended format, regardless of whether they have any
 * extended fields.  Thanks to Sammy for ideas on this bit of code.
 */
void medit_save_to_disk(int zone_num)
{
  int i, rmob_num, zone, top;
  FILE *mob_file;
  char fname[64];
  struct char_data *mob;
  struct path_info *path = NULL;
  struct mob_attack_data *new_attack;

  zone = zone_table[zone_num].number;
  top = zone_table[zone_num].top;

  sprintf(fname, "%s/%d.new", MOB_PREFIX, zone);
  if (!(mob_file = fopen(fname, "w"))) {
    mudlog("SYSERR: OLC: Cannot open mob file!", BRF, LVL_BUILDER, TRUE);
    return;
  }
  /*
   * Seach the database for mobs in this zone and save them.
   */
  for (i = zone * 100; i <= top; i++) {
    if ((rmob_num = real_mobile(i)) != -1) {
      if (fprintf(mob_file, "#%d\n", i) < 0) {
	mudlog("SYSERR: OLC: Cannot write mob file!\r\n", BRF, LVL_BUILDER, TRUE);
	fclose(mob_file);
	return;
      }
      mob = (mob_proto + rmob_num);

      /*
       * Clean up strings.
       */
      strcpy(buf1, (GET_LDESC(mob) && *GET_LDESC(mob)) ? GET_LDESC(mob) : "undefined");
      strip_string(buf1);
      strcpy(buf2, (GET_DDESC(mob) && *GET_DDESC(mob)) ? GET_DDESC(mob) : "undefined");
      strip_string(buf2);

      fprintf(mob_file, "%s~\n"
	      "%s~\n"
	      "%s~\n"
	      "%s~\n"
	      "%ld %ld %ld %ld %ld %d %d %d %d E\n"
	      "%d %d %d %dd%d+%d %d %d %d\n"
	      "%d %d %d\n"
	      "%d %d %d\n",
      (GET_ALIAS(mob) && *GET_ALIAS(mob)) ? GET_ALIAS(mob) : "undefined",
      (GET_SDESC(mob) && *GET_SDESC(mob)) ? GET_SDESC(mob) : "undefined",
	      buf1, buf2, MOB_FLAGS(mob), AFF_FLAGS(mob), AFF2_FLAGS(mob), AFF3_FLAGS(mob), AFF4_FLAGS(mob),
	GET_ALIGNMENT(mob), GET_SIZE(mob), GET_RACE(mob), GET_CLASS(mob),
	      GET_LEVEL(mob), GET_HITROLL(mob),
	    GET_AC(mob), GET_LEVEL(mob), GET_HPDIE(mob), GET_HPPLUS(mob),
	      GET_DAMROLL(mob), GET_MAGIC_RESISTANCE(mob),
	      GET_MOB_THAC0(mob), GET_MOB_TREASURE(mob),
	      GET_EXP(mob), GET_HOMETOWN(mob), GET_POS(mob), GET_DEFAULT_POS(mob), GET_SEX(mob)
	  );

      /*
       * Deal with Extra stats in case they are there.
       */
      if (GET_STR(mob) != 11)
	fprintf(mob_file, "Str: %d\n", GET_STR(mob));
      if (GET_ADD(mob) != 0)
	fprintf(mob_file, "StrAdd: %d\n", GET_ADD(mob));
      if (GET_DEX(mob) != 11)
	fprintf(mob_file, "Dex: %d\n", GET_DEX(mob));
      if (GET_INT(mob) != 11)
	fprintf(mob_file, "Int: %d\n", GET_INT(mob));
      if (GET_WIS(mob) != 11)
	fprintf(mob_file, "Wis: %d\n", GET_WIS(mob));
      if (GET_CON(mob) != 11)
	fprintf(mob_file, "Con: %d\n", GET_CON(mob));
      if (GET_CHA(mob) != 11)
	fprintf(mob_file, "Cha: %d\n", GET_CHA(mob));
      if (GET_MOB_ORIG_VNUM(mob) != GET_MOB_VNUM(mob))
	fprintf(mob_file, "PrototypeOf: %d\n", GET_MOB_ORIG_VNUM(mob));

      /*
       * XXX: Add E-mob handlers here.
       */

      fprintf(mob_file, "E\n");

      CREATE(new_attack, struct mob_attack_data, 1);
      new_attack = mob->mob_specials.mob_attack;
      if (new_attack) {
	while (new_attack) {
	  fprintf(mob_file, "%d %d %d\n",
		  new_attack->min_damage,
		  new_attack->max_damage,
		  new_attack->attack_type);
	  new_attack = new_attack->next;
	}
      } else {
	fprintf(mob_file, "0 0 0\n");
      }
      fprintf(mob_file, "A\n");

      script_save_to_disk(mob_file, mob, MOB_TRIGGER);
      quest_save_to_file(mob_file, mob);
#if 1
      /* time to write out paths. Since I haven't defined it anywhere else,
         I'll make it incredibly arbitrary, and handle it within a
         slightly modified
         Path
         <hrmin> <day> <week> <month> <type>
         <either target or arguments>
         reads in the bottom line as a string, and converts based on type */

      if (GET_PATH(mob)) {
	path = GET_PATH(mob);
	if (path->type != PATH_NONE) {
	  fprintf(mob_file, "!\n"
		  "%d %d %d %d %d\n",
		  path->performance->hrmin,
		  path->performance->day,
		  path->performance->week,
		  path->performance->month,
		  path->type);
	  if (path->type == PATH_GOTO)
	    fprintf(mob_file, "%d\n", path->target_room);
	  else {
	    if (path->arguments)
	      fprintf(mob_file, "%s\n", path->arguments);
	    else
	      fprintf(mob_file, "say Help! I need my path to be set!\n");
	  }
	}
      }
#endif
    }
  }
  fprintf(mob_file, "$\n");
  fclose(mob_file);
  sprintf(buf2, "%s/%d.mob", MOB_PREFIX, zone);
  /*
   * We're fubar'd if we crash between the two lines below.
   */
  remove(buf2);
  rename(fname, buf2);

  olc_remove_from_save_list(zone_table[zone_num].number, OLC_SAVE_MOB);
}

/**************************************************************************
 Menu functions 
 **************************************************************************/

void medit_display_attack(struct descriptor_data *d)
{
  extern struct attack_hit_type attack_hit_text[];

  struct mob_attack_data *new_attack;
  int i = 0;

  CREATE(new_attack, struct mob_attack_data, 1);
  new_attack = OLC_MOB(d)->mob_specials.mob_attack;
  strcpy(buf, "[Attack description]\r\n");
  if (new_attack) {
    while (new_attack) {
      sprintf(buf, "%s  &N(%d) &+c%2d&n-&+c%3d&n [&+c%s&n]\r\n",
	      buf, i,
	      new_attack->min_damage,
	      new_attack->max_damage,
	      attack_hit_text[new_attack->attack_type].singular);
      new_attack = new_attack->next;
      i++;
    }
  } else {
    sprintf(buf, "%s  None\r\n", buf);
  }
  send_to_char(buf, d->character);
}

/*
 * Display spells pre-memmed
 */
void medit_disp_spells_menu(struct descriptor_data *d)
{
  *buf = '\0';
  get_char_cols(d->character);
  send_to_char("[H[J", d->character);
  display_mob_spells(OLC_MOB(d), d->character);
  sprintf(buf2, "\r\n%sEnter spells name (0 to return) : ", nrm);
  send_to_char(buf2, d->character);
}

/*
 * Display positions. (sitting, standing, etc)
 */
void medit_disp_positions(struct descriptor_data *d)
{
  int i;

  get_char_cols(d->character);
  send_to_char("[H[J", d->character);
  for (i = 0; *position_types[i] != '\n'; i++) {
    sprintf(buf, "%s%2d%s) %s\r\n", grn, i, nrm, position_types[i]);
    send_to_char(buf, d->character);
  }
  send_to_char("Enter position number : ", d->character);
}

/*-------------------------------------------------------------------*/

/*
 * Display the gender of the mobile.
 */
void medit_disp_sex(struct descriptor_data *d)
{
  int i;

  get_char_cols(d->character);
  send_to_char("[H[J", d->character);
  for (i = 0; i < NUM_GENDERS; i++) {
    sprintf(buf, "%s%2d%s) %s\r\n", grn, i, nrm, genders[i]);
    send_to_char(buf, d->character);
  }
  send_to_char("Enter gender number : ", d->character);
}

/*-------------------------------------------------------------------*/

/*
 * Display attack types menu.
 */
void medit_disp_attack_types(struct descriptor_data *d)
{
  int i;

  get_char_cols(d->character);
  send_to_char("[H[J", d->character);
  for (i = 0; i < NUM_ATTACK_TYPES; i++) {
    sprintf(buf, "%s%2d%s) %s\r\n", grn, i, nrm, attack_hit_text[i].singular);
    send_to_char(buf, d->character);
  }
  send_to_char("Enter attack type : ", d->character);
}

/*-------------------------------------------------------------------*/


/*
 * Display class menu.
 */
void medit_disp_class(struct descriptor_data *d)
{
  int i, columns = 0;

  get_char_cols(d->character);
  send_to_char("[H[J", d->character);
  for (i = 0; i < NUM_CLASSES; i++) {
    sprintf(buf, "%s%2d%s) %-20.20s  %s", grn, i, nrm, class_types[i],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter class (-1 for None): ", d->character);
}

/*-------------------------------------------------------------------*/


/*
 * Display intelligence menu.
 */
void medit_disp_int_menu(struct descriptor_data *d)
{
  int i, columns = 0;

  get_char_cols(d->character);
  send_to_char("[H[J", d->character);
  for (i = 0; i < NUM_INT_RATINGS; i++) {
    sprintf(buf, "%s%2d%s) %-20.20s  %s", grn, i, nrm, int_ratings[i],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter rating: ", d->character);
}

/*-------------------------------------------------------------------*/
/*
 * Display alignment menu.
 */
void medit_disp_alignment(struct descriptor_data *d)
{
  int i;

  get_char_cols(d->character);
#if defined(CLEAR_SCREEN)
  send_to_char("[H[J", d->character);
#endif
  for (i = 0; i < NUM_ALIGNMENT; i++) {
    sprintf(buf, "%s%2d%s) %s\r\n", grn, i, nrm, alignment_types[i]);
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter alignment : ", d->character);
}

/*-------------------------------------------------------------------*/
/*
 * Display race menu.
 */
void medit_disp_races(struct descriptor_data *d)
{
  int i, columns = 0;

  get_char_cols(d->character);
#if defined(CLEAR_SCREEN)
  send_to_char("[H[J", d->character);
#endif
  strcpy(buf, "");
  for (i = 0; i < NUM_RACES; i++) {
    sprintf(buf, "%s%s%2d%s) %-20.20s  %s", buf, grn, i, nrm, race_types[i],
	    !(++columns % 2) ? "\r\n" : "");
  }
  strcat(buf, "\r\nEnter race : ");
  page_string(d->character->desc, buf, 1);
}


/*-------------------------------------------------------------------*/
/*
 * Display mob-flags menu.
 */
void medit_disp_mob_flags(struct descriptor_data *d)
{
  int i, columns = 0;

  get_char_cols(d->character);
#if defined(CLEAR_SCREEN)
  send_to_char("[H[J", d->character);
#endif
  for (i = 0; i < NUM_MOB_FLAGS; i++) {
    sprintf(buf, "%s%2d%s) %-20.20s  %s", grn, i + 1, nrm, action_bits[i],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit(MOB_FLAGS(OLC_MOB(d)), action_bits, buf1);
  sprintf(buf, "\r\nCurrent flags : %s%s%s\r\nEnter mob flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
}

/*-------------------------------------------------------------------*/

/*
 * Display affection flags menu.
 */
void medit_disp_aff_flags(struct descriptor_data *d)
{
  int i, columns = 0;

  get_char_cols(d->character);
#if defined(CLEAR_SCREEN)
  send_to_char("[H[J", d->character);
#endif
  for (i = 0; i < NUM_AFF_FLAGS; i++) {
    sprintf(buf, "%s%2d%s) %-20.20s  %s", grn, i + 1, nrm, affected_bits[i],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit(AFF_FLAGS(OLC_MOB(d)), affected_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter aff flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
}

/*-------------------------------------------------------------------*/

/*
 * Display affection2 flags menu.
 */
void medit_disp_aff2_flags(struct descriptor_data *d)
{
  int i, columns = 0;

  get_char_cols(d->character);
#if defined(CLEAR_SCREEN)
  send_to_char("[H[J", d->character);
#endif
  for (i = 0; i < NUM_AFF2_FLAGS; i++) {
    sprintf(buf, "%s%2d%s) %-20.20s  %s", grn, i + 1, nrm, affected2_bits[i],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit(AFF2_FLAGS(OLC_MOB(d)), affected2_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter aff flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
}

/*-------------------------------------------------------------------*/

/*
 * Display affection3 flags menu.
 */
void medit_disp_aff3_flags(struct descriptor_data *d)
{
  int i, columns = 0;

  get_char_cols(d->character);
#if defined(CLEAR_SCREEN)
  send_to_char("[H[J", d->character);
#endif
  for (i = 0; i < NUM_AFF3_FLAGS; i++) {
    sprintf(buf, "%s%2d%s) %-20.20s  %s", grn, i + 1, nrm, affected3_bits[i],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit(AFF3_FLAGS(OLC_MOB(d)), affected3_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter aff flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
}

/*-------------------------------------------------------------------*/

/*
 * Display affection4 flags menu.
 */
void medit_disp_aff4_flags(struct descriptor_data *d)
{
  int i, columns = 0;

  get_char_cols(d->character);
#if defined(CLEAR_SCREEN)
  send_to_char("[H[J", d->character);
#endif
  for (i = 0; i < NUM_AFF4_FLAGS; i++) {
    sprintf(buf, "%s%2d%s) %-20.20s  %s", grn, i + 1, nrm, affected4_bits[i],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit(AFF4_FLAGS(OLC_MOB(d)), affected4_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter aff flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
}

/*-------------------------------------------------------------------*/

/*
 * Display main menu.
 */
void medit_disp_menu(struct descriptor_data *d)
{
  struct char_data *mob;

  mob = OLC_MOB(d);
  get_char_cols(d->character);

  if (GET_CLASS(mob) == CLASS_UNDEFINED)
    strcpy(buf1, "None");
  else
    strcpy(buf1, class_types[(int) GET_CLASS(mob)]);

  sprintf(buf,
#if defined(CLEAR_SCREEN)
	  "[H[J"
#endif
	  "-- Mob Number:  [%s%d%s]\r\n"
	  "-- CopyOfVnum:  [%s%d%s]\r\n"
	  "%s1%s) Sex: %s%-7.7s%s	         %s2%s) Alias: %s%s\r\n"
	  "%s3%s) S-Desc: %s%s\r\n"
	  "%s4%s) L-Desc:-\r\n%s%s"
	  "%s5%s) D-Desc:-\r\n%s%s"
	  "%s6%s) Race : %s%s\r\n"
	  "%s7%s) Class: %s%s\r\n"
	  "%s8%s) HD: [%s%4d%s] %s9%s) Size HP Dice: [%s%4d%s] %sB%s) HP Bonus: [%s%5d%s]\r\n"
	  "%sV%s) THAC0:        [%s%3d%s]\r\n"
	  "%sX%s) Intelligence: [%s%s%s]\r\n"
	  "%sC%s) Hitroll:     [%s%4d%s]  %sE%s) Damroll:   [%s%4d%s]\r\n"
	  "%sF%s) Magic res.:   [%s%3d%s]  %sG%s) Alignment: [%s%s%s]\r\n"
	  "%sH%s) Armor Class: [%s%4d%s]  %sI%s) Exp:       [%s%9d%s]  %sJ%s) Gold: [%s%8d%s]\r\n"
	  "%sY%s) Hometown/Kingdom: [%s%s%s]\r\n",

	  cyn, OLC_NUM(d), nrm,
	  cyn, OLC_PROTO(d), nrm,
	  grn, nrm, yel, genders[(int) GET_SEX(mob)], nrm,
	  grn, nrm, nrm, GET_ALIAS(mob),
	  grn, nrm, nrm, GET_SDESC(mob),
	  grn, nrm, nrm, GET_LDESC(mob),
	  grn, nrm, nrm, GET_DDESC(mob),
	  grn, nrm, yel, race_types[(int) GET_RACE(mob)],
	  grn, nrm, yel, buf1,
	  grn, nrm, cyn, GET_LEVEL(mob), nrm,
	  grn, nrm, cyn, GET_HPDIE(mob), nrm,
	  grn, nrm, cyn, GET_HPPLUS(mob), nrm,
	  grn, nrm, cyn, GET_MOB_THAC0(mob), nrm,
	  grn, nrm, cyn, FathomMobIntelligence(mob, 0), nrm,
	  grn, nrm, cyn, GET_HITROLL(mob), nrm,
	  grn, nrm, cyn, GET_DAMROLL(mob), nrm,
	  grn, nrm, cyn, GET_MAGIC_RESISTANCE(mob), nrm,
	  grn, nrm, cyn, alignment_types[GET_ALIGNMENT(mob)], nrm,
	  grn, nrm, cyn, GET_AC(mob), nrm,
	  grn, nrm, cyn, GET_EXP(mob), nrm,
	  grn, nrm, cyn, GET_MOB_TREASURE(mob), nrm,
	  grn, nrm, nrm, kingdom_list[GET_HOMETOWN(mob)], nrm
      );
  send_to_char(buf, d->character);

  medit_display_attack(d);
  sprintf(buf, "  %sA%s) Add\r\n"
	  "  %sD%s) Delete\r\n"
	  "  %sM%s) Modify (for now delete then add)\r\n",
	  grn, nrm,
	  grn, nrm,
	  grn, nrm);
  send_to_char(buf, d->character);

  sprintbit(MOB_FLAGS(mob), action_bits, buf1);
  sprintbit(AFF_FLAGS(mob), affected_bits, buf2);
  sprintf(buf,
	  "%sK%s) Position   : %s%s\r\n"
	  "%sL%s) Default    : %s%s\r\n"
	  "%sN%s) NPC Flags  : %s%s\r\n"
	  "%sO%s) AFF Flags  : %s%s\r\n",

	  grn, nrm, yel, position_types[(int) GET_POS(mob)],
	  grn, nrm, yel, position_types[(int) GET_DEFAULT_POS(mob)],
	  grn, nrm, cyn, buf1,
	  grn, nrm, cyn, buf2
      );
  send_to_char(buf, d->character);

  sprintbit(AFF2_FLAGS(mob), affected2_bits, buf1);
  sprintbit(AFF3_FLAGS(mob), affected3_bits, buf2);
  sprintf(buf,
	  "%sP%s) AFF2 Flags : %s%s\r\n"
	  "%sR%s) AFF3 Flags : %s%s\r\n",

	  grn, nrm, cyn, buf1,
	  grn, nrm, cyn, buf2
      );
  send_to_char(buf, d->character);


  sprintbit(AFF4_FLAGS(mob), affected4_bits, buf1);
  sprintf(buf,
	  "%sS%s) AFF4 Flags : %s%s\r\n"
	  "%sT%s) Path Editor: %s%s\r\n"
	  "%sU%s) Pre-Memmed Spells: %s%s\r\n"
	  "%sW%s) AI Script  : %s%s\r\n"
	  "%s+%s) Edit Next         %s-%s) Edit Previous\r\n"
	  "%sQ%s) Quit\r\n"
	  "Enter choice : ",

	  grn, nrm, cyn, buf1,
	  grn, nrm, cyn, (OLC_PATH(d) ? "&+gSet&n" : "&+RNone&n"),
	  grn, nrm, cyn, (mob->char_specials.saved.spells[0] >= MEMINFO_READY) ? "Set." : "Not Set.",
	  grn, nrm, cyn, mob->proto_script ? "Set." : "Not Set.",
	  grn, nrm, grn, nrm,
	  grn, nrm
      );
  send_to_char(buf, d->character);


  OLC_MODE(d) = MEDIT_MAIN_MENU;
}

/************************************************************************
 *			The GARGANTAUN event handler			*
 ************************************************************************/

void medit_parse(struct descriptor_data *d, char *arg)
{
  int i = 0, number, real_num;
  int pos;

  if (OLC_MODE(d) > MEDIT_NUMERICAL_RESPONSE) {
    if (!*arg || (!isdigit(arg[0]) && ((*arg == '-') && (!isdigit(arg[1]))))) {
      send_to_char("Field must be numerical, try again : ", d->character);
      return;
    }
  }
  switch (OLC_MODE(d)) {
  case MEDIT_CONFIRM_DELETE:
    switch (*arg) {
    case 'y':
    case 'Y':
      medit_delete(d);
      sprintf(buf, "OLC: %s deleted mob %d", GET_NAME(d->character), OLC_NUM(d));
      mudlog(buf, CMP, MAX(LVL_BUILDER, GET_INVIS_LEV(d->character)), TRUE);
      cleanup_olc(d, CLEANUP_STRUCTS);
      return;
    default:
      cleanup_olc(d, CLEANUP_ALL);
      return;
    }
    break;
/*-------------------------------------------------------------------*/
  case MEDIT_CONFIRM_SAVESTRING:
    /*
     * Ensure mob has MOB_ISNPC set or things will go pair shaped.
     */
    SET_BIT(MOB_FLAGS(OLC_MOB(d)), MOB_ISNPC);
    switch (*arg) {
    case 'y':
    case 'Y':
      /*
       * Save the mob in memory and to disk.
       */
      send_to_char("Saving mobile to memory.\r\n", d->character);
      medit_save_internally(d);
      sprintf(buf, "OLC: %s edits mob %d", GET_NAME(d->character), OLC_NUM(d));
      mudlog(buf, CMP, MAX(LVL_BUILDER, GET_INVIS_LEV(d->character)), TRUE);
      /* FALL THROUGH */
    case 'n':
    case 'N':
      cleanup_olc(d, CLEANUP_ALL);
      return;
    default:
      send_to_char("Invalid choice!\r\n", d->character);
      send_to_char("Do you wish to save the mobile? : ", d->character);
      return;
    }
    break;

/*-------------------------------------------------------------------*/
  case MEDIT_MAIN_MENU:
    i = 0;
    switch (*arg) {
    case 'q':
    case 'Q':
      if (OLC_VAL(d)) {		/* Anything been changed? */
	send_to_char("Do you wish to save the changes to the mobile? (y/n) : ", d->character);
	OLC_MODE(d) = MEDIT_CONFIRM_SAVESTRING;
      } else
	cleanup_olc(d, CLEANUP_ALL);
      return;
    case '+':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      number = OLC_NUM(d) + 1;
      medit_save_internally(d);
      cleanup_olc(d, CLEANUP_ALL);
      OLC_NUM(d) = number;
      if ((real_num = real_mobile(number)) >= 0)
	medit_setup_existing(d, real_num);
      else
	medit_setup_new(d);
      STATE(d) = CON_MEDIT;
      break;
    case '-':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      number = OLC_NUM(d) - 1;
      medit_save_internally(d);
      cleanup_olc(d, CLEANUP_ALL);
      OLC_NUM(d) = number;
      if ((real_num = real_mobile(number)) >= 0)
	medit_setup_existing(d, real_num);
      else
	medit_setup_new(d);
      STATE(d) = CON_MEDIT;
      break;
    case '1':
      OLC_MODE(d) = MEDIT_SEX;
      medit_disp_sex(d);
      return;
    case '2':
      OLC_MODE(d) = MEDIT_ALIAS;
      i--;
      break;
    case '3':
      OLC_MODE(d) = MEDIT_S_DESC;
      i--;
      break;
    case '4':
      OLC_MODE(d) = MEDIT_L_DESC;
      i--;
      break;
    case '5':
      OLC_MODE(d) = MEDIT_D_DESC;
      SEND_TO_Q("Enter mob description: (/s saves /h for help)\r\n\r\n", d);
      d->backstr = NULL;
      if (OLC_MOB(d)->player.description) {
	SEND_TO_Q(OLC_MOB(d)->player.description, d);
	d->backstr = str_dup(OLC_MOB(d)->player.description);
      }
#ifndef MUDFTP
      d->str = &OLC_MOB(d)->player.description;
      d->max_str = MAX_MOB_DESC;
      d->mail_to = 0;
#else
      string_write(d, &OLC_MOB(d)->player.description,
		   MAX_MOB_DESC, 0, NULL);
#endif
      OLC_VAL(d) = 1;
      return;
    case '6':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_RACE;
      medit_disp_races(d);
      return;
      break;
    case '7':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_CLASS;
      medit_disp_class(d);
      return;
      break;
    case '8':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_LEVEL;
      i++;
      break;
    case '9':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_SIZE_HP_DICE;
      i++;
      break;
    case 'a':
    case 'A':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_ATTACK_MIN;
      medit_add_command(d);
      return;
      break;
    case 'b':
    case 'B':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_ADD_HP;
      i++;
      break;
    case 'c':
    case 'C':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_HITROLL;
      i++;
      break;
    case 'd':
    case 'D':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_DELETE_ATTACK;
      send_to_char("\r\nWhich attack do you want to delete?\r\n", d->character);
      return;
      break;
    case 'e':
    case 'E':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_DAMROLL;
      i++;
      break;
    case 'f':
    case 'F':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_MAGIC_RESISTANCE;
      i++;
      break;
    case 'g':
    case 'G':
      OLC_MODE(d) = MEDIT_ALIGNMENT;
      medit_disp_alignment(d);
      return;
      break;
    case 'h':
    case 'H':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_AC;
      i++;
      break;
    case 'i':
    case 'I':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_EXP;
      i++;
      break;
    case 'j':
    case 'J':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_GOLD;
      i++;
      break;
    case 'k':
    case 'K':
      OLC_MODE(d) = MEDIT_POS;
      medit_disp_positions(d);
      return;
    case 'l':
    case 'L':
      OLC_MODE(d) = MEDIT_DEFAULT_POS;
      medit_disp_positions(d);
      return;
    case 'm':
    case 'M':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_CHANGE_ATTACK;
      send_to_char("\r\nWhich attack do you want to change?\r\n", d->character);
      break;
    case 'n':
    case 'N':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_NPC_FLAGS;
      medit_disp_mob_flags(d);
      return;
    case 'o':
    case 'O':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_AFF_FLAGS;
      medit_disp_aff_flags(d);
      return;
    case 'p':
    case 'P':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_AFF2_FLAGS;
      medit_disp_aff2_flags(d);
      return;
    case 'r':
    case 'R':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_AFF3_FLAGS;
      medit_disp_aff3_flags(d);
      return;
    case 's':
    case 'S':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_AFF4_FLAGS;
      medit_disp_aff4_flags(d);
      return;
    case 't':
    case 'T':
      OLC_MODE(d) = MEDIT_PATH;
      OLC_SUBMODE(d) = MEDIT_SUB_PATH;
      paedit_disp_path_editor(d);
      return;
    case 'u':
    case 'U':
      OLC_MODE(d) = MEDIT_ADD_SPELL;
      medit_disp_spells_menu(d);
      return;
    case 'v':
    case 'V':
      if (GET_LEVEL(d->character) < LVL_GOD6) {
	medit_disp_menu(d);
	return;
      }
      OLC_MODE(d) = MEDIT_THAC0;
      i++;
      break;
    case 'w':
    case 'W':
      OLC_SCRIPT_EDIT_MODE(d) = SCRIPT_MAIN_MENU;
      dg_script_menu(d);
      return;
    case 'x':
    case 'X':
      OLC_MODE(d) = MEDIT_INTELLIGENCE;
/*      medit_disp_int_menu(d); */
      send_to_char("\r\nEnter the numerical value of intelligence (0, 25) : ", d->character);
      return;
    case 'y':
    case 'Y':
      OLC_MODE(d) = MEDIT_HOMETOWN;
      disp_kingdom_menu(d);
      return;
    default:
      medit_disp_menu(d);
      return;
    }
    if (i != 0) {
      send_to_char(i == 1 ? "\r\nEnter new value : " :
		   i == -1 ? "\r\nEnter new text :\r\n] " :
		   "\r\nOops...:\r\n", d->character);
      return;
    }
    break;

/*-------------------------------------------------------------------*/
  case MEDIT_ATTACK_MIN:
    pos = atoi(arg);
    if (isdigit(*arg)) {
      medit_add_min(d, pos);
    }
    return;
    break;

  case MEDIT_ATTACK_MAX:
    pos = atoi(arg);
    if (isdigit(*arg)) {
      medit_add_max(d, pos);
    }
    return;
    break;

  case MEDIT_ATTACK_TYPE:
    pos = atoi(arg);
    if (isdigit(*arg)) {
      medit_add_type(d, pos);
    }
    return;
    break;

/*-------------------------------------------------------------------*/
  case MEDIT_CHANGE_ATTACK:
    break;

/*-------------------------------------------------------------------*/
  case MEDIT_DELETE_ATTACK:
    pos = atoi(arg);
    if (isdigit(*arg)) {
      medit_delete_command(d, pos);
    }
    break;

/*-------------------------------------------------------------------*/
  case OLC_SCRIPT_EDIT:
    if (dg_script_edit_parse(d, arg))
      return;
    break;
/*-------------------------------------------------------------------*/
  case MEDIT_ALIAS:
    if (GET_ALIAS(OLC_MOB(d)))
      FREE(GET_ALIAS(OLC_MOB(d)));
    GET_ALIAS(OLC_MOB(d)) = str_dup((arg && *arg) ? arg : "undefined");
    break;
/*-------------------------------------------------------------------*/
  case MEDIT_S_DESC:
    if (GET_SDESC(OLC_MOB(d)))
      FREE(GET_SDESC(OLC_MOB(d)));
    GET_SDESC(OLC_MOB(d)) = str_dup((arg && *arg) ? arg : "undefined");
    break;
/*-------------------------------------------------------------------*/
  case MEDIT_L_DESC:
    if (GET_LDESC(OLC_MOB(d)))
      FREE(GET_LDESC(OLC_MOB(d)));
    if (arg && *arg) {
      strcpy(buf, arg);
      strcat(buf, "\r\n");
      GET_LDESC(OLC_MOB(d)) = str_dup(buf);
    } else
      GET_LDESC(OLC_MOB(d)) = str_dup("undefined");

    break;
/*-------------------------------------------------------------------*/
  case MEDIT_D_DESC:
    /*
     * We should never get here.
     */
    cleanup_olc(d, CLEANUP_ALL);
    mudlog("SYSERR: OLC: medit_parse(): Reached D_DESC case!",
	   BRF, LVL_BUILDER, TRUE);
    send_to_char("Oops...\r\n", d->character);
    break;
/*-------------------------------------------------------------------*/

  case MEDIT_NPC_FLAGS:
    if ((i = atoi(arg)) == 0)
      break;
    else if (!((i < 0) || (i > NUM_MOB_FLAGS)))
      TOGGLE_BIT(MOB_FLAGS(OLC_MOB(d)), 1 << (i - 1));
    medit_disp_mob_flags(d);
    return;
/*-------------------------------------------------------------------*/
  case MEDIT_AFF_FLAGS:
    if ((i = atoi(arg)) == 0)
      break;
    else if (!((i < 0) || (i > NUM_AFF_FLAGS)))
      TOGGLE_BIT(AFF_FLAGS(OLC_MOB(d)), 1 << (i - 1));
    medit_disp_aff_flags(d);
    return;
/*-------------------------------------------------------------------*/
  case MEDIT_AFF2_FLAGS:
    if ((i = atoi(arg)) == 0)
      break;
    else if (!((i < 0) || (i > NUM_AFF2_FLAGS)))
      TOGGLE_BIT(AFF2_FLAGS(OLC_MOB(d)), 1 << (i - 1));
    medit_disp_aff2_flags(d);
    return;
/*-------------------------------------------------------------------*/
  case MEDIT_AFF3_FLAGS:
    if ((i = atoi(arg)) == 0)
      break;
    else if (!((i < 0) || (i > NUM_AFF3_FLAGS)))
      TOGGLE_BIT(AFF3_FLAGS(OLC_MOB(d)), 1 << (i - 1));
    medit_disp_aff3_flags(d);
    return;
/*-------------------------------------------------------------------*/
  case MEDIT_AFF4_FLAGS:
    if ((i = atoi(arg)) == 0)
      break;
    else if (!((i < 0) || (i > NUM_AFF4_FLAGS)))
      TOGGLE_BIT(AFF4_FLAGS(OLC_MOB(d)), 1 << (i - 1));
    medit_disp_aff4_flags(d);
    return;
/*-------------------------------------------------------------------*/
  case MEDIT_PATH:
    path_subfunctions(d, arg);
    if (OLC_SUBMODE(d) == MEDIT_SUB_NONE)
      break;
    return;

  case MEDIT_SEX:
    GET_SEX(OLC_MOB(d)) = MAX(0, MIN(NUM_GENDERS - 1, atoi(arg)));
    break;

  case MEDIT_RACE:
    GET_RACE(OLC_MOB(d)) = MAX(0, MIN(NUM_RACES - 1, atoi(arg)));
    break;

  case MEDIT_THAC0:
    GET_MOB_THAC0(OLC_MOB(d)) = MAX(-10, MIN(10, atoi(arg)));
    break;

  case MEDIT_INTELLIGENCE:
    GET_INT(OLC_MOB(d)) = atoi(FathomMobIntelligence(NULL, atoi(arg)));
    break;

  case MEDIT_HOMETOWN:
    GET_HOMETOWN(OLC_MOB(d)) = BOUNDED(0, atoi(arg), NUM_KINGDOMS);
    break;

  case MEDIT_ADD_SPELL:
    if (isdigit(*arg))
      number = atoi(arg);
    else
      number = find_spells_num(arg);
    if (number < NUM_SPELLS && number > 0) {
      GET_SPELL(OLC_MOB(d), number) = MEMINFO_READY;
      medit_disp_spells_menu(d);
      return;
    }
    break;
  case MEDIT_CLASS:
    GET_CLASS(OLC_MOB(d)) = MAX(-1, MIN(NUM_CLASSES - 1, atoi(arg)));
    break;

  case MEDIT_HITROLL:
    GET_HITROLL(OLC_MOB(d)) = MAX(0, MIN(20, atoi(arg)));
    break;

  case MEDIT_DAMROLL:
    GET_DAMROLL(OLC_MOB(d)) = MAX(0, MIN(20, atoi(arg)));
    break;

  case MEDIT_SIZE_HP_DICE:
    GET_HPDIE(OLC_MOB(d)) = MAX(0, MIN(10, atoi(arg)));
    break;

  case MEDIT_MAGIC_RESISTANCE:
    GET_MAGIC_RESISTANCE(OLC_MOB(d)) = MAX(0, MIN(100, atoi(arg)));
    break;

  case MEDIT_ADD_HP:
    GET_HPPLUS(OLC_MOB(d)) = MAX(0, MIN(100, atoi(arg)));
    break;

  case MEDIT_AC:
    GET_AC(OLC_MOB(d)) = MAX(-10, MIN(10, atoi(arg)));
    break;

  case MEDIT_EXP:
    GET_EXP(OLC_MOB(d)) = MAX(0, atoi(arg));
    break;

  case MEDIT_GOLD:
    GET_MOB_TREASURE(OLC_MOB(d)) = MAX(0, atoi(arg));
    break;

  case MEDIT_POS:
    SET_POS(OLC_MOB(d), MAX(0, MIN(NUM_POSITIONS - 1, atoi(arg))) + STAT_NORMAL);
    break;

  case MEDIT_DEFAULT_POS:
    GET_DEFAULT_POS(OLC_MOB(d)) = MAX(0, MIN(NUM_POSITIONS - 1, atoi(arg)));
    break;

  case MEDIT_LEVEL:
    GET_LEVEL(OLC_MOB(d)) = MAX(1, MIN(34, atoi(arg)));
    break;

  case MEDIT_ALIGNMENT:
    SET_ALIGNMENT(OLC_MOB(d), MAX(0, MIN(NUM_ALIGNMENT, atoi(arg))));
    break;

/*-------------------------------------------------------------------*/
  default:
    /*
     * We should never get here.
     */
    cleanup_olc(d, CLEANUP_ALL);
    mudlog("SYSERR: OLC: medit_parse(): Reached default case!", BRF, LVL_BUILDER, TRUE);
    send_to_char("Oops...\r\n", d->character);
    break;
  }
/*-------------------------------------------------------------------*/

/*
 * END OF CASE 
 * If we get here, we have probably changed something, and now want to
 * return to main menu.  Use OLC_VAL as a 'has changed' flag  
 */

  OLC_VAL(d) = 1;
  medit_disp_menu(d);
}
/*
 * End of medit_parse(), thank god.
 */

void paedit_disp_path_editor(struct descriptor_data *d)
{
  struct path_info *paedit = OLC_PATH(d);

#if defined(CLEAR_SCREEN)
  send_to_char("[H[J", d->character);
#endif

  OLC_PTOTAL(d) = 1;

  sprintf(buf, "\r\nPaths\r\n%s-----%s\r\n", cyn, nrm);
  send_to_char(buf, d->character);
  if (paedit) {
    while (paedit) {
      switch (paedit->type) {

      case PATH_NONE:
	sprintf(buf, "%s%d%s) %sPath not set.%s\r\n", grn,
		OLC_PTOTAL(d), nrm, yel, nrm);
	send_to_char(buf, d->character);
	paedit = paedit->next;
	OLC_PTOTAL(d)++;
	break;
      case PATH_GOTO:

	sprintf(buf, "%s%d%s) %sGo to room %d at %s%s.\r\n",
		grn, OLC_PTOTAL(d), nrm, cyn, paedit->target_room,
		paedit_parse_performance(paedit->performance), nrm);

	send_to_char(buf, d->character);
	OLC_PTOTAL(d)++;
	paedit = paedit->next;
	break;

      case PATH_ACTION:
	sprintf(buf, "%s%d%s) %sPerform '%s' at%s %s\r\n",
		grn, OLC_PTOTAL(d), nrm, cyn, paedit->arguments, nrm,
		paedit_parse_performance(paedit->performance));
	send_to_char(buf, d->character);
	OLC_PTOTAL(d)++;
	paedit = paedit->next;
	break;
      default:
	send_to_char("Whoops. Bad case reached in paedit_disp_path_Editor\r\n", d->character);
	paedit = paedit->next;
	OLC_PTOTAL(d)++;
	break;
      }
    }
  } else {
    sprintf(buf, "%sNo Paths exist%s.\r\n", yel, nrm);
    send_to_char(buf, d->character);
  }
  sprintf(buf, "%s-----%s\r\n"
	  "%sA%s) Create new path\r\n"
	  "%sB%s) Purge existing path. \r\n"
	  "Enter command or # to edit [0 to exit]:  ",
	  cyn, nrm,
	  grn, nrm,
	  grn, nrm
      );
  send_to_char(buf, d->character);
}

void paedit_change_path(struct descriptor_data *d)
{

#if defined(CLEAR_SCREEN)
  send_to_char("[H[J", d->character);
#endif

  sprintf(buf, "\r\nPath\r\n%s-----%s\r\n", cyn, nrm);
  send_to_char(buf, d->character);
  sprintf(buf, "%s1%s) Occurance: %s\r\n"
	  "%s2%s) Type: %s\r\n"
	  "%s3%s) Action: %s\r\n"
	  "%s0%s) Stop editing this path.\r\n"
	  "Enter choice : ",

	  grn, nrm, paedit_parse_performance(OLC_PATH(d)->performance),
	  grn, nrm, paedit_parse_type(OLC_PATH(d)),
	  grn, nrm, paedit_parse_action(OLC_PATH(d)),
	  grn, nrm);
  send_to_char(buf, d->character);

}

char *paedit_parse_type(struct path_info *path)
{
  char *send = NULL;
  char intmp[MAX_STRING_LENGTH];

  if (!path) {
    mudlog("SYSERR: OLC: paedit_parse_type(): No path sent!",
	   BRF, LVL_BUILDER, TRUE);
    sprintf(intmp, "\r\nERROR. Please discontinue use and report.\r\n");
    CREATE(send, char, strlen(intmp) + 1);
    strcpy(send, intmp);
    return send;
  }
  if (path->type) {
    switch (path->type) {
    case PATH_GOTO:
      sprintf(intmp, "%sPATH_GOTO%s", cyn, nrm);
      break;
    default:
      sprintf(intmp, "%sPATH_ACTION%s", cyn, nrm);
    }
  } else
    sprintf(intmp, "%sNo type set.%s", yel, nrm);
  CREATE(send, char, strlen(intmp) + 1);
  strcpy(send, intmp);
  return send;
}

char *paedit_parse_action(struct path_info *path)
{
  char mytmp[MAX_STRING_LENGTH];
  char *send = NULL;

  if (!path) {
    mudlog("SYSERR: OLC: paedit_parse_action(): No path sent!",
	   BRF, LVL_BUILDER, TRUE);
    sprintf(mytmp, "\r\nERROR. Please discontinue use and report.\r\n");
    CREATE(send, char, strlen(mytmp) + 1);
    strcpy(send, mytmp);
    return send;
  }
  if (path->type) {
    switch (path->type) {
    case PATH_GOTO:
      if (path->target_room && path->target_room > 0) {
	sprintf(mytmp, "%sGo to room %d%s", cyn, path->target_room, nrm);
      } else
	sprintf(mytmp, "%sRoom not set.%s", yel, nrm);
      break;
    case PATH_ACTION:
      if (path->arguments)
	sprintf(mytmp, "%s%s%s", cyn, path->arguments, nrm);
      else
	sprintf(mytmp, "%s(Not Set)%s", yel, nrm);
      break;
    default:
      sprintf(mytmp, "%sNone - set 'type' first.%s", yel, nrm);
      break;
    }
  } else {
    sprintf(mytmp, "%sNone - choose a type first.%s", yel, nrm);
  }
  CREATE(send, char, strlen(mytmp) + 1);
  strcpy(send, mytmp);
  return send;
}

char *paedit_hrmin_disp(int hrmin)
{
  char *send = NULL;
  char inbuf[MAX_STRING_LENGTH];
  int min, hour;

  min = (hrmin % 60);
  hour = (((hrmin / 60)) % 12);
  if (hour == 0)
    hour = 12;

  if (min != 0) {
    sprintf(inbuf, "%d:%s%d (%s)", hour, (min < 9) ? "0" : "", min,
	    (hrmin > 720) ? "P.M." : "A.M.");
  } else {
    sprintf(inbuf, "%d O'Clock (%s)", hour, (hrmin > 720) ? "P.M." :
	    "A.M.");
  }

  CREATE(send, char, strlen(inbuf) + 1);
  strcpy(send, inbuf);
  return send;
}

char *paedit_parse_performance(struct path_perform *perform)
{
  char mytmp[MAX_STRING_LENGTH];
  char *dsuf, *wsuf, *msuf, *ysuf, *send = NULL;
  int i;

  *buf = '\0';

  /* assumption - if we have a minute, we'll have an hour too? */
  /* do we really want to do this? */

  if (perform->hrmin == -1) {
    sprintf(mytmp, "%sAs soon as possible%s", yel, nrm);
    CREATE(send, char, strlen(mytmp) + 1);
    strcpy(send, mytmp);
    return send;
  }
  i = ((perform->hrmin) % 60);
  if (i > 20)
    i = i % 20;

  if (i == 1)
    dsuf = "st";
  else if (i == 2)
    dsuf = "nd";
  else if (i == 3)
    dsuf = "rd";
  else
    dsuf = "th";

  if (perform->day == -1) {
    sprintf(mytmp, "%s%s each day.%s", cyn,
	    paedit_hrmin_disp(perform->hrmin), nrm);
    CREATE(send, char, strlen(mytmp) + 1);
    strcpy(send, mytmp);
    return send;
  }
/* i know, it looks odd - but if it occurs once a week, it does so on 1
   day - so, week usage is defined by a single day being set, and a specific 
   week well, not being set */

  i = perform->day;

  if (i == 1)
    wsuf = "st";
  else if (i == 2)
    wsuf = "nd";
  else if (i == 3)
    wsuf = "rd";
  else
    wsuf = "th";

  if (perform->week == -1) {
    sprintf(mytmp, "%s%s every %d%s day of the week%s",
	    cyn, paedit_hrmin_disp(perform->hrmin),
	    perform->day, wsuf, nrm);
    CREATE(send, char, strlen(mytmp) + 1);
    strcpy(send, mytmp);
    return send;
  }
  i = perform->week;

  if (i == 1)
    msuf = "st";
  else if (i == 2)
    msuf = "nd";
  else if (i == 3)
    msuf = "rd";
  else
    msuf = "th";

  if (perform->month == -1) {
    sprintf(mytmp,
	    "%s%s on the %d%s day of the %d%s week each month.%s",
	    cyn, paedit_hrmin_disp(perform->hrmin),
	    perform->day, wsuf, perform->week, msuf, nrm);
    CREATE(send, char, strlen(mytmp) + 1);
    strcpy(send, mytmp);
    return send;
  }
  i = perform->month;

  if (i == 1)
    ysuf = "st";
  else if (i == 2)
    ysuf = "nd";
  else if (i == 3)
    ysuf = "rd";
  else
    ysuf = "th";

  sprintf(mytmp,
	  "%s%s on the %d%s day of the %d%s week every %d%s month%s",
	  cyn, paedit_hrmin_disp(perform->hrmin),
     perform->day, wsuf, perform->week, msuf, perform->month, ysuf, nrm);
  CREATE(send, char, strlen(mytmp) + 1);
  strcpy(send, mytmp);
  return send;
}


void paedit_list_months(struct descriptor_data *d)
{
  int i;
  extern char *month_name[];
  char month[MAX_STRING_LENGTH];

  sprintf(month, "Months\r\n%s------%s\r\n", cyn, nrm);
  for (i = 1; *month_name[i - 1] != '\n'; i++) {
    sprintf(month, "%s%s%d%s) %s\r\n", month,
	    grn, i, nrm, month_name[i - 1]);
  }
  sprintf(month, "%sChoose a month : ", month);
  send_to_char(month, d->character);
}


void paedit_list_days(struct descriptor_data *d)
{
  int i;
  extern char *weekdays[];
  char days[MAX_STRING_LENGTH];

  sprintf(days, "Days\r\n%s----%s\r\n", cyn, nrm);
  for (i = 1; *weekdays[i - 1] != '\n'; i++) {
    sprintf(days, "%s%s%d%s) %s\r\n", days,
	    grn, i, nrm, weekdays[i - 1]);
  }
  sprintf(days, "%sChoose a day : ", days);
  send_to_char(days, d->character);

}

void paedit_time_edit_disp(struct descriptor_data *d)
{
  sprintf(buf, "\r\nCurrent Path: %s\r\n\r\n"
	  "%s1%s) Hour/Minute Select\r\n"
	  "%s2%s) Day Select\r\n"
	  "%s3%s) Week Select\r\n"
	  "%s4%s) Month Select\r\n"
	  "%s0%s) Return to previous menu.\r\n"
	  "Enter choice : ",
	  paedit_parse_performance(OLC_PATH(d)->performance),
	  grn, nrm,
	  grn, nrm,
	  grn, nrm,
	  grn, nrm,
	  grn, nrm);
  send_to_char(buf, d->character);
}

void path_subfunctions(struct descriptor_data *d, char *arg)
{
  int i = 0, j = 0;
  char *tmp;			/* god. I hate temporary variables */

/* rules for subfunction */
/* 1) all submodes go in the below switch statement.            */
/* 2) If you change modes, you must                             */
/*       a) output the query prompt for the new mode            */
/*       b) use 'break' to leave the current switch             */
/* 3) to exit subfunction set OLC_MODE(d)=MEDIT_MAIN_MENU       */
/*    and OLC_SUBMODE(d) = MEDIT_SUB_NONE;                      */
/* 4) Do _not_ make any calls to any non-path subfunctions, or  */
/*    add functions which make calls to any of the other medit  */
/*    functions - doing so induces loops.                       */

  switch (OLC_SUBMODE(d)) {
    /* this is the initial screen */
  case MEDIT_SUB_PATH:
    if (LOWER(*arg) != 'a' && LOWER(*arg) != 'b' &&
	((i = atoi(arg)) >= OLC_PTOTAL(d) || i < 0)) {
      send_to_char("That is not an option.\r\n", d->character);
      paedit_disp_path_editor(d);
    } else if (LOWER(*arg) == 'a') {
      struct path_info *temp;

      CREATE(temp, struct path_info, 1);
      temp->next = OLC_PATH_LIST(d);
      temp->target_room = -1;
      temp->arguments = NULL;
      temp->type = 0;
      CREATE(temp->performance, struct path_perform, 1);
      temp->performance->hrmin = -1;
      temp->performance->day = -1;
      temp->performance->week = -1;
      temp->performance->month = -1;

      OLC_PATH(d) = temp;
      OLC_PATH_LIST(d) = temp;
      paedit_change_path(d);
      OLC_SUBMODE(d) = MEDIT_SUB_EDIT_ONE_PATH;
    } else if (LOWER(*arg) == 'b') {
      send_to_char("Which path do you want to purge? ", d->character);
      OLC_SUBMODE(d) = MEDIT_SUB_PURGE_SELECTION;
    } else if (i < OLC_PTOTAL(d) && i != 0) {
      struct path_info *temp;
      int x = 1;
      for (temp = OLC_PATH_LIST(d); temp && x < i; temp = temp->next)
	x++;
      OLC_PATH(d) = temp;
      paedit_change_path(d);

      OLC_SUBMODE(d) = MEDIT_SUB_EDIT_ONE_PATH;
    } else if (i == 0) {
      OLC_MODE(d) = MEDIT_MAIN_MENU;
      OLC_SUBMODE(d) = MEDIT_SUB_NONE;
    } else {
      /* no, i have no idea how you could get here. */
      OLC_MODE(d) = MEDIT_MAIN_MENU;
      OLC_SUBMODE(d) = MEDIT_SUB_NONE;
    }
    break;

  case MEDIT_SUB_EDIT_PATH:
    paedit_change_path(d);
    OLC_SUBMODE(d) = MEDIT_SUB_EDIT_ONE_PATH;
    break;

  case MEDIT_SUB_EDIT_ONE_PATH:
    /* this is simply showing the path info */
    if ((i = atoi(arg)) == 0) {
      paedit_disp_path_editor(d);
      OLC_SUBMODE(d) = MEDIT_SUB_PATH;
    } else if (i == 1) {
      OLC_SUBMODE(d) = MEDIT_SUB_TIME_EDIT;
      paedit_time_edit_disp(d);
    } else if (i == 2) {
      sprintf(buf, "\r\n%s1%s) Goto\r\n"
	      "%s2%s) Action\r\n"
	      "Enter path type : ", grn, nrm, grn, nrm);
      send_to_char(buf, d->character);
      OLC_SUBMODE(d) = MEDIT_SUB_TYPE;
    } else if (i == 3) {
      if (OLC_PATH(d)->type == PATH_NONE) {
	send_to_char("You must select a type first.\r\n", d->character);
	paedit_change_path(d);
	OLC_SUBMODE(d) = MEDIT_SUB_EDIT_ONE_PATH;
      } else if (OLC_PATH(d)->type == PATH_GOTO) {
	send_to_char("Enter room to travel to: ", d->character);
	OLC_SUBMODE(d) = MEDIT_SUB_COMMAND;
      } else {			/* this is path_action */
	send_to_char("Enter Path Commands :\r\n", d->character);

	/* here we asked ourselves, can we just have one submode for input for
	   commands, even if it could be a different type? Sure, why not */
	OLC_SUBMODE(d) = MEDIT_SUB_COMMAND;

      }
    } else {
      /* this is any other gobblety-gook */
      send_to_char("That is not an option.\r\n", d->character);
      paedit_change_path(d);
    }

    break;

  case MEDIT_SUB_EDIT_SELECTION:
    break;

  case MEDIT_SUB_TYPE:
    if ((i = atoi(arg)) != 1 && i != 2) {
      send_to_char("Type must be either 1 or 2.\r\n", d->character);
      sprintf(buf, "%s1%s) Goto\r\n"
	      "%s2%s) Action\r\n"
	      "Enter path type : ", grn, nrm, grn, nrm);
      send_to_char(buf, d->character);
    } else {
      if (i == 1)
	OLC_PATH(d)->type = PATH_GOTO;

      else
	OLC_PATH(d)->type = PATH_ACTION;


      OLC_SUBMODE(d) = MEDIT_SUB_EDIT_ONE_PATH;
      paedit_change_path(d);
    }
    break;

  case MEDIT_SUB_COMMAND:
    switch (OLC_PATH(d)->type) {
    case PATH_NONE:
      /* we should never get here. Ever. Oh well. */
      send_to_char("Error. Aborting.\r\n", d->character);
      OLC_MODE(d) = MEDIT_MAIN_MENU;
      OLC_SUBMODE(d) = MEDIT_SUB_NONE;
      break;
    case PATH_ACTION:
      if (*arg)
	OLC_PATH(d)->arguments = str_dup(arg);
      else
	OLC_PATH(d)->arguments = NULL;

      break;

    case PATH_GOTO:
      OLC_PATH(d)->target_room = atoi(arg);
      break;
    }
    paedit_change_path(d);
    OLC_SUBMODE(d) = MEDIT_SUB_EDIT_ONE_PATH;
    break;

  case MEDIT_SUB_TIME_EDIT:
    /* standard non numerical response.. */
    if ((i = atoi(arg)) < 0 || i > 4) {
      send_to_char("That is not an option.\r\n", d->character);
      paedit_time_edit_disp(d);
    } else if (i == 0) {	/* this is breakout/quit */
      paedit_change_path(d);
      OLC_SUBMODE(d) = MEDIT_SUB_EDIT_ONE_PATH;
    } else if (i == 1) {
      send_to_char("Enter the hour and minute as hh:mm: ", d->character);
      OLC_SUBMODE(d) = MEDIT_SUB_HRMIN_SELECT;
      /* deal with hour:minute selection */
    } else if (i == 2) {
      /* deal with day selection */
      paedit_list_days(d);
      OLC_SUBMODE(d) = MEDIT_SUB_DAY_SELECT;
    } else if (i == 3) {
      send_to_char("There are 5 weeks in each month - which one to select? ",
		   d->character);
      OLC_SUBMODE(d) = MEDIT_SUB_WEEK_SELECT;
      /* deal with week selection */
    } else {
      /* deal with month selection */
      paedit_list_months(d);
      OLC_SUBMODE(d) = MEDIT_SUB_MONTH_SELECT;
    }
    break;

  case MEDIT_SUB_DAY_SELECT:
    if ((i = atoi(arg)) < 1 || i > 7) {
      send_to_char("You must choose between 1 and 7.\r\n", d->character);
      paedit_list_days(d);
    } else {
      OLC_PATH(d)->performance->day = i;
      paedit_time_edit_disp(d);
      OLC_SUBMODE(d) = MEDIT_SUB_TIME_EDIT;
    }
    break;

  case MEDIT_SUB_MONTH_SELECT:
    if ((i = atoi(arg)) < 1 || i > 17) {
      send_to_char("You must choose between 1 and 17.\r\n", d->character);
      paedit_list_months(d);
    } else {
      OLC_PATH(d)->performance->month = i;
      paedit_time_edit_disp(d);
      OLC_SUBMODE(d) = MEDIT_SUB_TIME_EDIT;
    }
    break;

  case MEDIT_SUB_WEEK_SELECT:
    if ((i = atoi(arg)) > 5 || i < 1) {
      send_to_char("You must choose a week between 1 and 5.\r\n",
		   d->character);
    } else {
      OLC_PATH(d)->performance->week = i;
      paedit_time_edit_disp(d);
      OLC_SUBMODE(d) = MEDIT_SUB_TIME_EDIT;
    }
    break;

  case MEDIT_SUB_HRMIN_SELECT:
    if (!strstr(arg, ":")) {
      send_to_char("Improper syntax. The time must be in format hh:mm.\r\n", d->character);
      send_to_char("Enter the hour and minute as hh:mm: ", d->character);
    } else {
      /* do some string manipulation to tell if we're doing okay. */
      tmp = (strtok(arg, ":"));
      if (!tmp)
	i = 13;			/* sure'n its easier than another flag */
      else
	i = atoi(tmp);

      tmp = strtok(NULL, "\n");
      if (!tmp) {
	j = 61;
      } else
	j = atoi(tmp);

      if ((i < 1 || i > 12) || (j < 0 || j > 59)) {
	send_to_char("Hours must be from 1 to 12, and minutes from 0-59\r\n",
		     d->character);
	send_to_char("Enter the hour and minute as hh:mm: ", d->character);
      } else {			/* i guess we're actually all good here then. */
	sprintf(buf, "\r\nAM or PM\r\n"
		"%s--------%s\r\n"
		"%s1%s) AM\r\n"
		"%s2%s) PM\r\n"
		"Enter choice : ",
		cyn, nrm,
		grn, nrm,
		grn, nrm);
	send_to_char(buf, d->character);
	OLC_SUBMODE(d) = MEDIT_SUB_AMPM_SELECT;
	if (i == 12) {
	  i = 0;
	}
	OLC_PATH(d)->performance->hrmin = ((i) * 60 + j);
      }


    }
    break;

  case MEDIT_SUB_AMPM_SELECT:
    if ((i = atoi(arg)) != 1 && i != 2) {
      send_to_char("You must enter either 1 or 2 for the time.\r\n",
		   d->character);
      sprintf(buf, "\r\nAM or PM\r\n"
	      "%s--------%s\r\n"
	      "%s1%s) AM\r\n"
	      "%s2%s) PM\r\n"
	      "Enter choice : ",
	      cyn, nrm,
	      grn, nrm,
	      grn, nrm);
      send_to_char(buf, d->character);
    } else {
      if (i == 2)
	OLC_PATH(d)->performance->hrmin += 720;
      paedit_time_edit_disp(d);
      OLC_SUBMODE(d) = MEDIT_SUB_TIME_EDIT;
    }
    break;

  case MEDIT_SUB_PURGE_SELECTION:
    if ((i = atoi(arg)) > 0 && i < OLC_PTOTAL(d)) {
      struct path_info *temp;

      j = 1;

      for (temp = OLC_PATH_LIST(d); temp && j < i; temp = temp->next)
	j++;

      OLC_PATH(d) = temp;
      REMOVE_FROM_LIST(OLC_PATH(d), OLC_PATH_LIST(d), next);

      FREE(OLC_PATH(d)->arguments);
      FREE(OLC_PATH(d)->performance);
      FREE(OLC_PATH(d));
      if (OLC_PATH_LIST(d))
	OLC_PATH(d) = OLC_PATH_LIST(d);
      else
	OLC_PATH(d) = NULL;
      OLC_VAL(d) = 1;
      OLC_PTOTAL(d)--;

      OLC_SUBMODE(d) = MEDIT_SUB_PATH;
    } else {
      sprintf(buf, "You must select a number between 1 and %d.\r\n",
	      OLC_PTOTAL(d) - 1);
      send_to_char(buf, d->character);
    }
    paedit_disp_path_editor(d);
    OLC_SUBMODE(d) = MEDIT_SUB_PATH;
    break;

  default:
    /* we don't know what to do - so bail out. */
    mudlog("Whoops. Reached default case in path_subfunctions. Prepare for crash.", BRF, LVL_BUILDER, TRUE);
    OLC_MODE(d) = MEDIT_MAIN_MENU;
    OLC_SUBMODE(d) = MEDIT_SUB_NONE;
    break;
  }


}

void medit_delete_command(struct descriptor_data *d, int pos)
{
  struct mob_attack_data *new_attack, *prev_attack;
  int i = 0;

  new_attack = OLC_MOB(d)->mob_specials.mob_attack;
  prev_attack = OLC_MOB(d)->mob_specials.mob_attack;
  if (new_attack) {
    while (new_attack) {
      if (i == pos) {
	if (i == 0) {
	  OLC_MOB(d)->mob_specials.mob_attack = new_attack->next;
	  FREE(new_attack);
	  OLC_VAL(d) = 1;
	  return;
	} else {
	  prev_attack->next = new_attack->next;
	  FREE(new_attack);
	  OLC_VAL(d) = 1;
	  return;
	}
      } else {
	prev_attack = new_attack;
	new_attack = new_attack->next;
	i++;
      }
    }
  }
  send_to_char("Wrong number!\r\n", d->character);
}

void medit_add_command(struct descriptor_data *d)
{
  struct mob_attack_data *new_attack;
  struct mob_attack_data *add_attack;

  new_attack = OLC_MOB(d)->mob_specials.mob_attack;
  CREATE(add_attack, struct mob_attack_data, 1);

  if (new_attack) {
    add_attack->next = new_attack;
    OLC_MOB(d)->mob_specials.mob_attack = add_attack;
  } else {
    add_attack->next = NULL;
    OLC_MOB(d)->mob_specials.mob_attack = add_attack;
  }

  OLC_MODE(d) = MEDIT_ATTACK_MIN;
  OLC_VAL(d) = 1;

  send_to_char("Enter min. damage:", d->character);

  add_attack->min_damage = 0;
  add_attack->max_damage = 0;
  add_attack->attack_type = 0;

}

void medit_add_min(struct descriptor_data *d, int pos)
{
  struct mob_attack_data *add_attack;

  add_attack = OLC_MOB(d)->mob_specials.mob_attack;

  OLC_MODE(d) = MEDIT_ATTACK_MAX;

  send_to_char("Enter max. damage:", d->character);

  add_attack->min_damage = pos;

}

void medit_add_max(struct descriptor_data *d, int pos)
{
  struct mob_attack_data *add_attack;

  add_attack = OLC_MOB(d)->mob_specials.mob_attack;

  OLC_MODE(d) = MEDIT_ATTACK_TYPE;

  medit_disp_attack_types(d);

  add_attack->max_damage = pos;

}

void medit_add_type(struct descriptor_data *d, int pos)
{
  struct mob_attack_data *add_attack;

  add_attack = OLC_MOB(d)->mob_specials.mob_attack;

  add_attack->attack_type = pos;
  OLC_MODE(d) = MEDIT_MAIN_MENU;
  medit_disp_menu(d);
}

void medit_delete(struct descriptor_data *d)
{
  int found = FALSE;
  int num_delete = 0;
  int rmob_num;
  struct char_data *new_proto;
  struct index_data *new_index;
  struct char_data *live_mob;

  rmob_num = real_mobile(OLC_NUM(d));

  sprintf(buf, "Deleting mob (%d) %s\r\n", OLC_NUM(d), mob_proto[rmob_num].player.name);
  send_to_char(buf, d->character);

  for (live_mob = character_list; live_mob; live_mob = live_mob->next) {
    if (GET_MOB_RNUM(live_mob) == rmob_num) {
      num_delete++;
      extract_char(live_mob);
    }
  }

  CREATE(new_proto, struct char_data, top_of_mobt);
  CREATE(new_index, struct index_data, top_of_mobt);

  for (rmob_num = 0; rmob_num <= top_of_mobt; rmob_num++) {
    if (!found) {		/* Is this the place? */
      if (mob_index[rmob_num].virtual == OLC_NUM(d)) {
	found = TRUE;
      } else {
	new_index[rmob_num] = mob_index[rmob_num];
	new_proto[rmob_num] = mob_proto[rmob_num];
      }
    } else {
      new_index[rmob_num - 1] = mob_index[rmob_num];
      new_proto[rmob_num - 1] = mob_proto[rmob_num];
      GET_MOB_RNUM(new_proto + rmob_num - 1) = rmob_num - 1;
    }
  }
  FREE(mob_index);
  FREE(mob_proto);
  mob_index = new_index;
  mob_proto = new_proto;
  top_of_mobt--;

  if (num_delete > 0) {
    sprintf(buf, "%d mobiles deleted from game.\r\n", num_delete);
    send_to_char(buf, d->character);
  }
  send_to_char("Saving file!\r\n", d->character);
  medit_save_to_disk(OLC_ZNUM(d));
}


ACMD(do_copym)
{
  extern void olc_add_to_save_list(int zone, byte type);
  int fmob = 0, tmob = 0, mobnum;

  two_arguments(argument, buf1, buf2);

  fmob = atoi(buf1);
  tmob = atoi(buf2);

  mobnum = real_mobile(fmob);
  if (mobnum < 1) {
    send_to_char("Error finding from_mob", ch);
    return;
  }
  CREATE(ch->desc->olc, struct olc_data, 1);
  STATE(ch->desc) = CON_MEDIT;
  medit_setup_existing(ch->desc, mobnum);
  OLC_NUM(ch->desc) = tmob;
  if (tmob <= 5000)
    OLC_PROTO(ch->desc) = tmob;
  else
    OLC_PROTO(ch->desc) = fmob;
  OLC_VAL(ch->desc) = 1;	/* So code knows its changed */
  olc_add_to_save_list(tmob / 100, OLC_SAVE_MOB);
}
