
/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "db.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "spells.h"
#include "olc.h"
#include "events.h"

/* external structs */
extern struct char_data *character_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct room_data *world;
extern struct str_app_type str_app[];
extern struct char_data *mob_proto;
extern struct time_info_data time_info;
extern int is_empty(int zone_nr);
extern int find_first_step(int src, int target);
extern char *dirs[];
extern char *race_types[];
extern int rev_dir[];
extern struct zone_data *zone_table;

extern void move_closer(struct char_data *ch, struct char_data *vict, struct obj_data *obj);
extern ACMD(do_stand);
extern ACMD(do_backstab);
extern ACMD(do_flee);
extern ACMD(do_move);
extern ACMD(do_sneak);
extern ACMD(do_hide);
extern bool perform_bash(struct char_data *ch, struct char_data *vict);
extern bool perform_kick(struct char_data *ch, struct char_data *vict);
extern int DamageOneItem(struct char_data *ch, int dam_type, struct obj_data *obj, int dam);
extern void MakeScrap(struct char_data *ch, struct obj_data *obj, int dam_type);
extern void SuddenDeath(struct char_data *ch, struct char_data *victim, char *kill_type);
extern bool CastMageSpell(struct char_data *ch);
extern bool CastClericSpell(struct char_data *ch);

/* locals */
void MobCombat(struct char_data *ch, struct char_data *new_vict);

byte is_aggro_to(struct char_data *ch, struct char_data *vict)
{

  /* this overrides any other considerations */
  if (MOB_FLAGGED(ch, MOB_AGGRESSIVE))
    return YES;

  /* a few races that are group defined */
  if (IS_UNDEAD(ch) && !IS_SUNLIT(ch->in_room))
    return YES;

  switch (GET_RACE(ch)) {
  case RACE_NONE:
  case RACE_HUMAN:
  case RACE_HALFELF:
  case RACE_ELF:
  case RACE_DWARF:
  case RACE_HALFLING:
  case RACE_GNOME:
  case RACE_HALFORC:
    if (GET_ALIGNMENT(ch) > ALIGN_NEUTRAL_NEUTRAL && GET_LEVEL(ch) > GET_LEVEL(vict))
      return YES;
    break;
  case RACE_DROW:
    if (IS_PC_HUMANOID(vict))
      return YES;
    break;
  case RACE_BEHOLDER:
    return YES;
  case RACE_BIRD:
    if (IS_HUMANOID(vict) && GET_SIZE(ch) > GET_SIZE(vict) + 1)
      return YES;
    break;

  case RACE_INSECT:
    if (IS_HUMANOID(vict) && GET_SIZE(ch) >= GET_SIZE(vict) - 1)
      return YES;
    break;

  case RACE_RAT:
    if (IS_HUMANOID(vict))
      return YES;
    break;
  case RACE_SNAKE:
    if (IS_HUMANOID(vict) && GET_SIZE(ch) >= (GET_SIZE(vict) - 2))
      return YES;
    break;
  case RACE_SPIDER:
    if (IS_HUMANOID(vict) && GET_SIZE(ch) >= GET_SIZE(vict))
      return YES;
    break;

    /* uncompleted below here */
  case RACE_ANKHEG:
  case RACE_AARAKOCRA:
  case RACE_ABOLETH:
  case RACE_ARCANE:
  case RACE_ARGOS:
  case RACE_BAATEZU:
  case RACE_AURUMVORAX:
  case RACE_BASILISK:
  case RACE_BEETLE_GIANT:
  case RACE_BEHIR:
  case RACE_BRAIN_MOLE:
  case RACE_BROWNIE:
  case RACE_BUGBEAR:
  case RACE_BULETTE:
  case RACE_BULLYWUG:
  case RACE_CARRION_CRAWLER:
  case RACE_CATOBLEPAS:
  case RACE_CAVE_FISHER:
  case RACE_CENTAUR:
  case RACE_CENTIPEDE:
  case RACE_CHIMERA:
  case RACE_GORGIMERA:
  case RACE_CLOAKER:
  case RACE_COCKATRICE:
  case RACE_COUATL:
  case RACE_CRABMAN:
  case RACE_CRAWLING_CLAW:
  case RACE_CROCODILE:
  case RACE_CRUSTACEAN_GIANT:
  case RACE_DEEPSPAWN:
  case RACE_DINOSAUR:
  case RACE_DISPLACER_BEAST:
  case RACE_DOPPLEGANGER:
  case RACE_DRAGON:
  case RACE_FAERIE_DRAGON:
  case RACE_FIREDRAKE:
  case RACE_PSEUDODRAGON:
  case RACE_DRAGONNE:
  case RACE_DRYAD:
  case RACE_ELEMENTAL_AIR:
  case RACE_ELEMENTAL_EARTH:
  case RACE_ELEMENTAL_FIRE:
  case RACE_ELEMENTAL_WATER:
  case RACE_ETTERCAP:
  case RACE_EYEWING:
  case RACE_FEYR:
  case RACE_FISH:
  case RACE_FROG:
  case RACE_FUNGUS:
  case RACE_GALEB_DUHR:
  case RACE_GARGOYLE:
  case RACE_GENIE:
  case RACE_GIANT:
  case RACE_GIBBERLING:
  case RACE_GIFF:
  case RACE_GITH:
  case RACE_GITHYANKI:
  case RACE_GITHZERAI:
  case RACE_GNOLL:
  case RACE_GOBLIN:
  case RACE_GOLEM:
  case RACE_GORGON:
  case RACE_GRELL:
  case RACE_GREMLIN:
  case RACE_GRIFFON:
  case RACE_GRIMLOCK:
  case RACE_GRIPPLI:
  case RACE_HAG:
  case RACE_HARPY:
  case RACE_HATORI:
  case RACE_HELL_HOUND:
  case RACE_HIPPOCAMPUS:
  case RACE_HIPPOGRIFF:
  case RACE_HOBGOBLIN:
  case RACE_HOMONCULOUS:
  case RACE_HOOK_HORROR:
  case RACE_HYDRA:
  case RACE_IMP:
  case RACE_INTELLECT_DEVOURER:
  case RACE_INVISIBLE_STALKER:
  case RACE_IXITXACHITL:
  case RACE_JACKELWERE:
  case RACE_KENKU:
  case RACE_KI_RIN:
  case RACE_KIRRE:
  case RACE_KOBOLD:
  case RACE_KUO_TOA:
  case RACE_LAMIA:
  case RACE_LAMMASU:
  case RACE_LEECH:
  case RACE_LEPRECHAUN:
  case RACE_LEUCROTTA:
  case RACE_LIVING_WALL:
  case RACE_LIZARD:
  case RACE_LIZARD_MAN:
  case RACE_LOCATHAH:
  case RACE_LURKER:
  case RACE_LYCANTHROPE:
  case RACE_MANSCORPION:
  case RACE_MANTICORE:
  case RACE_MEDUSA:
  case RACE_MEADAR:
  case RACE_MERMAN:
  case RACE_MIMIC:
  case RACE_MIND_FLAYER:
  case RACE_MINOTAUR:
  case RACE_MIST:
  case RACE_MOLD:
  case RACE_MONGRELMAN:
  case RACE_MORKOTH:
  case RACE_MUCKDWELLER:
  case RACE_MUDMAN:
  case RACE_MYCONID:
  case RACE_NAGA:
  case RACE_NEOGI:
  case RACE_NIGHTMARE:
  case RACE_NYMPH:
  case RACE_OCTOPUS:
  case RACE_OGRE:
  case RACE_OGRE_MAGE:
  case RACE_SLIME:
  case RACE_ORC:
  case RACE_OTYUGH:
  case RACE_OWLBEAR:
  case RACE_PEGASUS:
  case RACE_PERYTON:
  case RACE_PHANTOM:
  case RACE_PHOENIX:
  case RACE_PIERCER:
  case RACE_PLANT:
  case RACE_PUDDING:
  case RACE_QUAGGOTH:
  case RACE_RAKSHASA:
  case RACE_REMORHAZ:
  case RACE_ROC:
  case RACE_ROPER:
  case RACE_RUST_MONSTER:
  case RACE_SAHUAGIN:
  case RACE_SATYR:
  case RACE_SCORPION:
  case RACE_SELKIE:
  case RACE_SHEDU:
  case RACE_SIRINE:
  case RACE_SLAAD:
  case RACE_SPHINX:
  case RACE_SPRITE:
  case RACE_SQUID:
  case RACE_STIRGE:
  case RACE_SU_MONSTER:
  case RACE_SWANMAY:
  case RACE_TABAXI:
  case RACE_TAKO:
  case RACE_TANARRI:
  case RACE_TARRASQUE:
  case RACE_TASLOI:
  case RACE_THOUGHT_EATER:
  case RACE_THRI_KREEN:
  case RACE_TITAN:
  case RACE_TOAD:
  case RACE_TREANT:
  case RACE_TRITON:
  case RACE_TROGLODYTE:
  case RACE_TROLL:
  case RACE_UMBER_HULK:
  case RACE_UNICORN:
  case RACE_URCHIN:
  case RACE_WEMIC:
  case RACE_WHALE:
  case RACE_WILL_O_WISP:
  case RACE_WOLF:
  case RACE_WOLFWERE:
  case RACE_WORM:
  case RACE_WYVERN:
  case RACE_XORN:
  case RACE_YETI:
  case RACE_YAUN_TI:
  case RACE_YUGOLOTH:
  case RACE_ZARATAN:
    return NO;
  }
  return NO;
}

void HandleMobAggresion(struct char_data *ch)
{
  int found = FALSE;
  struct char_data *vict, *vict_next;

  for (vict = world[ch->in_room].people; vict && !found; vict = vict_next) {
    vict_next = vict->next_in_room;
    if (IS_NPC(vict) || !CAN_SEE(ch, vict) || PRF_FLAGGED(vict, PRF_NOHASSLE))
      continue;
    if (MOB_FLAGGED(ch, MOB_WIMPY) && AWAKE(vict) && GET_HIT(ch) <= GET_MAX_HIT(ch) / 3)
      continue;

    if (is_aggro_to(ch, vict)) {
      move_closer(ch, vict, NULL);
      MobCombat(ch, vict);
      found = TRUE;
    }
  }
}

/**** general mob fighting utilities ****/
bool valid_attack(struct char_data *ch, struct char_data *vict, int type)
{
  /* I dont care about vict right now, but I may eventually */
  if (GET_MOB_LAST_ATTACK_TYPE(ch) == type)
    if (GET_MOB_LAST_ATTACK_SUCCESS(ch) == FALSE)
      return FALSE;

  return TRUE;
}

int NumAttackers(struct char_data *ch)
{
  struct char_data *tch;
  int total = 0;

  for (tch = world[ch->in_room].people; tch; tch = tch->next_in_room)
    if (FIGHTING(tch) && FIGHTING(tch) == ch)
      total++;

  return total;
}

bool
are_together(struct char_data * ch1, struct char_data * ch2)
{

  if (!ch1 || !ch2)
    return FALSE;

  if ((ch1 == ch2->master) || (ch2 == ch1->master))
    return TRUE;
  else if (ch1->master && ch2->master && (ch1->master == ch2->master))
    return TRUE;

  return FALSE;
}

bool
has_help(struct char_data * ch)
{
  struct char_data *tmp = NULL;

  if (!ch)
    return (FALSE);

  for (tmp = world[ch->in_room].people; tmp; tmp = tmp->next_in_room) {
    if (tmp == ch)
      continue;
    if (are_together(ch, tmp))
      return (TRUE);
  }
  return (FALSE);
}

/* very simple, assume we are fighting a group, even if all of them aren't fighting us, chances are the
   one we are actually hitting, is the one that group WANTS us to hit, which means it's probably a really
   bad choice, this routine finds another victim. */

struct char_data *
 Juiciest(struct char_data *ch)
{
  struct char_data *tch, *juiciest = NULL;
  int tenderness, t_tend;

  if (!IS_FIGHTING(ch))
    return NULL;

  for (tch = world[ch->in_room].people; tch; tch = tch->next_in_room)
    if (((IS_FIGHTING(tch) && (FIGHTING(tch) == ch)) ||
	 are_together(tch, FIGHTING(ch))) && CAN_SEE(ch, tch)) {
      t_tend = GET_HIT(tch) * (100 - GET_AC(tch));
      if (!MIN_POS(tch, POS_STANDING + STAT_NORMAL)) {
	if (MIN_POS(tch, POS_SITTING + STAT_RESTING))
	  t_tend = t_tend * 3 / 2;
	else if (MIN_POS(tch, POS_KNEELING + STAT_DEAD))
	  t_tend = t_tend * 7 / 4;
	else			/* prone and/or not awake */
	  t_tend /= 2;
      }
      if (IS_IMMATERIAL(tch))
	t_tend *= 2;
      if (IS_AFFECTED2(tch, AFF2_FIRESHIELD) && !IS_AFFECTED2(ch, AFF2_GLOBE))
	t_tend *= 4;

      if (!IS_WARRIOR(tch))
	t_tend /= 2;

      if (!IS_AFFECTED(tch, AFF_CHARM))
	t_tend *= 2;

      if (!juiciest || (t_tend < tenderness)) {
	juiciest = tch;
	tenderness = t_tend;
      }
    }
  return juiciest;
}

int CountToughness(struct char_data *ch, struct char_data *victim)
{
  int val, foo;

  if (IS_PC(ch))
    return -1;

  val = GET_HIT(victim);

  if (IS_PRIEST(victim) || IS_MAGE(victim))
    val = val * 2 / 3;
  else if (IS_WARRIOR(victim))
    val = val * 2;
  foo = MAX(0, 60 - GET_LEVEL(victim));
  if (!IS_FIGHTING(victim))
    val = val / (IS_ROGUE(ch) ? 4 : 2);
//  if ((IS_AFFECTED(victim, AFF_AWARE) ||
  //       IS_AFFECTED(victim, AFF_SKILL_AWARE)) && IS_ROGUE(ch))
  //    val = val * 1.5;
  val = val * 100 / number(100 - foo, 100 + foo);
  return val;
}

#define MAX_TARGETS 20

/* used to pick a player to attack, checks all the piddly things that need
   to be checked, and if it finds a likely target, returns pointer to that
   ch, else returns NULL.
   Assumes: ch is agg (at least to some things), and ch can fight.  JAB */

struct char_data *
 PickTarget(struct char_data *ch)
{
  struct char_data *t_ch;
  int target_table[MAX_TARGETS + 1];
  struct char_data *target_addr[MAX_TARGETS + 1];
  int a, b, c, d, n_a;

  if (!ch)
    return NULL;

  if (IS_STUNNED(ch) || IS_AFFECTED4(ch, AFF4_BOUND) || !AWAKE(ch) ||
      IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS))
    return NULL;

  a = 0;
  n_a = NumAttackers(ch);

  for (t_ch = world[ch->in_room].people; t_ch; t_ch = t_ch->next_in_room) {
    if (t_ch == ch)
      continue;
/*    if (n_a && (!IS_FIGHTING(t_ch) || FIGHTING(t_ch) != ch))
   continue; */
//    if (!is_aggr_to(ch, t_ch))
    //      continue;
    if (a < MAX_TARGETS) {
      target_table[a] = CountToughness(ch, t_ch);
      target_addr[a] = t_ch;
      target_table[a + 1] = -1;
      target_addr[a + 1] = NULL;
      a++;
    } else
      break;
  }

  if (a == 0)
    return NULL;

  if (IS_PC(ch))
    return (target_addr[number(0, (a - 1))]);

  b = -2;
  c = -1;
  for (d = 0; d < a; d++)
    if ((target_table[d] < b) || (b == -2)) {
      c = d;
      b = target_table[d];
    }
  if (c != -1)
    return target_addr[c];

  /* nope, no likely targets in room */
  return NULL;
}
/**** end of mob utilities ****/



/* Do special warrior attacks */

bool
MobWarrior(struct char_data * ch)
{
  struct char_data *tch = NULL, *t_ch2, *juiciest;
  int n_atkr;

  if (IS_PC(ch) || !IS_HUMANOID(ch))
    return FALSE;

  if (IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS))
    return FALSE;

  n_atkr = NumAttackers(ch);

//  send_to_char("DEBUG: Considering a warrior fight move\r\n", ch);
  /* First off, what we do if we are fighting a group */

  if (n_atkr > 1) {
    /* let's see if there is a caster we can knock on his ass */
    for (tch = world[ch->in_room].people; tch; tch = tch->next_in_room) {
      if (((FIGHTING(tch) == ch) || are_together(tch, FIGHTING(ch))) &&
	  (IS_MAGE(tch) || IS_PRIEST(tch)) && (GET_POS(ch) > POS_SITTING) && number(0, 1) &&
       !IS_IMMATERIAL(tch) && ((GET_WEIGHT(ch) * 3) > GET_WEIGHT(tch)) &&
	  (GET_HEIGHT(tch) * 3 >= GET_HEIGHT(ch)))
	break;
    }

    if (valid_attack(ch, tch, ATTACK_BASH)) {
      if ((ch && tch) && (ch->in_room == tch->in_room)) {
	GET_MOB_LAST_ATTACK_TYPE(ch) = ATTACK_BASH;
	if (perform_bash(ch, tch))
	  GET_MOB_LAST_ATTACK_SUCCESS(ch) = TRUE;
	return TRUE;
      }
    }
  }
  /* From this point, they are fighting one-on-one */

  if (!number(0, 3) && (GET_POS(FIGHTING(ch)) == POS_STANDING)) {
    if (!IS_IMMATERIAL(FIGHTING(ch)) && ((GET_WEIGHT(ch) * 3) > GET_WEIGHT(FIGHTING(ch))) &&
	(GET_HEIGHT(ch) <= GET_HEIGHT(FIGHTING(ch)) * 3) && !IS_IMMATERIAL(ch)) {
      if (valid_attack(ch, FIGHTING(ch), ATTACK_BASH)) {
	GET_MOB_LAST_ATTACK_TYPE(ch) = ATTACK_BASH;
	if (perform_bash(ch, FIGHTING(ch)))
	  GET_MOB_LAST_ATTACK_SUCCESS(ch) = TRUE;
	if (!ch || !FIGHTING(ch) || (GET_POS(ch) < POS_STANDING) ||
	    (GET_POS(FIGHTING(ch)) < POS_STANDING))
	  return TRUE;
      }
    }
  }
  if (valid_attack(ch, FIGHTING(ch), ATTACK_KICK))
    if (!number(0, 2) && !IS_IMMATERIAL(FIGHTING(ch))) {
      GET_MOB_LAST_ATTACK_TYPE(ch) = ATTACK_KICK;
      if (perform_kick(ch, FIGHTING(ch)))
	GET_MOB_LAST_ATTACK_SUCCESS(ch) = TRUE;
      return TRUE;
    }
  if ((n_atkr > 2) && (GET_LEVEL(ch) > 14) && number(0, 2)) {
    /* psuedo hitall func, takes a swing at all chars fighting 'ch' */
    for (tch = world[ch->in_room].people; tch; tch = t_ch2) {
      t_ch2 = tch->next_in_room;
      if (tch != ch)
	continue;
      if ((FIGHTING(tch) == ch) || are_together(tch, FIGHTING(ch))) {
	if (number(0, 135) > MAX(99, ((GET_LEVEL(ch) - 10) * 9))) {
	  hit(ch, tch, TYPE_UNDEFINED);
	  if (GET_STAT(ch) < STAT_NORMAL)	/* fireshield, for example */
	    return TRUE;
	}
      }
    }
    return TRUE;
  }
  if (((number(1, 100) < (GET_LEVEL(ch) * 3))) && ((n_atkr > 1) || has_help(FIGHTING(ch)))) {
    /* The anti-tank clause.  The way players set things up, there is a real good chance that the person
       we are currently beating on, is the WORST person to be attacking (from our viewpoint), so let's see
       about switching to a juicier target.  JAB */

    juiciest = Juiciest(ch);
    if (juiciest && (juiciest != FIGHTING(ch))) {
      MobCombat(ch, juiciest);	/* switch to new target */
      return TRUE;
    }
  }
  hit(ch, FIGHTING(ch), TYPE_UNDEFINED);

  /* A final bit of fancy, mostly for show */
  /* nothing happens after this, so MobCombat will handle the facing opponent */
  if (!number(0, 2))
    do_move(ch, 0, NUM_DIRS + 1, SCMD_ROLL);	/* command needs to be set so this will be */
  else if (!number(0, 1))
    do_move(ch, 0, NUM_DIRS + 1, SCMD_CIRCLE);	/* considered a nonstandard move             */

  return TRUE;
}


/* thief mobs in combat, really suck, so we are gonna make them a just a bit
   smarter.  JAB */

bool
MobThief(struct char_data * ch)
{
  struct obj_data *bs_weap = NULL;
  struct obj_data *t_obj;
  bool weapon_pos = FALSE;

  /* basic strategy, flee/wield a backstab weapon if we have one/hide and
     wait for attackers to come looking, then let memory or agg flags handle
     backstabbing the miscreants.  Here we go... */

  /* first check if they have a backstabbing weapon either in hand or in
     inven.  If not, they may as well stand and fight, no advantage in
     fleeing (wimpy can still kick in, this flee is tactical). */

  if (IS_PC(ch))
    return FALSE;

  buf[0] = '\0';
  if (FIGHTING(ch))		// We set this now, in case we stop fighting in here

    sprintf(buf, "%s", GET_NAME(FIGHTING(ch)));

//  send_to_char("DEBUG: Considering a rogue fight move\r\n", ch);
  if ((ch->equipment[WEAR_WIELD]) && (GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) == 11) &&
      (CAN_WEAR(ch->equipment[WEAR_WIELD], ITEM_TAKE)) &&
      (CAN_WEAR(ch->equipment[WEAR_WIELD], ITEM_WEAR_WIELD))) {
    bs_weap = ch->equipment[WEAR_WIELD];
    weapon_pos = TRUE;
  } else if ((ch->equipment[WEAR_WIELD2]) && (GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD2), 3) == 11) &&
	     (CAN_WEAR(ch->equipment[WEAR_WIELD2], ITEM_TAKE)) &&
	     (CAN_WEAR(ch->equipment[WEAR_WIELD2], ITEM_WEAR_WIELD))) {
    bs_weap = ch->equipment[WEAR_WIELD2];
    weapon_pos = TRUE;
  } else {
    /* let's see if we are carrying something suitable */
    for (t_obj = ch->carrying; t_obj && !bs_weap; t_obj = t_obj->next_content)
      if ((GET_OBJ_TYPE(t_obj) == ITEM_WEAPON) && (GET_OBJ_VAL(t_obj, 3) == 11) &&
      (CAN_WEAR(t_obj, ITEM_TAKE)) && (CAN_WEAR(t_obj, ITEM_WEAR_WIELD)))
	bs_weap = t_obj;
    /* I should also loop bagged goods here */
  }

  /* bah, lets keep going */
  //if (!bs_weap)
  //  return FALSE;             /* nope, nothing we can do */

  /* ok, we can backstab, good deal, let's work at it */

  if (!IS_AFFECTED(ch, AFF_HIDE) &&
      (!IS_AFFECTED(ch, AFF_CHARM) || !(ch->master) || (ch->master->in_room != ch->in_room))) {
    if (!number(0, 1))
      do_move(ch, 0, NUM_DIRS + 1, SCMD_ROLL);	/* command needs to be set so this will be */
    else
      do_move(ch, 0, NUM_DIRS + 1, SCMD_CIRCLE);	/* considered a nonstandard move           */

    if (!IS_AFFECTED(ch, AFF_SNEAK))
      do_sneak(ch, "", 0, 0);
  }
  if (bs_weap && !weapon_pos) {
    obj_from_char(bs_weap);
    /* got to WEAR_WIELD our bs weapon (maybe unWIELDing current) */
    if (!ch->equipment[WEAR_WIELD])
      equip_char(ch, bs_weap, WEAR_WIELD);
    else if (!ch->equipment[WEAR_WIELD2] && !NEEDS_TWO_HANDS(ch, GET_EQ(ch, WEAR_WIELD)))
      equip_char(ch, bs_weap, WEAR_WIELD2);
    else {
      /* got to clear a weapon slot */
      if (NEEDS_TWO_HANDS(ch, GET_EQ(ch, WEAR_WIELD))) {
	if (!IS_OBJ_STAT(GET_EQ(ch, WEAR_WIELD), ITEM_NODROP)) {
	  obj_to_char(unequip_char(ch, WEAR_WIELD), ch);
	  equip_char(ch, bs_weap, WEAR_WIELD);
	} else
	  return TRUE;		/* hosed */
      } else if (!IS_OBJ_STAT(GET_EQ(ch, WEAR_WIELD2), ITEM_NODROP)) {
	obj_to_char(unequip_char(ch, WEAR_WIELD2), ch);
	equip_char(ch, bs_weap, WEAR_WIELD2);
      } else if (!IS_OBJ_STAT(GET_EQ(ch, WEAR_WIELD), ITEM_NODROP)) {
	obj_to_char(unequip_char(ch, WEAR_WIELD), ch);
	equip_char(ch, bs_weap, WEAR_WIELD);
      } else {
	/* both weapon slots filled with cursed non-bs weapons, we be hosed */
	obj_to_char(bs_weap, ch);
	return TRUE;
      }
    }
  }
  /* now we be cooking with gas, escaped, got bs weapon in hand */
  /* or not, we still move this way */
  if (!IS_AFFECTED(ch, AFF_HIDE))
    do_hide(ch, "", 0, 0);

  if (buf && bs_weap)
    do_backstab(ch, buf, 0, 0);

  return TRUE;
}

bool
MobGeneric(struct char_data * ch)
{
  if (IS_PC(ch))
    return FALSE;

  if (IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS))
    return FALSE;

  // Ouch, this sucks
  send_to_char("DEBUG: attacking generically\r\n", ch);
  hit(ch, FIGHTING(ch), TYPE_UNDEFINED);

  return TRUE;
}

bool RustMonsterCombat(struct char_data * ch)
{
  struct char_data *vict;
  struct obj_data *t_item;
  int t_pos;

  /* find a victim */
  if (FIGHTING(ch))
    vict = FIGHTING(ch);
  else
    vict = PickTarget(ch);
  if (!vict)
    return FALSE;

  /* choose an item of armor or a weapon that is metal */

  /* choose a weapon first, then if no weapon, choose a shield,
   * if no shield, choose breast plate, then leg plate, sleeves, helm
   */

  if (vict->equipment[WEAR_WIELD]) {
    t_item = vict->equipment[WEAR_WIELD];
    t_pos = WEAR_WIELD;
  } else if (vict->equipment[WEAR_WIELD2]) {
    t_item = vict->equipment[WEAR_WIELD2];
    t_pos = WEAR_WIELD2;
  } else if (vict->equipment[WEAR_SHIELD]) {
    t_item = vict->equipment[WEAR_SHIELD];
    t_pos = WEAR_SHIELD;
  } else if (vict->equipment[WEAR_BODY]) {
    t_item = vict->equipment[WEAR_BODY];
    t_pos = WEAR_BODY;
  } else if (vict->equipment[WEAR_LEGS]) {
    t_item = vict->equipment[WEAR_LEGS];
    t_pos = WEAR_LEGS;
  } else if (vict->equipment[WEAR_ARMS]) {
    t_item = vict->equipment[WEAR_ARMS];
    t_pos = WEAR_ARMS;
  } else if (vict->equipment[WEAR_HEAD]) {
    t_item = vict->equipment[WEAR_HEAD];
    t_pos = WEAR_HEAD;
  } else
    return FALSE;

  // item makes save (or not) 
  if (DamageOneItem(vict, TYPE_ACID, t_item, 0)) {
    t_item = unequip_char(vict, t_pos);
    if (t_item) {
      // if it doesn't make save, falls into a pile of scraps 
      MakeScrap(vict, t_item, TYPE_ACID);
    }
  }
  return FALSE;
}

/* newish attack type, this is a sweep (with a dragon's tail to start with) which will knock down
   several opponents (like bash) */

void SweepAttack(struct char_data *ch)
{
  struct char_data *tch, *next;
  int dam, distance, dir = -1;

  if (!ch || (ch->in_room == NOWHERE))
    return;

  distance = 3;			// This is where we should be setting location data etc of the tail

  for (tch = world[ch->in_room].people; tch; tch = next) {
    next = tch->next_in_room;
    if (IS_FIGHTING(tch) && (FIGHTING(tch) == ch) && (GET_WEIGHT(tch) * 10 < GET_WEIGHT(ch))) {
      if (WithinProximity(ch, NULL, tch, NULL, NULL, -1, &distance, &dir, FALSE)) {
	if (number(0, 25) > GET_DEX(ch)) {
	  dam = MIN(dice(2, (GET_LEVEL(ch) / 5)) + 5, GET_HIT(ch) + 8);
	  /* fall down, go boom */
	  if (GET_POS(tch) > POS_SITTING)
	    SET_POS(tch, POS_SITTING + GET_STAT(tch));
	  WAIT_STATE(tch, PULSE_VIOLENCE * 2);
	  if ((GET_HIT(tch) - dam) < -10) {
	    act("The powerful sweep knocks the remaining life from you!", FALSE, tch, 0, 0, TO_CHAR);
	    act("$n crashes lifelessly to the ground! ", FALSE, tch, 0, 0, TO_ROOM);
	    SuddenDeath(tch, ch, "Tail Sweep");
	    continue;
	  }
	  act("The powerful sweep sends you crashing to the ground!", FALSE, tch, 0, 0, TO_CHAR);
	  act("$n crashes to the ground!", FALSE, tch, 0, 0, TO_ROOM);
	  GET_HIT(tch) -= dam;
	} else
	  send_to_char("You nimbly dodge the sweep!\n", tch);
      }
    }
  }
}

#ifdef MOB_COMBAT
void spell_fire_breath(int level, struct char_data *ch, struct char_data *victim, struct obj_data *obj)
{
  int dam;
  struct obj_data *burn = NULL;

  dam = dice(level, 8) + level;

  if (makes_save(victim, SAVING_SPELL, 0))
    dam >>= 1;
//  else if (affected_by_spell(victim, SPELL_FIRESHIELD))
  //    dam <<= 1;

  if (IS_AFFECTED(victim, AFF_PROT_FIRE))
    dam >>= 1;

//  if (damage(ch, victim, dam, SPELL_FIRE_BREATH))
  //    return;

  if (IS_AFFECTED(victim, AFF_PROT_FIRE))
    return;

  /*
   * And now for the damage on inventory
   */
  if (number(0, 40) < GET_LEVEL(ch)) {
    if (!saves_spell(victim, SAVING_BREATH)) {
      for (burn = victim->carrying;
	   burn && (((burn->type != ITEM_SCROLL) &&
		     (burn->type != ITEM_WAND) &&
		     (burn->type != ITEM_STAFF) &&
		     (burn->type != ITEM_NOTE)) ||
		    number(0, 2));
	   burn = burn->next_content);
      if (burn) {
	act("&+r$p is turned into ash!", FALSE, victim, burn, 0, TO_CHAR);
	extract_obj(burn);
	burn = NULL;
      }
    }
  }
}

void spell_frost_breath(int level, struct char_data *ch, struct char_data *victim, struct obj_data *obj)
{
  int dam, mod;
  struct obj_data *frozen = NULL;
  struct affected_type af;

  dam = dice(level, 8) + level;

  if (makes_save(victim, SAVING_SPELL, affected_by_spell(victim, SPELL_COLDSHIELD) ? -2 : 0))
    dam >>= 1;
  else if (affected_by_spell(victim, SPELL_COLDSHIELD))
    dam <<= 1;

  if (IS_AFFECTED2(victim, AFF2_PROT_COLD))
    dam >>= 1;

  if (damage(ch, victim, dam, SPELL_FROST_BREATH))
    return;

#if 0
  if (IS_THRIKREEN(victim) && !(IS_TRUSTED(victim))) {
    mod = (20 * dam) / GET_MAX_HIT(victim);
    if (!makes_save(victim, SAVING_PARA, mod)) {
      send_to_char("&+CThe intense cold causes your entire body to freeze!\r\n", victim);
      act("&+CThe intense cold causes $n&+C's entire body to freeze!\r\n",
	  FALSE, victim, 0, 0, TO_ROOM);

      bzero(&af, sizeof(af));
      af.type = SPELL_MAJOR_PARALYSIS;
      af.duration = 2;
      af.bitvector2 = AFF2_MAJOR_PARALYSIS;

      affect_to_char(victim, &af);

      if (IS_FIGHTING(victim))
	stop_fighting(victim);
    }
  }
#endif
  if (IS_AFFECTED2(victim, AFF2_PROT_COLD))
    return;

  /*
   * And now for the damage on inventory
   */
  if (number(0, 40) < GET_LEVEL(ch)) {
    if (!saves_spell(victim, SAVING_BREATH)) {
      for (frozen = victim->carrying;
	   frozen && ((GET_OBJ_TYPE(frozen) != ITEM_DRINKCON &&
		       GET_OBJ_TYPE(frozen) != ITEM_FOOD &&
		       GET_OBJ_TYPE(frozen) != ITEM_POTION) ||
		      number(0, 2));
	   frozen = frozen->next_content);
      if (frozen) {
	act("&+W$p shatters into frozen scraps.", FALSE, victim, frozen, 0, TO_CHAR);
	extract_obj(frozen);
	frozen = NULL;
      }
    }
  }
}

void spell_acid_breath(int level, struct char_data *ch, struct char_data *victim, struct obj_data *obj)
{
  int dam;

  dam = dice(level, 8) + level;

  if (saves_spell(victim, SAVING_BREATH))
    dam >>= 1;
  if (IS_AFFECTED2(victim, AFF2_PROT_ACID))
    dam >>= 1;

  if (damage(ch, victim, dam, SPELL_ACID_BREATH))
    return;

  if (IS_AFFECTED2(victim, AFF2_PROT_ACID))
    return;

  /*
   * And now for the damage on equipment
   */

#if 0
  if (number(0, 40) < GET_LEVEL(ch)) {
    if (!saves_spell(victim, SAVING_BREATH)) {
      for (damaged = 0; (damaged < MAX_WEAR) &&
	   !((victim->equipment[damaged]) &&
	     (victim->equipment[damaged]->type == ITEM_ARMOR) &&
	     (victim->equipment[damaged]->value[0] > 0) &&
	     number(0, 1)); damaged++);
      if (damaged < MAX_WEAR) {
	act("&+L$p corrodes.", FALSE, victim, victim->equipment[damaged], 0, TO_CHAR);
	GET_AC(victim) -= apply_ac(victim, damaged);
	victim->equipment[damaged]->value[0] -= number(1, 7);
	GET_AC(victim) += apply_ac(victim, damaged);
	victim->equipment[damaged]->cost = 0;
      }
    }
  }
#endif
}

void spell_gas_breath(int level, struct char_data *ch, struct char_data *victim, struct obj_data *obj)
{
  int dam, mult = 1;

  dam = dice(level, 2) + (level / 4);

  if (!saves_spell(victim, SAVING_BREATH))
    mult <<= 1;
  if (!IS_AFFECTED2(victim, AFF2_PROT_GAS)) {
    mult <<= 1;
    spell_poison(level * mult, ch, victim, 0);
  }
  if (char_in_list(victim) && (GET_STAT(victim) != STAT_DEAD))
    damage(ch, victim, dam, SPELL_GAS_BREATH);
}

void spell_lightning_breath(int level, struct char_data *ch, struct char_data *victim, struct obj_data *obj)
{
  int dam;

  dam = dice(level, 8) + level;

  if (saves_spell(victim, SAVING_BREATH))
    dam >>= 1;
  if (IS_AFFECTED2(victim, AFF2_PROT_LIGHTNING))
    dam >>= 1;

  damage(ch, victim, dam, SPELL_LIGHTNING_BREATH);
}
/* New routine to handle dragons in combat, this will make dragons without
   unique specials.  Hopefully will make fighting ANY dragon (even babies!)
   a real challenge.  Called from 2 places, start_fighting(), and MobCombat().
   The call from start_fighting invokes 'Dragon Fear'.  Returns TRUE if dragon
   does something, else FALSE.  -JAB */

bool
DragonCombat(struct char_data *ch, int awe)
{
  struct char_data *tchar1 = NULL, *tchar2 = NULL, *vict;
  int i, breath_chance, attacktype = 0;

  void (*funct) (int, struct char_data *, struct char_data *, struct obj_data *);

  breath_chance = MAX(2, (1 + NumAttackers(ch) + (GET_MAX_HIT(ch) / MAX(1, GET_HIT(ch)))) >> 1);

  if (!isname("tiamat", GET_NAME(ch))) {
    if (awe) {			/* Tiamat is kept waaay down below here */

      /* means we were called from start_fighting, dragons get a special initial
         'fear' and 'awe' attack at start on combat, low levels are going to be
         sent fleeing in terror, and maybe some not-so low levels too.  Even
         young dragons are nasty opponents */

      act("$n ROARS!", 1, ch, 0, 0, TO_ROOM);
      for (tchar1 = world[ch->in_room].people; tchar1; tchar1 = tchar2) {
	tchar2 = tchar1->next_in_room;
	if ((tchar1 == ch) || are_together(ch, tchar1))
	  continue;
	if ((GET_RACE(tchar1) != RACE_DRAGON)) {
	  if (GET_LEVEL(tchar1) < (GET_LEVEL(ch) / 3))
	    do_flee(tchar1, "", 0, 0);
	  if (GET_LEVEL(tchar1) >= GET_LEVEL(ch)) {
	    if (!makes_save(tchar1, SAVING_PARA, -2))
	      do_flee(tchar1, "", 0, 0);
	  } else if (!(IS_TRUSTED(tchar1) && !makes_save(tchar1, SAVING_PARA, 1)))
	    do_flee(tchar1, "", 0, 0);
	}
	if (ch->in_room != tchar1->in_room)
	  if (IS_FIGHTING(tchar1))
	    stop_fighting(tchar1);
      }
      return TRUE;
    }
    /* basically, breath is their best weapon, so we use that 50% or more of the time, based on how many
       attackers and how hurt they are.  If they have lots of attackers (say 10), and aren't hurt much, they will
       breathe 6 out of 7 times, when they get very hurt, that will climb. */

    /* Ok, let's add a sweep with the tail 1 in 5 */
    if (((GET_RACE(ch) != RACE_DRAGON) || (number(0, breath_chance) < 2)) && !number(0, 4)) {
      act("$n lashes out with $s mighty tail!", FALSE, ch, 0, 0, TO_ROOM);
      SweepAttack(ch);
      return TRUE;
    }
    /* failing all that, they do nothing special 3 in 4, else they breathe, only real dragons can
       breathe here, other races need a breath_weapon or breath_attack proc. */
    if (GET_RACE(ch) != RACE_DRAGON)
      return FALSE;

    i = 0;
    if (isname("gold", GET_NAME(ch)))
      if (number(0, 1) == 0)
	i = 1;			/* fire */
      else
	i = 5;			/* gas */
    if (isname("brass", GET_NAME(ch)))	/* gas: sleep or fear */
      if (number(0, 1) == 0)
	i = 6;			/* sleep gas */
      else
	i = 7;			/* fear gas */
    if (isname("bronze", GET_NAME(ch)))
      if (number(0, 1) == 0)
	i = 2;			/* lightning */
      else
	i = 7;			/* repulsion gas */
    if (isname("silver", GET_NAME(ch)))
      if (number(0, 1) == 0)
	i = 3;			/* cold */
      else
	i = 8;			/* para gas */
    if (isname("copper", GET_NAME(ch)))
      if (number(0, 1) == 0)
	i = 4;			/* acid */
      else
	i = 5;			/* gas */
    if (isname("red", GET_NAME(ch)))
      i = 1;			/* fire */
    if (isname("blue", GET_NAME(ch)))
      i = 2;			/* lightning */
    if (isname("white", GET_NAME(ch)))
      i = 3;			/* cold */
    if (isname("black", GET_NAME(ch)))
      i = 4;			/* acid */
    if (isname("green", GET_NAME(ch)))
      i = 5;			/* gas */
    if (i == 0)
      i = number(1, 5);

    switch (i) {
    case 1:
      act("$n breathes fire!", 1, ch, 0, 0, TO_ROOM);
      funct = spell_fire_breath;
      break;
    case 2:
      act("$n breathes lightning!", 1, ch, 0, 0, TO_ROOM);
      funct = spell_lightning_breath;
      break;
    case 3:
      act("$n breathes frost!", 1, ch, 0, 0, TO_ROOM);
      funct = spell_frost_breath;
      break;
    case 4:
      act("$n breathes acid!", 1, ch, 0, 0, TO_ROOM);
      funct = spell_acid_breath;
      break;
    case 5:
      act("$n breathes poison gas!", 1, ch, 0, 0, TO_ROOM);
      funct = spell_gas_breath;
      break;
    case 6:
      act("$n breathes gas!", 1, ch, 0, 0, TO_ROOM);
      funct = spell_sleep;
      break;
    case 7:
      act("$n breathes gas!", 1, ch, 0, 0, TO_ROOM);
      funct = spell_fear;
      break;
    case 8:
      act("$n breathes gas!", 1, ch, 0, 0, TO_ROOM);
      funct = spell_minor_paralysis;
      break;
    }

    for (tchar1 = world[ch->in_room].people; tchar1; tchar1 = tchar2) {
      tchar2 = tchar1->next_in_room;
      /* dragon breath targets ALL in room, except other dragons */

      if (AreaAffectCheck(ch, tchar1))
	funct(GET_LEVEL(ch), ch, tchar1, 0);
    }
    return TRUE;
  }
  /* End of normal dragons, beginning of queen bitch... */
  if (awe) {
    act("$n&N &+RROARS, filling your heart with sheer terror!", TRUE, ch, 0, 0, TO_ROOM);
    for (tchar1 = world[ch->in_room].people; tchar1; tchar1 = tchar2) {
      tchar2 = tchar1->next_in_room;
      if ((GET_RACE(tchar1) != RACE_DRAGON)) {
	if (GET_LEVEL(tchar1) < 41)	/* 40 and below, see ya... */
	  do_flee(tchar1, "", 0, 0);
	else if (GET_LEVEL(tchar1) < 46) {	/* 41-45, slight chance */
	  if (!makes_save(tchar1, SAVING_PARA, -5))
	    do_flee(tchar1, "", 0, 0);
	} else if (GET_LEVEL(tchar1) < 50) {	/* 46-49, good chance of staying */
	  if (!makes_save(tchar1, SAVING_PARA, -2))
	    do_flee(tchar1, "", 0, 0);
	}
      }
      if (ch->in_room != tchar1->in_room)
	if (IS_FIGHTING(tchar1))
	  stop_fighting(tchar1);
    }
    return TRUE;
  }
  /* End of fear */

  /* Attacks. 5 heads; breath, cast or bite.
     1 tail; sweeps all, slaps one, or backstabs one.
     2 claws; handled by MobCombat().
   */

  /* Tail... */
  switch (number(1, 3)) {
  case 1:			/* Knock em all on their ass */
    act("&+LTiamat lashes out with her mighty tail!", FALSE, ch, 0, 0, TO_ROOM);
    for (vict = world[ch->in_room].people; vict; vict = tchar1) {
      tchar1 = vict->next_in_room;
      if ((GET_RACE(vict) != RACE_DRAGON) &&
	  IS_FIGHTING(vict) && (FIGHTING(vict) == ch)) {
	if (number(0, 25) > GET_DEX(ch)) {
	  SET_POS(vict, POS_SITTING + GET_STAT(vict));
	  WAIT_STATE(vict, PULSE_VIOLENCE * 2);
	  act("&+LThe powerful sweep sends you crashing to the ground!", FALSE, vict, 0, 0, TO_CHAR);
	  act("$n&+L crashes to the ground!", FALSE, vict, 0, 0, TO_ROOM);
	  alter_hit(vict, ((dice(2, 10)) + 5), -1, 0);
	  if (GET_HIT(vict) < -10) {
	    SuddenDeath(vict, ch, "Tiamat's tail");
	    continue;
	  }
	} else
	  act("&+LYou nimbly dodge the sweep!", FALSE, vict, 0, 0, TO_CHAR);
      }
      update_pos(vict);
      if (GET_STAT(vict) == STAT_DEAD)
	SuddenDeath(vict, ch, "tiamat's tail");
    }
    break;
  case 2:
    if (!(vict = char_in_room(ch->in_room)))	/* Bitch slap someone */
      vict = FIGHTING(ch);
    if (vict && (GET_RACE(vict) != RACE_DRAGON) &&
	IS_FIGHTING(vict) && (FIGHTING(vict) == ch)) {
      if (!StatSave(vict, APPLY_DEX, -2)) {
	SET_POS(vict, POS_SITTING + GET_STAT(vict));
	WAIT_STATE(vict, PULSE_VIOLENCE * 2);
	act("&+LHer immense tail broadsides you in the face!", FALSE, vict, 0, 0, TO_CHAR);
	act("$n&+L is slapped by Tiamat's tail, and crashes to the ground!", FALSE, vict, 0, 0, TO_ROOM);
	alter_hit(vict, ((dice(5, 10)) + 5), -1, 0);
	if (GET_HIT(vict) < -10) {
	  SuddenDeath(vict, ch, "Tiamat's tail");
	  break;
	}
      } else if (vict) {
	act("&+LYou nimbly jump $n's tail sweep!", FALSE, ch, 0, vict, TO_VICT);
	act("$N &+Lnimbly jumps $n's fierce tail sweep!", TRUE, ch, 0, vict, TO_NOTVICT);
      }
      update_pos(vict);
      if (GET_STAT(vict) == STAT_DEAD)
	SuddenDeath(vict, ch, "tiamat's tail");
    }
    break;
  case 3:
  default:
    if (!(vict = char_in_room(ch->in_room)))	/* Bitch stab someone */
      vict = FIGHTING(ch);
    if (vict && (GET_RACE(vict) != RACE_DRAGON) &&
	IS_FIGHTING(vict) && (FIGHTING(vict) == ch)) {
      act("&+RHer tail lashes out, stabbing you deep in the back!", FALSE, vict, 0, 0, TO_CHAR);
      act("$n makes a strange sound, as &+LTiamats&N tail stabs $s in the back!", 1, vict, 0, 0, TO_ROOM);
      spell_poison(59, ch, vict, NULL);
      alter_hit(vict, (dice(10, 10)), -1, 0);
      if (GET_HIT(vict) < -10) {
	SuddenDeath(vict, ch, "Poisoned by Tiamat");
	break;
      }
      update_pos(vict);
      if (GET_STAT(vict) == STAT_DEAD)
	SuddenDeath(vict, ch, "tiamat's tail");
    }
    break;
  }

  /* End of tail */

  for (i = 0; i < 5; i++) {	/* Loop through the 5 heads */
    switch (number(1, 2)) {
    case 1:			/* Bite someone */
      if (!(vict = char_in_room(ch->in_room)))
	vict = FIGHTING(ch);
      if (vict) {
	switch (i) {
	case 1:		// These used to be type_tiamat, dunno if spell_ works

	  attacktype = SPELL_FIRE_BREATH;
	  act("&+RFire courses through your blood, as she bites deep!", FALSE, vict, 0, 0, TO_CHAR);
	  break;
	case 2:
	  attacktype = SPELL_LIGHTNING_BREATH;
	  act("&+BYour heart skips a beat or three, as her mighty teeth clamp down upon your arm!", FALSE, vict, 0, 0, TO_CHAR);
	  break;
	case 3:
	  attacktype = SPELL_FROST_BREATH;
	  act("&+WShe rips into your skin, sending shivers of intense cold through your body!", FALSE, vict, 0, 0, TO_CHAR);
	  break;
	case 4:
	  attacktype = SPELL_ACID_BREATH;
	  act("&+LShe lashes out at you, the acid from her fangs burning immensly!", FALSE, vict, 0, 0, TO_CHAR);
	  break;
	case 5:
	  attacktype = SPELL_GAS_BREATH;
	  act("&+GYou feel a wave of poisonous nausea, as her teeth sink deep!", FALSE, vict, 0, 0, TO_CHAR);
	  break;
	}
	damage(ch, vict, dice(6, 6) + 30, attacktype);
      }
      break;

    case 2:
      switch (i) {
      case 1:
	act("&+LTiamat's&N&+R red head breathes fire!", 1, ch, 0, 0, TO_ROOM);
	funct = spell_fire_breath;
	break;
      case 2:
	act("&+LTiamat's&N&+B blue head breathes &=LBelectricity!", 1, ch, 0, 0, TO_ROOM);
	funct = spell_lightning_breath;
	break;
      case 3:
	act("&+LTiamat's&N&+W white head breathes frost!", 1, ch, 0, 0, TO_ROOM);
	funct = spell_frost_breath;
	break;
      case 4:
	act("&+LTiamat's black head breathes acid!", 1, ch, 0, 0, TO_ROOM);
	funct = spell_acid_breath;
	break;
      case 5:
      default:
	act("&+LTiamat's&N&+g green head breathes poison gas!", 1, ch, 0, 0, TO_ROOM);
	funct = spell_gas_breath;
	break;
      }
      for (vict = world[ch->in_room].people; vict; vict = tchar1) {
	tchar1 = vict->next_in_room;
	if (!((GET_RACE(vict) == RACE_DRAGON) && (FIGHTING(vict) != ch))) {
	  funct(59, ch, vict, 0);
	  if (GET_STAT(vict) != STAT_DEAD) {
	    switch (i) {
	    case 1:
	      if (IS_AFFECTED(vict, AFF_BARKSKIN)) {
		act("&+RThe burst of flame causes your bark-like skin to catch fire!",
		    0, vict, 0, 0, TO_CHAR);
		act("$n &+Rcatches on fire!", 1, vict, 0, 0, TO_ROOM);
		alter_hit(vict, (dice(1, 20)), -1, 0);
		affect_from_char(vict, SPELL_BARKSKIN);
		if (GET_HIT(vict) < -10) {
		  SuddenDeath(vict, ch, "Roasted by Tiamat");
		  continue;
		}
	      }
	      break;
	    case 2:
	      if (IS_AFFECTED(vict, AFF_HASTE)) {
		act("&+BBetween your spell of haste, and &N&+LTiamats&N&+B electrical discharge, your heart cracks under the strain!", 0, vict, 0, 0, TO_CHAR);
		act("$n's&+B face turns a shade of blue, as $s heart stops for a moment!",
		    1, vict, 0, 0, TO_ROOM);
		alter_hit(vict, (dice(1, 20)), -1, 0);
		if (affected_by_spell(vict, SPELL_HASTE))
		  affect_from_char(vict, SPELL_HASTE);
		if (GET_HIT(vict) < -10) {
		  SuddenDeath(vict, ch, "Tiamat's heart attack");
		  continue;
		}
	      }
	      break;
	    case 3:
	      if (IS_AFFECTED(vict, AFF_STONE_SKIN)) {
		act("&+WThe intense coldness cracks your stone-like skin!", 0, vict, 0, 0, TO_CHAR);
		act("$n &+Wscreams in pain as $s stone-like skin cracks under the intense cold!",
		    0, vict, 0, 0, TO_ROOM);
		affect_from_char(vict, SPELL_STONE_SKIN);
		alter_hit(vict, (dice(1, 20)), -1, 0);
		if (GET_HIT(vict) < -10) {
		  SuddenDeath(vict, ch, "Tiamat's chisel");
		  continue;
		}
	      }
	      break;
	    case 4:
	      break;
	    case 5:
	      break;
	    }
	    update_pos(vict);
	    if (GET_STAT(vict) == STAT_DEAD)
	      SuddenDeath(vict, ch, "tiamat's breath special");
	  }
	}
      }
    }
  }
  return TRUE;
}

bool
DemonCombat(struct char_data * ch)
{
  if (CastClericSpell(ch))
    return TRUE;
  else
    return FALSE;
}


void GhostFearEffect(struct char_data *ch)
{
  struct char_data *tch;

  /* what this does is.. go thru list of people in room, and if
     they don't save, and they are pcs, _and_ not clerics (>10lvl)
     the fellow gets screwed 'a bit'. :-) */
  /* over-all, meaning of this is triple-fold; first, it creates market
     for potions. second, undead get nasty imago. third, it makes people
     consider once or twice before intentionally charing a ghost. */

  for (tch = world[ch->in_room].people; tch; tch = tch->next_in_room)
    if (IS_PC(tch) && (GET_CLASS(tch) != CLASS_CLERIC || GET_LEVEL(tch) <= 10))
      if (!makes_save(ch, SAVING_SPELL, 0) && CAN_SEE(tch, ch)) {
	act("$N is gripped by great fear after seeing $n's visage!", TRUE, ch, 0, tch, TO_NOTVICT);
	act("$N is gripped by great fear after seeing your visage!", TRUE, ch, 0, tch, TO_CHAR);
	act("You are gripped by great fear after seeing $n's visage!", TRUE, ch, 0, tch, TO_VICT);
	send_to_char("You feel older..\n", tch);
	AgeChar(tch, 10);
      }
}
int UndeadCombat(struct char_data *ch)
{
  /* this fellows undead.. undead do wicked things. */
  /*
     among 'features' we have..
     - aging effect of ghosts
     - biting of vampires
     - vampires' gaseous form
   */
  struct char_data *victim = FIGHTING(ch);

  if (ch->in_room < 0)
    return TRUE;

  if (GET_RACE(ch) == RACE_GHOST && number(1, 1000) <= GET_LEVEL(ch))
    GhostFearEffect(ch);

  if (GET_RACE(ch) == RACE_VAMPIRE) {
    /*
       vampires are smart.. their ability to shapeshift makes them..
       interesting.. foes. _NEVER_ place vampire as sentinel mob to
       block something tho. when fight goes awry, they flee like
       hell, sentinel or not.

       tactic of the day:
       - bite occasionally (con + dex + hp drain, if reaches 0, something
       .. fascinating.. may occur)
       - if hps < max/3, flee.
       - if hps < max/4, change into wraithform and flee
     */
    if (number(1, 300) <= GET_LEVEL(ch) && victim) {
      act("$n bites savagely $N's neck!", TRUE, ch, 0, victim, TO_NOTVICT);
      act("You bite savagely $N's neck!", TRUE, ch, 0, victim, TO_CHAR);
      act("$n bites savagely at YOUR NECK!", TRUE, ch, 0, victim, TO_VICT);
      if (!damage(ch, victim, dice(2, GET_LEVEL(ch)), SPELL_FEAR)) {
	send_to_char("You are stunned and unable to defend yourself properly!\n", victim);
	/* ok, now he's .. bit .. damaged. but more ofcourse occurs! *cackle* */

#if 0				/* disabled, adding affects without a skill to reference them is bad. JAB */
	/* con + dex + hpmax drain, remember? *EVIL GRIN* */
	bzero(&af, sizeof(af));
	af.duration = BOUNDED(4, GET_LEVEL(ch), 72);
	af.location = APPLY_CON;
	af.modifier = 0 - MAX(1, GET_LEVEL(ch) / 10);
	affect_join(victim, &af, TRUE, FALSE);
	af.location = APPLY_DEX;
	affect_join(victim, &af, TRUE, FALSE);
	af.modifier = af.modifier * 10;
	af.location = APPLY_HIT;
	affect_join(victim, &af, TRUE, FALSE);
#endif
	WAIT_STATE(victim, 4 * PULSE_VIOLENCE);
      }
      WAIT_STATE(ch, 2 * PULSE_VIOLENCE);
      return TRUE;
    }
#if 0
    if (GET_HIT(ch) < (GET_MAX_HIT(ch) / 4) && number(0, 1)) {
      /* wraithform is .. a kludge. :-) */
      spell_wraithform(51, ch, ch, 0);
      WAIT_STATE(ch, 20 * PULSE_VIOLENCE);
      return TRUE;
    }
#endif
  }
  return FALSE;
}

#endif

  /* This handles all types of mob combat, from starting a fight, to
   * a running battle. Use new_vict to begin a fight, leave it null
   * otherwise. From here, we will break out into race/class specifics.
   */

void MobCombat(struct char_data *ch, struct char_data *new_vict)
{
  struct char_data *tch;
  int i;

  if (IS_PC(ch))
    return;
/*
   if ( IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)
   || IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS))
   return;
 */
  if (!FIGHTING(ch))
    if (new_vict)
      set_fighting(ch, new_vict);

  /* mounts flee, unless they are battle trained */
  if (IS_SET(MOB_FLAGS(ch), MOB_MOUNTABLE) && NumAttackers(ch)
      && (!ch->master || (GET_CLASS(ch->master) != CLASS_PALADIN))) {
    do_flee(ch, "", 0, 0);
    return;
  }
  if (GET_POS(ch) < POS_STANDING) {
    send_to_char("Hmm, I was not ready. Attempting to stand now.\r\n", ch);
    do_stand(ch, "", 0, 0);
    return;
  }
  /* Always face your opponent */
  if (FIGHTING(ch))
    SET_FACING(ch, angleBetweenCharAndVict(ch, FIGHTING(ch)));

  /* do we need to check specials here as well as in the combat event? */
//  if (!no_specials)
  //      if (GET_MOB_SPEC(ch) != NULL)
  //      if (GET_MOB_SPEC(ch) (ch, ch, 0, "", "", PROC_COMBAT);
  //        return;

  if (!MIN_POS(ch, POS_STANDING + STAT_NORMAL))
    return;			/* not much we can do, if they can't stand */

  /* Decide who we should be fighting, and then decide if we should switch to
   * them or not.
   */
  if (!new_vict && (number(1, 50) <= (GET_INT(ch) + GET_LEVEL(ch))) && !GET_ACTION(ch) && !IS_ANIMAL(ch)) {
    tch = PickTarget(ch);
    if (tch && (tch != FIGHTING(ch))) {
      stop_fighting(ch);
      MobCombat(ch, tch);	/* recursive call, sets new vict */
      return;
    }
    if (!FIGHTING(ch))
      return;
    /* Not fighting somehow, so pick someone */
  } else if (!new_vict && !FIGHTING(ch)) {
    tch = PickTarget(ch);
    if (!tch)
      return;
    if (FIGHTING(ch))
      stop_fighting(ch);
    MobCombat(ch, tch);		/* recursive call, sets new vict */
  }
  /* Decide if we still want to be in this fight */

  if (MOB_FLAGGED(ch, MOB_WIMPY) && (GET_HIT(ch) < (GET_MAX_HIT(ch) / 4))) {
    send_to_char("Valor gets the better of you, and you attempt to retreat the area.\r\n", ch);
    do_flee(ch, "", 0, 0);
  }
  /* This is the main section. We should have a target, and we are battle ready. */
  /* From here, we will drop out to the various sub routines */

  if (GET_RACE(ch) == RACE_RUST_MONSTER)
    if (RustMonsterCombat(ch))
      return;

//  if ((GET_RACE(ch) == RACE_DRAGON) || (GET_RACE(ch) == RACE_DRAGONKIN) || (GET_RACE(ch) == RACE_REPTILE))
  //    if (DragonCombat(ch, FALSE))
  //      return;

//  if ((GET_RACE(ch) == RACE_DEMON) || (GET_RACE(ch) == RACE_DEVIL))
  //   if (DemonCombat(ch))
  //    return;

//  if (IS_UNDEAD(ch))
  //   if (UndeadCombat(ch))
  //    return;

  /* below here is primarily humanoid attack forms, although a few others may
   * be allowed to use them, so we don't make that a requirement.
   */

  /* First thing we do, is heal ourselves if we know how */
  if ((GET_MAX_HIT(ch) / GET_HIT(ch)) > 2)
    if (CastClericSpell(ch))
      return;

  if (IS_WARRIOR(ch))
    if (MobWarrior(ch))
      return;

  if (IS_ROGUE(ch))
    if (MobThief(ch))
      return;

  /* get a bit fancy with casters */
  i = 0;
  if (IS_PRIEST(ch))
    i = 2;
  if (IS_MAGE(ch))
    i++;

  if (i == 3) {
    /* has both, pick one */
    /* lean towards cleric if hurt */
    if (!number(0, 1 + ((GET_MAX_HIT(ch) / GET_HIT(ch)) > 2) ? 1 : 0))
      i = 1;
    else
      i = 2;
  }
  switch (i) {
  case 1:
    if (CastMageSpell(ch))
      return;
    break;
  case 2:
    if (CastClericSpell(ch))
      return;
    break;
  }
  /* Well, if we made it this far, we've been lazy and haven't written a more
     appropriate function for the mob. Let's hit somehow anyways...
   */
  MobGeneric(ch);

  return;
}
