
/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <signal.h>

#include "structs.h"

#include "utils.h"
#include "db.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "spells.h"
#include "map.h"
#include "olc.h"

/* external structs */
extern struct char_data *character_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct room_data *world;
extern struct str_app_type str_app[];
extern struct char_data *mob_proto;
extern struct zone_data *zone_table;
extern struct time_info_data time_info;
extern int is_empty(int zone_nr);
extern int find_first_step(int src, int target);
extern char *dirs[];
extern char *race_types[];
extern char *rev_dir[];
ACMD(do_flee);
ACMD(do_say);
int find_eq_pos(struct char_data *ch, struct obj_data *obj, char *arg);
byte ProximityCheck(struct char_data *ch, struct obj_data *obj, struct char_data *target, int door);
void perform_put(struct char_data *ch, struct obj_data *obj, struct obj_data *cont);
void perform_wear(struct char_data *ch, struct obj_data *obj, int where);
void CheckEqWorthUsing(struct char_data *ch, struct obj_data *obj);
void HandleMobMoves(struct char_data *ch);
void move_closer(struct char_data *ch, struct char_data *vict, struct obj_data *obj);
void load_treasure_by_type(struct char_data *mob, int type, int room);
void HandleMobAggresion(struct char_data *ch);
void name_mobile(struct char_data *mob);
extern char *random_name(void);
void GreetPeople(struct char_data *ch);

void mobile_activity(int pulse)
{
  register struct char_data *ch, *next_ch, *vict;
  struct obj_data *obj, *best_obj;
  int found, max, i, n;
  static int mobs_to_run = 0;
  memory_rec *names;

  extern int no_specials;

  ACMD(do_get);

  if (++mobs_to_run > MOBILE_PERCENT)
    mobs_to_run = 1;

  for (ch = character_list, i = 1; ch; ch = next_ch, i++) {
    next_ch = ch->next;

    if (i > MOBILE_PERCENT)
      i = 1;
    if (i != mobs_to_run)
      continue;

    if (!IS_MOB(ch))
      continue;
    if (!IS_NPC(ch))
      continue;
    if (MORPH_MOB(ch))		/* how annoying to auto do shit */
      continue;
    if (zone_table[world[ch->in_room].zone].closed)
      continue;

    /* Examine call for special procedure */
    if (GET_MOB_SPEC(ch) != NULL && !no_specials)
      if ((mob_index[GET_MOB_RNUM(ch)].func) (ch, ch, 0, "", "", PROC_NORMAL))
	continue;		/* go to next char */

    /* If the mob has no specproc, do the default actions */
    if (!AWAKE(ch))
      continue;

    if (FIGHTING(ch) && !GET_ACTION(ch))	// Lost our fight_event

      set_fighting(ch, FIGHTING(ch));

    if (IS_AFFECTED(ch, AFF_FLY) && !GET_Z_CORD(ch))
      GET_Z_CORD(ch) = 1;
    if (IS_SWIMMING_RACE(ch))
      if (IS_WATER_CORD(IN_ROOM(ch), GET_X_CORD(ch), GET_Y_CORD(ch)))
	GET_Z_CORD(ch) = -1;

    /* Scavenger (picking up objects) */
    if (!MOB_FLAGGED(ch, MOB_SENTINEL) &&
	(GET_POS(ch) == POS_STANDING) &&
	(IS_HUMANOID(ch) || IS_SLIME(ch)))
      if (world[ch->in_room].contents && !number(0, 2)) {
	max = 0;
	best_obj = NULL;
	for (obj = world[ch->in_room].contents; obj; obj = obj->next_content)
	  if (CAN_GET_OBJ(ch, obj) && GET_OBJ_COST(obj) > max &&
	      GET_OBJ_WEIGHT(obj) < 15) {
	    best_obj = obj;
	    max = GET_OBJ_COST(obj);
	  }
	if (best_obj != NULL) {
	  move_closer(ch, NULL, best_obj);
	  obj_from_room(best_obj);
	  obj_to_char(best_obj, ch);
	  if (IS_HUMANOID(ch)) {
	    act("$n gets $p.", FALSE, ch, best_obj, 0, PROXIMITY);
	    CheckEqWorthUsing(ch, best_obj);
	  } else if (IS_SLIME(ch))
	    act("$n absorbs $p into its body.", FALSE, ch, best_obj, 0, PROXIMITY);
	}
      }
    /* Sort objects around */
    if (ch->carrying) {
      for (obj = ch->carrying; obj; obj = obj->next_content) {
	if (GET_OBJ_TYPE(obj) == ITEM_CONTAINER)
	  CheckEqWorthUsing(ch, obj);
      }
      for (n = 0; n < NUM_WEARS; n++)
	if (GET_EQ(ch, n) && GET_OBJ_TYPE(GET_EQ(ch, n)) == ITEM_CONTAINER)
	  CheckEqWorthUsing(ch, GET_EQ(ch, n));
    }
    /* Mob Movement */
    if (!GET_MOB_MOVE(ch) && !FIGHTING(ch))
      HandleMobMoves(ch);

    /* Aggressive Mobs */
    HandleMobAggresion(ch);

    /* Mob Memory */
    if (MOB_FLAGGED(ch, MOB_MEMORY) && MEMORY(ch)) {
      found = FALSE;
      for (vict = world[ch->in_room].people; vict && !found; vict = vict->next_in_room) {
	if (IS_NPC(vict) || !CAN_SEE(ch, vict) || PRF_FLAGGED(vict, PRF_NOHASSLE))
	  continue;
	for (names = MEMORY(ch); names && !found; names = names->next)
	  if (names->id == GET_IDNUM(vict)) {
	    found = TRUE;
	    if (GET_HIT(ch) <= GET_MAX_HIT(ch) / 4)
	      do_flee(ch, "", 0, 0);
	    else if (ProximityCheck(ch, NULL, vict, -1)) {
	      act("'Hey!  You're the fiend that attacked me!!!', exclaims $n.",
		  FALSE, ch, 0, 0, PROXIMITY);
	      hit(ch, vict, TYPE_UNDEFINED);
	    } else {
	      act("$n appears to notice $N and moves towards $M.", TRUE, ch, 0, vict, PROXIMITY);
	      HUNTING(ch) = vict;
	    }
	  }
      }
    }
    /* Helper Mobs */
    if (MOB_FLAGGED(ch, MOB_HELPER)) {
      found = FALSE;
      for (vict = world[ch->in_room].people; vict && !found; vict = vict->next_in_room)
	if (ch != vict && IS_NPC(vict) && FIGHTING(vict) &&
	    IS_PC(FIGHTING(vict)) && ch != FIGHTING(vict)) {
	  act("$n jumps to the aid of $N!", FALSE, ch, 0, vict, PROXIMITY);
	  hit(ch, FIGHTING(vict), TYPE_UNDEFINED);
	  found = TRUE;
	}
      return;
    } else {
      found = FALSE;
      for (vict = world[ch->in_room].people; vict && !found; vict = vict->next_in_room)
	if (ch != vict && IS_NPC(vict) && FIGHTING(vict) &&
	    !IS_NPC(FIGHTING(vict)) && ch != FIGHTING(vict) &&
	    GET_MOB_VNUM(ch) == GET_MOB_VNUM(vict)) {
	  act("$n jumps to the aid of $N!", FALSE, ch, 0, vict, PROXIMITY);
	  hit(ch, FIGHTING(vict), TYPE_UNDEFINED);
	  found = TRUE;
	}
      return;
    }

    /* friendly townsfolks */
    if (IS_CITIZEN(ch))
      GreetPeople(ch);

    /* Add new mobile actions here */

  }				/* end for() */
}



/* Mob Memory Routines */

/* make ch remember victim */
void remember(struct char_data *ch, struct char_data *victim)
{
  memory_rec *tmp;
  bool present = FALSE;

  if (!IS_NPC(ch) || IS_NPC(victim) || (GET_LEVEL(victim) >= LVL_IMMORT))
    return;

  for (tmp = MEMORY(ch); tmp && !present; tmp = tmp->next)
    if (tmp->id == GET_IDNUM(victim))
      present = TRUE;

  if (!present) {
    CREATE(tmp, memory_rec, 1);
    tmp->next = MEMORY(ch);
    tmp->id = GET_IDNUM(victim);
    tmp->name = GET_NAME(victim);
    MEMORY(ch) = tmp;
  }
}


/* make ch forget victim */
void forget(struct char_data *ch, struct char_data *victim)
{
  memory_rec *curr, *prev = NULL;

  if (!(curr = MEMORY(ch)))
    return;

  while ((curr && curr->id != GET_IDNUM(victim)) ||
	 (curr && curr->name != GET_NAME(victim))) {
    prev = curr;
    curr = curr->next;
  }

  if (!curr)
    return;			/* person wasn't there at all. */

  if (curr == MEMORY(ch))
    MEMORY(ch) = curr->next;
  else
    prev->next = curr->next;

  FREE(curr);
}


/* erase ch's memory */
void clearMemory(struct char_data *ch)
{
  memory_rec *curr, *next;

  curr = MEMORY(ch);

  while (curr) {
    next = curr->next;
    FREE(curr);
    curr = next;
  }

  MEMORY(ch) = NULL;
}

/*
 * To put it simply, this routine does a couple of things. first, it does
 * check on if eq worn is crappier than some item in inventory. If not,
 * it'll try to store stuff in a container which is in inventory as well.
 * If not possible, it lurks. Solution about this needs to be found,
 * mayhap just cold-bloodedly make it junk stuff? hmm. Else mobs will be
 * fulla eq before too long, and that is _NOT_ how it should be.
 * Ha, new cleverness _was_ invented: It won't process new items more than
 * once, but the problem of stuff still lurking in the inventory needs to
 * be fixed perhaps one day. 
 * Damn, I'm clever. Now it bypasses some dumbass checks, and loots stuff
 * nastier than ever! CACKLE. :) -Still the same
 */


int RateObject(struct char_data *ch, int a, struct obj_data *obj)
{
  int manap, damp, hitp, value, tmp;

  value = 0;
  damp = 100;
  hitp = 75;
  manap = 0;

  if (obj == NULL)
    return -1;
  if (IS_MAGE(ch))
    manap += 75;
  if (IS_PRIEST(ch))
    manap += 75;
  if (IS_WARRIOR(ch) || IS_ROGUE(ch)) {
    hitp += 75;
    damp += 50;
  }
  /*
   * Ok, now all should be _at_max_ 150, and at min, 0.
   */

  if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_NODROP))
    value -= 350;

  if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_TRANSIENT))
    value -= 500;

  if ((GET_OBJ_TYPE(obj) != ITEM_QUIVER) && (GET_OBJ_TYPE(obj) != ITEM_WORN) &&
      (GET_OBJ_TYPE(obj) < ITEM_SCROLL || GET_OBJ_TYPE(obj) > ITEM_FIREWEAPON))
    return value;		/*
				 * Practically, 'toy' items that are
				 * nodrop won't be ever used
				 */
  switch (GET_OBJ_TYPE(obj)) {
    /*
     * Mob can presently only value armor, and weapon(s). Other eq's
     * value comes to mob from their +dam/+tohit/+mana
     */
  case ITEM_LIGHT:
    if (CAN_SEE(ch, ch))
      value += 100;
    break;
#if 0
  case ITEM_ARMOR:
    if (GET_AC(ch) > -100)
      value += 5 * GET_OBJ_VAL(obj, 0);
    break;
#endif
  case ITEM_WORN:
    /* dumb, but I can see no easy way to force them to wear things */
    value += 50;
    break;
  case ITEM_WEAPON:
    value += (GET_OBJ_VAL(obj, 1) * GET_OBJ_VAL(obj, 2)) * damp / 10;
    break;
  default:
    break;
  }
  for (tmp = 0; tmp < MAX_OBJ_AFFECT; tmp++) {
    switch (obj->affected[tmp].location) {
    case APPLY_DAMROLL:
      value += obj->affected[tmp].modifier * damp / 10;
      break;
    case APPLY_HITROLL:
      value += obj->affected[tmp].modifier * hitp / 10;
      break;
#if 0
    case APPLY_ARMOR:
      if (GET_AC(ch) > -100)
	value += (0 - obj->affected[tmp].modifier * 5);
      break;
#endif
    case APPLY_STR:
    case APPLY_DEX:
    case APPLY_INT:
    case APPLY_WIS:
    case APPLY_CON:
      if (GET_LEVEL(ch) <= 50)
	value += obj->affected[tmp].modifier * 5;
      break;

      /*
       * This apply_hit thing rules: street cleaners etc will prefer
       * +hitpoint stuff, other mobs will go for nastier +dam etc.
       * :)
       */
    case APPLY_HIT:
      value += 400 * obj->affected[tmp].modifier / GET_MAX_HIT(ch);
      break;
    case APPLY_SAVING_PARA:
    case APPLY_SAVING_ROD:
    case APPLY_SAVING_PETRI:
    case APPLY_SAVING_BREATH:
    case APPLY_SAVING_SPELL:
      value += (0 - 30) * obj->affected[tmp].modifier;
    default:
      break;
    }
  }
  if (IS_ROGUE(ch) && (GET_OBJ_TYPE(obj) == ITEM_WEAPON) && (GET_OBJ_VAL(obj, 3) == 11))
    value = value * 3 / 2;

  value = MAX(1, value);	/* avoid division by zero */
  value = value * 100 / number(90, 110);
  return value;
}

int AlignRestriction(struct char_data *ch, struct obj_data *obj)
{
  if (IS_EVIL(ch) && IS_SET(GET_OBJ_EXTRA(obj), ITEM_ANTI_EVIL))
    return 1;
  if (IS_GOOD(ch) && IS_SET(GET_OBJ_EXTRA(obj), ITEM_ANTI_GOOD))
    return 1;
  if (!IS_GOOD(ch) && !IS_EVIL(ch) && IS_SET(GET_OBJ_EXTRA(obj),
					     ITEM_ANTI_NEUTRAL))
    return 1;
  return 0;
}

int IsBetterObject(struct char_data *ch, struct obj_data *obj, int foo)
{
  int a = 0, l;
  struct obj_data *foob = NULL, *foo2 = NULL;

  if (CAN_WEAR(obj, find_eq_pos(ch, obj, 0)) && !AlignRestriction(ch, obj)) {
    if (ch->equipment[find_eq_pos(ch, obj, 0)] != NULL) {
      l = find_eq_pos(ch, obj, 0);
      foob = ch->equipment[l];
      if ((RateObject(ch, a, obj) >= RateObject(ch, a, foob)) &&
	  (GET_OBJ_RNUM(obj) != GET_OBJ_RNUM(foob)) &&
	  !IS_SET(GET_OBJ_EXTRA(foob), ITEM_NODROP)) {
	act("You stop using $p.", FALSE, ch, foob, 0, TO_CHAR);
	act("$n stops using $p.", TRUE, ch, foob, 0, TO_ROOM);
	foo2 = unequip_char(ch, l);
	obj_to_char(foo2, ch);
	perform_wear(ch, obj, find_eq_pos(ch, obj, 0));
	IsBetterObject(ch, foo2, a + 1);	/*
						 * Nifty recursive
						 * call to make of
						 * use item if it
						 * can be used
						 * elsewhere
						 */
	return 1;
      }
    } else {			/* if (RateObject(ch, a, obj) >= 0) */
      perform_wear(ch, obj, find_eq_pos(ch, obj, 0));
      return 1;
    }
  }
  return 0;
}

void CheckEqWorthUsing(struct char_data *ch, struct obj_data *obj)
{
  struct obj_data *ob = NULL, *ob2 = NULL;

  if (!obj || !ch)
    return;

  if (IS_ANIMAL(ch))
    return;

  if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_NODROP)) {
    /*
     * If a cleric, let's just simulate remove curse here - nothing as
     * complicated the real thing, just remove the flag + continue,
     */
    if (IS_PRIEST(ch))
      REMOVE_BIT(GET_OBJ_EXTRA(obj), ITEM_NODROP);
    return;
  }
  /*
   * Keep containers around, for our mobs 'collections'..
   * muhahahahahaaa!
   */
  if (GET_OBJ_TYPE(obj) == ITEM_CONTAINER) {
    /* Just for sake of .. fun .., mobs archive stuff in containers.  */
    for (ob = ch->carrying; ob; ob = ob2) {
      ob2 = ob->next_content;
      if (!IS_CONTAINER(ob) && obj)
	/* logic so we dont spam ourselves with errors */
	if (GET_OBJ_WEIGHT(obj) + GET_OBJ_WEIGHT(ob) > GET_OBJ_VAL(obj, 0))
	  perform_put(ch, ob, obj);
    }
    return;
  }
  if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_NODROP)) {
    /*
     * If a cleric, let's just simulate remove curse here - nothing as
     * complicated the real thing, just remove the flag + continue,
     */
    if (IS_PRIEST(ch))
      REMOVE_BIT(GET_OBJ_EXTRA(obj), ITEM_NODROP);
    else
      return;
  }
  if (IsBetterObject(ch, obj, 0)) {
    return;
  }
  for (ob = ch->carrying; ob; ob = ob2) {
    ob2 = ob->next_content;
    if (GET_OBJ_TYPE(ob) == ITEM_CONTAINER) {
      perform_put(ch, obj, ob);
      return;			/* problem solved, item in bag.  */
    }
  }
  if (RateObject(ch, 0, obj) <= 50) {
    extract_obj(obj);		/* didn't bag, not worth much, get rid of */
  }
}

int ItemsIn(struct obj_data *obj)
{
  struct obj_data *ob;
  int a;

  a = 0;
  for (ob = obj->contains; ob; ob = ob->next_content)
    a++;
  return a;
}


/************ This is temp stuff, to aid in converting mobs ********/
int find_mob_name(char *searchname, char *mob_name)
{
  char pattern[4096], mobile_name[4096];

  strcpy(pattern, searchname);
  strToLower(pattern);
  strcpy(mobile_name, mob_name);
  strToLower(mobile_name);

  if (strstr(mobile_name, pattern) != NULL) {
    return 1;
  }
  return 0;
}

void convert_mob_if_needed(int i)
{
  int x;
  struct char_data *mob = mob_proto + i;

  /* validate the race */
  if (GET_RACE(mob) == RACE_NONE)
    for (x = 0; x < NUM_RACES; x++) {
      strcpy(buf, race_types[x]);
      if (find_mob_name(buf, GET_NAME(mob)))
	GET_RACE(mob) = x;
    }
  /* add racial flags */
  if (GET_RACE(mob) == RACE_BIRD) {
    SET_BIT(AFF_FLAGS(mob), AFF_FLY);
    GET_Z_CORD(mob) = 1;
  }
  if (IS_SWIMMING_RACE(mob)) {
    SET_BIT(AFF3_FLAGS(mob), AFF3_SWIMMING);
    if (IS_WATER_CORD(IN_ROOM(mob), GET_X_CORD(mob), GET_Y_CORD(mob)))
      GET_Z_CORD(mob) = -1;
    SET_BIT(MOB_FLAGS(mob), MOB_STAY_SECTOR);
  }
#if 0
  /* Check valid treasure types */
  if (GET_MOB_TREASURE(mob) > 26 || GET_MOB_TREASURE(mob) < 0)
    GET_MOB_TREASURE(mob) = -1;
  load_treasure_by_type(mob, GET_MOB_TREASURE(mob), 0);

  /* positions */
  SET_POS(mob, POS_STANDING + STAT_NORMAL);
  GET_DEFAULT_POS(mob) = POS_STANDING;
#endif
  /* size */
  if (!GET_SIZE(mob))
    GET_SIZE(mob) = SIZE_SMALL;

  /* drunk? */
  if (isname("drunk", mob->player.name)) {
    GET_COND(mob, DRUNK) = number(1, 24);
  } else
    GET_COND(mob, DRUNK) = -1;

  /* Do we need a name? */
  if (isname("%rand", mob->player.name)) {
    name_mobile(mob);
  }
  /* prototypes */
  if (!GET_MOB_ORIG_VNUM(mob))
    GET_MOB_ORIG_VNUM(mob) = GET_MOB_VNUM(mob);
}

/***********  End of temp stuff ************************************/
void mobsay(struct char_data *ch, const char *msg)
{
  char Gbuf[MAX_INPUT_LENGTH];

  if (!msg || !*msg) {
    log("SYSERR: No text in mobsay()");
    raise(SIGSEGV);
  }
  if (strlen(msg) > (MAX_INPUT_LENGTH - 1)) {
    log("SYSERR: text too long in mobsay()");
    raise(SIGSEGV);
  }
  strcpy(Gbuf, msg);
  do_say(ch, Gbuf, 0, 0);
}

void mobaction(struct char_data *ch, char *string, char *action)
{
  sprintf(buf, "%s %s", action, string);
  command_interpreter(ch, buf);
}


/* called only by GreetPeople() */
void Submit(struct char_data *ch, struct char_data *t)
{
  char Gbuf[MAX_STRING_LENGTH];

  switch (number(1, 5)) {
  case 1:
    sprintf(Gbuf, "bow %s", GET_NAME(t));
    command_interpreter(ch, Gbuf);
    break;
  case 2:
    sprintf(Gbuf, "smile %s", GET_NAME(t));
    command_interpreter(ch, Gbuf);
    break;
  case 3:
    sprintf(Gbuf, "wink %s", GET_NAME(t));
    command_interpreter(ch, Gbuf);
    break;
  case 4:
    sprintf(Gbuf, "wave %s", GET_NAME(t));
    command_interpreter(ch, Gbuf);
    break;
  default:
    act("$n nods $s head at you", 0, ch, 0, t, TO_VICT);
    act("$n nods $s head at $N", 0, ch, 0, t, TO_NOTVICT);
    break;
  }
}

/* called only by GreetPeople() */
void SayNiceHello(struct char_data *ch, struct char_data *t)
{
  char Gbuf[MAX_STRING_LENGTH], Gbuf2[MAX_STRING_LENGTH];
  struct weather_data *cond;
  struct climate *clime;

  clime = &zone_table[world[ch->in_room].zone].climate;
  cond = &zone_table[world[ch->in_room].zone].conditions;

  switch (number(1, 10)) {
  case 1:
    mobsay(ch, "Greetings, adventurer");
    break;
  case 2:
    if (t->player.sex == SEX_FEMALE)
      mobsay(ch, "Good day, m'lady");
    else
      mobsay(ch, "Good day, m'lord");
    break;
  case 3:
    if (t->player.sex == SEX_FEMALE)
      mobsay(ch, "Pleasant journey, mistress");
    else
      mobsay(ch, "Pleasant journey, master");
    break;
  case 4:
    if (t->player.sex == SEX_FEMALE)
      sprintf(Gbuf, "Make way!  Make way for the lady %s!", GET_NAME(t));
    else
      sprintf(Gbuf, "Make way!  Make way for the lord %s!", GET_NAME(t));
    mobsay(ch, Gbuf);
    break;
  case 5:
    mobsay(ch, "May the prophets smile upon you.");
    break;
  case 6:
    mobsay(ch, "It is a pleasure to see you again.");
    break;
  case 7:
    mobsay(ch, "You are always welcome here, great one.");
    break;
  case 8:
    mobsay(ch, "My lord bids you greetings.");
    break;
  case 9:
    if (time_info.hours > 6 && time_info.hours < 12)
      sprintf(Gbuf, "Good morning, %s", GET_NAME(t));
    else if (time_info.hours >= 12 && time_info.hours < 20)
      sprintf(Gbuf, "Good afternoon, %s", GET_NAME(t));
    else if (time_info.hours >= 20 && time_info.hours <= 24)
      sprintf(Gbuf, "Good evening, %s", GET_NAME(t));
    else
      sprintf(Gbuf, "Up for a midnight stroll, %s?\n", GET_NAME(t));
    mobsay(ch, Gbuf);
    break;
  case 10:{
      if (time_info.hours < 6)
	strcpy(Gbuf2, "evening");
      else if (time_info.hours < 12)
	strcpy(Gbuf2, "morning");
      else if (time_info.hours < 20)
	strcpy(Gbuf2, "afternoon");
      else
	strcpy(Gbuf2, "evening");

      if (!cond->precip_rate) {
	if (cond->humidity > 25)
	  sprintf(Gbuf, "Nice %s to go for a walk, %s.", Gbuf2, GET_NAME(t));
	else if (cond->humidity)
	  sprintf(Gbuf, "Lovely weather we're having this %s, isn't it, %s.", Gbuf2, GET_NAME(t));
      } else if (cond->precip_rate) {
	if (cond->temp <= 0)
	  sprintf(Gbuf, "I hope this %s's snow clears up.. don't you %s?", Gbuf2, GET_NAME(t));
	else
	  sprintf(Gbuf, "I hope this %s's rain clears up.. don't you %s?", Gbuf2, GET_NAME(t));
      } else if (cond->temp > 95) {
	sprintf(Gbuf, "How can you be out on such a hot %s, %s!", Gbuf2, GET_NAME(t));
      } else {
	sprintf(Gbuf, "Such a pleasant %s, don't you think?", Gbuf2);
      }
      mobsay(ch, Gbuf);
    }
    break;
  }
}

/* called only by GreetPeople() */
void SayBlehHello(struct char_data *ch, struct char_data *t)
{

  switch (number(1, 10)) {
  case 1:
    act("$n looks for a second, then looks away quickly.", TRUE, ch, 0, 0, TO_ROOM);
    break;
  case 2:
    mobsay(ch, "Hello.");
    break;
  case 3:
    act("$n purposefully averts his gaze.", TRUE, ch, 0, 0, TO_ROOM);
    break;
  case 4:
    act("$n whistles softly to himself.", TRUE, ch, 0, 0, TO_ROOM);
    break;
  case 5:
    act("$n smiles warmly at you.", TRUE, ch, 0, 0, TO_ROOM);
    break;
  case 6:
    mobsay(ch, "Good day to you.");
    break;
  case 7:
    act("$n looks around for something or someone.", TRUE, ch, 0, 0, TO_ROOM);
    break;
  case 8:
    mobsay(ch, "Have you seen a small boy, around eight years old, with red hair?");
    mobsay(ch, "I can't find him anywhere!");
    break;
  default:
    break;
  }
}

void GreetPeople(struct char_data *ch)
{
  struct char_data *tch, *tch_next;

  if (!IS_PC_HUMANOID(ch))
    return;

  if (GET_MOB_SPEC_VALUE(ch, 3) >= 50) {
    for (tch = world[ch->in_room].people; tch; tch = tch_next) {
      tch_next = tch->next_in_room;
      if (IS_PC(tch)) {
	if (tch && GET_LEVEL(tch) > GET_LEVEL(ch) && number(0, 3)) {
	  if (CAN_SEE(ch, tch)) {
	    Submit(ch, tch);
	    SayNiceHello(ch, tch);
	    GET_MOB_SPEC_VALUE(ch, 0) = 0;
	  }
	  return;
	} else {
	  SayBlehHello(ch, tch);
	}
      }
    }
  } else
    GET_MOB_SPEC_VALUE(ch, 3)++;
}


void mobact_rescueHandle(struct char_data *mob, struct char_data *attacker)
{
#define comrade_inSameGroup(A, B)  ((IS_NPC(A) && IS_NPC(B) && ((A)->nr == (B)->nr)))

  int door, opp_dir, to_room;
  struct char_data *pl, *next_pl;
  char Gbuf[MAX_STRING_LENGTH];

  if ((GET_STAT(mob) < STAT_SLEEPING) || (GET_STAT(attacker) < STAT_SLEEPING))
    return;

  if (IS_FIGHTING(mob))
    return;

/*
 * if (!CHAR_IN_TOWN(mob))  act("$n screams 'HELP ME COMRADES... I AM
 * BEING ATTACKED!!'", FALSE, mob, 0, 0, TO_ROOM);
 */

  for (pl = world[mob->in_room].people; pl; pl = next_pl) {
    next_pl = pl->next_in_room;

    if (IS_PC(pl) || IS_MORPH(pl, NULL) || FIGHTING(pl) || (GET_POS(pl) < POS_STANDING))
      continue;
    if (!comrade_inSameGroup(mob, pl) || !IS_GUARD(pl) || IS_ANIMAL(pl))
      continue;

    if (pl != mob) {
      if (IS_GUARD(pl))
	sprintf(Gbuf, "Argh! Fighting right in front of me? Have you a deathwish?\r\n");
      else
	sprintf(Gbuf, "Hey! That's my pal you're messing with!\r\n");
      mobsay(pl, buf);
      hit(pl, attacker, TYPE_UNDEFINED);

      if ( /*!char_in_list(attacker) || */ (attacker->in_room != pl->in_room))
	return;
    }
  }

  for (door = 0; door < NUM_EXITS; door++) {

    if ((door == UP) || (door == DOWN))
      continue;
    if (!EXIT(mob, door))
      continue;
    to_room = EXIT(mob, door)->to_room;
    if (to_room == NOWHERE)
      continue;

    opp_dir = (int) rev_dir[door];

    if (!world[to_room].dir_option[opp_dir])
      continue;
    if (world[to_room].dir_option[opp_dir]->to_room != mob->in_room)
      continue;

    if (IS_SET(world[to_room].dir_option[opp_dir]->exit_info, EX_CLOSED))
      continue;

    for (pl = world[to_room].people; pl; pl = next_pl) {
      next_pl = pl->next_in_room;

      if (IS_PC(pl) || IS_MORPH(pl, NULL))
	continue;

      if (FIGHTING(pl) || (GET_POS(pl) != POS_STANDING) || IS_SET(MOB_FLAGS(pl), MOB_SENTINEL))
	continue;

      if (IS_GUARD(pl))
	sprintf(buf, "shout No citizen dies on my shift!\r\n");
      else
	sprintf(buf, "say Cool! A fight to the %s!\r\n", dirs[opp_dir]);

//      AddEvent(EVENT_DELAYED_COMMAND, 1, TRUE, pl, buf);
      /*      sprintf(buf2, "%s", dirs[opp_dir]); */
/*
 * AddEvent(EVENT_DELAYED_COMMAND, 2, TRUE, pl, buf2);
 */

    }
  }
}
