

/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <stdarg.h>
#include <math.h>
#include <signal.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "db.h"
#include "spells.h"
#include "handler.h"
#include "interpreter.h"


struct char_data *morph(struct char_data *ch, int rnum)
{
  struct char_data *mob;

  if (!ch || IS_NPC(ch) || MORPH_MOB(ch)) {
    log("morph: Bogus ch passed");
    raise(SIGSEGV);
  }
  mob = read_mobile(rnum, REAL);
  if (!mob) {
    vlogf("morph: Unable to load mob %d", rnum);
    raise(SIGSEGV);
  }
  /* no division by zero here! */
  if (GET_HIT(ch) < 1)
    GET_HIT(ch) = 1;
  if (GET_MAX_HIT(ch) < 1)
    GET_MAX_HIT(ch) = 1;

  /* set their hitpoints PROPORTIONALLY to what they had.. */
  GET_HIT(mob) = ((GET_HIT(ch) + 1) * 100 / GET_MAX_HIT(ch)) *
      GET_MAX_HIT(mob) / 100;

  if (ch->desc) {
    ch->desc->character = mob;
    ch->desc->original = ch;
    mob->desc = ch->desc;
    ch->desc = NULL;
  }
  ch->switched = mob;
  mob->switched = ch;

  SET_BIT(AFF3_FLAGS(ch), AFF3_POLYMORPH);

  MORPH_MOB(mob) = ch;
  /*
   * Void is probably a safe place to PC for time being... assuming no
   * idiot gods don't fuck off in there 
   */

  put_char_to_room(mob, ch->in_room, -1, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
  char_from_room(ch);
  char_to_room(ch, 0, -1);

  /* force a save at this point, simply because its a smart thing */
  Crash_crashsave(ch);

  return mob;
}

struct char_data *un_morph(struct char_data *mob)
{
  struct char_data *ch;

  if (!mob || IS_PC(mob) || !IS_MORPH(mob, NULL)) {
    log("un_morph: Bogus morph passed");
    raise(SIGSEGV);
  }
  ch = MORPH_MOB(mob);

  REMOVE_BIT(AFF3_FLAGS(ch), AFF3_POLYMORPH);
  ch->switched = NULL;
  mob->switched = NULL;

  /* the "owning" player. We use this instead of desc->original just in
   * case they are linkless */
  if (!ch) {
    log("un_morph: redundant orig pointer is null. Argh!");
    raise(SIGSEGV);
  }
  if (mob->desc) {

    /* * sanity check */
    if (ch != mob->desc->original) {
      log("un_morph: redundant original pointers don't match!");
      raise(SIGSEGV);
    }
    /* move a snoop from the morph to the owning player */
/*
   if (mob->desc->snoop_by)
   mob->desc->snoop_by->snooping = mob->desc->original;
 */
    /* desc points at the original char... */
    mob->desc->character = ch;
    /* and the character gets his/her desc back */
    ch->desc = mob->desc;
    mob->desc = NULL;
    /* clean up the desc... */
    ch->desc->original = 0;
  }
  MORPH_MOB(mob) = NULL;

  /* set their hitpoints PROPORTIONALLY to what they had.. */

  if (GET_HIT(mob) < 1)
    GET_HIT(mob) = 1;

  GET_HIT(ch) = ((GET_HIT(mob) + 1) * 100 / GET_MAX_HIT(mob)) *
      GET_MAX_HIT(ch) / 100 + 1;
  if (GET_HIT(ch) > GET_MAX_HIT(ch))
    GET_HIT(ch) = GET_MAX_HIT(ch);

  /* move the char back to their room */
  char_from_room(ch);
  put_char_to_room(ch, mob->in_room, -1, GET_X_CORD(mob), GET_Y_CORD(mob), GET_Z_CORD(mob));
  extract_char(mob);
  Crash_crashsave(ch);
  return ch;
}


const char *shape_list_animal[] =
{
  "chipmunk",
  "ibex",
  "horse",
  "hippopotamus",
  "elephant",
  "robin",
  "finch",
  "crow",
  "eagle",
  "dodo",
  "lizard",
  "snake",
  "turtle",
  "dragon",
  "trout",
  "catfish",
  "pike",
  "\n"
};

#if 0
struct shape_data_struct_animal {
  int skill;
  int vnums;
};

struct shape_data_struct_animal shape_data_animal[] =
{
  {SKILL_USAGE_SCHANGE_MAMMAL, 400},
  {SKILL_USAGE_SCHANGE_MAMMAL, 401},
  {SKILL_USAGE_SCHANGE_MAMMAL, 402},
  {SKILL_USAGE_SCHANGE_MAMMAL, 403},
  {SKILL_USAGE_SCHANGE_MAMMAL, 404},
  {SKILL_USAGE_SCHANGE_BIRD, 405},
  {SKILL_USAGE_SCHANGE_BIRD, 406},
  {SKILL_USAGE_SCHANGE_BIRD, 407},
  {SKILL_USAGE_SCHANGE_BIRD, 408},
  {SKILL_USAGE_SCHANGE_BIRD, 409},
  {SKILL_USAGE_SCHANGE_REPTILE, 410},
  {SKILL_USAGE_SCHANGE_REPTILE, 411},
  {SKILL_USAGE_SCHANGE_REPTILE, 412},
  {SKILL_USAGE_SCHANGE_REPTILE, 414},
  {SKILL_USAGE_SCHANGE_FISH, 415},
  {SKILL_USAGE_SCHANGE_FISH, 416},
  {SKILL_USAGE_SCHANGE_FISH, 417}
};

const char *shape_list[] =
{
  "bird",
  "reptile",
  "mammal",
  "fish",
  "\n"
};

struct shape_data_struct {
  int skill;
  int choices;
  int vnums[10];
};

struct shape_data_struct shape_data[] =
{
  {SKILL_USAGE_SCHANGE_BIRD, 5,
   {405, 406, 407, 408, 409, 0, 0, 0, 0, 0}
  },

  {SKILL_USAGE_SCHANGE_REPTILE, 5,
   {410, 411, 412, 413, 414, 0, 0, 0, 0, 0}
  },

  {SKILL_USAGE_SCHANGE_MAMMAL, 5,
   {400, 401, 402, 403, 404, 0, 0, 0, 0, 0}
  },

  {SKILL_USAGE_SCHANGE_FISH, 3,
   {415, 416, 417, 0, 0, 0, 0, 0, 0, 0}
  }
};

const char *disguise_list[] =
{
  "halfling",
  "gnome",
  "dwarf",
  "elf",
  "half-elf",
  "human",
  "duergar",
  "drow",
  "orc",
  "\n"
};

struct disguise_list_data_struct {
  int size;
  int race;
  const char *name[3];
};

struct disguise_list_data_struct disguise_list_data[] =
{
  {SIZE_SMALL, RACE_HALFLING,
   {"a halfling", "&+ga halfling scout&N", "&+ya halfling merchant&N"}},
  {SIZE_SMALL, RACE_GNOME,
   {"&+La gnome merchant&N", "a gnome", "&+Ra gnome peasant&N"}},
  {SIZE_MEDIUM, RACE_MOUNTAIN,
   {"a dwarf", "&+ra dwarf guard&N", "&+Ya dwarf merchant&N"}},
  {SIZE_MEDIUM, RACE_GREY,
   {"a elf", "&+Gan elven merchant&N", "&+gan elven scout&N"}},
  {SIZE_MEDIUM, RACE_HALFELF,
   {"a half-elf", "&+Ga half-elf merchant&N", "&+La half-elf guard&N"}},
  {SIZE_MEDIUM, RACE_HUMAN,
   {"a human", "&+Ca human merchant&N", "&+Ga human traveler&N"}},
  {SIZE_SMALL, RACE_DUERGAR,
   {"a duergar", "&+ra clansdwarf&N", "&+La duergar merchant&N"}},
  {SIZE_MEDIUM, RACE_DROW,
   {"a drow", "&+ma drow guard&N", "&+La drow merchant&N"}},
  {SIZE_MEDIUM, RACE_ORC,
   {"an orc", "&+yan orcich merchant&N", "&+gan orcish guard&N"}}
};

void do_disguise(struct char_data *ch, char *arg, int cmd)
{
  int skl_lvl = 0;
  int percent = 0;
  int i, the_size;
  char Gbuf1[MAX_STRING_LENGTH];
  char name[MAX_STRING_LENGTH];
  struct char_data *target = NULL;
  struct obj_data *temp;
  bool equipped;
  bool disguise_npc = FALSE;

  if (!ch)
    return;

/*  if (!IS_TRUSTED(ch)) {
   send_to_char("Not ready yet!\r\n", ch);
   return;
   } */

  if (FIGHTING(ch)) {
    send_to_char("Can't do that while fighting!\r\n", ch);
    return;
  }
  if (IS_NPC(ch))
    return;

  skl_lvl = GET_CHAR_SKILL(ch, SKILL_DISGUISE) * 2;

  if (!skl_lvl) {
    send_to_char("You don't know how to.\r\n", ch);
    return;
  }
  if (!*arg) {
    if (IS_DISGUISE(ch)) {
      send_to_char("You remove your disguise!\r\n", ch);
      IS_DISGUISE_PC(ch) = FALSE;
      IS_DISGUISE_NPC(ch) = FALSE;
      ch->disguise.name = NULL;
      ch->disguise.class = 0;
      ch->disguise.race = 0;
      ch->disguise.level = 0;
      ch->disguise.title = NULL;
      act("$n starts removing his disguise.", FALSE,
	  ch, 0, ch, TO_ROOM);
      WAIT_STATE(ch, PULSE_VIOLENCE * 3);
      disguiseEvent(ch, TRUE);
      return;
    } else {
      send_to_char("Disguise as who or what?\r\n", ch);
      return;
    }
  }
  if ((*arg) && IS_DISGUISE(ch)) {
    send_to_char("You need to remove your disguise before trying to disguise again!\r\n", ch);
    return;
  }
  one_argument(arg, name);

  i = search_block(name, disguise_list, 0);
  if (i != -1) {
    disguise_npc = TRUE;
  } else {

    target = mm_get(dead_mob_pool);
    target->only.pc = mm_get(dead_pconly_pool);

    if ((restoreCharOnly(target, skip_spaces(arg)) < 0) || !target) {
      if (target) {
	free_char(target);
	target = NULL;
      }
    }
    if (!target) {
      send_to_char("Disguise as who?\r\n", ch);
      return;
    }
    if (target == ch) {
      send_to_char("It is too hard to disguise into that loser!\r\n", ch);
      if (target)
	free_char(target);
      return;
    }
    if (!IS_TRUSTED(ch) && target && GET_LEVEL(target) > 56) {
      send_to_char("You really don't want to disguise yourself as a God!\r\n", ch);
      if (target)
	free_char(target);
      return;
    }
    if (IS_TRUSTED(ch)) {
      if (GET_LEVEL(ch) < GET_LEVEL(target)) {
	send_to_char("No you don't!\r\n", ch);
	if (target)
	  free_char(target);
	return;
      }
    }
    if (!IS_TRUSTED(ch)) {
      if (GET_RACE(target) == RACE_ILLITHID || GET_RACE(target) == RACE_CENTAUR ||
	  GET_RACE(target) == RACE_THRIKREEN || GET_RACE(target) == RACE_MINOTAUR ||
	  GET_RACE(target) == RACE_TROLL) {
	send_to_char("Cannot disguise into that.\r\n", ch);
	if (target)
	  free_char(target);
	return;
      }
    }
  }

  equipped = FALSE;

  if (!IS_TRUSTED(ch)) {
    if (!(temp = get_obj_in_list_vis(ch, "_disguise_kit_", ch->carrying))) {
      temp = ch->equipment[HOLD];
      equipped = TRUE;
      if ((temp == 0) || !isname("_disguise_kit_", temp->name)) {
	act("You need a disguise kit.", FALSE, ch, 0, 0, TO_CHAR);
	if (target)
	  free_char(target);
	return;
      }
    }
  }
  if (IS_TRUSTED(ch))
    skl_lvl = 200;

  percent = number(1, 101);

  if (target) {
    percent += 30;

    if (racewar(ch, target))
      percent += 20;

    if (GET_RACE(ch) != GET_RACE(target))
      percent += 30;
    if (GET_SEX(ch) != GET_SEX(target))
      percent += 10;
  } else {
    if (GET_RACE(ch) != disguise_list_data[i].race)
      percent += 30;
  }

  if (percent > skl_lvl) {
    send_to_char("You do a horrid job", ch);
    CharSkillGainChance(ch, SKILL_DISGUISE, 35);
    justice_witness(ch, NULL, CRIME_DISGUISE);
    WAIT_STATE(ch, PULSE_VIOLENCE * 3);
    if (!IS_TRUSTED(ch) && number(0, 1)) {
      send_to_char(", and ruin your disguise kit in the process.\r\n", ch);
      if (equipped)
	unequip_char(ch, HOLD);
      extract_obj(temp);
    } else
      send_to_char(", but manage to salvage the rest of the kit's supplies.\r\n", ch);
    if (target)
      free_char(target);

    return;
  }
  if (target)
    the_size = GET_SIZE(target);
  else
    the_size = disguise_list_data[i].size;
  if ((GET_ALT_SIZE(ch) < (the_size - 1)) && !IS_TRUSTED(ch)) {
    send_to_char("You're too small for that!\r\n", ch);
    WAIT_STATE(ch, PULSE_VIOLENCE * 2);
  } else if ((GET_ALT_SIZE(ch) > (the_size + 1)) && !IS_TRUSTED(ch)) {
    send_to_char("You're too big for that!\r\n", ch);
    WAIT_STATE(ch, PULSE_VIOLENCE * 2);
  } else {
    if (target) {
      IS_DISGUISE_PC(ch) = TRUE;
      ch->disguise.name = str_dup(GET_NAME(target));
      ch->disguise.class = GET_CLASS(target);
      ch->disguise.race = GET_RACE(target);
      ch->disguise.level = GET_LEVEL(target);
      if (GET_TITLE(target))
	ch->disguise.title = str_dup(GET_TITLE(target));
      sprintf(Gbuf1, "You disguise yourself into %s.\r\n", GET_NAME(target));
      send_to_char(Gbuf1, ch);
      sprintf(Gbuf1, "%s starts disguising into %s.", GET_NAME(ch), GET_NAME(target));
    } else {
      IS_DISGUISE_NPC(ch) = TRUE;
      ch->disguise.name = str_dup(disguise_list_data[i].name[number(0, 2)]);
      ch->disguise.race = disguise_list_data[i].race;
      sprintf(Gbuf1, "You disguise yourself into a %s.\r\n", disguise_list[i]);
      send_to_char(Gbuf1, ch);
      sprintf(Gbuf1, "%s starts disguising into a %s.", GET_NAME(ch), disguise_list[i]);
    }
    AddEvent(EVENT_DISGUISE, 7200, TRUE, ch, 0);
    SET_BIT(ch->specials.act, PLR_NOWHO);
    justice_witness(ch, NULL, CRIME_DISGUISE);
    act(Gbuf1, FALSE, ch, 0, target, TO_ROOM);
    if (!IS_TRUSTED(ch)) {
      if (equipped)
	unequip_char(ch, HOLD);
      extract_obj(temp);
    }
    CharSkillGainChance(ch, SKILL_DISGUISE, 6);
    WAIT_STATE(ch, PULSE_VIOLENCE * 5);
  }
  if (target)
    free_char(target);
  return;
}


#endif

inline int IS_MORPH(struct char_data *mob, struct obj_data *obj)
{
  if (mob && MORPH_MOB(mob))
    return TRUE;
  else if (obj && MORPH_OBJ(obj))
    return TRUE;
  else
    return FALSE;
}

void polymorph_obj(struct char_data *ch, int obj_num)
{
  struct obj_data *obj;

  obj = read_object(obj_num, REAL);
  if (!obj) {
    vlogf("polymorph_into_object(): obj %d not loadable", obj_num);
    return;
  }
  put_obj_to_room(obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
  vlogf("Load: Item polymorph is in %d", obj->in_room);
  OBJ_POLYMORPH(obj) = ch;
  CHAR_POLYMORPH_OBJ(ch) = obj;

  act("$n &+Wsuddenly warps shape, changing into &n$p&+W!", TRUE, ch, obj, 0, TO_ROOM);
  act("&+WWith an incredibly unsettling sensation, you change into &n$p&+W.", TRUE, ch, obj, 0, TO_CHAR);

  char_from_room(ch);
  char_to_room(ch, 0, -1);
}

void return_from_poly_obj(struct char_data *ch)
{
  struct obj_data *obj;

  obj = CHAR_POLYMORPH_OBJ(ch);
  vlogf("Unload: Item polymorph is in %d", obj->in_room);
  if (!OBJ_POLYMORPH(obj)) {
    send_to_char("polymorph weirdness #1.  tell a god\r\n", ch);
    if (IS_TRUSTED(ch)) {
      char_from_room(ch);
      put_char_to_room(ch, obj->in_room, -1, GET_OBJ_X_CORD(obj), GET_OBJ_Y_CORD(obj), GET_OBJ_Z_CORD(obj));
      OBJ_POLYMORPH(obj) = NULL;
      CHAR_POLYMORPH_OBJ(ch) = NULL;
      extract_obj(obj);
    }
    return;
  }
  if (obj->in_room <= NOWHERE) {
    send_to_char("damn, obj is nowhere.  you're screwed.  tell a god\r\n", ch);
    return;
  } else {
    char_from_room(ch);
    char_to_room(ch, obj->in_room, -1);

    act("$p&n suddenly changes shape, turning into $n!", FALSE, ch, 0, 0, TO_ROOM);
    send_to_char("&+WAn exceedingly queasy feeling, and you're back to normal.\r\n", ch);

    OBJ_POLYMORPH(obj) = NULL;
    CHAR_POLYMORPH_OBJ(ch) = NULL;

    extract_obj(obj);
    return;
  }
}


ACMD(do_shapechange)
{
  char mobname[MAX_INPUT_LENGTH], objname[MAX_INPUT_LENGTH];
  struct obj_data *obj;
  int mob_num = 0, j = 0, animal = FALSE, obj_num = 0;
  time_t now, tmp_time;
  char Gbuf1[MAX_STRING_LENGTH];

  if (!ch->desc)
    return;

  if (RIDING(ch)) {
    send_to_char("While mounted?  I don't think so...\r\n", ch);
    return;
  }
  if (RIDDEN_BY(ch)) {
    send_to_char("You can't change your form when you have a rider!\r\n", ch);
    return;
  }
  if (subcmd == SCMD_MOB) {
    if ((GET_CLASS(ch) != CLASS_DRUID) && !IS_TRUSTED(ch) &&
	!MORPH_MOB(ch)) {
      send_to_char("You don't know how to change your form!\r\n", ch);
      return;
    }
    one_argument(argument, mobname);

    if (!str_cmp(mobname, "me")) {
      if (!ch->desc->original) {
	if (GET_CLASS(ch) == CLASS_DRUID)
	  send_to_char("But you're already in your natural form.\r\n", ch);
	else
	  send_to_char("You don't know how!\r\n", ch);
	return;
      }
      /* can't change back when fighting a PC */
      if (FIGHTING(ch)) {
	send_to_char("You can't concentrate on that right now.\r\n", ch);
	return;
      }
      if (!MORPH_MOB(ch)) {
	send_to_char("You need to switch back using 'return' command.\r\n", ch);
	return;
      }
      if (IS_AFFECTED4(ch, AFF4_NO_UNMORPH)) {
	send_to_char("Some mysterious force prevents you from returning to your normal form.\r\n", ch);
	return;
      }
      send_to_char("You return to your original body.\r\n", ch);
      act("$n &nbegins to writhe and twist, re-shaping into $N.",
	  FALSE, ch, 0, ch->desc->original, TO_ROOM);

      un_morph(ch);

      return;
    }
    /*
     * they must return to their original form before transmuting into
     * another 
     */
    if (ch->desc->original) {
      if (MORPH_MOB(ch))
	send_to_char("Your powers are too weak in this subordinate form.\r\n", ch);
      else
	send_to_char("Note: You must \"return\" from a \"switch\" before using this skill.\r\n", ch);
      return;
    }
    /* assume form changed into will never have snooping capabilities */

    if (ch->desc->snooping) {
      send_to_char("No snooping AND shapechanging at any one time.\r\n", ch);
      return;
    }
    if (FIGHTING(ch)) {
      send_to_char("You can't muster the mental energy to do that right now.\r\n", ch);
      return;
    }
    /* if no arg, give a list of forms available, and time left for each */
#if 0
    if (!*argument) {
      now = time(0);
      for (j = 0; *shape_list[j] != '\n'; j++) {
	if (IS_FULLY_RESTED(ch, shape_data[j].skill, now)) {
	  sprintf(Gbuf1, "%-15s You feel able to assume this form.\r\n",
		  shape_list[j]);
	} else {
	  tmp_time = SKILL_HOURS_TO_REST(ch, shape_data[j].skill, now);
	  if (tmp_time >= 20) {
	    sprintf(Gbuf1, "%-15s You just returned from this form.  Your mind is still VERY weak.\r\n",
		    shape_list[j]);
	  } else if (tmp_time >= 16) {
	    sprintf(Gbuf1, "%-15s Your mind still needs A LOT of rest.\r\n",
		    shape_list[j]);
	  } else if (tmp_time >= 12) {
	    sprintf(Gbuf1, "%-15s Your mind still needs some solid rest.\r\n",
		    shape_list[j]);
	  } else if (tmp_time >= 8) {
	    sprintf(Gbuf1, "%-15s You still have some hours ahead of you.\r\n",
		    shape_list[j]);
	  } else if (tmp_time >= 4) {
	    sprintf(Gbuf1, "%-15s You're almost ready.\r\n",
		    shape_list[j]);
	  } else {
	    sprintf(Gbuf1, "%-15s You're on the brink of feeling 100%%.\r\n",
		    shape_list[j]);
	  }
	}
	send_to_char(Gbuf1, ch);
      }
      return;
    }
#endif
    if (is_number(mobname) && IS_TRUSTED(ch)) {
      mob_num = atoi(mobname);

      if (!mob_num) {
	send_to_char("What vnum?\r\n", ch);
	return;
      }
      if ((mob_num = real_mobile(mob_num)) <= 0) {
	send_to_char("Invalid vnum\r\n", ch);
	return;
      }
    }
#if 0
    else {
      int i;
      i = search_block(mobname, shape_list, 0);		/* first we check for general type (mammal etc) */
      if (i == -1) {		/* no match */
	i = search_block(mobname, shape_list_animal, 0);	/* second we check for specific type (horse etc) */
	if (i == -1) {		/* no match */
	  send_to_char("Umm.. what form are you trying to change to?\r\n", ch);
	  return;
	}
	if (check_skill_usage(ch, shape_data_animal[i].skill) == FALSE) {
	  send_to_char("Your mind needs more rest.\r\n", ch);
	  return;
	}
	mob_num = shape_data_animal[i].vnums;
	animal = TRUE;
	if ((mob_num = real_mobile0(mob_num)) == 0) {
	  send_to_char("Oops!  There is an error in the list of this type of mob!\r\n",
		       ch);
	  logit(LOG_DEBUG, "SHAPECHANGE: Error list shape data for type %s",
		mobname);
	  return;
	}
      }
      if (animal == FALSE) {
	if (check_skill_usage(ch, shape_data[i].skill) == FALSE) {
	  send_to_char("Your mind needs more rest.\r\n", ch);
	  return;
	}
	j = number(0, shape_data[i].choices - 1);
	mob_num = shape_data[i].vnums[j];
	if ((mob_num = real_mobile0(mob_num)) == 0) {
	  send_to_char("Oops!  There is an error in the list of this type of mob!\r\n",
		       ch);
	  logit(LOG_DEBUG, "SHAPECHANGE: Error list shape data for type %s",
		mobname);
	  return;
	}
      }
    }
#endif
    send_to_char("You feel yourself metamorphing into your desired new form.\r\n",
		 ch);

    if (morph(ch, mob_num)) {
      act("$N's body begins to re-form into $n.", FALSE,
	  ch->switched, 0, ch, TO_ROOM);
    } else {
      act("$N's body begins to re-form, then suddenly stops.", FALSE,
	  ch, 0, 0, TO_ROOM);
      send_to_char("You suddenly stop reforming!\r\n", ch);
    }
  } else if (subcmd == SCMD_OBJ) {

    one_argument(argument, objname);

    if (!str_cmp(objname, "me")) {
      return_from_poly_obj(ch);
      return;
    }
    if (*objname && !is_number(objname)) {
#if 0
      for (i = 0; minor_create_name_list[i].keyword[0]; i++) {
	if (isname(arg, minor_create_name_list[i].keyword)) {
	  obj_num = minor_create_name_list[i].obj_number;
	  break;
	}
      }
#endif
    } else if (is_number(objname) && IS_TRUSTED(ch)) {
      obj_num = real_object(atoi(objname));

      if (obj_num == 0) {
	send_to_char("You cannot fathom the power needed to polymorph into that.\r\n", ch);
	return;
      }
      if (CHAR_POLYMORPH_OBJ(ch)) {
	send_to_char("Strange, you already seem to be polymorphed.\r\n", ch);
	return;
      }
      polymorph_obj(ch, obj_num);
    }
  }
}
