/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "db.h"
#include "utils.h"
#include "spells.h"
#include "interpreter.h"
#include "handler.h"

int alignment_ok_race[NUM_PC_RACES][NUM_ALIGNMENT] =
{

  /*    N LG NG CG LN NN CN LE NE CE */
  /* human */
  {Y, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* half elf */
  {Y, Y, Y, Y, Y, Y, Y, N, N, N},
  /* elf */
  {Y, Y, Y, Y, Y, Y, Y, N, N, N},
  /* dwarf */
  {Y, Y, Y, Y, Y, Y, Y, N, N, N},
  /* halfling */
  {Y, Y, Y, Y, Y, Y, Y, N, N, N},
  /* gnome */
  {Y, Y, Y, Y, Y, Y, Y, N, N, N},
  /* halforc */
  {Y, N, Y, N, N, Y, Y, N, Y, N},
  /* barbarian */
  {Y, N, Y, N, N, Y, Y, N, Y, N},
  /* drow */
  {N, N, N, N, N, N, Y, Y, Y, Y},
  /* duergar */
  {N, N, N, N, N, N, Y, Y, Y, Y},
  /* ogre */
  {N, N, N, N, N, N, N, Y, Y, Y},
  /* troll */
  {N, N, N, N, N, N, N, Y, Y, Y},
  /* illithid */
  {N, N, N, N, N, N, N, Y, N, N},
  /* orc */
  {Y, N, N, N, N, Y, Y, Y, Y, Y},
  /* goblin */
  {Y, N, N, N, Y, Y, Y, Y, Y, Y},
};

const char *race_types[] =
{
  "None",
  "Human",
  "Half elf",
  "Elf",
  "Dwarf",
  "Halfling",
  "Gnome",
  "Half orc",
  "Barbarian",
  "Drow",
  "Duergar",
  "Ogre",
  "Troll",
  "Illithid",
  "Orc",
  "Goblin",

  "None16", "None17", "None18", "None19",
  "None20", "None21", "None22", "None23", "None24", "None25",
  "None26", "None27", "None28", "None29",

  "Ankheg",
  "Aarakocra",
  "Aboleth",
  "Arcane",
  "Argos",
  "Baatezu",
  "Aurumvorax",
  "Drow",
  "Banshee",
  "Basilisk",
  "Bat",
  "Bear",
  "Beetle_Giant",
  "Beholder",
  "Behir",
  "Bird",
  "Brain_Mole",
  "Brownie",
  "Bugbear",
  "Bulette",
  "Bullywug",
  "Carrion_Crawler",
  "Cat_Great",
  "Catoblepas",
  "Cave_Fisher",
  "Centaur",
  "Centipede",
  "Chimera",
  "Gorgimera",
  "Cloaker",
  "Cockatrice",
  "Couatl",
  "Crabman",
  "Crawling_Claw",
  "Crocodile",
  "Crustacean_Giant",
  "Crypt_Thing",
  "Death_Knight",
  "Deepspawn",
  "Dinosaur",
  "Displacer_Beast",
  "Dog",
  "Dolphin",
  "Doppleganger",
  "Dracolich",
  "Dragon",
  "Faerie_Dragon",
  "Firedrake",
  "Pseudodragon",
  "Dragonne",
  "Dryad",
  "Elemental_Air",
  "Elemental_Earth",
  "Elemental_Fire",
  "Elemental_Water",
  "Elephant",
  "Ettercap",
  "Eyewing",
  "Feyr",
  "Fish",
  "Frog",
  "Fungus",
  "Galeb_Duhr",
  "Gargoyle",
  "Genie",
  "Ghost",
  "Ghoul",
  "Giant",
  "Gibberling",
  "Giff",
  "Gith",
  "Githyanki",
  "Githzerai",
  "Gnoll",
  "Goblin",
  "Golem",
  "Gorgon",
  "Grell",
  "Gremlin",
  "Griffon",
  "Grimlock",
  "Grippli",
  "Hag",
  "Harpy",
  "Hatori",
  "Haunt",
  "Hell_Hound",
  "Heucuva",
  "Hippocampus",
  "Hippogriff",
  "Hobgoblin",
  "Homonculous",
  "Hook_Horror",
  "Horse",
  "Hydra",
  "Imp",
  "Insect",
  "Intellect_Devourer",
  "Invisible_Stalker",
  "Ixitxachitl",
  "Jackelwere",
  "Kenku",
  "Ki-Rin",
  "Kirre",
  "Kobold",
  "Kuo-Toa",
  "Lamia",
  "Lammasu",
  "Leech",
  "Leprechaun",
  "Leucrotta",
  "Lich",
  "Living_Wall",
  "Lizard",
  "Lizard_Man",
  "Locathah",
  "Lurker",
  "Lycanthrope",
  "Mammal",
  "Manscorpion",
  "Manticore",
  "Medusa",
  "Meadar",
  "Merman",
  "Mimic",
  "Mind_Flayer",
  "Minotaur",
  "Mist",
  "Mold",
  "Mongrelman",
  "Morkoth",
  "Muckdweller",
  "Mudman",
  "Mummy",
  "Myconid",
  "Naga",
  "Neogi",
  "Nightmare",
  "Nymph",
  "Octopus",
  "Ogre",
  "Ogre_Mage",
  "Slime",
  "Orc",
  "Otyugh",
  "Owlbear",
  "Pegasus",
  "Peryton",
  "Phantom",
  "Phoenix",
  "Piercer",
  "Plant",
  "Poltergeist",
  "Pudding",
  "Quaggoth",
  "Rakshasa",
  "Rat",
  "Remorhaz",
  "Revenant",
  "Roc",
  "Roper",
  "Rust_Monster",
  "Sahuagin",
  "Satyr",
  "Scorpion",
  "Sea_Lion",
  "Selkie",
  "Shadow",
  "Shedu",
  "Sirine",
  "Skeleton",
  "Slaad",
  "Slug",
  "Snake",
  "Spectre",
  "Sphinx",
  "Spider",
  "Sprite",
  "Squid",
  "Stirge",
  "Su_Monster",
  "Swanmay",
  "Tabaxi",
  "Tako",
  "Tanarri",
  "Tarrasque",
  "Tasloi",
  "Thought-Eater",
  "Thri-Kreen",
  "Titan",
  "Toad",
  "Treant",
  "Triton",
  "Troglodyte",
  "Troll",
  "Umber_Hulk",
  "Unicorn",
  "Urchin",
  "Vampire",
  "Wemic",
  "Whale",
  "Wight",
  "Will_O_Wisp",
  "Wolf",
  "Wolfwere",
  "Worm",
  "Wraith",
  "Wyvern",
  "Xorn",
  "Yeti",
  "Yaun-Ti",
  "Yugoloth",
  "Zaratan",
  "Zombie",
  "\n"
};

int parse_race(char arg)
{
  arg = LOWER(arg);

  switch (arg) {
  case 'a':
    return RACE_HUMAN;
  case 'b':
    return RACE_HALFELF;
  case 'c':
    return RACE_ELF;
  case 'd':
    return RACE_DWARF;
  case 'e':
    return RACE_HALFLING;
  case 'f':
    return RACE_GNOME;
  case 'g':
    return RACE_HALFORC;
  case 'h':
    return RACE_BARBARIAN;
  case 'i':
    return RACE_P_DROW;
  case 'j':
    return RACE_P_DUERGAR;
  case 'k':
    return RACE_P_OGRE;
  case 'l':
    return RACE_P_TROLL;
  case 'm':
    return RACE_P_ORC;
  case 'n':
    return RACE_P_GOBLIN;
  case 'o':
    return RACE_P_ILLITHID;
  default:
    return RACE_NONE;
  }
}
