/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <signal.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"


/*   external vars  */
extern struct room_data *world;
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct time_info_data time_info;
extern char *dirs[];
extern int rev_dir[];
extern char *activating_commands[];
extern char *coin_stringv(int amount);
ACMD(do_drop);
ACMD(do_grab);
ACMD(do_remove);
ACMD(do_action);
ACMD(do_gen_door);


/* extern functions */
extern void add_follower(struct char_data *ch, struct char_data *leader);
extern void smallmap_look(struct char_data *ch, int size, int dif_room);
extern bool transact(struct char_data *from, struct obj_data *merchandise, struct char_data *to, int value);
extern void master_parser(char *buff, struct char_data *ch, struct room_data *room, struct obj_data *obj, struct char_data *mob);
extern void mobaction(struct char_data *ch, char *string, char *action);

struct social_type {
  char *cmd;
  int next_line;
};


/* ********************************************************************
   *  Special procedures for mobiles                                     *
   ******************************************************************** */

int spell_sort_info[MAX_SPELLS + 1];

extern char *spells[];

void sort_spells(void)
{
  int a, b, tmp;

  /* initialize array */
  for (a = 1; a < MAX_SPELLS; a++)
    spell_sort_info[a] = a;

  /* Sort.  'a' starts at 1, not 0, to remove 'RESERVED' */
  for (a = 1; a < MAX_SPELLS - 1; a++)
    for (b = a + 1; b < MAX_SPELLS; b++)
      if (strcmp(spells[spell_sort_info[a]], spells[spell_sort_info[b]]) > 0) {
	tmp = spell_sort_info[a];
	spell_sort_info[a] = spell_sort_info[b];
	spell_sort_info[b] = tmp;
      }
}


char *how_good(int percent)
{
  static char buf[256];

  if (percent == 0)
    strcpy(buf, " (not learned)");
  else if (percent <= 10)
    strcpy(buf, " (awful)");
  else if (percent <= 20)
    strcpy(buf, " (bad)");
  else if (percent <= 40)
    strcpy(buf, " (poor)");
  else if (percent <= 55)
    strcpy(buf, " (average)");
  else if (percent <= 70)
    strcpy(buf, " (fair)");
  else if (percent <= 80)
    strcpy(buf, " (good)");
  else if (percent <= 85)
    strcpy(buf, " (very good)");
  else
    strcpy(buf, " (superb)");

  return (buf);
}

SPECIAL(dump)
{
  struct obj_data *k;
  int value = 0;

  for (k = world[ch->in_room].contents; k; k = world[ch->in_room].contents) {
    act("$p vanishes in a puff of smoke!", FALSE, 0, k, 0, PROXIMITY);
    extract_obj(k);
  }

  if (!CMD_IS("drop"))
    return 0;

  do_drop(ch, argument, cmd, 0);

  for (k = world[ch->in_room].contents; k; k = world[ch->in_room].contents) {
    act("$p vanishes in a puff of smoke!", FALSE, 0, k, 0, PROXIMITY);
    value += MAX(1, MIN(50, GET_OBJ_COST(k) / 10));
    extract_obj(k);
  }

  if (value) {
    act("You are awarded for outstanding performance.", FALSE, ch, 0, 0, TO_CHAR);
    act("$n has been awarded for being a good citizen.", TRUE, ch, 0, 0, PROXIMITY);

    if (GET_LEVEL(ch) < 3)
      gain_exp(ch, value);
    else
      ADD_MONEY(ch, value);
  }
  return 1;
}


SPECIAL(mayor)
{
  static char open_path[] =
  "W3a3003b33000c111d0d111Oe333333Oe22c222112212111a1S.";

  static char close_path[] =
  "W3a3003b33000c111d0d111CE333333CE22c222112212111a1S.";

  static char *path;
  static int index;
  static bool move = FALSE;

  if (!move) {
    if (time_info.hours == 6) {
      move = TRUE;
      path = open_path;
      index = 0;
    } else if (time_info.hours == 20) {
      move = TRUE;
      path = close_path;
      index = 0;
    }
  }
  if (cmd || !move || (GET_STAT(ch) < STAT_SLEEPING) ||
      IS_FIGHTING(ch))
    return FALSE;

  switch (path[index]) {
  case '0':
  case '1':
  case '2':
  case '3':
    perform_move(ch, path[index] - '0', 1);
    break;

  case 'W':
    SET_POS(ch, POS_STANDING + STAT_NORMAL);
    act("$n awakens and groans loudly.", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'S':
    SET_POS(ch, POS_PRONE + STAT_SLEEPING);
    act("$n lies down and instantly falls asleep.", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'a':
    act("$n says 'Hello Honey!'", FALSE, ch, 0, 0, PROXIMITY);
    act("$n smirks.", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'b':
    act("$n says 'What a view!  I must get something done about that dump!'",
	FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'c':
    act("$n says 'Vandals!  Youngsters nowadays have no respect for anything!'",
	FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'd':
    act("$n says 'Good day, citizens!'", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'e':
    act("$n says 'I hereby declare the bazaar open!'", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'E':
    act("$n says 'I hereby declare Midgaard closed!'", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'O':
    do_gen_door(ch, "gate", 0, SCMD_UNLOCK);
    do_gen_door(ch, "gate", 0, SCMD_OPEN);
    break;

  case 'C':
    do_gen_door(ch, "gate", 0, SCMD_CLOSE);
    do_gen_door(ch, "gate", 0, SCMD_LOCK);
    break;

  case '.':
    move = FALSE;
    break;

  }

  index++;
  return FALSE;
}


/* ********************************************************************
   *  General special procedures for mobiles                             *
   ******************************************************************** */


void npc_steal(struct char_data *ch, struct char_data *victim)
{
  int gold;

  if (IS_NPC(victim))
    return;
  if (GET_LEVEL(victim) >= LVL_IMMORT)
    return;

  if (AWAKE(victim) && (number(0, GET_LEVEL(ch)) == 0)) {
    act("You discover that $n has $s hands in your wallet.", FALSE, ch, 0, victim, TO_VICT);
    act("$n tries to steal gold from $N.", TRUE, ch, 0, victim, TO_NOTVICT);
  } else {
    /* Steal some gold coins */
    gold = (int) ((GET_MONEY(victim) * number(1, 10)) / 100);
    if (gold > 0) {
      ADD_MONEY(ch, gold);
      SUB_MONEY(victim, gold);
    }
  }
}




/* ********************************************************************
   *  Special procedures for mobiles                                      *
   ******************************************************************** */


#define PET_PRICE(pet) (GET_LEVEL(pet) * 300)

SPECIAL(pet_shops)
{
  char buf[MAX_STRING_LENGTH], pet_name[256];
  int pet_room;
  struct char_data *pet;

  pet_room = ch->in_room + 1;

  if (CMD_IS("list")) {
    send_to_char("Available pets are:\r\n", ch);
    for (pet = world[pet_room].people; pet; pet = pet->next_in_room) {
      sprintf(buf, "%8d - %s\r\n", PET_PRICE(pet), GET_NAME(pet));
      send_to_char(buf, ch);
    }
    return (TRUE);
  } else if (CMD_IS("buy")) {

    argument = one_argument(argument, buf);
    argument = one_argument(argument, pet_name);

    if (!(pet = get_char_room(buf, pet_room))) {
      send_to_char("There is no such pet!\r\n", ch);
      return (TRUE);
    }
    if (GET_MONEY(ch) < PET_PRICE(pet)) {
      send_to_char("You don't have enough gold!\r\n", ch);
      return (TRUE);
    }
    SUB_MONEY(ch, PET_PRICE(pet));

    pet = read_mobile(GET_MOB_RNUM(pet), REAL);
    GET_EXP(pet) = 0;
    SET_BIT(AFF_FLAGS(pet), AFF_CHARM);

    if (*pet_name) {
      sprintf(buf, "%s %s", pet->player.name, pet_name);
      /* FREE(pet->player.name); don't free the prototype! */
      pet->player.name = str_dup(buf);

      sprintf(buf, "%sA small sign on a chain around the neck says 'My name is %s'\r\n",
	      pet->player.description, pet_name);
      /* FREE(pet->player.description); don't free the prototype! */
      pet->player.description = str_dup(buf);
    }
    char_to_room(pet, ch->in_room, -1);
    add_follower(pet, ch);

    /* Be certain that pets can't get/carry/use/wield/wear items */
    IS_CARRYING_W(pet) = 1000;
    IS_CARRYING_N(pet) = 100;

    send_to_char("May you enjoy your pet.\r\n", ch);
    act("$n buys $N as a pet.", FALSE, ch, 0, pet, PROXIMITY);

    return 1;
  }
  /* All commands except list and buy */
  return 0;
}



/* ********************************************************************
   *  Special procedures for objects                                     *
   ******************************************************************** */


SPECIAL(changelog)
{
  FILE *f = NULL;
  char o_buf[MAX_STRING_LENGTH];
  char *ret;

  if (!ch || !cmd)
    return FALSE;

  if (!CMD_IS("read"))
    return FALSE;

  skip_spaces(&argument);

  if (!str_cmp(argument, "book")) {
    send_to_char("The book of files contain: log, misc, todo, magic, prototypes.\r\n", ch);
    return TRUE;
  }
  o_buf[0] = '\0';

  if (!str_cmp(argument, "log"))
    f = fopen("../src/ChangeLog", "r");
  else if (!str_cmp(argument, "todo"))
    f = fopen("../src/TODO", "r");
  else if (!str_cmp(argument, "misc"))
    f = fopen("text/misc.txt", "r");
  else if (!str_cmp(argument, "magic"))
    f = fopen("../src/MAGIC", "r");
  else if (!str_cmp(argument, "prototypes"))
    f = fopen("../src/PROTOTYPES", "r");
  if (!f) {
    return FALSE;
  }
  do {
    ret = fgets(buf, MAX_STRING_LENGTH, f);
    if (ret)
      if ((strlen(o_buf) + strlen(buf)) >= (MAX_STRING_LENGTH - 10)) {
	ret = NULL;
      } else {
	if (isalpha(buf[0]) && isupper(buf[0]))
	  strcat(o_buf, "&+B");
	else
	  strcat(o_buf, "&+c");
	strcat(o_buf, buf);
	strcat(o_buf, "\r&n");
      }
  } while (ret);
  strcat(o_buf, "&n\r\n");

  fclose(f);
  page_string(ch->desc, o_buf, 1);
  return TRUE;
}



/*
   use object with ITEM_SWITCH type:
   value[0] = command to trigger
   value[1] = room containing the blocked exit
   value[2] = direction of the exit in the room
   value[3] = 0:wall moves, 1:switch item moves
 */

SPECIAL(item_switch)
{
  int door, in_room, bits, back, cmd_given;
  struct char_data *dummy;
  struct obj_data *object, *obj;

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;

  obj = me;

  skip_spaces(&argument);

  bits = generic_find(argument, FIND_OBJ_INV | FIND_OBJ_EQUIP | FIND_OBJ_ROOM, ch, &dummy, &object);

  if (obj != object) {
    send_to_char("&+CDEBUG:&n obj != object in item_switch.\r\n", ch);
    return FALSE;
  }
  if (GET_OBJ_TYPE(obj) != ITEM_SWITCH) {
    send_to_char("&+CDEBUG:&n not a switch\r\n", ch);
    return FALSE;
  }
  if ((cmd_given = search_block(command, activating_commands, TRUE)) < 0)
    return FALSE;

  if (GET_OBJ_VAL(object, 0) != cmd_given++) {
    sprintf(buf, "&+CDEBUG:&n wrong command %d %d\r\n", GET_OBJ_VAL(obj, 0), cmd_given);
    send_to_char(buf, ch);
    return FALSE;
  }
  in_room = real_room(GET_OBJ_VAL(object, 1));
  if (in_room < 0) {
    send_to_char("This item is broken.  Talk to a god! #1\r\n", ch);
    return TRUE;
  }
  door = GET_OBJ_VAL(object, 2);
  if ((door < 0) || (door > 5)) {
    send_to_char("This item is broken.  Talk to a god! #2\r\n", ch);
    return TRUE;
  }
  if (!world[in_room].dir_option[door]) {
    send_to_char("This item is broken.  Talk to a god! #3\r\n", ch);
    return TRUE;
  }
  if (!IS_SET(world[in_room].dir_option[door]->exit_info, EX_BLOCKED)) {
    if (object->in_room != in_room) {
      send_to_char("You hear a rumbling sound in the distance.\r\n", ch);
      act("You hear a rumbling sound in the distance.", FALSE, ch, 0, 0, PROXIMITY);
    }
    SET_BIT(world[in_room].dir_option[door]->exit_info, EX_BLOCKED);
    SET_BIT(world[in_room].dir_option[door]->exit_info, EX_CLOSED);
    back = world[in_room].dir_option[door]->to_room;

    if (IS_SET(world[in_room].dir_option[door]->exit_info, EX_SECRET)) {
      if (GET_OBJ_VAL(object, 3) == 1) {
	sprintf(buf, "%s moves a bit, covering the wall again.\r\n",
		obj->short_description);
	CAP(buf);
	send_to_room(buf, in_room);
      } else {
	sprintf(buf, "The %s wall seems to be moving.\r\n", dirs[door]);
	send_to_room(buf, in_room);
      }
    } else {
      if (back == in_room)
	REMOVE_BIT(world[back].dir_option[(int) rev_dir[door]]->exit_info, EX_BLOCKED);

      if (GET_OBJ_VAL(object, 3) == 1) {
	sprintf(buf, "%s moves to close off the passageway.\r\n",
		obj->short_description);
	CAP(buf);
	send_to_room(buf, in_room);
      } else {
	sprintf(buf, "The %s wall moves to close off the passageway.\r\n",
		dirs[door]);
	send_to_room(buf, in_room);
      }
    }
    return TRUE;
  }
  if (object->in_room != in_room) {
    send_to_char("You hear a rumbling sound in the distance.\r\n", ch);
    act("You hear a rumbling sound in the distance.", FALSE, ch, 0, 0, PROXIMITY);
  }
  REMOVE_BIT(world[in_room].dir_option[door]->exit_info, EX_BLOCKED);
  REMOVE_BIT(world[in_room].dir_option[door]->exit_info, EX_CLOSED);
  back = world[in_room].dir_option[door]->to_room;

  if (IS_SET(world[in_room].dir_option[door]->exit_info, EX_SECRET)) {
    if (GET_OBJ_VAL(object, 3) == 1) {
      sprintf(buf, "%s moves aside, revealing a wall behind.\r\n",
	      obj->short_description);
      CAP(buf);
      send_to_room(buf, in_room);
    } else {
      sprintf(buf, "The %s wall seems to be moving.\r\n", dirs[door]);
      send_to_room(buf, in_room);
    }
  } else {
    if (back == in_room)
      REMOVE_BIT(world[back].dir_option[(int) rev_dir[door]]->exit_info, EX_BLOCKED);

    if (GET_OBJ_VAL(object, 3) == 1) {
      sprintf(buf, "%s moves aside, revealing a passageway.\r\n",
	      obj->short_description);
      CAP(buf);
      send_to_room(buf, in_room);
    } else {
      sprintf(buf, "!!SOUND(secretdoor* T=general)The %s wall moves aside, revealing a passageway.\r\n",
	      dirs[door]);
      send_to_room(buf, in_room);
    }
  }
  return TRUE;
}

/*
   use object with ITEM_TELEPORT type:
   value[0] = where this door leads to
   value[1] = type (door, portal, wormhole, etc)
   value[2] = charges
   value[3] = 
 */
SPECIAL(portal_object)
{
  int bits;
  int to_room;
  struct char_data *dummy;
  struct obj_data *obj, *object = NULL;

  if (!ch || !argument)
    return FALSE;

  obj = me;

  skip_spaces(&argument);

  bits = generic_find(argument, FIND_OBJ_ROOM, ch, &dummy, &object);

  if (obj != object)
    return FALSE;

  if (GET_OBJ_TYPE(obj) != ITEM_TELEPORT)
    return FALSE;
/***/

  if (strstr(command, "look")) {
    while (*argument == ' ')
      argument++;
    if (!*argument)
      return FALSE;
    if (strn_cmp(argument, "in ", 3))
      return FALSE;
    argument += 3;
  }
  to_room = real_room(GET_OBJ_VAL(obj, 0));
  if (to_room == NOWHERE) {
    send_to_char("Bug with portal!!! Tell a god.\r\n", ch);
    return (TRUE);
  }
  if (strstr("look", command)) {
    act("You peer into $p and see...", 0, ch, obj, 0, TO_CHAR);
    if (0)
      send_to_char("It is pitch black over there...\r\n", ch);
    else {
      send_to_char(world[to_room].name, ch);
      send_to_char("\r\n", ch);
    }
    return TRUE;
  }
  /* otherwise cmd == enter */

  if (!obj->action_description) {
    act("&+W$p suddenly glows brightly!", FALSE, ch, obj, 0, PROXIMITY);
    act("$n enters $p and disappears among the mist.", FALSE, ch, obj, 0, PROXIMITY);
  } else
    act(obj->action_description, FALSE, ch, obj, 0, PROXIMITY);

  char_from_room(ch);
  char_to_room(ch, to_room, -1);
  act("$n has arrived.", FALSE, ch, obj, 0, PROXIMITY);

  if (ch->desc != NULL) {
    smallmap_look(ch, view_distance(ch), -1);
    look_at_room(ch, 2);
  }
  if (IS_SET(world[to_room].room_flags, ROOM_DEATH) && !IS_TRUSTED(ch))
    extract_char(ch);

#if 0
  if (obj->value[2] > 0) {	/* gates have no charges */
    obj->value[2]--;
    if (obj->value[2] == 0) {	/* last one */
      Decay(obj);
    }
  }
#endif
  return TRUE;
}

/*
   use object with ITEM_PIT type:
   value[0] = where this pit leads to
   value[1] = type (open, wieght activated)
   value[2] = weight it can hold before opening
   value[3] = 
 */
SPECIAL(pit_object)
{
  int to_room;
  struct obj_data *obj;

  if (!ch)
    return FALSE;

  obj = me;

  if (GET_OBJ_TYPE(obj) != ITEM_PIT)
    return FALSE;

  if (GET_OBJ_X_CORD(obj) != GET_X_CORD(ch) ||
      GET_OBJ_Y_CORD(obj) != GET_Y_CORD(ch) ||
      GET_OBJ_Z_CORD(obj) != GET_Z_CORD(ch))
    return FALSE;

  /* requires weight check if this has a value */
  if (GET_OBJ_VAL(obj, 1)) {
    if ((GET_WEIGHT(ch) + IS_CARRYING_W(ch)) < GET_OBJ_VAL(obj, 2))
      return FALSE;
  }
  to_room = real_room(GET_OBJ_VAL(obj, 0));
  if (to_room == NOWHERE)
    return (TRUE);

  if (!obj->action_description) {
    act("$p suddenly opens beneath $n's feet!", FALSE, ch, obj, 0, PROXIMITY);
    act("$n falls into $p with a slight scream.", FALSE, ch, obj, 0, PROXIMITY);
  } else
    act(obj->action_description, FALSE, ch, obj, 0, PROXIMITY);

  char_from_room(ch);
  char_to_room(ch, to_room, 4);

  act("$n falls in from above.", FALSE, ch, obj, 0, PROXIMITY);

  if (ch->desc != NULL) {
    smallmap_look(ch, view_distance(ch), -1);
    look_at_room(ch, 2);
  }
  if (IS_SET(world[to_room].room_flags, ROOM_DEATH) && !IS_TRUSTED(ch))
    extract_char(ch);

  return TRUE;
}

/*
   use object with ITEM_DOOR type:
   value[0] = where this door leads to
   value[1] = x it leads to
   value[2] = y it leads to
   value[3] = z it leads to
 */
SPECIAL(door_object)
{
  int to_room;
  struct obj_data *obj;

  if (!ch || IS_MOB(ch))
    return FALSE;

  if (!MOVE_CMD(cmd))
    return FALSE;

  obj = me;

  if (GET_OBJ_TYPE(obj) != ITEM_DOOR)
    return FALSE;

  if (GET_OBJ_X_CORD(obj) != GET_X_CORD(ch) ||
      GET_OBJ_Y_CORD(obj) != GET_Y_CORD(ch) ||
      GET_OBJ_Z_CORD(obj) != GET_Z_CORD(ch))
    return FALSE;

  to_room = real_room(GET_OBJ_VAL(obj, 0));
  if (to_room == NOWHERE)
    return (TRUE);

  if (!obj->action_description) {
    act("$n enters the $p.", FALSE, ch, obj, 0, PROXIMITY);
  } else
    act(obj->action_description, FALSE, ch, obj, 0, PROXIMITY);

  char_from_room(ch);
  put_char_to_room(ch, to_room, -1, GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 2), GET_OBJ_VAL(obj, 3));

  if (!obj->action_description) {
    act("$n exits the $p.", FALSE, ch, obj, 0, PROXIMITY);
  } else
    act(obj->action_description, FALSE, ch, obj, 0, PROXIMITY);

  if (ch->desc != NULL) {
    smallmap_look(ch, view_distance(ch), -1);
    look_at_room(ch, 2);
  }
  if (IS_SET(world[to_room].room_flags, ROOM_DEATH) && !IS_TRUSTED(ch))
    extract_char(ch);

  return TRUE;
}

/* this one links outdoor zones with the map */
/* val 0 = room it connects to */
/* val 1 = direction it commands */
SPECIAL(outdoor_zone_connecter_object)
{
  int to_room, x, y, z;
  struct obj_data *obj;

  if (!ch || IS_MOB(ch))
    return FALSE;

  if (!MOVE_CMD(cmd))
    return FALSE;

  obj = me;

  if (GET_OBJ_TYPE(obj) != ITEM_MAP_CONNECTOR)
    return FALSE;

  to_room = real_room(GET_OBJ_VAL(obj, 0));
  if (to_room == NOWHERE)
    return FALSE;

  switch (GET_OBJ_VAL(obj, 1)) {
  case NORTH:
    if (GET_X_CORD(ch) > 0) {
      return FALSE;
    }
    if (!CMD_IS("north") && !CMD_IS("n")) {
      return FALSE;
    }
    if (GET_Y_CORD(ch) > GET_Y_SIZE(to_room) - 1) {
      return FALSE;
    }
    x = GET_X_SIZE(to_room) - 1;
    y = BOUNDED(0, GET_Y_CORD(ch), GET_Y_SIZE(to_room) - 1);
    z = GET_Z_CORD(ch);
    break;
  case SOUTH:
    if (GET_X_CORD(ch) < GET_X_SIZE(ch->in_room) - 1)
      return FALSE;
    if (!CMD_IS("south") && !CMD_IS("s"))
      return FALSE;
    if (GET_Y_CORD(ch) > GET_Y_SIZE(to_room) - 1)
      return FALSE;
    x = 0;
    y = BOUNDED(0, GET_Y_CORD(ch), GET_Y_SIZE(to_room) - 1);
    z = GET_Z_CORD(ch);
    break;
  case WEST:
    if (GET_Y_CORD(ch) > 0)
      return FALSE;
    if (!CMD_IS("west") && !CMD_IS("w"))
      return FALSE;
    if (GET_X_CORD(ch) > GET_X_SIZE(to_room) - 1)
      return FALSE;
    x = BOUNDED(0, GET_X_CORD(ch), GET_X_SIZE(to_room) - 1);
    y = GET_Y_SIZE(to_room) - 1;
    z = GET_Z_CORD(ch);
    break;
  case EAST:
    if (GET_Y_CORD(ch) < GET_Y_SIZE(ch->in_room) - 1)
      return FALSE;
    if (!CMD_IS("east") && !CMD_IS("e"))
      return FALSE;
    if (GET_X_CORD(ch) > GET_X_SIZE(to_room) - 1)
      return FALSE;
    x = BOUNDED(0, GET_X_CORD(ch), GET_X_SIZE(to_room) - 1);
    y = 0;
    z = GET_Z_CORD(ch);
    break;
  case UP:
    if (GET_Z_CORD(ch) < GET_Z_SIZE(ch->in_room))
      return FALSE;
    if (!CMD_IS("up") && !CMD_IS("u"))
      return FALSE;
    if (GET_X_CORD(ch) > GET_X_SIZE(to_room) - 1)
      return FALSE;
    if (GET_Y_CORD(ch) > GET_Y_SIZE(to_room) - 1)
      return FALSE;
    x = BOUNDED(0, GET_X_CORD(ch), GET_X_SIZE(to_room) - 1);
    y = BOUNDED(0, GET_Y_CORD(ch), GET_Y_SIZE(to_room) - 1);
    z = 0;
    break;
  case DOWN:
    if (GET_Z_CORD(ch) > 0)
      return FALSE;
    if (!CMD_IS("down") && !CMD_IS("d"))
      return FALSE;
    if (GET_X_CORD(ch) > GET_X_SIZE(to_room) - 1)
      return FALSE;
    if (GET_Y_CORD(ch) > GET_Y_SIZE(to_room) - 1)
      return FALSE;
    x = BOUNDED(0, GET_X_CORD(ch), GET_X_SIZE(to_room) - 1);
    y = BOUNDED(0, GET_Y_CORD(ch), GET_Y_SIZE(to_room) - 1);
    z = GET_Z_SIZE(to_room);
    break;
  default:
    return FALSE;
  }

  x = BOUNDED(0, x + GET_OBJ_VAL(obj, 2), GET_X_SIZE(to_room) - 1);
  y = BOUNDED(0, y + GET_OBJ_VAL(obj, 3), GET_Y_SIZE(to_room) - 1);
  z = BOUNDED(0, z + GET_OBJ_VAL(obj, 4), GET_Z_SIZE(to_room) - 1);

  char_from_room(ch);
  put_char_to_room(ch, to_room, -1, x, y, z);

  if (ch->desc != NULL) {
    smallmap_look(ch, view_distance(ch), -1);
    look_at_room(ch, 2);
  }
  if (IS_SET(world[to_room].room_flags, ROOM_DEATH) && !IS_TRUSTED(ch))
    extract_char(ch);

  return TRUE;
}

/* support for the merchant proc */
struct obj_data *find_first_bagged_item(struct char_data *ch)
{
  struct obj_data *obj = NULL;

  if (ch->equipment[WEAR_HOLD] &&
      GET_OBJ_TYPE(ch->equipment[WEAR_HOLD]) == ITEM_CONTAINER)
    if (ch->equipment[WEAR_HOLD]->contains)
      obj = ch->equipment[WEAR_HOLD]->contains;
  if (!obj && ch->equipment[WEAR_HOLD2] &&
      GET_OBJ_TYPE(ch->equipment[WEAR_HOLD2]) == ITEM_CONTAINER)
    if (ch->equipment[WEAR_HOLD2]->contains)
      obj = ch->equipment[WEAR_HOLD2]->contains;
  if (!obj && ch->equipment[WEAR_ATTACH_1] &&
      GET_OBJ_TYPE(ch->equipment[WEAR_ATTACH_1]) == ITEM_CONTAINER)
    if (ch->equipment[WEAR_ATTACH_1]->contains)
      obj = ch->equipment[WEAR_ATTACH_1]->contains;
  if (!obj && ch->equipment[WEAR_ATTACH_2] &&
      GET_OBJ_TYPE(ch->equipment[WEAR_ATTACH_2]) == ITEM_CONTAINER)
    if (ch->equipment[WEAR_ATTACH_2]->contains)
      obj = ch->equipment[WEAR_ATTACH_2]->contains;
  if (!obj && ch->equipment[WEAR_ATTACH_3] &&
      GET_OBJ_TYPE(ch->equipment[WEAR_ATTACH_3]) == ITEM_CONTAINER)
    if (ch->equipment[WEAR_ATTACH_3]->contains)
      obj = ch->equipment[WEAR_ATTACH_3]->contains;
  if (!obj && ch->carrying)
    obj = ch->carrying;		/* last resort, perhaps they have nowhere to bag it */

  if (obj && (GET_OBJ_TYPE(obj) == ITEM_FOOD || GET_OBJ_TYPE(obj) == ITEM_MONEY ||
	      GET_OBJ_TYPE(obj) == ITEM_KEY))
    obj = NULL;			/* don't want to sell those items */
  return obj;
}
/*
 * The salesman, always after a tidy profit and the satisfaction of a sale, is
 * a social individual...Maybe overly so... <grin>
 */

SPECIAL(merchant)
{
  char arg1[MAX_STRING_LENGTH], arg2[MAX_STRING_LENGTH], buf3[MAX_STRING_LENGTH];
  int i_val = 0;
  struct obj_data *selling = 0, *s_item = 0, *obj = 0;
  struct char_data *pl = NULL, *c_obj = 0, *k, *old_follow = NULL;

  arg1[0] = '\0';
  arg2[0] = '\0';

  pl = me;			/* me is us, when a player types command */

  if (FIGHTING(pl)) {
    mobsay(pl, "Help! I'm being attacked");
    return FALSE;
  }
  /* Find out if we have anything to sell */


  selling = find_first_bagged_item(pl);

  if (selling)
    i_val = (selling->obj_flags.cost + (selling->obj_flags.cost * (GET_LEVEL(pl) / 100)));

  if (i_val < 10)
    i_val = 10;

  /* Handle cases where we might care what a player does: buy, steal, etc. */
  if (cmd != 0) {
    argument_interpreter(argument, arg1, arg2);

    if (*arg1)
      c_obj = get_char_room(arg1, ch->in_room);

    if (CMD_IS("kill") || CMD_IS("hit")
	|| CMD_IS("murder") || CMD_IS("kick")) {
      /* Actions of questionable intent */
      if (c_obj != ch || GET_STAT(ch) <= STAT_SLEEPING)
	return FALSE;
      sprintf(buf1, "mcallforguards %s att_murder", fname(GET_NAME(ch)));
      command_interpreter(pl, buf1);

      if (pl->player.sex != 2) {
	act("$N slaps you before you can even begin.",
	    FALSE, pl, 0, ch, TO_CHAR);
	act("$N slaps $n for his naughty intentions.",
	    FALSE, pl, 0, ch, TO_ROOM);
      }
      return TRUE;
    } else if (CMD_IS("smile")) {
      /* The salesman suffers from acceptance anxiety :-) */
      if (GET_STAT(pl) <= STAT_SLEEPING)
	return FALSE;
      do_action(pl, arg1, cmd, 0);
      act("$n smiles too, trying to join in on the fun.",
	  TRUE, pl, 0, 0, TO_ROOM);
      return TRUE;
    } else if (CMD_IS("poke")) {
      /* An exchange of poking */
      if (c_obj != pl || GET_STAT(pl) <= STAT_SLEEPING)
	return FALSE;
/*      do_action(pl, arg, "poke", 0); */
      act("$N pokes you back.", FALSE, ch, 0, pl, TO_CHAR);
      act("$N pokes $n back.", TRUE, ch, 0, pl, TO_ROOM);
      return TRUE;
    } else if (CMD_IS("backstab")) {
      if (c_obj != ch || GET_STAT(ch) <= STAT_SLEEPING)
	return FALSE;
      act("Oof! $N knocks you down.", FALSE, ch, 0, pl, TO_CHAR);
      act("$n tries to backstab $N who, more alert than \
he appears, slams\n\r$m down on the ground instead.", FALSE, pl, 0, ch, TO_ROOM);
      mobsay(ch, "I've dealt with your kind before!");
      SET_POS(pl, POS_SITTING + GET_STAT(pl));
      return TRUE;
    } else if (CMD_IS("list")) {
      if (!AWAKE(pl))
	return FALSE;

      if (selling) {
	strcpy(buf3, selling->short_description);
	master_parser(buf3, NULL, NULL, selling, NULL);
	sprintf(buf1, "%s, I'm selling %s, for a mere %s!", GET_NAME(ch), buf3, coin_stringv(i_val));
      } else
	sprintf(buf1, "Sorry %s, I have nothing currently for sale.", GET_NAME(ch));
      mobsay(pl, buf1);
      return TRUE;
    } else if (CMD_IS("buy")) {
      if (!AWAKE(pl))
	return FALSE;
      /* "To buy is to be" is the salesman's motto... */
      if (strlen(arg1) == 0 || GET_STAT(ch) <= STAT_SLEEPING || !selling)
	return FALSE;
      if (c_obj == ch) {
	mobsay(ch, "I'm not for sale, idiot!");
	return TRUE;
      }
      if (isname(arg1, selling->name))
	s_item = selling;
      else
	s_item = NULL;
      if (strlen(arg2) == 0) {
	if (!get_char_room_vis(pl, "2.merchant")) {
	  if (s_item == selling) {
	    if (transact(pl, s_item, ch, i_val)) {
	      if (ch->master)
		stop_follower(ch);
	      REMOVE_BIT(MOB_FLAGS(ch), MOB_SENTINEL);
	    } else if (s_item) {
	      sprintf(buf1, "$n grins and says, 'Buy the %s \
first, and then I'll\n\rconsider selling you the %s.", selling->short_description,
		      s_item->short_description);
	      act(buf1, FALSE, ch, 0, 0, TO_ROOM);
	    } else {
	      mobsay(pl, "Sell you what?");
	    }
	    return (TRUE);
	  } else if (ch == get_char_room_vis(ch, "merchant")) {
	    sprintf(buf, "$n arches an eyebrow and says, 'Who \
\n\r'ya talkin' to, %s?'\n\rUse: BUY <OBJ> [FROM] <SELLER>.", pl->player.name);
	    act(buf, FALSE, ch, 0, 0, TO_ROOM);
	  } else {
	    mobsay(pl, "Yeah, we can't ALL oblige you, ya know?");
	    return TRUE;
	  }
	} else {
	  if (ch == get_char_room_vis(pl, arg2)) {
	    if (s_item == selling) {
	      if (transact(ch, s_item, pl, i_val)) {
		if (ch->master)
		  stop_follower(ch);
		REMOVE_BIT(MOB_FLAGS(ch), MOB_SENTINEL);
	      }
	    } else
	      mobsay(ch, "I don't have that.");
	    return (TRUE);
	  }
	}
      } else
	return FALSE;
    }
  } else if (IS_LIGHT(ch->in_room)) {
    if (GET_STAT(ch) == STAT_SLEEPING) {
      SET_POS(ch, POS_STANDING + STAT_NORMAL);
      act("$n becomes alert, ready to assault any \
unsuspecting\n\rpersons with $s salesmanship.", TRUE, ch, 0, 0, TO_ROOM);
    }
    if (ch->master) {
      if (!selling) {
	/* No items to sell, must have had item stolen */
	act("$n looks puzzled.", TRUE, ch, 0, 0, TO_ROOM);
	send_to_char("We may have been robbed.\r\n", ch);
	stop_follower(ch);
	REMOVE_BIT(MOB_FLAGS(ch), MOB_SENTINEL);
	return (FALSE);
      }
      i_val = (selling->obj_flags.cost + (selling->obj_flags.cost * (GET_LEVEL(pl) / 100)));
      if (ch->master->in_room == ch->in_room) {
	if (GET_MOB_SPEC_VALUE(ch, 0) > 150) {	/* Number is not fixed in stone */
	  act("$n throws $s hands up in disgust.", TRUE,
	      ch, 0, 0, TO_ROOM);
	  old_follow = ch->master;
	  stop_follower(ch);
	  send_to_char("spec value above 108, we give up. They aren't buying.\r\n", ch);
	  REMOVE_BIT(MOB_FLAGS(ch), MOB_SENTINEL);
	} else {
	  if (GET_STAT(ch->master) <= STAT_SLEEPING)
	    return (0);
	  switch (number(0, 20)) {
	  case 0:
	    act("$n says 'I tell you this $o is of the finest quality.'", FALSE, ch, selling, 0, TO_ROOM);
	    break;
	  case 1:
	    sprintf(buf1, "Only %s - a bargain!", coin_stringv(i_val));
	    mobsay(ch, buf1);
	    break;
	  case 2:
	    act("$N splutters 'You know, I have six\
 hungry wives\n\rand a child to feed...'", FALSE, ch->master, 0, ch, TO_CHAR);
	    break;
	  case 3:
	    act("$N waves $p in your face.",
		FALSE, ch->master, selling, ch, TO_CHAR);
	    act("$N waves $p in $n's face.",
		FALSE, ch->master, selling, ch, TO_ROOM);
	    break;
	  case 4:
	    act("The merchant demonstrates the unique usefulness of the $o.", TRUE, ch, selling, 0, TO_ROOM);
	    break;
	  case 5:
	    sprintf(buf1, "Com'on folks, I really need this sale to meet my quota!");
	    mobsay(ch, buf1);
	    break;
	  case 6:
	    sprintf(buf1, "What else could you possibly buy for %s, nicer than this %s?",
		    coin_stringv(i_val), GET_OBJ_SHORT(selling));
	    mobsay(ch, buf1);
	    break;
	  default:
	    break;
	  }
	  if (buf[0]) {
	    switch (number(0, 4)) {
	    case 0:
	      act("$n tries to sell the $o to $N.", FALSE, ch, selling, ch->master, TO_NOTVICT);
	      break;
	    case 1:
	      act("$n continues sales-pitching $o to $N.", FALSE, ch, selling, ch->master, TO_NOTVICT);
	      break;
	    case 2:
	      act("Looking a bit flustered, $n makes a heavy sales pitch to $N.", FALSE, ch, selling, ch->master, TO_NOTVICT);
	      break;
	    case 3:
	      act("$N looks over $o offered by $n.", FALSE, ch, selling, ch->master, TO_NOTVICT);
	      break;
	    case 4:
	      act("$n offers $o to $n.", FALSE, ch, selling, ch->master, TO_NOTVICT);
	      break;
	    }
	    GET_MOB_SPEC_VALUE(ch, 0) += 1;
	  }
	}
      } else {
	stop_follower(ch);
	REMOVE_BIT(MOB_FLAGS(ch), MOB_SENTINEL);
      }
    }
    if (!ch->master && !selling) {
      SET_BIT(MOB_FLAGS(ch), MOB_SCAVENGER);
      if (number(0, 5) == 0) {
	act("$n scans the floor.", TRUE, ch, 0, 0, TO_ROOM);
	send_to_char("We have no master, nor anything to sell. Let's scan the floor for something.\r\n", ch);
      }
    } else if (!ch->master) {
      REMOVE_BIT(MOB_FLAGS(ch), MOB_SCAVENGER);
      send_to_char("We have something to sell, let's find a buyer.\r\n", ch);
      for (k = world[ch->in_room].people; k; k = k->next_in_room) {
	pl = k;
	if (number(0, 4) && CAN_SEE(ch, pl) && IS_PC(pl) &&
	    !circle_follow(ch, pl) && old_follow != pl
	    && !isname("peddler", pl->player.name)
	    && !isname("merchant", pl->player.name)) {
	  add_follower(ch, pl);
	  SET_BIT(MOB_FLAGS(ch), MOB_SENTINEL);
	  sprintf(buf, "The salesman saunters up to you and says, 'Hey %s! \
Have I got a\n\rdeal for you! Take a look at this magnificent $o.\n\rIsn't it \
just a dream? And it can be yours for just %s!'", pl->player.name, coin_stringv(i_val));
	  act(buf, FALSE, ch->master, selling, 0, TO_CHAR);
	  act("$N makes a sales pitch to $n.", FALSE, ch->master,
	      0, ch, TO_ROOM);
	  act("You make a sales pitch to $N.", FALSE, ch,
	      0, ch->master, TO_CHAR);
	  GET_MOB_SPEC_VALUE(ch, 0) = 50;
	  break;
	}
      }
    }
  } else if (GET_STAT(ch) > STAT_SLEEPING) {
    if (!ch->equipment[WEAR_HOLD]) {	/* See if we have a light source */
      for (obj = ch->carrying; obj; obj = obj->next_content) {
	if (obj->obj_flags.type_flag == ITEM_LIGHT) {
	  do_grab(ch, obj->name, 0, 0);
	  return (FALSE);
	}
      }
    } else {
      //see if this is crashing
      if (obj) {
	do_remove(ch, obj->name, 0, 0);
	do_drop(ch, obj->name, 0, 0);
      }				//if

      return (FALSE);
    }
    if (number(0, 5) == 0) {
      SET_POS(ch, POS_PRONE + STAT_SLEEPING);
      mobsay(ch, "Ah, must be time to go to bed!");
      act("The sounds of violent snoring filter through the area.", FALSE,
	  ch, 0, 0, TO_ROOM);
      if (ch->master) {
	stop_follower(ch);
	REMOVE_BIT(MOB_FLAGS(ch), MOB_SENTINEL);
      }
    }
  }
  return (FALSE);
}

SPECIAL(rentacleric)
{
  int i, diff, cost, spl;
  struct obj_data *obj, *next_obj;
  struct char_data *vict;
  char buf[MAX_STRING_LENGTH];
  struct price_info {
    short int number;
    char name[25];
    int price;
  } prices[] =
  {
    /* Spell Num (defined)      Name shown        Price  */
    {
      SPELL_CURE_CRITIC, "healing        ", 8932
    },
    {
      SPELL_ARMOR, "armor          ", 1000
    },
    {
      SPELL_BLESS, "bless          ", 1500
    },
    {
      SPELL_NEUT_POISON, "poison removal ", 10921
    },
    {
      SPELL_REMOVE_CURSE, "curse removal  ", 12294
    },
    {
      SPELL_CURE_BLINDNESS, "blindness cure ", 11925
    },
    {
      -1, "\r\n", -1
    }
  };

  vict = ch;			// bah, I messed up the variables. I should fix this here....

  ch = me;

  if (!ch || !AWAKE(ch) || IS_FIGHTING(ch))
    return FALSE;

  if (!cmd) {
    switch (number(0, 10)) {
    case 0:
      mobsay(ch, "For a small donation, I can heal what ails you.");
      break;
    }
  }
  if (CMD_IS("buy")) {
    argument = one_argument(argument, buf);
    if (*buf) {
      for (i = 0; prices[i].number > -1; i++)
	if (is_abbrev(buf, prices[i].name)) {
	  /* resur is special case. Just find any corpse and raise it :) */
	  /*
	     if (prices[i].number == SPELL_RESURRECT) {
	     for (obj = world[ch->in_room].contents; obj; obj = next_obj) {
	     next_obj = obj->next_content;
	     if ((obj->type == ITEM_CORPSE) && IS_SET(obj->value[1], PC_CORPSE))
	     break;
	     }
	     if (!obj) {
	     mobsay(ch, "Did you perhaps forget to bring your friend?");
	     return TRUE;
	     }
	     }
	   */
	  diff = (GET_ALIGNMENT(ch) - GET_ALIGNMENT(vict)) * 6;
	  cost = prices[i].price + diff;
	  spl = prices[i].number;
	  if (transact(vict, NULL, ch, cost)) {
	    // MobCastSpell(ch, vict, NULL, spl, 60);
	    return TRUE;
	  } else
	    return TRUE;
	}
      mobsay(ch, "Sorry, I don't know of that spell.");
      return TRUE;
    } else {
      mobsay(ch, "Here is a listing of the prices for my services.");
      for (i = 0; prices[i].number > -1; i++) {
	diff = (GET_ALIGNMENT(ch) - GET_ALIGNMENT(vict)) * 6;
	cost = prices[i].price + diff;
	sprintf(buf, "%s%s\r\n", prices[i].name, coin_stringv(cost));
	send_to_char(buf, vict);
      }
      return TRUE;
    }
  } else if (CMD_IS("list")) {
    mobsay(ch, "Here is a listing of the prices for my services.");
    for (i = 0; prices[i].number > -1; i++) {
      diff = (GET_ALIGNMENT(ch) - GET_ALIGNMENT(vict)) * 6;
      cost = prices[i].price + diff;
      sprintf(buf, "%s%s\r\n", prices[i].name, coin_stringv(cost));
      send_to_char(buf, vict);
    }
    return TRUE;
  }
  return FALSE;
}
