/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "weather.h"
#include "utils.h"
#include "comm.h"
#include "handler.h"
#include "interpreter.h"
#include "db.h"

/* uses */
extern struct time_info_data time_info;
extern struct zone_data *zone_table;
extern int top_of_zone_table;
extern struct char_data *character_list;
extern struct room_data *world;
extern struct descriptor_data *descriptor_list;
extern struct time_info_data time_info;
extern char *month_name[];
extern char *season_names[];

/* In this part. */
void weather_and_time(int mode);
void another_hour(int mode);
void weather_change(int mode);
void calc_light_zone(struct zone_data *zone);
sh_int get_season(struct zone_data *zone);
void blow_out_torches(void);
void send_to_zone_outdoor(int zone_rnum, char *messg);
void weather_setup(void);
ACMD(do_weather);
extern void play_sound(const char *mess, struct char_data *ch, int where, int xcord, int ycord, int zcord, int type);
extern void sound_to_outdoor(char *messg);
extern void sound_to_zone_outdoor(int zone_rnum, char *messg);

void weather_and_time(int mode)
{
  another_hour(mode);
  weather_change(mode);
}

void set_minute(pulse)
{
  time_info.minutes = (int) (pulse %
			     (SECS_PER_MUD_HOUR * PASSES_PER_SEC))
      / (float) (SECS_PER_MUD_HOUR * PASSES_PER_SEC) * 60;
  if (time_info.minutes >= 60) {
    time_info.minutes = 0;
  }
}

void another_hour(int mode)
{
  time_info.hours++;
  /* this should already be done, but just in case */
  time_info.minutes = 0;
  if (mode) {
    switch (time_info.hours) {
    case 5:
      sunlight = SUN_RISE;
      send_to_outdoor("&+BThe sun rises in the east.&n\r\n");
      break;
    case 6:
      sunlight = SUN_LIGHT;
      send_to_outdoor("&+CThe day has begun.&n\r\n");
      sound_to_outdoor("!!SOUND(daybreak* T=weather)");
      break;
    case 21:
      sunlight = SUN_SET;
      send_to_outdoor("&+bThe sun slowly disappears in the west.&n\r\n");
      break;
    case 22:
      sunlight = SUN_DARK;
      sound_to_outdoor("!!SOUND(cricket.* L=3 T=weather)");
      send_to_outdoor("&+LThe night has begun.&n\r\n");
      break;
    default:
      break;
    }
  }
  if (time_info.hours > 23) {
    time_info.hours -= 24;
    time_info.day++;

    if (time_info.day > 27) {
      time_info.day = 0;
      time_info.month++;

      if (time_info.month > 11) {
	time_info.month = 0;
	time_info.year++;
      }
    }
  }
}

void weather_change(int mode)
{
  int number(int from, int to);
  int zon, magic, old_wind;
  signed char old_temp;
  unsigned char old_precip, season_num;
  struct climate *clime;
  struct weather_data *cond;

  if (mode) {

    /* Scan through each zone of the zone_table */
    for (zon = 0; zon < top_of_zone_table; zon++) {
      clime = &zone_table[zon].climate;
      cond = &zone_table[zon].conditions;
      old_temp = cond->temp;
      old_precip = cond->precip_rate;
      old_wind = cond->windspeed;

      /* Which season is it? */
      season_num = get_season(&zone_table[zon]);

      /* Clear the control weather bit */
      REMOVE_BIT(cond->flags, WEATHER_CONTROLLED);

      /* Create changes for this hour */
      cond->free_energy = MAX(3000,
			      clime->energy_add + cond->free_energy);
      cond->free_energy = MIN(50000, cond->free_energy);
      switch (clime->season_wind[season_num]) {
      case SEASON_CALM:
	if (cond->windspeed > 25)
	  cond->windspeed -= 5;
	else
	  cond->windspeed += number(-2, 1);
	break;
      case SEASON_BREEZY:
	if (cond->windspeed > 40)
	  cond->windspeed -= 5;
	else
	  cond->windspeed += number(-2, 2);
	break;
      case SEASON_UNSETTLED:
	if (cond->windspeed < 5)
	  cond->windspeed += 5;
	else if (cond->windspeed > 60)
	  cond->windspeed -= 5;
	else
	  cond->windspeed += number(-6, 6);
	break;
      case SEASON_WINDY:
	if (cond->windspeed < 15)
	  cond->windspeed += 5;
	else if (cond->windspeed > 80)
	  cond->windspeed -= 5;
	else
	  cond->windspeed += number(-6, 6);
	break;
      case SEASON_CHINOOK:
	if (cond->windspeed < 25)
	  cond->windspeed += 5;
	else if (cond->windspeed > 110)
	  cond->windspeed -= 5;
	else
	  cond->windspeed += number(-15, 15);
	break;
      case SEASON_VIOLENT:
	if (cond->windspeed < 40)
	  cond->windspeed += 5;
	else
	  cond->windspeed += number(-8, 8);
	break;
      case SEASON_HURRICANE:
	cond->windspeed = 100;
	break;
      default:
	break;
      }
      cond->free_energy += cond->windspeed;	/* + or - ? */
      if (cond->free_energy < 0)
	cond->free_energy = 0;
      else if (cond->free_energy > 20000)
	cond->windspeed += number(-10, -1);
      cond->windspeed = MAX(0, cond->windspeed);
      switch (clime->season_wind_variance[season_num]) {
      case 0:
	cond->wind_dir = clime->season_wind_dir[season_num];
	break;
      case 1:
	if (dice(2, 15) * 1000 < cond->free_energy)
	  cond->wind_dir = number(0, 3);
	break;
      }
      switch (clime->season_temp[season_num]) {
      case SEASON_FROSTBITE:
	if (cond->temp > -20)
	  cond->temp -= 4;
	else
	  cond->temp += number(-3, 3);
	break;
      case SEASON_NIPPY:
	if (cond->temp < -40)
	  cond->temp += 2;
	else if (cond->temp > 5)
	  cond->temp -= 3;
	else
	  cond->temp += number(-3, 3);
	break;
      case SEASON_FREEZING:
	if (cond->temp < -20)
	  cond->temp += 2;
	else if (cond->temp > 0)
	  cond->temp -= 2;
	else
	  cond->temp += number(-2, 2);
	break;
      case SEASON_COLD:
	if (cond->temp < -10)
	  cond->temp += 1;
	else if (cond->temp > 5)
	  cond->temp -= 2;
	else
	  cond->temp += number(-2, 2);
	break;
      case SEASON_COOL:
	if (cond->temp < -3)
	  cond->temp += 2;
	else if (cond->temp > 14)
	  cond->temp -= 2;
	else
	  cond->temp += number(-3, 3);
	break;
      case SEASON_MILD:
	if (cond->temp < 7)
	  cond->temp += 2;
	else if (cond->temp > 26)
	  cond->temp -= 2;
	else
	  cond->temp += number(-2, 2);
	break;
      case SEASON_WARM:
	if (cond->temp < 19)
	  cond->temp += 2;
	else if (cond->temp > 33)
	  cond->temp -= 2;
	else
	  cond->temp += number(-3, 3);
	break;
      case SEASON_HOT:
	if (cond->temp < 24)
	  cond->temp += 3;
	else if (cond->temp > 46)
	  cond->temp -= 2;
	else
	  cond->temp += number(-3, 3);
	break;
      case SEASON_BLUSTERY:
	if (cond->temp < 34)
	  cond->temp += 3;
	else if (cond->temp > 53)
	  cond->temp -= 2;
	else
	  cond->temp += number(-5, 5);
	break;
      case SEASON_HEATSTROKE:
	if (cond->temp < 44)
	  cond->temp += 5;
	else if (cond->temp > 60)
	  cond->temp -= 5;
	else
	  cond->temp += number(-3, 3);
	break;
      case SEASON_BOILING:
	if (cond->temp < 80)
	  cond->temp += 5;
	else if (cond->temp > 120)
	  cond->temp -= 5;
	else
	  cond->temp += number(-6, 6);
	break;
      default:
	break;
      }
      if (cond->flags & SUN_VISIBLE)
	cond->temp += 2;
      else if (!(clime->flags & NO_SUN_EVER))
	cond->temp -= 2;
      switch (clime->season_precip[season_num]) {
      case SEASON_NO_PRECIP_EVER:
	if (cond->precip_rate > 0)
	  cond->precip_rate /= 2;
	cond->humidity = 0;
	break;
      case SEASON_ARID:
	if (cond->humidity > 30)
	  cond->humidity -= 3;
	else
	  cond->humidity += number(-3, 2);
	if (old_precip > 20)
	  cond->precip_rate -= 8;
	break;
      case SEASON_DRY:
	if (cond->humidity > 50)
	  cond->humidity -= 3;
	else
	  cond->humidity += number(-4, 3);
	if (old_precip > 35)
	  cond->precip_rate -= 6;
	break;
      case SEASON_LOW_PRECIP:
	if (cond->humidity < 13)
	  cond->humidity += 3;
	else if (cond->humidity > 91)
	  cond->humidity -= 2;
	else
	  cond->humidity += number(-5, 4);
	if (old_precip > 45)
	  cond->precip_rate -= 10;
	break;
      case SEASON_AVG_PRECIP:
	if (cond->humidity < 30)
	  cond->humidity += 3;
	else if (cond->humidity > 80)
	  cond->humidity -= 2;
	else
	  cond->humidity += number(-9, 9);
	if (old_precip > 55)
	  cond->precip_rate -= 10;
	break;
      case SEASON_HIGH_PRECIP:
	if (cond->humidity < 40)
	  cond->humidity += 3;
	else if (cond->humidity > 90)
	  cond->humidity -= 2;
	else
	  cond->humidity += number(-8, 8);
	if (old_precip > 65)
	  cond->precip_rate -= 10;
	break;
      case SEASON_STORMY:
	if (cond->humidity < 50)
	  cond->humidity += 4;
	else
	  cond->humidity += number(-6, 6);
	if (old_precip > 80)
	  cond->precip_rate -= 10;
	break;
      case SEASON_TORRENT:
	if (cond->humidity < 60)
	  cond->humidity += 4;
	else
	  cond->humidity += number(-6, 9);
	if (old_precip > 100)
	  cond->precip_rate -= 15;
	break;
      case SEASON_CONSTANT_PRECIP:
	cond->humidity = 100;
	if (cond->precip_rate == 0)
	  cond->precip_rate = number(5, 12);
	break;
      default:
	break;
      }
      cond->humidity = MIN(100, cond->humidity);
      cond->humidity = MAX(0, cond->humidity);

      cond->pressure_change += number(-3, 3);
      cond->pressure_change = MIN(8, cond->pressure_change);
      cond->pressure_change = MAX(-8, cond->pressure_change);
      cond->pressure += cond->pressure_change;
      cond->pressure = MIN(cond->pressure, 1040);
      cond->pressure = MAX(cond->pressure, 960);

      cond->free_energy += cond->pressure_change;

      /* The numbers that follow are truly magic since  */
      /* they have little bearing on reality and are an */
      /* attempt to create a mix of precipitation which */
      /* will seem reasonable for a specified climate   */
      /* without any complicated formulae that could    */
      /* cause a performance hit. To get more specific  */
      /* or exacting would certainly not be "Diku..."   */

      magic = ((1240 - cond->pressure) * cond->humidity >> 4) +
	  cond->temp + old_precip * 2 +
	  (cond->free_energy - 10000) / 100;

      if (old_precip == 0) {
	if (magic > MAGIC_PRECIP_START) {
	  cond->precip_rate += 1;
	  if (cond->temp > 0) {
	    sound_to_zone_outdoor(zon, "!!SOUND(rain* T=weather)");
	    send_to_zone_outdoor(zon, "It begins to rain.\n\r");
	  } else
	    send_to_zone_outdoor(zon,
				 "It starts to snow.\n\r");
	} else if (!old_wind && cond->windspeed) {
	  sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	  send_to_zone_outdoor(zon, "The wind begins to blow\n\r");
	} else if (cond->windspeed - old_wind > 10) {
	  sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	  send_to_zone_outdoor(zon, "The wind picks up some.\n\r");
	} else if (cond->windspeed - old_wind < -10)
	  send_to_zone_outdoor(zon,
			       "The wind calms down a bit.\n\r");
	else if (cond->windspeed > 60) {
	  if (cond->temp > 50) {
	    sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	    send_to_zone_outdoor(zon,
				 "A violent scorching wind "
				 "blows hard in the face "
				 "of any poor travellers "
				 "in the area.\n\r");
	  } else if (cond->temp > 21) {
	    sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	    send_to_zone_outdoor(zon,
				 "A hot wind gusts wildly " \
				 "through the area.\n\r");
	  } else if (cond->temp > 0) {
	    sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	    send_to_zone_outdoor(zon,
				 "A fierce wind cuts the air " \
				 "like a razor-sharp knife.\n\r");
	  } else if (cond->temp > -10) {
	    sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	    send_to_zone_outdoor(zon,
				 "A freezing gale blasts " \
				 "through the area.\n\r");
	  } else {
	    sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	    send_to_zone_outdoor(zon,
				 "An icy wind drains the " \
				 "warmth from all in sight.\n\r");
	  }
	} else if (cond->windspeed > 25) {
	  if (cond->temp > 50) {
	    sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	    send_to_zone_outdoor(zon,
				 "A hot, dry breeze blows " \
				 "languidly around.\n\r");
	  } else if (cond->temp > 22) {
	    sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	    send_to_zone_outdoor(zon,
				 "A warm pocket of air " \
				 "is rolling through here.\n\r");
	  } else if (cond->temp > 10) {
	    sound_to_zone_outdoor(zon, "!!SOUND(wind* T=weather)");
	    send_to_zone_outdoor(zon, "It's breezy.\n\r");
	  } else if (cond->temp > 2)
	    send_to_zone_outdoor(zon,
				 "A cool breeze wafts by.\n\r");
	  else if (cond->temp > -5)
	    send_to_zone_outdoor(zon,
				 "A slight wind blows a " \
				 "chill into living tissue.\n\r");
	  else if (cond->temp > -15)
	    send_to_zone_outdoor(zon,
				 "A freezing wind blows " \
				 "gently, but firmly " \
				 "against all obstacles in " \
				 "the area.\n\r");
	  else
	    send_to_zone_outdoor(zon,
				 "The wind isn't very strong " \
				 "here, but the cold makes " \
				 "it quite noticeable.\n\r");
	} else if (cond->temp > 52)
	  send_to_zone_outdoor(zon,
			   "It's hotter than anyone could imagine.\n\r");
	else if (cond->temp > 37)
	  send_to_zone_outdoor(zon,
			       "It's really, really hot here. A " \
			       "slight breeze would really " \
			       "improve things.\n\r");
	else if (cond->temp > 25)
	  send_to_zone_outdoor(zon,
			       "It's hot out here.\n\r");
	else if (cond->temp > 19)
	  send_to_zone_outdoor(zon,
			       "It's nice and warm out.\n\r");
	else if (cond->temp > 9)
	  send_to_zone_outdoor(zon,
			       "It's mild out today.\n\r");
	else if (cond->temp > 1)
	  send_to_zone_outdoor(zon,
			       "It's cool out here.\n\r");
	else if (cond->temp > -5)
	  send_to_zone_outdoor(zon,
			       "It's a bit nippy here.\n\r");
	else if (cond->temp > -20)
	  send_to_zone_outdoor(zon,
			       "It's cold!\n\r");
	else if (cond->temp > -25)
	  send_to_zone_outdoor(zon,
			       "It's really c-c-c-cold!!\n\r");
	else
	  send_to_zone_outdoor(zon,
			       "Better get inside - this is too " \
			       "cold for man or -most- beasts.\n\r");
      } else if (magic < MAGIC_PRECIP_STOP) {
	cond->precip_rate = 0;
	if (old_temp > 0)
	  send_to_zone_outdoor(zon,
			       "The rain stops.\n\r");
	else
	  send_to_zone_outdoor(zon,
			       "It stops snowing.\n\r");
      } else {
	/* Still precip'ing, update the rate */
	/* Check rain->snow or snow->rain */
	if (cond->free_energy > 10000)
	  cond->precip_change += number(-3, 4);
	else
	  cond->precip_change += number(-4, 2);
	cond->precip_change = MAX(-10, cond->precip_change);
	cond->precip_change = MIN(10, cond->precip_change);
	cond->precip_rate += cond->precip_change;
	cond->precip_rate = MAX(1, cond->precip_rate);
	cond->precip_rate = MIN(100, cond->precip_rate);

	cond->free_energy -= cond->precip_rate * 3 -
	    abs(cond->precip_change);

	if (old_temp > 0 && cond->temp <= 0)
	  send_to_zone_outdoor(zon,
			       "The rain turns to snow.\n\r");
	else if (old_temp <= 0 && cond->temp > 0)
	  send_to_zone_outdoor(zon,
			       "The snow turns to a cold rain.\n\r");
	else if (cond->precip_change > 5) {
	  if (cond->temp > 0)
	    send_to_zone_outdoor(zon,
				 "It rains a bit harder.\n\r");
	  else
	    send_to_zone_outdoor(zon,
				 "The snow is coming down " \
				 "faster now.\n\r");
	} else if (cond->precip_change < -5) {
	  if (cond->temp > 0)
	    send_to_zone_outdoor(zon,
				 "The rain is falling less " \
				 "heavily now.\n\r");
	  else
	    send_to_zone_outdoor(zon,
				 "The snow has let up a " \
				 "little.\n\r");
	} else if (cond->temp > 0) {
	  if (cond->precip_rate > 80) {
	    if (cond->windspeed > 80) {
	      sound_to_zone_outdoor(zon,
				    "!!SOUND(wind* L=-1 T=weather)" \
				    "!!SOUND(rain* L=-1 T=weather)");
	      send_to_zone_outdoor(zon,
				   "There's a hurricane " \
				   "out here!\n\r");
	    } else if (cond->windspeed > 40) {
	      sound_to_zone_outdoor(zon,
				    "!!SOUND(wind* L=-1 T=weather)" \
				    "!!SOUND(rain* L=-1 T=weather)");
	      send_to_zone_outdoor(zon,
				   "The wind and the rain" \
				   " are nearly too much " \
				   "to handle.\n\r");
	    } else {
	      sound_to_zone_outdoor(zon, "!!SOUND(rain* L=-1 T=weather)");
	      send_to_zone_outdoor(zon,
				   "It's raining really " \
				   "hard right now.\n\r");
	    }
	  } else if (cond->precip_rate > 50) {
	    if (cond->windspeed > 60) {
	      sound_to_zone_outdoor(zon, "!!SOUND(rain* L=-1 T=weather)");
	      send_to_zone_outdoor(zon, "What a rainstorm!\n\r");
	    } else if (cond->windspeed > 30) {
	      sound_to_zone_outdoor(zon,
				    "!!SOUND(rain* L=-1 T=weather)" \
				    "!!SOUND(wind* L=-1 T=weather)");
	      send_to_zone_outdoor(zon,
				   "The wind is lashing " \
				   "this wild rain seemi" \
				   "ngly straight into y" \
				   "our face.\n\r");
	    } else {
	      sound_to_zone_outdoor(zon, "!!SOUND(rain* L=-1 T=weather)");
	      send_to_zone_outdoor(zon,
				   "It's raining pretty " \
				   "hard.\n\r");
	    }
	  } else if (cond->precip_rate > 30) {
	    if (cond->windspeed > 50) {
	      sound_to_zone_outdoor(zon,
				    "!!SOUND(rain* L=-1 T=weather)" \
				    "!!SOUND(wind* L=-1 T=weather)");
	      send_to_zone_outdoor(zon,
				   "A respectable rain " \
				   "is being thrashed " \
				   "about by a vicious " \
				   "wind.\n\r");
	    } else if (cond->windspeed > 25) {
	      send_to_zone_outdoor(zon,
				   "It's rainy and windy " \
				   "but, altogether not " \
				   "too uncomfortable.\n\r");
	    } else
	      send_to_zone_outdoor(zon,
				   "Hey, it's raining...\n\r");
	  } else if (cond->precip_rate > 10) {
	    if (cond->windspeed > 50) {
	      sound_to_zone_outdoor(zon,
				    "!!SOUND(wind* L=-1 T=weather)");
	      send_to_zone_outdoor(zon,
				   "The light rain here " \
				   "is nearly unnoticeab" \
				   "le compared to the h" \
				   "orrendous wind.\n\r");
	    } else if (cond->windspeed > 24) {
	      sound_to_zone_outdoor(zon,
				    "!!SOUND(wind* L=-1 T=weather)");
	      send_to_zone_outdoor(zon,
				   "A light rain is bei" \
				   "ng driven fiercely " \
				   "by the wind.\n\r");
	    } else
	      send_to_zone_outdoor(zon,
				   "It's raining lightly.\n\r");
	  } else if (cond->windspeed > 55) {
	    sound_to_zone_outdoor(zon,
				  "!!SOUND(wind* L=-1 T=weather)");
	    send_to_zone_outdoor(zon,
				 "A few drops of rain are fall" \
				 "ing admidst a fierce windsto" \
				 "rm.\n\r");
	  } else if (cond->windspeed > 30) {
	    sound_to_zone_outdoor(zon,
				  "!!SOUND(wind* L=-1 T=weather)");
	    send_to_zone_outdoor(zon,
				 "The wind and a bit of rain " \
				 "hint at the possibility of " \
				 "a storm.\n\r");
	  } else
	    send_to_zone_outdoor(zon,
				 "A light drizzle is falling " \
				 "here.\n\r");
	} else if (cond->precip_rate > 70) {
	  if (cond->windspeed > 50) {
	    sound_to_zone_outdoor(zon,
				  "!!SOUND(wind* L=-1 T=weather)");
	    send_to_zone_outdoor(zon,
				 "This must be the worst " \
				 "blizzard ever.\n\r");
	  } else if (cond->windspeed > 25) {
	    sound_to_zone_outdoor(zon,
				  "!!SOUND(wind* L=-1 T=weather)");
	    send_to_zone_outdoor(zon,
				 "There's a blizzard out " \
				 "here, making it quite " \
				 "difficult to see.\n\r");
	  } else
	    send_to_zone_outdoor(zon,
				 "It's snowing very hard.\n\r");
	} else if (cond->precip_rate > 40) {
	  if (cond->windspeed > 60)
	    send_to_zone_outdoor(zon,
				 "!!SOUND(wind* L=-1 T=weather)" \
				 "The heavily falling snow is " \
				 "being whipped up to a frenzy" \
				 " by a ferocious wind.\n\r");
	  else if (cond->windspeed > 35)
	    send_to_zone_outdoor(zon,
				 "!!SOUND(wind* L=-1 T=weather)" \
				 "A heavy snow is being blown" \
				 " randomly about by a brisk " \
				 "wind.\n\r");
	  else if (cond->windspeed > 18)
	    send_to_zone_outdoor(zon,
				 "!!SOUND(wind* L=-1 T=weather)" \
				 "Drifts in the snow are " \
				 "being formed by the wind.\n\r");
	  else
	    send_to_zone_outdoor(zon,
				 "The snow's coming down " \
				 "pretty fast now.\n\r");
	} else if (cond->precip_rate > 19) {
	  if (cond->windspeed > 70)
	    send_to_zone_outdoor(zon,
				 "!!SOUND(wind* L=-1 T=weather)" \
				 "The snow wouldn't be too " \
				 "bad, except for the awful " \
				 "wind blowing it in every " \
				 "possible directon.\n\r");
	  else if (cond->windspeed > 45)
	    send_to_zone_outdoor(zon,
				 "!!SOUND(wind* L=-1 T=weather)" \
				 "There's a minor blizzard " \
				 "here, more wind than snow.\n\r");
	  else if (cond->windspeed > 12)
	    send_to_zone_outdoor(zon,
				 "!!SOUND(wind* L=-1 T=weather)" \
				 "Snow is being blown about " \
				 "by a stiff breeze.\n\r");
	  else
	    send_to_zone_outdoor(zon,
				 "It is snowing here.\n\r");
	} else if (cond->windspeed > 60)
	  send_to_zone_outdoor(zon,
			       "!!SOUND(wind* L=-1 T=weather)" \
			       "A light snow is being tossed about " \
			       "by a fierce wind.\n\r");
	else if (cond->windspeed > 42)
	  send_to_zone_outdoor(zon,
			       "!!SOUND(wind* L=-1 T=weather)" \
			       "A lightly falling snow is being " \
			       "driven by a strong wind.\n\r");
	else if (cond->windspeed > 18)
	  send_to_zone_outdoor(zon,
			       "!!SOUND(wind* L=-1 T=weather)" \
			       "A light snow is falling admidst " \
			       "an unsettled wind.\n\r");
	else
	  send_to_zone_outdoor(zon,
			       "It is lightly snowing.\n\r");
      }

      /* Handle celestial objects */
      if (!(clime->flags & NO_SUN_EVER)) {
	if (time_info.hours < 6 || time_info.hours > 18
	    || cond->humidity > 90 || cond->precip_rate
	    > 80)
	  cond->flags &= ~SUN_VISIBLE;
	else
	  cond->flags |= SUN_VISIBLE;
      }
      if (!(clime->flags & NO_MOON_EVER)) {
	if ((time_info.hours > 5 && time_info.hours < 19)
	    || cond->humidity > 80 || cond->precip_rate
	    > 70 || time_info.day < 3 || time_info.day
	    > 31)
	  cond->flags &= ~MOON_VISIBLE;
	else if (!(cond->flags & MOON_VISIBLE)) {
	  cond->flags |= MOON_VISIBLE;
	  if (time_info.day == 17)
	    send_to_zone_outdoor(zon,
				 "The full moon floods the " \
				 "area with light.\n\r");
	  else
	    send_to_zone_outdoor(zon,
				 "The moon casts a little " \
				 "bit of light on the ground.\n\r");
	}
      }
      calc_light_zone(&zone_table[zon]);
    }				/* End zone iteration */
    blow_out_torches();
  }				/* End if(mode) */
}

void blow_out_torches()
{
  struct char_data *i, *i_next;
  struct obj_data *obj;
  for (i = character_list; i; i = i_next) {
    i_next = i->next;
    if (!IS_SET(world[i->in_room].room_flags, ROOM_INDOORS) && i->equipment[WEAR_HOLD]) {
      obj = i->equipment[WEAR_HOLD];
      if (obj->obj_flags.value[3]) {
	if (obj->obj_flags.value[3] < zone_table[world[i->in_room].zone].conditions.windspeed && number(0, 1)) {
	  act("$p goes out", TRUE, i, obj, NULL, TO_ROOM);
	  act("Your $p goes out and you put it away.", TRUE, i, obj, NULL, TO_CHAR);
	  log("blowing out a light source");
	  unequip_char(i, WEAR_HOLD);
	  obj_to_char(obj, i);
	}
      }
    }
    if (!IS_SET(world[i->in_room].room_flags, ROOM_INDOORS) && i->equipment[WEAR_HOLD2]) {
      obj = i->equipment[WEAR_HOLD2];
      if (obj->obj_flags.value[3]) {
	if (obj->obj_flags.value[3] < zone_table[world[i->in_room].zone].conditions.windspeed && number(0, 1)) {
	  act("$p goes out", TRUE, i, obj, NULL, TO_ROOM);
	  act("Your $p goes out and you put it away.", TRUE, i, obj, NULL, TO_CHAR);
	  log("blowing out a light source");
	  unequip_char(i, WEAR_HOLD2);
	  obj_to_char(obj, i);
	}
      }
    }
  }
}

void water_log_torches(struct char_data *i)
{
  struct obj_data *obj;

  if (i->equipment[WEAR_HOLD]) {
    obj = i->equipment[WEAR_HOLD];
    if (GET_OBJ_TYPE(obj) == ITEM_LIGHT && obj->obj_flags.value[3]) {
      act("The water soaks your $p, so you put it away.", TRUE, i, obj, NULL, TO_CHAR);
      unequip_char(i, WEAR_HOLD);
      obj_to_char(obj, i);
    }
  }
  if (i->equipment[WEAR_HOLD2]) {
    obj = i->equipment[WEAR_HOLD2];
    if (GET_OBJ_TYPE(obj) == ITEM_LIGHT && obj->obj_flags.value[3]) {
      act("The water soaks your $p, so you put it away.", TRUE, i, obj, NULL, TO_CHAR);
      unequip_char(i, WEAR_HOLD2);
      obj_to_char(obj, i);
    }
  }
}

void calc_light_zone(struct zone_data *zone)
{
  char light_sum = 0, temp, temp2;

  if (!(zone->climate.flags & NO_SUN_EVER)) {
    temp = time_info.hours;
    if (temp > 11)
      temp = 23 - temp;
    temp -= 5;
    if (temp > 0) {
      zone->conditions.flags |= SUN_VISIBLE;
      light_sum += temp * 10;
    } else
      zone->conditions.flags &= ~SUN_VISIBLE;
  }
  if (!(zone->climate.flags & NO_MOON_EVER)) {
    temp = abs(time_info.hours - 12);
    temp -= 7;
    if (temp > 0) {
      temp2 = 17 - abs(time_info.day - 17);
      temp2 -= 3;
      if (temp2 > 0) {
	zone->conditions.flags |= MOON_VISIBLE;
	light_sum += temp * temp2 / 2;
      } else
	zone->conditions.flags &= ~MOON_VISIBLE;
    }
  }
  light_sum -= zone->conditions.precip_rate;
  light_sum = MAX(0, light_sum);
  light_sum = MIN(100, light_sum);
  if (zone->kingdom == KINGDOM_GREYHAWK)
    light_sum += 10;
  zone->conditions.ambient_light = light_sum;
}

sh_int get_season(struct zone_data *zone)
{
  return (time_info.month < 3) ? 0 :
      (time_info.month < 6) ? 1 :
      (time_info.month < 9) ? 2 : 3;

}

void weather_setup()
{
  int zon, weather_sector;
  sh_int s, bonus;

  /* default conditions for season values */
  const char winds[6] =
  {2, 12, 30, 40, 50, 80};
  const char precip[15] =
  {0, 0, 0, 1, 1, 3, 5, 8, 10, 13, 16, 23, 30, 37, 45};
  const char humid[9] =
  {4, 10, 20, 30, 40, 50, 60, 75, 100};
  const char temps[16] =
  {-30, -20, -10, -5, 0, 2, 7, 10, 17, 23, 29, 37, 43, 55, 75, 95};

  for (zon = 0; zon <= top_of_zone_table; zon++) {
/*
 * Here is where I should check zone number vs equator
 * and calc the temp etc values based on the seasons
 *
 */
    if (IS_MAP_ZONE(zon))
      weather_sector = BOUNDED(0, zon / 21 / 2, 8);
    else			/* link zones to maps here */
      weather_sector = 4;

    for (s = 0; s < 4; s++) {
      if (s == 0)
	bonus = 1;
      else if (s == 3)
	bonus = 5;
      else
	bonus = number(2, 4);

      zone_table[zon].climate.season_temp[s] = temps[weather_sector + bonus];
      /* stupid */
      zone_table[zon].climate.season_precip[s] =
	  precip[BOUNDED(0, weather_sector + number(-5, 10), 14)];
    }
    /* get the season */
    s = get_season(&zone_table[zon]);

    /* These are pretty standard start values */
    zone_table[zon].conditions.pressure = 980;
    zone_table[zon].conditions.free_energy = 10000;
    zone_table[zon].conditions.precip_depth = 0;
    if (zon >= 1000 && zon <= 3000)	/* wilderness */
      zone_table[zon].conditions.flags = SUN_VISIBLE;

    /* These use the default conditions above */
    zone_table[zon].conditions.windspeed =
	winds[(int) zone_table[zon].climate.season_wind[s]];

    zone_table[zon].conditions.wind_dir =
	zone_table[zon].climate.season_wind_dir[s];

    zone_table[zon].conditions.precip_rate =
	precip[(int) zone_table[zon].climate.season_precip[s]];

    zone_table[zon].conditions.temp =
	temps[(int) zone_table[zon].climate.season_temp[s]];

    zone_table[zon].conditions.humidity =
	humid[(int) zone_table[zon].climate.season_precip[s]];

    /* Set ambient light */
    calc_light_zone(&zone_table[zon]);
  }
}

ACMD(do_weather)
{
  struct weather_data *cond;
  struct climate *clime;
  char *wind_patterns[7] =
  {
    "Calm ",
    "Breezy ",
    "Unsettled ",
    "Windy ",
    "Chinook ",
    "Violent ",
    "Hurricane "
  };
  char *wind_dir[4] =
  {"North. ", "East. ", "South. ", "West. "};
  char *precip_patterns[9] =
  {
    "No precip. ",
    "Arid. ",
    "Dry. ",
    "Low precip. ",
    "Avg precip. ",
    "High precip. ",
    "Stormy. ",
    "Torrent. ",
    "Const precip. "
  };
  char *temp_patterns[11] =
  {
    "Frostbite. ",
    "Nippy. ",
    "Freezing. ",
    "Cold. ",
    "Cool. ",
    "Mild. ",
    "Warm. ",
    "Hot. ",
    "Blustery. ",
    "Heatstroke. ",
    "Boiling. "
  };
  int seas, weather_sector;

  clime = &zone_table[world[ch->in_room].zone].climate;
  cond = &zone_table[world[ch->in_room].zone].conditions;

  if (IS_TRUSTED(ch)) {
    seas = get_season(&zone_table[world[ch->in_room].zone]);
    if (WILDERNESS(ch->in_room))
      weather_sector = BOUNDED(0, world[ch->in_room].zone / 21 / 2, 8);
    else			/* need to relate zones to the maps here */
      weather_sector = -1;

    sprintf(buf, "Weather Dump for sector %d:\r\n", weather_sector);
    send_to_char(buf, ch);

    sprintf(buf, "Current season (#%d) %s   Current Month (%d) %s\r\n",
	    seas, season_names[seas], (int) time_info.month, month_name[(int) time_info.month]);
    send_to_char(buf, ch);

    sprintf(buf, "Wind Patterns: (%d) %s", clime->season_wind[seas],
	    wind_patterns[(int) clime->season_wind[seas]]);
    send_to_char(buf, ch);

    if (clime->season_wind_variance[seas])
      send_to_char("variable. ", ch);
    else {
      send_to_char("from ", ch);
      send_to_char(wind_dir[(int) clime->season_wind_dir[seas]], ch);
    }
    send_to_char("\r\n", ch);
    sprintf(buf, "Precipitation: (%d) %s\r\n", clime->season_precip[seas],
	    precip_patterns[(int) clime->season_precip[seas]]);
    send_to_char(buf, ch);
    sprintf(buf, "Tempurature Patterns: (%d) %s\r\n", clime->season_temp[seas],
	    temp_patterns[(int) clime->season_temp[seas]]);
    send_to_char(buf, ch);

    sprintf(buf, "Temp: %d  Humidity: %d  Pressure: %d\r\n", cond->temp,
	    cond->humidity, cond->pressure);

    send_to_char(buf, ch);

    sprintf(buf, "Windspeed: %d  Direction: %d  Precip Rate: %d\r\n",
	    cond->windspeed, cond->wind_dir, cond->precip_rate);

    send_to_char(buf, ch);

    sprintf(buf, "Light: %d  Energy: %d  Pressure change: %d  Precip change: %d\r\n",
	    cond->ambient_light, cond->free_energy, cond->pressure_change,
	    cond->precip_change);

    send_to_char(buf, ch);
  }
  if (!OUTSIDE(ch)) {
    send_to_char("How can you know what the weather's like when you are inside?\r\n\r\n", ch);
    return;
  }
  buf[0] = 0;
  if (cond->precip_rate) {
    if (cond->temp <= 0)
      strcat(buf, "It's snowing");
    else
      strcat(buf, "It's raining");
    if (cond->precip_rate > 65)
      strcat(buf, " extremely hard");
    else if (cond->precip_rate > 50)
      strcat(buf, " very hard");
    else if (cond->precip_rate > 30)
      strcat(buf, " hard");
    else if (cond->precip_rate < 15)
      strcat(buf, " lightly");
    strcat(buf, ", ");
  } else {
    if (cond->humidity > 80)
      strcat(buf, "It's very cloudy, ");
    else if (cond->humidity > 55)
      strcat(buf, "It's cloudy, ");
    else if (cond->humidity > 25)
      strcat(buf, "It's partly cloudy, ");
    else if (cond->humidity)
      strcat(buf, "It's mostly clear, ");
    else
      strcat(buf, "It's clear, ");
  }
  if (cond->temp > 115)
    strcat(buf, "boiling, ");
  else if (cond->temp > 110)
    strcat(buf, "blistering, ");
  else if (cond->temp > 105)
    strcat(buf, "incredibly hot, ");
  else if (cond->temp > 95)
    strcat(buf, "very, very hot, ");
  else if (cond->temp > 85)
    strcat(buf, "very hot, ");
  else if (cond->temp > 80)
    strcat(buf, "hot, ");
  else if (cond->temp > 70)
    strcat(buf, "warm, ");
  else if (cond->temp > 60)
    strcat(buf, "mild, ");
  else if (cond->temp > 50)
    strcat(buf, "cool, ");
  else if (cond->temp > 40)
    strcat(buf, "cold, ");
  else if (cond->temp > 30)
    strcat(buf, "freezing, ");
  else if (cond->temp > 15)
    strcat(buf, "well past freezing, ");
  else
    strcat(buf, "numbingly frozen, ");

  strcat(buf, "and ");

  if (cond->windspeed <= 0)
    strcat(buf, "there is absolutely no wind");
  else if (cond->windspeed < 10)
    strcat(buf, "calm");
  else if (cond->windspeed < 20)
    strcat(buf, "breezy");
  else if (cond->windspeed < 35)
    strcat(buf, "windy");
  else if (cond->windspeed < 50)
    strcat(buf, "very windy");
  else if (cond->windspeed < 70)
    strcat(buf, "very, very windy");
  else if (cond->windspeed < 100)
    strcat(buf, "there is a gale blowing");
  else
    strcat(buf, "the wind is unbelievable");
  strcat(buf, ".\r\n");
  send_to_char(buf, ch);
  if (GET_CLASS(ch) == CLASS_CLERIC || GET_CLASS(ch) == CLASS_MAGIC_USER) {
    if (cond->free_energy > 40000)
      send_to_char("Wow! This place is bursting with energy!\r\n", ch);
    else if (cond->free_energy > 30000)
      send_to_char("The environs tingle your magical senses.\r\n", ch);
    else if (cond->free_energy > 20000)
      send_to_char("The area is rich with energy.\r\n", ch);
    else if (cond->free_energy < 4000)
      send_to_char("There is almost no magical energy here.\r\n", ch);
    else if (cond->free_energy < 5000)
      send_to_char("Your magical senses are dulled by the scarceness of energy here.\r\n", ch);
  }
  send_to_char("\r\n", ch);
}
