/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "interpreter.h"
#include "spells.h"
#include "handler.h"
#include "comm.h"
#include "events.h"
#include "db.h"
#include "dg_scripts.h"

#define DEBUG_GRANOR 1

struct spell_info_type spell_info[TOP_SPELL_DEFINE + 1];
extern struct room_data *world;
extern char *class_types[];
extern char *dirs[];

EVENTFUNC(cast_event);


/*
 * This arrangement is pretty stupid, but the number of spells is limited by
 * the playerfile.  We can arbitrarily increase the number of spells by
 * increasing the space in the playerfile. Meanwhile, this should provide
 * ample slots for spells.
 */

char *spells[] =
{
  "!RESERVED!",			/* 0 - reserved */
  "bless",
  "command",
  "create water",
  "cure light",
  "cause light",
  "detect evil",
  "detect good",
  "detect magic",
  "light",
  "darkness",
  "prot from evil",
  "prot from good",
  "purify food water",
  "contam food water",
  "remove fear",
  "fear",
  "resist cold",
  "sanctuary",
  "augury",
  "chant",
  "detect charm",
  "combine",
  "find traps",
  "hold person",
  "know align",
  "resist fire",
  "silence 15",
  "slow poison",
  "snake charm",
  "speak animals",
  "spirit hammer",
  "animate dead",
  "continual light",
  "continual darkness",
  "create food",
  "detect poison",
  "cure blindness",
  "cause blindness",
  "cure disease",
  "cause disease",
  "dispel magic",
  "feign death",
  "glyph warding",
  "locate object",
  "obscure object",
  "prayer",
  "remove curse",
  "curse",
  "speak with dead",
  "cure serious",
  "cause serious",
  "detect lie",
  "undet lie",
  "divination",
  "endure cold",
  "lower water",
  "raise water",
  "neut poison",
  "poison",
  "prot evil 10",
  "prot good 10",
  "speak plants",
  "sticks snakes",
  "snakes sticks",
  "tongues",
  "rev tongues",
  "atonement",
  "commune",
  "cure critic",
  "cause critic",
  "dispel evil",
  "dispel good",
  "flame strike",
  "insect plague",
  "plane shift",
  "quest",
  "raise dead",
  "slay living",
  "true seeing",
  "false seeing",
  "aerial servant",
  "animate object",
  "blade barrier",
  "conjure animals",
  "find path",
  "lose path",
  "heal",
  "harm",
  "part water",
  "speak monsters",
  "stone tell",
  "word recall",
  "astral spell",
  "control weather",
  "earthquake",
  "holyword",
  "unholyword",
  "regenerate",
  "wither",
  "restoration",
  "energy drain",
  "resurection",
  "destruction",
  "symbol",
  "wind walk",
  "animal friendship",
  "endure heat",
  "detect snares",
  "entangle",
  "faerie fire",
  "invis animals",
  "locate animal",
  "pass without trace",
  "invis undead",
  "magical stone",
  "aid",
  "shillelagh",
  "dust devil",
  "barkskin",
  "charm mammal",
  "fire trap",
  "heat metal",
  "chill metal",
  "locate plants",
  "obscurement",
  "produce flame",
  "trip",
  "warp wood",
  "call lightning",
  "hold animal",
  "plant growth",
  "prot fire",
  "pyrotechnics",
  "snare",
  "stone shape",
  "summon insects",
  "tree",
  "water breathing",
  "air breathing",
  "animal summ1",
  "call wood beings",
  "cont temp 10",
  "halluc forest",
  "hold plant",
  "plant door",
  "prod fire",
  "quench fire",
  "prot lightning",
  "repel insects",
  "animal growth",
  "reduce animal",
  "anim summ2",
  "anti plant shell",
  "commune nature",
  "control winds",
  "enthrall",
  "pass plant",
  "rock mud",
  "mud rock",
  "wall fire",
  "anim summ3",
  "anti anim shell",
  "conj fire elemental",
  "dismiss fire elemental",
  "flame blade",
  "fire seeds",
  "trans via plants",
  "turn wood",
  "wall thorns",
  "weather summoning",
  "animate rock",
  "chariot sustarre",
  "confusion",
  "conj earth elemental",
  "dismiss earth elemental",
  "goodberry",
  "creeping doom",
  "badberry",
  "fire storm",
  "fire quench",
  "reincarnate",
  "trans metal wood",
  "messenger",
  "withdraw",
  "wyvern watch",
  "flame walk",
  "magical vestment",
  "meld into stone",
  "neg plane prot",
  "remove para",
  "spike growth",
  "starshine",
  "water walk",
  "abjure",
  "cloak bravery",
  "free action",
  "giant insect",
  "small insect",
  "imbue spell ability",
  "reflecting pool",
  "spell immunity",
  "air walk",
  "magic font",
  "moonbeam",
  "rainbow",
  "spike stones",
  "forbiddance",
  "heros feast",
  "liveoak",
  "water to dust",
  "dust to water",
  "changestaff",
  "exaction",
  "gate",
  "sunray",
  "succor",
  "wood to metal",
  "affect normal fires",
  "alarm",
  "armor",
  "audible glamer",
  "burning hands",
  "cantrip",
  "change self",
  "chill touch",
  "color spray",
  "comprehend languages",
  "obscure languages",
  "dancing lights",
  "detect undead",
  "enlarge",
  "reduce",
  "erase",
  "feather fall",
  "find familiar",
  "friends",
  "gaze reflection",
  "grease",
  "hold portal",
  "hypnotism",
  "identify",
  "jump",
  "crystalbrittle",
  "magic missile",
  "mending",
  "message",
  "mount",
  "nystul magical aura",
  "phantasmal force",
  "read magic",
  "shield",
  "shocking grasp",
  "sleep",
  "spider climb",
  "spook",
  "taunt",
  "tenser floating disc",
  "unseen servant",
  "ventriloquism",
  "wall of fog",
  "wizard mark",
  "alter self",
  "bind",
  "blur",
  "deafness",
  "deeppockets",
  "detect invisibility",
  "esp",
  "flaming sphere",
  "fog cloud",
  "fools gold",
  "forget",
  "glitterdust",
  "hypnotic pattern",
  "improved phantasmal force",
  "invisibility",
  "irritation",
  "knock",
  "leomund trap",
  "levitate",
  "magic mouth",
  "melf acid arrow",
  "mirror image",
  "misdirection",
  "protection from cantrips",
  "weird",
  "ray of enfeeblement",
  "rope trick",
  "scare",
  "shatter",
  "spectral hand",
  "stinking cloud",
  "strength",
  "summon swarm",
  "tasha uncontrollable hideous laughter",
  "web",
  "whispering wind",
  "wizard lock",
  "blink",
  "clairaudience",
  "clairvoyance",
  "delude",
  "explosive runes",
  "fireball",
  "flame arrow",
  "fly",
  "gust of wind",
  "haste",
  "hold undead",
  "illusionary script",
  "infravision",
  "invisibility 10",
  "item",
  "leomund tiny hut",
  "lightning bolt",
  "melf minute meteors",
  "monster summoning I",
  "non detection",
  "phantom steed",
  "protection from normal missiles",
  "secret page",
  "sepia snake sigil",
  "slow",
  "spectral force",
  "suggestion",
  "vampiric touch",
  "wind wall",
  "wraithform",
  "charm monster",
  "foresight",
  "contagion",
  "detect scrying",
  "dig",
  "dimension door",
  "emotion",
  "enchanted weapon",
  "enervation",
  "evard black tentacles",
  "extension I",
  "wish",
  "fire charm",
  "fire shield",
  "imprisonment",
  "fumble",
  "hallucinatory terrain",
  "ice storm",
  "illusionary wall",
  "improved invisibility",
  "leomund secure shelter",
  "magic mirror",
  "massmorph",
  "minor creation",
  "minor globe of invulnerability",
  "monster summoning ii",
  "otiluke resilient sphere",
  "phantasmal killer",
  "meteor swarm",
  "polymorph other",
  "polymorph self",
  "rainbow pattern",
  "rary mnemonic enhancer",
  "shadow monsters",
  "shout",
  "solid fog",
  "stoneskin",
  "vacancy",
  "wall of fire",
  "wall of ice",
  "wizard eye",
  "advanced illusion",
  "airy water",
  "monster summoning VII",
  "avoidance",
  "bigby interposing hand",
  "chaos",
  "cloudkill",
  "cone of cold",
  "conjure elemental",
  "contact other plane",
  "demi shadow monsters",
  "dismissal",
  "distance distortion",
  "domination",
  "dream",
  "extension II",
  "fabricate",
  "false vision",
  "feeblemind",
  "hold monster",
  "leomund lamentable belaborment",
  "leomund secret chest",
  "magic jar",
  "major creation",
  "monster summoning III",
  "mordenkainen faithful hound",
  "passwall",
  "seeming",
  "sending",
  "shadow door",
  "shadow magic",
  "mordenkainen disjunction",
  "summon shadow",
  "telekinesis",
  "teleport",
  "transmute rock to mud",
  "transmute mud to rock",
  "wall of force",
  "wall of iron",
  "wall of stone",
  "anti magic shell",
  "bigby forceful hand",
  "chain lightning",
  "power word kill",
  "!UNUSED!",
  "contingency",
  "death fog",
  "death spell",
  "demi shadow magic",
  "disintegrate",
  "enchant an item",
  "ensnarement",
  "extension III",
  "eyebite",
  "geas",
  "glassee",
  "globe of invulnerability",
  "guards and wards",
  "invisible stalker",
  "legend lore",
  "mass suggestion",
  "mirage arcana",
  "mislead",
  "monster summoning IV",
  "mordenkainen lucubration",
  "move earth",
  "otiluke freezing sphere",
  "prismatic sphere",
  "permanent illusion",
  "programmed illusion",
  "project image",
  "reincarnation",
  "repulsion",
  "shades",
  "stone to flesh",
  "flesh to stone",
  "tenser transformation",
  "shape change",
  "veil",
  "banishment",
  "bigby grasping hand",
  "charm plants",
  "control undead",
  "delayed blast fireball",
  "drawmij instant summons",
  "duo dimension",
  "finger of death",
  "forcecage",
  "limited wish",
  "mass invisibility",
  "monster summoning V",
  "mordenkainen magnificent mansion",
  "mordenkainen sword",
  "phase door",
  "power word stun",
  "prismatic spray",
  "reverse gravity",
  "sequester",
  "shadow walk",
  "simulacrum",
  "spell turning",
  "statue",
  "teleport without error",
  "vanish",
  "vision",
  "antipathy sympathy",
  "bigby clenched fist",
  "binding",
  "clone",
  "demand",
  "glassteel",
  "incendiary cloud",
  "mass charm",
  "maze",
  "mind blank",
  "monster summoning VI",
  "otiluke telekinetic sphere",
  "otto irresistible dance",
  "permanency",
  "polymorph any object",
  "power word blind",
  "prismatic wall",
  "screen",
  "serten spell immunity",
  "sink",
  "temporal stasis",
  "trap the soul",
  "time stop",
  "bigby crushing hand",
  "cold shield", "blindness",	/* 500 */
  "charm person", "undetectable charm", "cause fear",
  "cause deafness", "cure deafness",	/* 505 */
  "!UNUSED!", "!UNUSED!", "!UNUSED!", "!UNUSED!", "!UNUSED!",	/* 510 */
  "!UNUSED!", "!UNUSED!", "!UNUSED!", "!UNUSED!", "!UNUSED!",	/* 515 */
  "!UNUSED!", "!UNUSED!", "!UNUSED!", "!UNUSED!", "!UNUSED!",	/* 520 */
  "\n"				/* the end */
};


struct syllable {
  char *org;
  char *new;
};


#define SYLS_SIZE 64

struct syllable syls[] =
{
  {" ", " "},
  {"ar", "abra"},
  {"ate", "i"},
  {"cau", "kada"},
  {"blind", "nose"},
  {"bur", "mosa"},
  {"cu", "judi"},
  {"de", "oculo"},
  {"dis", "mar"},
  {"ect", "kamina"},
  {"en", "uns"},
  {"gro", "cra"},
  {"light", "dies"},
  {"lo", "hi"},
  {"magi", "kari"},
  {"mon", "bar"},
  {"mor", "zak"},
  {"move", "sido"},
  {"ness", "lacri"},
  {"ning", "illa"},
  {"per", "duda"},
  {"ra", "gru"},
  {"re", "candus"},
  {"son", "sabru"},
  {"tect", "infra"},
  {"tri", "cula"},
  {"ven", "nofo"},
  {"word of", "inset"},
  {"a", "i"},
  {"b", "v"},
  {"c", "q"},
  {"d", "m"},
  {"e", "o"},
  {"f", "y"},
  {"g", "t"},
  {"h", "p"},
  {"i", "u"},
  {"j", "y"},
  {"k", "t"},
  {"l", "r"},
  {"m", "w"},
  {"n", "b"},
  {"o", "a"},
  {"p", "s"},
  {"q", "d"},
  {"r", "f"},
  {"s", "g"},
  {"t", "h"},
  {"u", "e"},
  {"v", "z"},
  {"w", "x"},
  {"x", "n"},
  {"y", "l"},
  {"z", "k"},
  {"0", "a"},
  {"1", "b"},
  {"2", "c"},
  {"3", "d"},
  {"4", "e"},
  {"5", "f"},
  {"6", "g"},
  {"7", "h"},
  {"8", "i"},
  {"9", "j"},
  {"", ""}
};



/* say_spell erodes buf, buf1, buf2 */
void say_spell(struct char_data *ch, int spellnum, struct char_data *tch,
	       struct obj_data *tobj)
{
  char lbuf[256];

  struct char_data *i;
  int j, ofs = 0;

  *buf = '\0';
  strcpy(lbuf, spells[spellnum]);

  while (*(lbuf + ofs)) {
    for (j = 0; *(syls[j].org); j++) {
      if (!strncmp(syls[j].org, lbuf + ofs, strlen(syls[j].org))) {
	strcat(buf, syls[number(0, SYLS_SIZE)].new);
	ofs += strlen(syls[j].org);
      }
    }
  }

  if (tch != NULL && tch->in_room == ch->in_room) {
    if (tch == ch)
      sprintf(lbuf, "$n closes $s eyes and utters the words, '%%s'.");
    else
      sprintf(lbuf, "$n stares at $N and utters the words, '%%s'.");
  } else if (tobj != NULL &&
	     ((tobj->in_room == ch->in_room) || (tobj->carried_by == ch)))
    sprintf(lbuf, "$n stares at $p and utters the words, '%%s'.");
  else
    sprintf(lbuf, "$n utters the words, '%%s'.");

  sprintf(buf1, lbuf, spells[spellnum]);
  sprintf(buf2, lbuf, buf);

  for (i = world[ch->in_room].people; i; i = i->next_in_room) {
    if (i == ch || i == tch || !i->desc || !AWAKE(i))
      continue;
    if (GET_CLASS(ch) == GET_CLASS(i))
      perform_act(buf1, ch, tobj, tch, i, TO_PROXIMITY);
    else
      perform_act(buf2, ch, tobj, tch, i, TO_PROXIMITY);
  }

  if (tch != NULL && tch != ch && tch->in_room == ch->in_room) {
    sprintf(buf1, "$n stares at you and utters the words, '%s'.",
	    GET_CLASS(ch) == GET_CLASS(tch) ? spells[spellnum] : buf);
    act(buf1, FALSE, ch, NULL, tch, TO_VICT);
  }
}


char *spell_name(int num)
{
  int i = 0;

  if (num <= 0) {
    if (num == -1)
      return "UNUSED";
    else
      return "UNDEFINED";
  }
  while (num && *spells[i] != '\n') {
    num--;
    i++;
  }

  if (*spells[i] != '\n')
    return spells[i];
  else
    return "UNDEFINED";
}


int find_spells_num(char *name)
{
  int index = 0, ok;
  char *temp, *temp2;
  char first[256], first2[256];

  while (*spells[++index] != '\n') {
    if (is_abbrev(name, spells[index]))
      return index;

    ok = 1;
    temp = any_one_arg(spells[index], first);
    temp2 = any_one_arg(name, first2);
    while (*first && *first2 && ok) {
      if (!is_abbrev(first2, first))
	ok = 0;
      temp = any_one_arg(temp, first);
      temp2 = any_one_arg(temp2, first2);
    }

    if (ok && !*first2)
      return index;
  }

  return -1;
}



/*
 * This function is the very heart of the entire magic system.  All
 * invocations of all types of magic -- objects, spoken and unspoken PC
 * and NPC spells, the works -- all come through this function eventually.
 * This is also the entry point for non-spoken or unrestricteded spells.
 * Spellnum 0 is legal but silently ignored here, to make callers simpler.
 */
int call_magic(struct char_data *caster, struct char_data *cvict,
	   struct obj_data *ovict, int spellnum, int level, int casttype,
	       struct location_data *loc)
{
  int savetype;

  if (spellnum < 1 || spellnum > TOP_SPELL_DEFINE)
    return 0;

  if (caster->nr != real_mobile(DG_CASTER_PROXY)) {
    if (ROOM_FLAGGED(caster->in_room, ROOM_NOMAGIC)) {
      send_to_char("Your magic fizzles out and dies.\r\n", caster);
      act("$n's magic fizzles out and dies.", FALSE, caster, 0, 0, PROXIMITY);
      return 0;
    }
    if (IS_SET(ROOM_FLAGS(caster->in_room), ROOM_PEACEFUL) &&
	(SINFO.violent)) {
      send_to_char("A flash of white light fills the room, dispelling your "
		   "violent magic!\r\n", caster);
      act("White light from no particular source suddenly fills the room, "
	  "then vanishes.", FALSE, caster, 0, 0, PROXIMITY);
      return 0;
    }
  }
  /* determine the type of saving throw */
  switch (casttype) {
  case CAST_STAFF:
  case CAST_SCROLL:
  case CAST_POTION:
  case CAST_WAND:
    savetype = SAVING_ROD;
    break;
  case CAST_SPELL:
    savetype = SAVING_SPELL;
    break;
  default:
    savetype = SAVING_BREATH;
    break;
  }

  ((*spell_info[spellnum].spell_pointer) (level, caster, casttype, savetype,
					  cvict, ovict, loc));
  return 1;
}

/*
 * mag_objectmagic: This is the entry-point for all magic items.  This should
 * only be called by the 'quaff', 'use', 'recite', etc. routines.
 *
 * For reference, object values 0-3:
 * staff  - [0] level   [1] max charges [2] num charges [3] spell num
 * wand   - [0] level   [1] max charges [2] num charges [3] spell num
 * scroll - [0] level   [1] spell num   [2] spell num   [3] spell num
 * potion - [0] level   [1] spell num   [2] spell num   [3] spell num
 *
 * Staves and wands will default to level 14 if the level is not specified;
 * the DikuMUD format did not specify staff and wand levels in the world
 * files (this is a CircleMUD enhancement).
 */

void mag_objectmagic(struct char_data *ch, struct obj_data *obj,
		     char *argument, int which_value)
{
  int i, k, empty;
  struct char_data *tch = NULL, *next_tch;
  struct obj_data *tobj = NULL;
  char *spell_name;

  one_argument(argument, arg);

  k = generic_find(arg, FIND_CHAR_ROOM | FIND_OBJ_INV | FIND_OBJ_ROOM |
		   FIND_OBJ_EQUIP, ch, &tch, &tobj);

  switch (GET_OBJ_TYPE(obj)) {
  case ITEM_STAFF:
    act("You tap $p three times on the ground.", FALSE, ch, obj, 0, TO_CHAR);
    if (obj->action_description)
      act(obj->action_description, FALSE, ch, obj, 0, PROXIMITY);
    else
      act("$n taps $p three times on the ground.", FALSE, ch, obj, 0, PROXIMITY);

    if (GET_OBJ_VAL(obj, 2) <= 0) {
      act("It seems powerless.", FALSE, ch, obj, 0, TO_CHAR);
      act("Nothing seems to happen.", FALSE, ch, obj, 0, PROXIMITY);
    } else {
      GET_OBJ_VAL(obj, 2)--;
      WAIT_STATE(ch, PULSE_VIOLENCE);
      for (tch = world[ch->in_room].people; tch; tch = next_tch) {
	next_tch = tch->next_in_room;
	if (ch == tch)
	  continue;
	if (GET_OBJ_VAL(obj, 0))
	  call_magic(ch, tch, NULL, GET_OBJ_VAL(obj, 3),
		     GET_OBJ_VAL(obj, 0), CAST_STAFF, NULL);
	else
	  call_magic(ch, tch, NULL, GET_OBJ_VAL(obj, 3),
		     DEFAULT_STAFF_LVL, CAST_STAFF, NULL);
      }
    }
    break;
  case ITEM_WAND:
    if (k == FIND_CHAR_ROOM) {
      if (tch == ch) {
	act("You point $p at yourself.", FALSE, ch, obj, 0, TO_CHAR);
	act("$n points $p at $mself.", FALSE, ch, obj, 0, PROXIMITY);
      } else {
	act("You point $p at $N.", FALSE, ch, obj, tch, TO_CHAR);
	if (obj->action_description != NULL)
	  act(obj->action_description, FALSE, ch, obj, tch, PROXIMITY);
	else
	  act("$n points $p at $N.", TRUE, ch, obj, tch, PROXIMITY);
      }
    } else if (tobj != NULL) {
      act("You point $p at $P.", FALSE, ch, obj, tobj, TO_CHAR);
      if (obj->action_description != NULL)
	act(obj->action_description, FALSE, ch, obj, tobj, PROXIMITY);
      else
	act("$n points $p at $P.", TRUE, ch, obj, tobj, PROXIMITY);
    } else {
      act("At what should $p be pointed?", FALSE, ch, obj, NULL, TO_CHAR);
      return;
    }

    if (GET_OBJ_VAL(obj, 2) <= 0) {
      act("It seems powerless.", FALSE, ch, obj, 0, TO_CHAR);
      act("Nothing seems to happen.", FALSE, ch, obj, 0, PROXIMITY);
      return;
    }
    GET_OBJ_VAL(obj, 2)--;
    WAIT_STATE(ch, PULSE_VIOLENCE);
    if (GET_OBJ_VAL(obj, 0))
      call_magic(ch, tch, tobj, GET_OBJ_VAL(obj, 3),
		 GET_OBJ_VAL(obj, 0), CAST_WAND, NULL);
    else
      call_magic(ch, tch, tobj, GET_OBJ_VAL(obj, 3),
		 DEFAULT_WAND_LVL, CAST_WAND, NULL);
    break;
  case ITEM_SCROLL:
    if (*arg) {
      if (!k) {
	act("There is nothing to here to affect with $p.", FALSE,
	    ch, obj, NULL, TO_CHAR);
	return;
      }
    } else
      tch = ch;

    if (which_value > 0) {
      sprintf(buf, "%s", spells[GET_OBJ_VAL(obj, which_value)]);
      spell_name = str_dup(buf);
    } else
      spell_name = str_dup("all the spells");

    sprintf(buf, "You recite %s from $p.", spell_name);
    act(buf, TRUE, ch, obj, 0, TO_CHAR);
    if (obj->action_description)
      act(obj->action_description, FALSE, ch, obj, NULL, PROXIMITY);
    else
      act("$n recites $p.", FALSE, ch, obj, NULL, PROXIMITY);

    WAIT_STATE(ch, PULSE_VIOLENCE);
    if (which_value > -1) {
      call_magic(ch, tch, tobj, GET_OBJ_VAL(obj, which_value), GET_OBJ_VAL(obj, 0), CAST_SCROLL, NULL);
      /* see if its empty now */
      empty = TRUE;
      for (i = 1; i < 8; i++)
	if (GET_OBJ_VAL(obj, i) > 0)
	  empty = FALSE;
    } else {			/* casting all at once */
      for (i = 1; i < 8; i++)
	if (!(call_magic(ch, tch, tobj, GET_OBJ_VAL(obj, i),
			 GET_OBJ_VAL(obj, 0), CAST_SCROLL, NULL))) {
	  GET_OBJ_VAL(obj, i) = 0;
	  break;
	}
      empty = TRUE;
    }

    if (empty && obj != NULL) {
      send_to_char("The empty parchment dissolves in your hands.\r\n", ch);
      extract_obj(obj);
    }
    break;
  case ITEM_POTION:
    tch = ch;
    act("You quaff $p.", FALSE, ch, obj, NULL, TO_CHAR);
    if (obj->action_description)
      act(obj->action_description, FALSE, ch, obj, NULL, PROXIMITY);
    else
      act("$n quaffs $p.", TRUE, ch, obj, NULL, PROXIMITY);

    WAIT_STATE(ch, PULSE_VIOLENCE);
    for (i = 1; i < 8; i++) {
      if (GET_OBJ_VAL(obj, i) > 0)
	if (!(call_magic(ch, ch, NULL, GET_OBJ_VAL(obj, i),
			 GET_OBJ_VAL(obj, 0), CAST_POTION, NULL))) {
	  GET_OBJ_VAL(obj, i) = 0;	/* dosage used */
	  break;
	}
    }
    /* see if its empty now */
    empty = TRUE;
    for (i = 1; i < 8; i++)
      if (GET_OBJ_VAL(obj, i) > 0)
	empty = FALSE;
    if (empty && obj != NULL) {
      send_to_char("The empty potion bottle dissolves in your hands.\r\n", ch);
      extract_obj(obj);
    }
    break;
  default:
    log("SYSERR: Unknown object_type in mag_objectmagic");
    break;
  }
}


/*
 * cast_spell is used generically to cast any spoken spell, assuming we
 * already have the target char/obj and spell number.  It checks all
 * restrictions, prints the words, then if all is well, returns control
 * to the event creator.
 *
 * Entry point for NPC casts is in mob_cast_spell, not here!
 */

int cast_spell(struct char_data *ch, struct char_data *tch,
	       struct obj_data *tobj, int spellnum)
{
  char buf[256];

  if (spellnum < 0 || spellnum > TOP_SPELL_DEFINE) {
    sprintf(buf, "SYSERR: cast_spell trying to call spellnum %d\n", spellnum);
    log(buf);
    return 0;
  }
  if (GET_STAT(ch) < SINFO.min_position) {
    switch (GET_STAT(ch)) {
    case STAT_SLEEPING:
      send_to_char("You dream about great magical powers.\r\n", ch);
      break;
    case STAT_RESTING:
      send_to_char("You cannot concentrate while resting.\r\n", ch);
      break;
    default:
      send_to_char("You can't do much of anything like this!\r\n", ch);
      break;
    }
    return 0;
  }
  if (IS_AFFECTED(ch, AFF_CHARM) && (ch->master == tch)) {
    send_to_char("You are afraid you might hurt your master!\r\n", ch);
    return 0;
  }
  if ((tch != ch) && IS_SET(SINFO.targets, TAR_SELF_ONLY)) {
    send_to_char("You can only cast this spell upon yourself!\r\n", ch);
    return 0;
  }
  if ((tch == ch) && IS_SET(SINFO.targets, TAR_NOT_SELF)) {
    send_to_char("You cannot cast this spell upon yourself!\r\n", ch);
    return 0;
  }
#if 0				/* leave for npc? */
  send_to_char(OK, ch);
  say_spell(ch, spellnum, tch, tobj);
  return (call_magic(ch, tch, tobj, spellnum, GET_LEVEL(ch), CAST_SPELL, NULL);
#else
  return 1;
#endif
}


int find_spell_location(struct char_data *ch, char *arg,
			struct location_data *loc, int spellnum)
{
  char target = 0;
  int angle = 0, i = -1, j = -1;
  char distance[MAX_STRING_LENGTH], direction[MAX_STRING_LENGTH];

  if ((arg == NULL) || (!*arg))
    return target;

  if ((SINFO.range + SINFO.range_by_level) == 0) {
    one_argument(arg, direction);
    if (!*direction) {
      send_to_char("Syntax: cast <spell> direction\r\n", ch);
      return target;
    }
    if (is_number(direction))
      j = atoi(direction);
  } else {
    two_arguments(arg, distance, direction);

    if ((!*distance) || !is_number(distance)) {
      send_to_char("Syntax: cast <spell> distance direction\r\n", ch);
      return target;
    }
    j = atoi(distance);
    if (j < 0) {
      send_to_char("Range must be greater than 0.\r\n", ch);
      send_to_char("DEBUG: arg = ", ch);
      send_to_char(arg, ch);
      send_to_char("\r\n", ch);
      return target;
    }
  }

  if (IS_SET(SINFO.targets, TAR_ROOM_WORLD)) {
    if (j < 0) {
      send_to_char("That is not a valid number.", ch);
      return target;
    }
    loc->real_room = j;
    return 1;
  }
  if (!is_number(direction)) {
    for (i = 0; i < NUM_OF_DIRS; i++) {
      if (str_cmp(direction, dirs[i]) == 0) {
	target = TRUE;
	break;
      }
    }
  } else {
    angle = atoi(direction);
    if (ch->player_specials->saved.angle_type)
      angle = -(angle - 90);
    target = TRUE;
  }

  if (!target) {
    send_to_char("You must specify either an angle or a cardinal direction.\r\n", ch);
    return target;
  }
  switch (i) {
  case NORTH:
    angle = 90;
    break;
  case EAST:
    angle = 0;
    break;
  case SOUTH:
    angle = -90;
    break;
  case WEST:
    angle = 180;
    break;
  case NORTHEAST:
    angle = 45;
    break;
  case NORTHWEST:
    angle = 135;
    break;
  case SOUTHEAST:
    angle = -45;
    break;
  case SOUTHWEST:
    angle = -135;
    break;
  default:
    break;
  }

  angle = MOD(angle, 360);

  loc->angle = angle;

  if (angle < 23 || angle > 337)
    i = EAST;
  else if (angle >= 23 && angle < 68)
    i = NORTHEAST;
  else if (angle >= 68 && angle < 113)
    i = NORTH;
  else if (angle >= 113 && angle < 158)
    i = NORTHWEST;
  else if (angle >= 158 && angle < 213)
    i = WEST;
  else if (angle >= 213 && angle < 248)
    i = SOUTHWEST;
  else if (angle >= 248 && angle < 303)
    i = SOUTH;
  else
    i = SOUTHEAST;

  SET_FACING(ch, i);

  unobstructed_line(ch, NULL, NULL, loc, j, angle, TRUE);

#if 0
  door_dir = i;
  t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
  c_alter = (world[ch->in_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
  switch (i) {
  case NORTH:
    loc->real_room = ch->in_room;
    loc->x_cord = GET_X_CORD(ch) - j;
    loc->y_cord = GET_Y_CORD(ch);
    loc->z_cord = GET_Z_CORD(ch);
    x = loc->x_cord;
    y = loc->y_cord;
    if (x < 1) {
      if (world[ch->in_room].dir_option[NORTH]) {
	loc->real_room = world[ch->in_room].dir_option[NORTH]->to_room;
	t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
	loc->x_cord += world[loc->real_room].x_size - t_alter;
      } else
	loc->x_cord = 1;
      loc->y_cord += GetYOffset(ch->in_room, NORTH);
      loc->z_cord += GetZOffset(ch->in_room, NORTH);
    }
    break;
  case EAST:
    loc->real_room = ch->in_room;
    loc->x_cord = GET_X_CORD(ch);
    loc->y_cord = GET_Y_CORD(ch) + j;
    loc->z_cord = GET_Z_CORD(ch);
    x = loc->x_cord;
    y = loc->y_cord;
    if (y >= world[ch->in_room].y_size - c_alter) {
      if (world[ch->in_room].dir_option[EAST]) {
	loc->real_room = world[ch->in_room].dir_option[EAST]->to_room;
	loc->y_cord -= world[ch->in_room].y_size - c_alter;
      } else
	loc->y_cord = world[ch->in_room].y_size - c_alter;
      loc->x_cord += GetXOffset(ch->in_room, EAST);
      loc->z_cord += GetZOffset(ch->in_room, EAST);
    }
    break;
  case SOUTH:
    loc->real_room = ch->in_room;
    loc->x_cord = GET_X_CORD(ch) + j;
    loc->y_cord = GET_Y_CORD(ch);
    loc->z_cord = GET_Z_CORD(ch);
    x = loc->x_cord;
    y = loc->y_cord;
    if (x >= world[ch->in_room].x_size - c_alter) {
      if (world[ch->in_room].dir_option[SOUTH]) {
	loc->real_room = world[ch->in_room].dir_option[SOUTH]->to_room;
	loc->x_cord -= world[ch->in_room].x_size - c_alter;
      } else
	loc->x_cord = world[ch->in_room].x_size - c_alter;
      loc->y_cord += GetYOffset(ch->in_room, SOUTH);
      loc->z_cord += GetZOffset(ch->in_room, SOUTH);
    }
    break;
  case WEST:
    loc->real_room = ch->in_room;
    loc->x_cord = GET_X_CORD(ch);
    loc->y_cord = GET_Y_CORD(ch) - j;
    loc->z_cord = GET_Z_CORD(ch);
    x = loc->x_cord;
    y = loc->y_cord;
    if (y < 1) {
      if (world[ch->in_room].dir_option[WEST]) {
	loc->real_room = world[ch->in_room].dir_option[WEST]->to_room;
	t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
	loc->y_cord += world[loc->real_room].y_size - t_alter;
      } else
	loc->y_cord = 1;
      loc->x_cord += GetXOffset(ch->in_room, WEST);
      loc->z_cord += GetZOffset(ch->in_room, WEST);
    }
    break;
  case UP:
  case DOWN:
    send_to_char("That is not a valid direction\r\n", ch);
    return 0;
    break;
  case NORTHEAST:
    loc->real_room = ch->in_room;
    loc->x_cord = GET_X_CORD(ch) - j;
    loc->y_cord = GET_Y_CORD(ch) + j;
    loc->z_cord = GET_Z_CORD(ch);
    x = loc->x_cord;
    y = loc->y_cord;
    if ((x < 1) && (y < world[ch->in_room].y_size - c_alter)) {
      if (world[ch->in_room].dir_option[NORTH]) {
	door_dir = NORTH;
	loc->real_room = world[ch->in_room].dir_option[NORTH]->to_room;
	t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
	loc->x_cord += world[loc->real_room].x_size - t_alter;
      } else
	loc->x_cord = 1;
      loc->y_cord += GetYOffset(ch->in_room, NORTH);
      loc->z_cord += GetZOffset(ch->in_room, NORTH);
    } else if ((x >= 1) && (y >= world[ch->in_room].y_size - c_alter)) {
      if (world[ch->in_room].dir_option[EAST]) {
	door_dir = EAST;
	loc->real_room = world[ch->in_room].dir_option[EAST]->to_room;
	loc->y_cord -= world[ch->in_room].y_size - c_alter;
      } else
	loc->y_cord = world[ch->in_room].y_size - c_alter;
      loc->x_cord += GetXOffset(ch->in_room, EAST);
      loc->z_cord += GetZOffset(ch->in_room, EAST);
    } else if ((x < 1) && (y >= world[ch->in_room].y_size - c_alter)) {
      if (world[ch->in_room].dir_option[NORTHEAST]) {
	loc->real_room = world[ch->in_room].dir_option[NORTHEAST]->to_room;
	t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
	loc->x_cord += world[loc->real_room].x_size - t_alter;
	loc->y_cord -= world[ch->in_room].y_size - c_alter;
      } else {
	loc->x_cord = 1;
	loc->y_cord = world[ch->in_room].y_size - c_alter;
      }
      loc->z_cord += GetZOffset(ch->in_room, NORTHEAST);
    }
    break;
  case NORTHWEST:
    loc->real_room = ch->in_room;
    loc->x_cord = GET_X_CORD(ch) - j;
    loc->y_cord = GET_Y_CORD(ch) - j;
    loc->z_cord = GET_Z_CORD(ch);
    x = loc->x_cord;
    y = loc->y_cord;
    if ((x < 1) && (y < world[ch->in_room].y_size - c_alter)) {
      if (world[ch->in_room].dir_option[NORTH]) {
	door_dir = NORTH;
	loc->real_room = world[ch->in_room].dir_option[NORTH]->to_room;
	t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
	loc->x_cord += world[loc->real_room].x_size - t_alter;
      } else
	loc->x_cord = 1;
      loc->y_cord += GetYOffset(ch->in_room, NORTH);
      loc->z_cord += GetZOffset(ch->in_room, NORTH);
    } else if ((x >= 1) && (y < 1)) {
      if (world[ch->in_room].dir_option[WEST]) {
	door_dir = WEST;
	loc->real_room = world[ch->in_room].dir_option[WEST]->to_room;
	t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
	loc->y_cord += world[loc->real_room].y_size - t_alter;
      } else
	loc->y_cord = 1;
      loc->x_cord += GetXOffset(ch->in_room, WEST);
      loc->z_cord += GetZOffset(ch->in_room, WEST);
    } else if ((x < 1) && (y < 1)) {
      if (world[ch->in_room].dir_option[NORTHWEST]) {
	loc->real_room = world[ch->in_room].dir_option[NORTHWEST]->to_room;
	t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
	loc->x_cord += world[loc->real_room].x_size - t_alter;
	loc->y_cord += world[loc->real_room].y_size - t_alter;
      } else {
	loc->x_cord = 1;
	loc->y_cord = 1;
      }
      loc->z_cord += GetZOffset(ch->in_room, NORTHWEST);
    }
    break;
  case SOUTHEAST:
    loc->real_room = ch->in_room;
    loc->x_cord = GET_X_CORD(ch) + j;
    loc->y_cord = GET_Y_CORD(ch) + j;
    loc->z_cord = GET_Z_CORD(ch);
    x = loc->x_cord;
    y = loc->y_cord;
    if ((x > world[ch->in_room].x_size - c_alter) &&
	(y < world[ch->in_room].y_size - c_alter)) {
      if (world[ch->in_room].dir_option[SOUTH]) {
	door_dir = SOUTH;
	loc->real_room = world[ch->in_room].dir_option[SOUTH]->to_room;
	loc->x_cord -= world[ch->in_room].x_size - c_alter;
      } else
	loc->x_cord = world[ch->in_room].x_size - c_alter;
      loc->y_cord += GetYOffset(ch->in_room, SOUTH);
      loc->z_cord += GetZOffset(ch->in_room, SOUTH);
    } else if ((x < world[ch->in_room].x_size - c_alter) &&
	       (y >= world[ch->in_room].y_size - c_alter)) {
      if (world[ch->in_room].dir_option[EAST]) {
	door_dir = EAST;
	loc->real_room = world[ch->in_room].dir_option[EAST]->to_room;
	loc->y_cord -= world[ch->in_room].y_size - c_alter;
      } else
	loc->y_cord = world[ch->in_room].y_size - c_alter;
      loc->x_cord += GetXOffset(ch->in_room, EAST);
      loc->z_cord += GetZOffset(ch->in_room, EAST);
    } else if ((x >= world[ch->in_room].x_size - c_alter)
	       && (y >= world[ch->in_room].y_size - c_alter)) {
      if (world[ch->in_room].dir_option[SOUTHEAST]) {
	loc->real_room = world[ch->in_room].dir_option[SOUTHEAST]->to_room;
	loc->x_cord -= world[ch->in_room].x_size - c_alter;
	loc->y_cord -= world[ch->in_room].y_size - c_alter;
      } else {
	loc->x_cord = world[ch->in_room].x_size - c_alter;
	loc->y_cord = world[ch->in_room].y_size - c_alter;
      }
      loc->z_cord += GetZOffset(ch->in_room, SOUTHEAST);
    }
    break;
  case SOUTHWEST:
    loc->real_room = ch->in_room;
    loc->x_cord = GET_X_CORD(ch) + j;
    loc->y_cord = GET_Y_CORD(ch) - j;
    loc->z_cord = GET_Z_CORD(ch);
    x = loc->x_cord;
    y = loc->y_cord;
    if ((x >= world[ch->in_room].x_size - c_alter) &&
	(y >= 1)) {
      if (world[ch->in_room].dir_option[SOUTH]) {
	door_dir = SOUTH;
	loc->real_room = world[ch->in_room].dir_option[SOUTH]->to_room;
	loc->x_cord -= world[ch->in_room].x_size - c_alter;
      } else
	loc->x_cord = world[ch->in_room].x_size - c_alter;
      loc->y_cord += GetYOffset(ch->in_room, SOUTH);
      loc->z_cord += GetZOffset(ch->in_room, SOUTH);
    } else if ((x < world[ch->in_room].x_size - c_alter) &&
	       (y < 1)) {
      if (world[ch->in_room].dir_option[WEST]) {
	door_dir = WEST;
	loc->real_room = world[ch->in_room].dir_option[WEST]->to_room;
	t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
	loc->y_cord += world[loc->real_room].y_size - t_alter;
      } else
	loc->y_cord = 1;
      loc->x_cord += GetXOffset(ch->in_room, WEST);
      loc->z_cord += GetZOffset(ch->in_room, WEST);
    } else if ((x >= world[ch->in_room].x_size - c_alter)
	       && (y < 1)) {
      if (world[ch->in_room].dir_option[SOUTHWEST]) {
	loc->real_room = world[ch->in_room].dir_option[SOUTHWEST]->to_room;
	loc->x_cord -= world[ch->in_room].x_size - c_alter;
	t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
	loc->y_cord += world[loc->real_room].y_size - t_alter;
      } else {
	loc->x_cord = world[ch->in_room].x_size - c_alter;
	loc->y_cord = 1;
      }
      loc->z_cord += GetZOffset(ch->in_room, SOUTHWEST);
    }
    break;
  }
  range_dir = -1;
  dist_door = 3;
  range = charWithinRangeofDoor(ch, door_dir, 3);
  dist_door = distBetweenCharAndDoor(ch, door_dir);
  range_dir = angleBetweenCharAndDoor(ch, door_dir);
  if (loc->real_room != ch->in_room) {
    if ((!range ||
	 IS_SET(EXIT(ch, i)->exit_info, EX_CLOSED)) && (i == range_dir)) {
      loc->real_room = ch->in_room;
      t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
      loc->x_cord = world[ch->in_room].dir_option[door_dir]->x_cord;
      loc->y_cord = world[ch->in_room].dir_option[door_dir]->y_cord;
      loc->z_cord = world[ch->in_room].dir_option[door_dir]->z_cord;
    } else if ((j < dist_door) || (i != range_dir)) {
      loc->real_room = ch->in_room;
      t_alter = (world[loc->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
      loc->x_cord = GET_X_CORD(ch);
      loc->y_cord = GET_Y_CORD(ch);
      loc->z_cord = GET_Z_CORD(ch);
      switch (i) {
      case NORTH:
	loc->x_cord = 1;
	break;
      case EAST:
	loc->y_cord = world[loc->real_room].y_size - t_alter;
	break;
      case SOUTH:
	loc->x_cord = world[loc->real_room].x_size - t_alter;
	break;
      case WEST:
	loc->y_cord = 1;
	break;
      case NORTHEAST:
	y_dif = (world[loc->real_room].y_size - t_alter) - loc->y_cord;
	dif = MIN(y_dif, loc->x_cord - 1);
	loc->x_cord -= dif;
	loc->y_cord += dif;
	break;
      case NORTHWEST:
	y_dif = loc->y_cord - 1;
	dif = MIN(y_dif, loc->x_cord - 1);
	loc->x_cord -= dif;
	loc->y_cord -= dif;
	break;
      case SOUTHEAST:
	y_dif = (world[loc->real_room].y_size - t_alter) - loc->y_cord;
	x_dif = (world[loc->real_room].x_size - t_alter) - loc->x_cord;
	dif = MIN(y_dif, x_dif);
	loc->x_cord += dif;
	loc->y_cord += dif;
	break;
      case SOUTHWEST:
	x_dif = loc->x_cord - 1;
	dif = MIN(x_dif, loc->y_cord - 1);
	loc->x_cord += dif;
	loc->y_cord -= dif;
	break;
      }
    }
  }
  loc->x_cord = BOUNDED(1, loc->x_cord, world[loc->real_room].x_size - t_alter);
  loc->y_cord = BOUNDED(1, loc->y_cord, world[loc->real_room].y_size - t_alter);
  loc->z_cord = BOUNDED(0, loc->z_cord, world[loc->real_room].z_size);
#endif
  return 1;
}

/*
 * finds the target of spell spellnum cast by ch with arg
 * if found, tobj or tch is set to the target.  returns
 * TRUE if target is found, else FALSE
 */
int find_spell_target(struct char_data *ch, char *arg, int spellnum,
		      struct char_data **tch, struct obj_data **tobj)
{
  int i, target = FALSE;

  if (IS_SET(SINFO.targets, TAR_IGNORE)) {
    target = TRUE;
  } else if (arg != NULL && *arg) {
    if (!target && (IS_SET(SINFO.targets, TAR_CHAR_ROOM))) {
      if ((*tch = get_char_room_vis(ch, arg)) != NULL)
	target = TRUE;
    }
    if (!target && IS_SET(SINFO.targets, TAR_CHAR_WORLD))
      if ((*tch = get_char_vis(ch, arg)))
	target = TRUE;

    if (!target && IS_SET(SINFO.targets, TAR_OBJ_INV))
      if ((*tobj = get_obj_in_list_vis(ch, arg, ch->carrying))) {
	target = TRUE;
      }
    if (!target && IS_SET(SINFO.targets, TAR_OBJ_EQUIP)) {
      for (i = 0; !target && i < NUM_WEARS; i++)
	if (GET_EQ(ch, i) && isname(arg, GET_EQ(ch, i)->name)) {
	  *tobj = GET_EQ(ch, i);
	  target = TRUE;
	}
    }
    if (!target && IS_SET(SINFO.targets, TAR_OBJ_ROOM))
      if ((*tobj = get_obj_in_list_vis(ch, arg, world[ch->in_room].contents)))
	target = TRUE;

    if (!target && IS_SET(SINFO.targets, TAR_OBJ_WORLD))
      if ((*tobj = get_obj_vis(ch, arg)))
	target = TRUE;

  } else {			/* if target string is empty */
    if (!target && IS_SET(SINFO.targets, TAR_FIGHT_SELF))
      if (FIGHTING(ch) != NULL) {
	*tch = ch;
	target = TRUE;
      }
    if (!target && IS_SET(SINFO.targets, TAR_FIGHT_VICT))
      if ((FIGHTING(ch) != NULL) && (IN_ROOM(ch) == IN_ROOM(FIGHTING(ch))) &&
	  !IS_SET(SINFO.targets, TAR_CHAR_WORLD)) {
	/* extra checks made to make sure target hasn't fleed */
	*tch = FIGHTING(ch);
	target = TRUE;
      }
    /* if no target specified, and the spell isn't violent, default to self */
    if (!target && IS_SET(SINFO.targets, TAR_CHAR_ROOM) &&
	!SINFO.violent) {
      *tch = ch;
      target = TRUE;
    }
    /* default */
    if (!target && SINFO.violent)
      if ((FIGHTING(ch) != NULL) && (IN_ROOM(ch) == IN_ROOM(FIGHTING(ch))) &&
	  !IS_SET(SINFO.targets, TAR_CHAR_WORLD)) {
	/* extra checks made to make sure target hasn't fleed */
	*tch = FIGHTING(ch);
	target = TRUE;
      }
  }

  return target;
}

/*
 * do_cast is the entry point for PC-casted spells.  It parses the arguments,
 * determines the spell number and finds a target, throws the die to see if
 * the spell can be cast, checks silly things with cast_spell(), then sets up
 * the event, which later on calls the actual spell.
 */

ACMD(do_cast)
{
  struct char_data *tch = NULL;
  struct obj_data *tobj = NULL;
  struct location_data *rtar = NULL;
  struct spell_event *se;
  char *s, *t, *range = NULL;
  int spellnum, target = 0, range_location = 0;
  int rge = 0;
  int real_range = 0;

  /* prevent from casting multiple spells at a time (e.g. by force) */
  if (GET_ACTION(ch)) {
//    send_to_char("You are too busy to do this!\r\n", ch);
    //    return;
    send_to_char("You stop what you were doing to cast a spell.\r\n", ch);
    event_cancel(GET_ACTION(ch));
  }
  /* get: blank, spell name, target name */
  s = strtok(argument, "'");

  if (s == NULL) {
    send_to_char("Cast what where?\r\n", ch);
    return;
  }
  s = strtok(NULL, "'");
  if (s == NULL) {
    send_to_char("Spell names must be enclosed in the Holy Magic Symbols: '\r\n", ch);
    return;
  }
  t = strtok(NULL, "\0");

  spellnum = find_spells_num(s);

  if ((spellnum < 1) || (spellnum > LAST_SPELL)) {
    send_to_char("That spell does not exist.\r\n", ch);
    return;
  }
  if ((SINFO.min_level[CLASS_CLERIC] > LVL_IMPL) &&
      (SINFO.min_level[CLASS_MAGIC_USER] > LVL_IMPL)) {
    send_to_char("That spell has not been implemented yet.\r\n", ch);
    return;
  }
  if (GET_SPELL(ch, spellnum) == 0 && !IS_TRUSTED(ch)) {
    send_to_char("You are unfamiliar with that spell.\r\n", ch);
    return;
  }
  if (GET_SPELL(ch, spellnum) < MEMINFO_READY && !IS_TRUSTED(ch)) {
    send_to_char("You don't have that spell ready.\r\n", ch);
    return;
  }
  if (t != NULL) {
    /* Find the target or range */
    range = str_dup(t);
    one_argument(strcpy(arg, t), t);
    skip_spaces(&t);
  }
  target = find_spell_target(ch, t, spellnum, &tch, &tobj);
  real_range = SINFO.range + SINFO.range_by_level * GET_LEVEL(ch);
  if (!real_range)
    real_range = 1;
  if (!target && (!t || !*t)) {
    sprintf(buf, "%s should the spell be cast?\r\n",
      IS_SET(SINFO.targets, TAR_OBJ_ROOM | TAR_OBJ_INV | TAR_OBJ_WORLD) ?
	    "Upon what" : IS_SET(SINFO.targets, TAR_ROOM_RANGE) ? "Where" : "Upon who");
    send_to_char(buf, ch);
    return;
  }
  /*if (target && (tch == ch) && SINFO.violent) {
     send_to_char("You shouldn't cast that on yourself -- could be bad for your health!\r\n", ch);
     return;
     } */
  if (target && tch && !victWithinRangeofChar(ch, tch, real_range) &&
      !IS_SET(SINFO.targets, TAR_CHAR_WORLD)) {
    send_to_char("That person is out of range!\r\n", ch);
    return;
  }
  if (target && tobj && !objWithinRangeofChar(ch, tobj, real_range) &&
      !IS_SET(SINFO.targets, TAR_OBJ_WORLD)) {
    send_to_char("That object is out of range!\r\n", ch);
    return;
  }
  if (!target && IS_SET(SINFO.targets, TAR_ROOM_RANGE | TAR_ROOM_WORLD)) {
    CREATE(rtar, struct location_data, 1);
    range_location = find_spell_location(ch, range, rtar, spellnum);
    if (!target && !range_location)
      return;			// We've already given an error message in find_spell_location

  }
  if (range_location && !WithinRange(ch, rtar, real_range, &rge)
      && !IS_SET(SINFO.targets, TAR_ROOM_WORLD)) {
    send_to_char("That location is out of range!\r\n", ch);
    FREE(rtar);
    FREE(range);
    return;
  }
  if (!target && !range_location) {
    send_to_char("Cannot find the target of your spell!\r\n", ch);
    return;
  }
  if (!cast_spell(ch, tch, tobj, spellnum))
    return;

/*** from here down, is copied in mob_cast_spell() ***/
  /* create and initialize the cast event */
  CREATE(se, struct spell_event, 1);
  se->ch = ch;
  se->in_loc = FALSE;

  if (tch) {
    //send_to_char("&+CDEBUG:&n target is a tch\r\n", ch);
    se->target = GET_ID(tch);
  } else if (tobj) {
    //send_to_char("&+CDEBUG:&n Target is a tobj.\r\n", ch);
    se->target = GET_ID(tobj);
  } else if (range_location) {
    se->in_loc = TRUE;
    se->target = rtar->real_room;
    se->location.real_room = rtar->real_room;
    se->location.x_cord = rtar->x_cord;
    se->location.y_cord = rtar->y_cord;
    se->location.z_cord = rtar->z_cord;
    se->location.angle = rtar->angle;
    FREE(rtar);
  } else
    se->target = -1;

  FREE(range);

  strcpy(se->arg, arg);
  se->spellnum = spellnum;
  if (IS_TRUSTED(ch) || (SINFO.delay == 0))
    cast_event(se);
  else {
    if (GET_ACTION(ch)) {
      /* odd, we already have this at top of func */
      //send_to_char("You are already busy.\r\n", ch);
      //return;
      //send_to_char("You abort what you were doing to cast your spell.\r\n", ch);
      //event_cancel(GET_ACTION(ch));
      //GET_ACTION(ch) = NULL;
    }
    GET_ACTION(ch) = event_create(cast_event, se, (SINFO.delay * PASSES_PER_SEC * SECS_PER_SEGMENT));

    SET_BIT(ch->char_specials.saved.affected_by2, AFF2_CASTING);

    /* it would be nice to have a different message for each class here */
    if (IS_PRIEST(ch) && IS_PRIEST_SPELL(spellnum)) {
      act("$n raises $s holy symbol and begins chanting.", FALSE, ch, NULL, NULL, PROXIMITY);
      act("You raise your holy symbol and begin chanting.", FALSE, ch, NULL, NULL, TO_CHAR);
    } else {
      act("$n begins to cast a spell.", FALSE, ch, NULL, NULL, PROXIMITY);
      act("You begin to cast your spell.", FALSE, ch, NULL, NULL, TO_CHAR);
    }
  }
}


void spell_level(int spell, int class, int level, int components)
{
  char buf[256];
  int bad = 0;

  if (spell < 0 || spell > TOP_SPELL_DEFINE) {
    sprintf(buf, "SYSERR: attempting assign to illegal spellnum %d", spell);
    log(buf);
    return;
  }
  if (class < 0 || class >= NUM_CLASSES) {
    sprintf(buf, "SYSERR: assigning '%s' to illegal class %d",
	    spell_name(spell), class);
    log(buf);
    bad = 1;
  }
  if (level < 1 || level > LVL_IMPL) {
    sprintf(buf, "SYSERR: assigning '%s' to illegal level %d",
	    spell_name(spell), level);
    log(buf);
    bad = 1;
  }
  if (!bad) {
    spell_info[spell].min_level[class] = level;
    spell_info[spell].mat_comp[class] = components;
  }
}


/* Assign the spells on boot up */
void spello(int spl, int range, int range_by_level, int aoe, int aoe_by_level,
	    int minpos, byte components, int violent, int targets,
	    int Sphere, int School, int shape, byte hits,
	    int delay, byte(*Routine_pointer) (int,
					       struct char_data *,
					       struct obj_data *,
					       struct char_data *,
					       struct obj_data *,
					       struct location_data *,
					       int),
	    void (*Spell_pointer) (int, struct char_data *, int, int,
				   struct char_data *, struct obj_data *,
				   struct location_data * loc))
{
  int i;

  for (i = 0; i < NUM_CLASSES; i++) {
    spell_info[spl].min_level[i] = LVL_IMPL + 1;
    spell_info[spl].mat_comp[i] = 0;
  }
  spell_info[spl].range = range;
  spell_info[spl].range_by_level = range_by_level;
  spell_info[spl].components = components;
  spell_info[spl].area_of_affect = aoe;
  spell_info[spl].area_by_level = aoe_by_level;
  spell_info[spl].min_position = minpos;
  spell_info[spl].targets = targets;
  spell_info[spl].violent = violent;
  spell_info[spl].routines = Routine_pointer;
  spell_info[spl].spell_pointer = Spell_pointer;
  spell_info[spl].sphere = Sphere;
  spell_info[spl].school = School;
  spell_info[spl].shape = shape;
  spell_info[spl].hits = hits;
  spell_info[spl].delay = delay;
}


void unused_spell(int spl)
{
  int i;

  for (i = 0; i < NUM_CLASSES; i++)
    spell_info[spl].min_level[i] = LVL_IMPL + 1;
  spell_info[spl].range = 0;
  spell_info[spl].range_by_level = 0;
  spell_info[spl].area_of_affect = 0;
  spell_info[spl].area_by_level = 0;
  spell_info[spl].duration = 0;
  spell_info[spl].components = 0;
  spell_info[spl].min_position = 0;
  spell_info[spl].targets = 0;
  spell_info[spl].violent = 0;
  spell_info[spl].sphere = 0;
  spell_info[spl].school = 0;
  spell_info[spl].hits = 0;
  spell_info[spl].shape = 0;
  spell_info[spl].delay = 0;
}

/*
 * Arguments for spello calls:
 *
 * spl: The spell number, as defined in spells.h
 * 
 * range,range_by_level : Distance spell can travel BEFORE it has its 
 * affects.
 *
 * aoe, aoe_by_level: Distance from spell center that the affects can
 * be felt.
 *
 * min_pos: The minimum position caster must be in to cast the spell.
 *
 * components: If the caster needs to speak, make motions, or needs to
 * use reagants. Joined with an |
 *
 * targets: How the spell can be targetted. Joined with an |
 *
 * Sphere/School: Appropriate school/sphere caster must have to be able
 * to cast this spell.
 *
 * Shape: The shape of the affect.
 *
 * hits: Who it affects, everyone, friends or just enemies.
 *
 * delay: casting time, in segments.
 *
 * routine_pointer: Some spells do more than set aff flags and do damage. If
 * you want these affects to have an area affect, you must make a routine.
 *
 * spell_pointer: The starting point of the casting process.
 *
 */

/*
 * NOTE: SPELL LEVELS ARE NO LONGER ASSIGNED HERE AS OF Circle 3.0 bpl9.
 * In order to make this cleaner, as well as to make adding new classes
 * much easier, spell levels are now assigned in class.c.  You only need
 * a spello() call to define a new spell; to decide who gets to use a spell
 * or spell, look in class.c.  -JE 5 Feb 1996
 */

void mag_assign_spells(void)
{
  int i;

  /* Do not change the loop below */
  for (i = 1; i <= TOP_SPELL_DEFINE; i++)
    unused_spell(i);
  /* Do not change the loop above */


/* First Level Magic User Spells */
/*
   spello(SPELL_AFFECT_NORMAL_FIRES, 5, 5, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 1, r_none, cast_spell_affect_normal_fires);

   spello(SPELL_ALARM, 10, 0, 7, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ABJURATION, SHAPE_SQUARE, HIT_ALL, 10, r_none,
   cast_spell_alarm);
 */
  spello(SPELL_ARMOR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 FALSE, TAR_CHAR_ROOM, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_armor,
	 cast_spell_armor);
/*

   spello(SPELL_AUDIBLE_GLAMOR, 60, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | 
   COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL,
   1, r_none, cast_spell_audible_glamor);
 */
  spello(SPELL_BURNING_HANDS, 0, 0, 2, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
	 TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_CONE, HIT_ALL,
	 1, r_none, cast_spell_burning_hands);
/*
   spello(SPELL_CANTRIP, 3, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, 0, SCHOOL_ALL, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, 
   cast_spell_cantrip);

   spello(SPELL_CHANGE_SELF, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL,
   1, r_none, cast_spell_change_self);
 */
  spello(SPELL_CHARM_PERSON, 120, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
	 TAR_CHAR_ROOM, 0, SCHOOL_ENCHANTMENT, 0, HIT_ALL, 1, r_charm,
	 cast_spell_charm_person);
/*
   spello(SPELL_CHILL_TOUCH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_CHAR_ROOM, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_chill_touch);

   spello(SPELL_COLOR_SPRAY, 20, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC 
   | COM_MATERIAL, TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_CONE, HIT_ALL,
   1, r_none, cast_spell_color_spray);

   spello(SPELL_COMPREHEND_LANGUAGES, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, 
   SHAPE_UNDEFINED, HIT_ALL, 10, r_none, cast_spell_comprehend_languages);

   spello(SPELL_CONFUSE_LANGUAGES, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_confuse_languages);

   spello(SPELL_DANCING_LIGHTS, 40, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 1, r_none, cast_spell_dancing_lights);
 */
  spello(SPELL_DETECT_MAGIC, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_LESSER_DIVIN, 0, HIT_ALL, 1, r_detect_magic,
	 cast_spell_detect_magic);
/*
   spello(SPELL_DETECT_UNDEAD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_detect_undead);

   spello(SPELL_ENLARGE, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_enlarge);

   spello(SPELL_ERASE, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_OBJ_ROOM
   | TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, 
   cast_spell_erase);

   spello(SPELL_FEATHER_FALL, 10, 10, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_CHAR_ROOM,
   0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_feather_fall);

   spello(SPELL_FIND_FAMILIAR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 1200, r_none,
   cast_spell_find_familiar);

   spello(SPELL_FRIENDS, 0, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_friends);
 */
  spello(SPELL_GAZE_REFLECTION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, 0, HIT_ALL, 1, r_gaze_reflection,
	 cast_spell_gaze_reflection);
/*
   spello(SPELL_GREASE, 10, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TAR_ROOM_RANGE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_grease);

   spello(SPELL_HOLD_PORTAL, 20, 20, 7, 7, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE, 0,
   SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_hold_portal);

   spello(SPELL_HYPNOTISM, 5, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, 
   TAR_CHAR_ROOM, 0, SCHOOL_ENCHANTMENT, SHAPE_SQURE, HIT_ALL, 1, r_none,
   cast_spell_hypnotism);

   spello(SPELL_IDENTIFY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_INV, 0, SCHOOL_LESSER_DIVIN, 0, 0, 200, r_none, cast_spell_identify);

   spello(SPELL_JUMP, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, 1, r_none, 
   cast_spell_jump);

   spello(SPELL_LIGHT, 60, 0, 7, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL, FALSE, 
   TAR_ROOM_RANGE | TAR_CHAR_ROOM | TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_SPHERE,
   1, r_none, cast_spell_light);
 */
  spello(SPELL_MAGIC_MISSILE, 60, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
	 TAR_CHAR_ROOM | TAR_NOT_SELF, 0, SCHOOL_EVOCATION, SHAPE_SPHERE, HIT_ALL, 1, r_none,
	 cast_spell_magic_missile);
/*
   spello(SPELL_MENDING, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   1, r_none, cast_spell_mending);

   spello(SPELL_MESSAGE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_message);

   spello(SPELL_MOUNT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 60, r_none,
   cast_spell_mount);

   spello(SPELL_NYSTUL_MAGICAL_AURA, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED,
   HIT_ALL, 10, r_none, cast_spell_nystul_magical_aura);

   spello(SPELL_PHANTASMAL_FORCE, 60, 10, 7, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED,
   HIT_ALL, 1, r_none, cast_spell_phantasmal_force);
 */
  spello(SPELL_PROT_FROM_EVIL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL |
	 COM_SOMATIC | COM_MATERIAL, FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, 0, HIT_ALL, 1,
	 r_protection_from_evil, cast_spell_prot_from_evil);

  spello(SPELL_PROT_FROM_GOOD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL |
	 COM_SOMATIC | COM_MATERIAL, FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, 0, HIT_ALL, 1,
	 r_protection_from_good, cast_spell_prot_from_good);
/*
   spello(SPELL_READ_MAGIC, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_read_magic);

   spello(SPELL_REDUCE, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_reduce);
 */
  spello(SPELL_SHIELD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_EVOCATION, 0, HIT_ALL, 1, r_shield,
	 cast_spell_shield);

  spello(SPELL_SHOCKING_GRASP, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
	 TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_shocking_grasp,
	 cast_spell_shocking_grasp);

  spello(SPELL_SLEEP, 30, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 TRUE, TAR_ROOM_RANGE | TAR_CHAR_ROOM, 0, SCHOOL_ENCHANTMENT, SHAPE_SPHERE, HIT_ALL, 1, r_sleep,
	 cast_spell_sleep);

  spello(SPELL_SPIDER_CLIMB, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 FALSE, TAR_CHAR_ROOM, 0, SCHOOL_EVOCATION, 0, HIT_ALL, 1, r_spider_climb,
	 cast_spell_spider_climb);
/*
   spello(SPELL_SPOOK, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
   0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_spook);

   spello(SPELL_TAUNT, 60, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SPHERE, HIT_ALL, 1, r_none,
   cast_spell_taunt);

   spello(SPELL_TENSER_FLOATING_DISC, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 
   1, r_none, cast_spell_tenser_floating_disc);

   spello(SPELL_UNSEEN_SERVANT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
   1, r_none, cast_spell_unseen_servant);

   spello(SPELL_VENTRILOQUISM, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL, FALSE,
   TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, 
   cast_spell_ventriloquism);

   spello(SPELL_WALL_OF_FOG, 30, 0, 7, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_SQUARE, HIT_ALL,
   1, r_none, cast_spell_wall_of_fog);

   spello(SPELL_WIZARD_MARK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ALTERATION, 
   SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_wizard_mark);
 */
/* Second Level Magic User Spells */
/*
   spello(SPELL_ALTER_SELF, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   2, r_none, cast_spell_alter_self);

   spello(SPELL_BIND, 30, 0, 17, 1, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL,
   2, r_none, cast_spell_bind);
 */
  spello(SPELL_BLINDNESS, 30, 10, 0, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_CHAR_ROOM,
   0, SCHOOL_ILLUSION, 0, HIT_ALL, 2, r_blindness, cast_spell_blindness);

  spello(SPELL_BLUR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL,
	 2, r_blur, cast_spell_blur);
/*
   spello(SPELL_CONTINUAL_DARKNESS, 60, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_ROOM_RANGE | TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION,
   SHAPE_SPHERE, HIT_ALL, 2, r_none, cast_spell_continual_darkness);

   spello(SPELL_CONTINUAL_LIGHT, 60, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_ROOM_RANGE | TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION,
   SHAPE_SPHERE, HIT_ALL, 2, r_none, cast_spell_continual_light);

   spello(SPELL_DARKNESS, 10, 10, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 2, r_none,
   cast_darkness);

   spello(SPELL_DEAFNESS, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none,
   cast_deafness);

   spello(SPELL_DEEPPOCKETS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_deeppockets);
 */
  spello(SPELL_DETECT_EVIL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_LESSER_DIVIN, SHAPE_SPHERE, HIT_ALL, 2,
	 r_detect_evil, cast_spell_detect_evil);

  spello(SPELL_DETECT_GOOD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_LESSER_DIVIN, SHAPE_SPHERE, HIT_ALL, 2,
	 r_detect_good, cast_spell_detect_good);

  spello(SPELL_DETECT_INVISIBILITY, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
	 | COM_MATERIAL, FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_LESSER_DIVIN,
	 SHAPE_LINE, HIT_ALL, 2, r_detect_invisibility, cast_spell_detect_invisibility);
/*
   spello(SPELL_ESP, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, 0, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 2, r_none, 
   cast_spell_esp);

   spello(SPELL_FLAMING_SPHERE, 10, 0, 1, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_SPHERE, HIT_ALL, 2, r_none,
   cast_spell_flaming_sphere);

   spello(SPELL_FOG_CLOUD, 10, 0, 7, 7, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, 
   TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_CLOUD, HIT_ALL, 2, r_none,
   cast_spell_fog_cloud);

   spello(SPELL_FOOLS_GOLD, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none, 
   cast_spell_fools_gold);

   spello(SPELL_FORGET, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, 0,
   SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 2, r_none, cast_spell_forget);

   spello(SPELL_GLITTERDUST, 10, 10, 7, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, 0, SCHOOL_CONJURATION, SHAPE_CLOUD, HIT_ALL, 2, r_none, cast_spell_glitterdust);

   spello(SPELL_HYPNOTIC_PATTERN, 30, 0, 10, 0, POS_STANDING, COM_SOMATIC | COM_MATERIAL, TRUE,
   0, SCHOOL_ILLUSION, SHAPE_SQUARE, HIT_ALL, 2, r_none, cast_spell_hypnotic_pattern);

   spello(SPELL_IMPROVED_PHANTASMAL_FORCE, 60, 10, 60, 16, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, 0, SCHOOL_ILLUSION, SHAPE_SQUARE, HIT_ALL, 2, r_none,
   cast_spell_improved_phantasmal_force);
 */
  spello(SPELL_INVISIBILITY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
  FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ILLUSION, 0, HIT_ALL, 2, r_invisibility,
	 cast_spell_invisibility);
/*
   spello(SPELL_IRRITATION, 10, 10, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 2, r_none,
   cast_spell_irritation);

   spello(SPELL_KNOCK, 60, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION,
   SHAPE_UNDEFINED, HIT_ALL, 2, r_none, cast_spell_knock);

   spello(SPELL_KNOW_ALIGNMENT, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM | TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_LESSER_DIVIN, HIT_ALL, 10, r_none,
   cast_spell_know_alignment);

   spello(SPELL_LEOMUND_TRAP, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ILLUSION, HIT_ALL, 30, r_none,
   cast_spell_leomund_trap);
 */
  spello(SPELL_LEVITATE, 20, 20, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
    TRUE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, 0, HIT_ALL, 2, r_levitate,
	 cast_spell_levitate);
/* 
   spello(SPELL_LOCATE_OBJECT, 0, 0, 7, 7, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM, 0, SCHOOL_LESSER_DIVIN, 0, 0, 2, r_none, cast_spell_locate_object);

   spello(SPELL_LOCK, 60, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION,
   SHAPE_UNDEFINED, HIT_ALL, 2, r_none, cast_spell_lock);

   spello(SPELL_MAGIC_MOUTH, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   2, r_none, cast_magic_mouth);

   spello(SPELL_MELF_ACID_ARROW, 180, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, 0, SCHOOL_CONJURATION, SHAPE_LINE, HIT_ALL, 2, r_none,
   cast_melf_acid_arrow);

   spello(SPELL_MIRROR_IMAGE, 0, 0, 2, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 2, r_mirror_image,
   cast_spell_mirror_image);

   spello(SPELL_MISDIRECTION, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none, cast_spell_misdirection);

   spello(SPELL_OBSCURE_OBJECT, 0, 0, 7, 7, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM, 0, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 2, r_none, 
   cast_spell_obscure_object);

   spello(SPELL_PROTECTION_FROM_CANTRIPS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_protection_from_cantrips);

   spello(SPELL_PYROTECHNICS, 120, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_OBJ_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none,
   cast_spell_pyrotechnics);
 */
  spello(SPELL_RAY_OF_ENFEEBLEMENT, 10, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
	 TRUE, TAR_CHAR_ROOM, 0, SCHOOL_ENCHANTMENT, SHAPE_LINE, HIT_ALL, 2, r_ray_of_enfeeblement,
	 cast_spell_ray_of_enfeeblement);
/*
   spello(SPELL_ROPE_TRICK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none,
   cast_spell_rope_trick);

   spello(SPELL_SCARE, 30, 10, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SPHERE, HIT_ALL, 2, r_none,
   cast_spell_scare);

   spello(SPELL_SHATTER, 30, 10, 1, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 2, r_none,
   cast_spell_shatter);

   spello(SPELL_SPECTRAL_HAND, 30, 2, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 2, r_none,
   cast_spell_spectral_hand);
 */
  spello(SPELL_STINKING_CLOUD, 30, 0, 7, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
	 COM_MATERIAL, TRUE, TAR_CHAR_ROOM | TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_CLOUD, HIT_ALL,
	 2, r_none, cast_spell_stinking_cloud);

  spello(SPELL_STRENGTH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_strength,
	 cast_spell_strength);
/*
   spello(SPELL_SUMMON_SWARM, 60, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_CONJURATION, SHAPE_SQUARE, HIT_ALL, 2, r_none,
   cast_spell_summon_swarm);

   spello(SPELL_TASHA_UNCONTROLLABLE_HIDEOUS_LAUGHTER, 60, 0, 10, 0, POS_STANDING, COM_VERBAL |
   COM_SOMATIC | COM_MATERIAL, TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SQUARE,
   HIT_ALL, 2, r_none, cast_spell_tasha_uncontrollable_hideous_laughter);

   spello(SPELL_UNDETECTABLE_ALIGNMENT, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
   FALSE, TAR_CHAR_ROOM | TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED,
   HIT_ALL, 10, r_none, cast_spell_undetectable_alignment);

   spello(SPELL_WEB, 5, 5, 7, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, TRUE,
   TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_SQUARE, HIT_ALL, 2, r_none,
   cast_spell_web);

   spello(SPELL_WHISPERING_WIND, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none,
   cast_spell_whispering_wind);

   spello(SPELL_WIZARD_LOCK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none,
   cast_spell_wizard_lock);
 */
/* Third Level Magic-User Spells */
/*
   spello(SPELL_AIR_BREATHING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | 
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   3, r_water_breathing, cast_spell_air_breathing);
 */
  spello(SPELL_BLINK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
	 1, r_blink, cast_spell_blink);
/*
   spello(SPELL_CLAIRAUDIENCE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_WORLD, 0, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 3, r_none, 
   cast_spell_clairaudience);
 */
  spello(SPELL_CLAIRVOYANCE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 FALSE, TAR_ROOM_WORLD, 0, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 3, r_none,
	 cast_spell_clairvoyance);
/*
   spello(SPELL_DELUDE, 0, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_ROOM_RANGE,
   0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 3, r_none, cast_spell_delude);

   spello(SPELL_DISPEL_MAGIC, 120, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM | TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED,
   HIT_ALL, 3, r_none, cast_spell_dispel_magic);

   spello(SPELL_EXPLOSIVE_RUNES, 0, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 3, r_none, 
   cast_spell_explosive_runes);

   spello(SPELL_FEIGN_DEATH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_feign_death);
 */
  spello(SPELL_FIREBALL, 10, 10, 7, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 TRUE, TAR_CHAR_ROOM | TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_SPHERE, HIT_ALL,
	 3, r_none, cast_spell_fireball);
/*
   spello(SPELL_FLAME_ARROW, 30, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 3, r_none,
   cast_spell_flame_arrow);
 */
  spello(SPELL_FLY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
	 TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, 0, HIT_ALL, 3, r_fly, cast_spell_fly);
/*
   spello(SPELL_GUST_OF_WIND, 3, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_LINE, HIT_ALL, 3, r_none,
   cast_spell_gust_of_wind);
 */
  spello(SPELL_HASTE, 60, 0, 13, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 FALSE, TAR_ROOM_RANGE | TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_SQUARE, HIT_ALL,
	 3, r_haste, cast_spell_haste);
/*
   spello(SPELL_HOLD_PERSON, 120, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SQUARE, HIT_ALL, 3, r_none,
   cast_spell_hold_person);

   spello(SPELL_HOLD_UNDEAD, 20, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
   cast_spell_hold_undead);

   spello(SPELL_ILLUSIONARY_SCRIPT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_OBJ_INV, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_illusionary_script);
 */
  spello(SPELL_INFRAVISION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, 0, HIT_ALL, 10, r_infravision,
	 cast_spell_infravision);

  spello(SPELL_INVISIBILITY_10, 0, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
	 COM_MATERIAL, FALSE, TAR_CHAR_ROOM | TAR_OBJ_ROOM, 0, SCHOOL_ILLUSION, SHAPE_SPHERE,
	 HIT_ALL, 3, r_invisibility, cast_spell_invisibility_10);
/*
   spello(SPELL_ITEM, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   3, r_none, cast_spell_item);

   spello(SPELL_LEOMUND_TINY_HUT, 0, 0, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 3,
   r_none, cast_spell_leomud_tiny_hut);

   spello(SPELL_LIGHTNING_BOLT, 40, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL,
   3, r_none, cast_spell_lightning_bolt);

   spello(SPELL_MELF_MINUTE_METEORS, 70, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_CHAR_ROOM, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL,
   3, r_none, cast_spell_melf_minute_meteors);

   spello(SPELL_MONSTER_SUMMONING_I, 0, 0, 30, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 3,
   r_none, cast_spell_monster_summoning_i);

   spello(SPELL_NONDETECTION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED,
   HIT_ALL, 3, r_none, cast_spell_nondetection);

   spello(SPELL_PHANTOM_STEED, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_phantom_steed);

   spello(SPELL_PROT_EVIL_10, 0, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, SHAPE_SPHERE, HIT_ALL, 3, r_none,
   cast_spell_prot_evil_10);

   spello(SPELL_PROT_GOOD_10, 0, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, SHAPE_SPHERE, HIT_ALL, 3, r_none,
   cast_spell_prot_good_10);

   spello(SPELL_PROTECTION_FROM_NORMAL_MISSILES, 0, 0, 0, 0, POS_STANDING, COM_VERBAL |
   COM_SOMATIC | COM_MATERIAL, FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED,
   HIT_ALL, 3, r_none, cast_spell_protection_from_normal_missiles);

   spello(SPELL_SECRET_PAGE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_INV, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_secret_page);

   spello(SPELL_SEPIA_SNAKE_SIGIL, 5, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_INV, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 3, r_none,
   cast_spell_sepia_snake_sigil);
 */
  spello(SPELL_SLOW, 90, 10, 13, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 TRUE, TAR_ROOM_RANGE | TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_SQUARE, HIT_ALL,
	 3, r_slow, cast_spell_slow);
/*
   spello(SPELL_SPECTRAL_FORCE, 60, 1, 10, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_SQUARE, HIT_ALL, 3, r_none,
   cast_spell_spectral_force);

   spello(SPELL_SUGGESTION, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL, TRUE, 
   TAR_CHAR_ROOM, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 3, r_none,
   cast_spell_suggestion);

   spello(SPELL_TONGUES, 0, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL, FALSE,
   TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   3, r_none, cast_spell_tongues);

   spello(SPELL_TONGUES_REVERSE, 0, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL, FALSE,
   TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   3, r_none, cast_spell_tongues_reverse);

   spello(SPELL_VAMPIRIC_TOUCH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL,
   3, r_none, cast_spell_vampiric_touch);
 */
  spello(SPELL_WATER_BREATHING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
	 COM_MATERIAL, FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
	 3, r_water_breathing, cast_spell_water_breathing);
/*
   spello(SPELL_WIND_WALL, 10, 10, 3, 1, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 3, r_none,
   cast_spell_wind_wall);
 */
  spello(SPELL_WRAITHFORM, 0, 0, 0, 0, POS_STANDING, COM_SOMATIC | COM_MATERIAL, FALSE,
	 TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
	 1, r_wraithform, cast_spell_wraithform);
/* Fourth Level Magic-User Spells */
/*
   spello(SPELL_BESTOW_CURSE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none,
   cast_spell_bestow_curse);

   spello(SPELL_CHARM_MONSTER, 60, 0, 7, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SPHERE, HIT_ALL, 4, r_none,
   cast_spell_charm_monster);
 */
  spello(SPELL_COLD_SHIELD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_EVOCATION, 0, HIT_ALL, 4, r_cold_shield,
	 cast_spell_cold_shield);
/*
   spello(SPELL_CONFUSION, 120, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SQUARE, HIT_ALL, 4, r_none,
   cast_spell_confusion);

   spello(SPELL_CONTAGION, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_CHAR_ROOM, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 4, r_none, 
   cast_spell_contagion);

   spello(SPELL_DETECT_SCRYING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 3, r_none,
   cast_spell_detect_scrying);

   spello(SPELL_DIG, 30, 0, 1, 1, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_SQUARE, HIT_ALL, 4, r_none, cast_spell_dig);

   spello(SPELL_DIMENSION_DOOR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE, 0,
   SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_dimension_door);

   spello(SPELL_EMOTION, 10, 10, 7, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_ROOM_RANGE,
   0, SCHOOL_ENCHANTMENT, SHAPE_SQUARE, HIT_ALL, 4, r_none, cast_spell_dimension_door);

   spello(SPELL_ENCHANT_WEAPON, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_INV, 0, SCHOOL_ENCHANTMENT, 0, 0, 10, r_none, cast_spell_enchant_weapon);

   spello(SPELL_ENERVATION, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_CHAR_ROOM,
   0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 4, r_none, cast_spell_enervation);

   spello(SPELL_EVARD_BLACK_TENTACLES, 30, 0, 10, 10, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, TRUE, TAR_ROOM_RANGE, 0, SCHOOL_CONJURATION, SHAPE_SQUARE, HIT_ALL, 10,
   r_none, cast_spell_evard_black_tentacles);

   spello(SPELL_EXTENSION_I, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE, 0, 
   SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none, cast_spell_extension_i);

   spello(SPELL_FEAR, 20, 0, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, TRUE,
   TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_CONE, HIT_ALL, 4, r_none, cast_spell_extension);

   spello(SPELL_FIRE_CHARM, 10, 0, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_OBJ_ROOM, 0, SCHOOL_ENCHANTMENT, SHAPE_SPHERE, HIT_ALL, 4, r_none, 
   cast_spell_fire_charm);
 */
  spello(SPELL_FIRE_SHIELD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_EVOCATION, 0, HIT_ALL, 4, r_fire_shield,
	 cast_spell_fire_shield);
/*
   spello(SPELL_FIRE_TRAP, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, 
   FALSE, TAR_OBJ_INV | TAR_OBJ_ROOM, 0, SCHOOL_ABJURATION, SHAPE_IGNORE, HIT_ALL, 60,
   r_none, cast_spell_fire_trap);

   spello(SPELL_FUMBLE, 10, 10, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SQUARE, HIT_ALL, 4, r_none, 
   cast_spell_fumble);

   spello(SPELL_HALLUCINATORY_TERRAIN, 20, 20, 3, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_SQUARE, HIT_ALL, 60,
   r_none, cast_spell_hallucinatory_terrain);

   spello(SPELL_ICE_STORM, 10, 10, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM | TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_SPHERE, HIT_ALL, 4,
   r_none, cast_spell_ice_storm);

   spello(SPELL_ILLUSIONARY_WALL, 30, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_SQUARE, HIT_ALL, 4, r_none,
   cast_spell_illusionary_wall);

   spello(SPELL_IMPROVED_INVISIBILITY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none, 
   cast_spell_improved_invisibility);

   spello(SPELL_LEOMUND_SECURE_SHELTER, 20, 0, 5, 5, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SQUARE, HIT_ALL,
   400, r_none, cast_spell_leomund_secure_shelter);

   spello(SPELL_MAGIC_MIRROR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL,
   600, r_none, cast_spell_magic_mirror);

   spello(SPELL_MASSMORPH, 10, 10, 3, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SQUARE, HIT_ALL, 4, r_none,
   cast_spell_massmorph);

   spello(SPELL_MINOR_CREATION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_minor_creation);

 */
  spello(SPELL_MINOR_GLOBE_OF_INVULNERABILITY, 0, 0, 2, 0, POS_STANDING, COM_VERBAL |
	 COM_SOMATIC | COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ABJURATION, SHAPE_SPHERE,
	 HIT_ALL, 4, r_minor_globe_of_invulnerability, cast_spell_minor_globe_of_invulnerability);

/*
   spello(SPELL_MONSTER_SUMMONING_II, 0, 0, 40, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 4,
   r_none, cast_spell_monster_summoning_ii);

   spello(SPELL_OTILUKE_RESILIENT_SPHERE, 20, 0, 3, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 4,
   r_none, cast_spell_otiluke_resilient_sphere);

   spello(SPELL_PHANTASMAL_KILLER, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_CHAR_ROOM, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none, 
   cast_spell_phantasmal_killer);

   spello(SPELL_PLANT_GROWTH, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, 
   TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none,
   cast_spell_plant_growth);

   spello(SPELL_POLYMORPH_OTHER, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none,
   cast_spell_polymorph_other);

   spello(SPELL_POLYMORPH_SELF, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_CHAR_ROOM |
   TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none,
   cast_spell_polymorph_self);

   spello(SPELL_RAINBOW_PATTERN, 10, 0, 5, 0, POS_STANDING, COM_SOMATIC | COM_MATERIAL, TRUE,
   TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SQUARE, HIT_ALL, 4, r_none,
   cast_spell_rainbow_pattern);

   spello(SPELL_RARY_MNEMONIC_ENHANCER, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 100, r_none, cast_spell_rary_mnemonic_enhancer);

   spello(SPELL_REMOVE_CURSE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, 
   TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none, 
   cast_spell_remove_curse);

   spello(SPELL_SHADOW_MONSTERS, 30, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_SQUARE, HIT_ALL, 4, r_none,
   cast_spell_shadow_monsters);

   spello(SPELL_SHOUT, 10, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL, TRUE, TAR_ROOM_RANGE,
   0, SCHOOL_EVOCATION, SHAPE_CONE, HIT_ALL, 1, r_none, cast_spell_shout);

   spello(SPELL_SOLID_FOG, 30, 0, 7, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_CLOUD, HIT_ALL, 4, r_none,
   cast_spell_solid_fog);

   spello(SPELL_STONESKIN, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_stoneskin);

   spello(SPELL_VACANCY, 10, 10, 3, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 4, r_none,
   cast_spell_vacancy);

   spello(SPELL_WALL_OF_FIRE, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none,
   cast_spell_wall_fire);

   spello(SPELL_WALL_OF_ICE, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none,
   cast_spell_wall_of_ice);

   spello(SPELL_WIZARD_EYE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_wizard_eye):
 */
/* Fifth Level Magic-User Spells */
/*
   spello(SPELL_ADVANCED_ILLUSION, 60, 10, 7, 1, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_SQUARE, HIT_ALL, 10,
   r_none, cast_spell_advanced_illusion);

   spello(SPELL_AIRY_WATER, 0, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 5, r_none,
   cast_spell_airy_water);

   spello(SPELL_ANIMAL_GROWTH, 60, 0, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SQUARE, HIT_ALL, 5, r_none,
   cast_spell_animal_growth);

   spello(SPELL_ANIMATE_DEAD, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 50, r_none,
   cast_spell_animate_dead);

   spello(SPELL_ATTRACTION, 10, 0, 1, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM, 0, SCHOOL_ABJURATION, SHAPE_SQUARE, HIT_ALL, 5, r_none,
   cast_spell_attraction);

   spello(SPELL_AVOIDANCE, 10, 0, 1, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_ROOM, 0, SCHOOL_ABJURATION, SHAPE_SQUARE, HIT_ALL, 5, r_none,
   cast_spell_avoidance);

   spello(SPELL_BIGBY_INTERPOSING_HAND, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
   | COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 5,
   r_none, cast_spell_bigby_interposing_hand);

   spello(SPELL_CHAOS, 5, 5, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SQUARE, HIT_ALL, 5, r_none,
   cast_spell_chaos);

   spello(SPELL_CLOUDKILL, 10, 0, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_CLOUD, HIT_ALL, 5, r_none,
   cast_spell_cloudkill);
 */
  spello(SPELL_CONE_OF_COLD, 2, 0, 1, 1, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 TRUE, TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_CONE, HIT_ALL, 5, r_none,
	 cast_spell_cone_of_cold);
/*
   spello(SPELL_CONJURE_ELEMENTAL, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
   100, r_none, cast_spell_conjure_elemental);

   spello(SPELL_CONTACT_OTHER_PLANE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE,
   0, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 100, r_none, cast_spell_contact_other_plane);

   spello(SPELL_DEMISHADOW_MONSTERS, 30, 0, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_SQUARE, 5, r_none, cast_spell_demishadow_monsters);

   spello(SPELL_DISMISSAL, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, 10, r_none,
   cast_spell_dismissal);

   spello(SPELL_DISTANCE_DISTORTION, 10, 10, 1, 1, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_SQUARE, 5, r_none,
   cast_spell_distance_distortion);

   spello(SPELL_DOMINATION, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_CHAR_ROOM, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, 5, r_none, 
   cast_spell_domination);

   spello(SPELL_DREAM, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
   0, SCHOOL_INVOCATION, SHAPE_UNDEFINED, 100, r_none, cast_spell_dream);

   spello(SPELL_EXTENSION_II, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE, 0,
   SCHOOL_ALTERATION, SHAPE_UNDEFINED, 4, r_none, cast_spell_extension_ii);

   spello(SPELL_FABRICATE, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, 10, r_none, 
   cast_spell_fabricate);

   spello(SPELL_FALSE_VISION, 0, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_SPHERE, 0, SCHOOL_GREATER_DIVIN, SHAPE_SPHERE, 5, r_none, 
   cast_spell_false_vision);
 */
  spello(SPELL_FEEBLEMIND, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 TRUE, TAR_CHAR_ROOM | TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, 0, HIT_ALL, 5, r_feeblemind,
	 cast_spell_feeblemind);
/*
   spello(SPELL_HOLD_MONSTER, 5, 5, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SQUARE, HIT_ALL, 5, r_none,
   cast_spell_hold_monster);

   spello(SPELL_LEOMUND_LAMENTABLE_BELABORMENT, 10, 0, 3, 0, POS_STANDING, COM_VERBAL, TRUE, 
   TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT, SHAPE_SPHERE, HIT_ALL, 5, r_none, 
   cast_spell_leomund_lamentable_belaborment);

   spello(SPELL_LEOMUND_SECRET_CHEST, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100,
   r_none, cast_spell_leomund_secret_chest);

   spello(SPELL_MAGIC_JAR, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_magic_jar);

   spello(SPELL_MAJOR_CREATION, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_major_creation);

   spello(SPELL_MONSTER_SUMMONING_III, 0, 0, 16, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 5, 
   r_none, cast_spell_monster_summoning_iii);

   spello(SPELL_MORDENKAINEN_FAITHFUL_HOUND, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 5,
   r_none, cast_spell_mordenkainen_faithful_hound);

   spello(SPELL_NIGHTMARE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
   0, SCHOOL_INVOCATION, SHAPE_UNDEFINED, 100, r_none, cast_spell_nightmare);

   spello(SPELL_PASSWALL, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
   cast_spell_passwall);

   spello(SPELL_SEEMING, 3, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
   0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, cast_spell_seeming);

   spello(SPELL_SENDING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none, cast_spell_sending);

   spello(SPELL_SHADOW_DOOR, 10, 0, 0, 0, POS_STANDING, COM_SOMATIC, FALSE, TAR_IGNORE, 0,
   SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none, cast_spell_shadow_door);

   spello(SPELL_SHADOW_MAGIC, 50, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, 
   cast_spell_shadow_magic);

   spello(SPELL_SHRINK_ANIMAL, 60, 0, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SQUARE, HIT_ALL, 5, r_none,
   cast_spell_shrink_animal);

   spello(SPELL_STONE_SHAPE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_stone_shape);

   spello(SPELL_SUMMON_SHADOW, 10, 0, 1, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
   cast_spell_summon_shadow);

   spello(SPELL_TELEKINESIS, 10, 10, 10, 10, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_OBJ_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, 
   cast_spell_telekinesis);

   spello(SPELL_TELEPORT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_CHAR | TAR_SELF_ONLY,
   0, SCHOOL_ALTERATION, 0, HIT_ALL, 2, r_none, cast_spell_fireball);

   spello(SPELL_TRANSMUTE_MUD_TO_ROCK, 10, 10, 3, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5,
   r_none, cast_spell_transmute_mud_to_rock);

   spello(SPELL_TRANSMUTE_ROCK_TO_MUD, 10, 10, 3, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5,
   r_none, cast_spell_transmute_rock_to_mud);

   spello(SPELL_WALL_OF_FORCE, 30, 0, 1, 1, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 5,
   r_none, cast_spell_wall_of_force);

   spello(SPELL_WALL_OF_IRON, 5, 5, 5, 5, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
   cast_spell_wall_of_force);

   spello(SPELL_WALL_OF_STONE, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, 
   cast_spell_wall_of_stone);
 */
/* Sixth Level Magic-User Spells */
/*
   spello(SPELL_ANTIMAGIC_SHELL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, 
   TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ABJURATION, SHAPE_SPHERE, HIT_ALL, 1,
   r_none, cast_spell_antimagic_shell);

   spello(SPELL_BIGBY_FORCEFUL_HAND, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TAR_CHAR_ROOM, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 6, 
   r_none, cast_spell_bigby_forceful_hand);

   spello(SPELL_CHAIN_LIGHTNING, 40, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
   cast_spell_chain_lightning);

   spello(SPELL_CONJURE_ANIMALS, 0, 0, 30, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 6, r_none, 
   cast_spell_conjure_animals);

   spello(SPELL_CONTINGENCY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_contingency);

   spello(SPELL_CONTROL_WEATHER, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_control_weather);

   spello(SPELL_DEATH_FOG, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_CLOUD, HIT_ALL, 6, r_none,
   cast_spell_death_fog);

   spello(SPELL_DEATH_SPELL, 10, 10, 5, 5, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_NECROMANCY, SHAPE_SQUARE, HIT_ALL, 6, r_none,
   cast_spell_death_spell);

   spello(SPELL_DEMISHADOW_MAGIC, 60, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 6, r_none, 
   cast_spell_demishadow_magic);

   spello(SPELL_DISINTEGRATE, 5, 5, 1, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 6, r_none, cast_spell_disintegrate);

   spello(SPELL_ENCHANT_AN_ITEM, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_OBJ_INV, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 10,
   r_none, cast_spell_enchant_an_item);

   spello(SPELL_ENSNAREMENT, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_ensnarement);

   spello(SPELL_EXTENSION_III, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE,
   0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 6, r_none, cast_spell_extension_iii);

   spello(SPELL_EYEBITE, 20, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_CHAR_ROOM,
   0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 6, r_none, cast_spell_eyebite);

   spello(SPELL_FLESH_TO_STONE, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | 
   COM_VERBAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, HIT_ALL, 6, r_none, 
   cast_spell_FLESH_TO_STONE);

   spello(SPELL_GEAS, 10, 0, 0, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_CHAR_ROOM, 0, 
   SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 4, r_none, cast_spell_geas);

   spello(SPELL_GLASSEE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_glassee);

   spello(SPELL_GLOBE_OF_INVULNERABILITY, 0, 0, 2, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ABJURATION, SHAPE_SPHERE, HIT_ALL, 
   10, r_globe_of_invulnerability, cast_spell_globe_of_invulnerability);

   spello(SPELL_GUARDS_AND_WARDS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 300,
   r_none, cast_spell_guards_and_wards);

   spello(SPELL_INVISIBLE_STALKER, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_invisible_stalker);

   spello(SPELL_LEGEND_LORE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_legend_lore);

   spello(SPELL_LOWER_WATER, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_lower_water);

   spello(SPELL_MASS_SUGGESTION, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL, TRUE,
   TAR_CHAR_ROOM, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 6, r_none, 
   cast_spell_mass_suggestion);

   spello(SPELL_MIRAGE_ARCANA, 10, 10, 3, 3, POS_STANDING, COM_VERBAL | COM_MATERIAL, FALSE,
   TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_mirage_arcana);

   spello(SPELL_MISLEAD, 10, 0, 0, 0, POS_STANDING, COM_SOMATIC, FALSE, TAR_IGNORE, 0, 
   SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_mislead);

   spello(SPELL_MONSTER_SUMMONING_IV, 0, 0, 60, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 6,
   r_none, cast_spell_monster_summoning_iv);

   spello(SPELL_MORDENKAINEN_LUCUBRATION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_mordenkainen_lucubration);

   spello(SPELL_MOVE_EARTH, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_move_earth);

   spello(SPELL_OTILUKE_FREEZING_SPHERE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_freezing_sphere);

   spello(SPELL_PART_WATER, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_part_water);

   spello(SPELL_PERMANENT_ILLUSION, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_permanent_illusion);

   spello(SPELL_PROGRAMMED_ILLUSION, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_programmed_illusion);

   spello(SPELL_PROJECT_IMAGE, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_project_image);

   spello(SPELL_RAISE_WATER, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_raise_water);

   spello(SPELL_REINCARNATION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | 
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 100,
   r_none, cast_spell_reincarnation);

   spello(SPELL_REPULSION, 0, 0, 3, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 6, r_none,
   cast_spell_repulsion);

   spello(SPELL_SHADES, 30, 0, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_IGNORE,
   0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 6, r_none, cast_spell_shades);

   spello(SPELL_STONE_TO_FLESH, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | 
   COM_VERBAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, HIT_ALL, 6, r_none, 
   cast_spell_stone_to_flesh);

   spello(SPELL_TENSER_TRANSFORMATION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, HIT_ALL,
   6, r_none, cast_spell_tenser_transformation);

   spello(SPELL_TRANSMUTE_DUST_TO_WATER, 60, 0, 1, 1, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, HIT_ALL, 5, r_none, 
   cast_spell_transmute_dust_to_water);

   spello(SPELL_TRANSMUTE_WATER_TO_DUST, 60, 0, 1, 1, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, HIT_ALL, 5, r_none, 
   cast_spell_transmute_water_to_dust);

   spello(SPELL_TRUE_SEEING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, 0, SCHOOL_GREATER_DIVIN, HIT_ALL, 10, r_none, cast_spell_true_seeing);

   spello(SPELL_VEIL, 10, 10, 3, 3, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   0, SCHOOL_ILLUSION, HIT_ALL, 6, r_none, cast_spell_veil);
 */
// Seventh Level Magic-User Spells 
  /*
     spello(SPELL_BANISHMENT, 20, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ABJURATION, SHAPE_SPHERE, HIT_ALL, 7, r_none,
     cast_spell_banishment);

     spello(SPELL_BIGBY_GRASPING_HAND, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
     COM_MATERIAL, TRUE, TAR_CHAR_ROOM, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL,
     7, r_none, cast_spell_bigby_grasping_hand);

     spello(SPELL_CHARM_PLANTS, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     TRUE, TAR_IGNORE, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
     cast_spell_charm_plants);

     spello(SPELL_CONTROL_UNDEAD, 20, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     TRUE, TAR_IGNORE, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 10, r_none, 
     cast_spell_control_undead);

     spello(SPELL_DELAYED_BLAST_FIREBALL, 100, 10, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
     COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_SPHERE, HIT_ALL, 7,
     r_none, cast_spell_delayed_blast_fireball);

     spello(SPELL_DRAWMIJ_INSTANT_SUMMONS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
     COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 1,
     r_none, cast_spell_drawmij_instant_summons);

     spello(SPELL_DUO_DIMENSION_ 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7,
     r_none, cast_spell_duo_dimension);

     spello(SPELL_FINGER_OF_DEATH, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
     TAR_CHAR_ROOM, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
     cast_spell_finger_of_death);

     spello(SPELL_FORCECAGE, 10, 5, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
     TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_SQUARE, HIT_ALL, 4, r_none,
     cast_spell_forcecage);

     spello(SPELL_LIMITED_WISH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE, 0,
     SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none, cast_spell_limited_wish);

     spello(SPELL_MASS_INVISIBILITY, 10, 10, 60, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | 
     COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ILLUSION, SHAPE_SQUARE, HIT_ALL, 7,
     r_none, cast_spell_mass_invisibility);

     spello(SPELL_MONSTER_SUMMONING_V, 0, 0, 70, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
     COM_MATERIAL, TRUE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 6,
     r_none, cast_spell_monster_summoning_v);

     spello(SPELL_MORDENKAINEN_MAGNIFICENT_MANSION, 10, 0, 0, 0, POS_STANDING, COM_VERBAL |
     COM_SOMATIC | COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_IGNORE,
     HIT_ALL, 70, r_none, cast_spell_mordenkainen_magnificent_mansion);

     spello(SPELL_MORDENKAINEN_SWORD, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
     COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_IGNORE, HIT_ALL, 7,
     r_none, cast_spell_mordenkainen_sword);

     spello(SPELL_PHASE_DOOR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE, 0,
     SCHOOL_ALTERATION, SHAPE_IGNORE, HIT_ALL, 7, r_none, cast_spell_phase_door);

     spello(SPELL_POWER_WORD_STUN, 5, 5, 0, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_CHAR_ROOM,
     0, SCHOOL_CONJURATION, SHAPE_IGNORE, HIT_ALL, 1, r_none, cast_spell_phase_door);

     spello(SPELL_PRISMATIC_SPRAY, 23, 0, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
     TRUE, TAR_ROOM_RANGE, 0, SCHOOL_CONJURATION, SHAPE_CONE, HIT_ALL, 7, r_none,
     cast_spell_prismatic_spray);

     spello(SPELL_REVERSE_GRAVITY, 5, 5, 10, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
     COM_MATERIAL, FALSE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_SQUARE, HIT_ALL,
     7, r_none, cast_spell_reverse_gravity);

     spello(SPELL_SEQUESTER, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 7, r_none,
     cast_spell_squester);

     spello(SPELL_SHADOW_WALK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
     0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_shadow_walk);

     spello(SPELL_SIMULACRUM, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, 0, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
     cast_spell_simulacrum);

     spello(SPELL_SPELL_TURNING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
     7, r_none, cast_spell_spell_turning);

     spello(SPELL_STATUE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7, r_none,
     cast_spell_statue);

     spello(SPELL_TELEPORT_WITHOUT_ERROR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, 
     TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
     cast_spell_teleport_without_error);

     spello(SPELL_VANISH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_OBJ_ROOM | TAR_OBJ_INV,
     0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none, cast_spell_vanish);

     spello(SPELL_VISION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, 0, SCHOOL_DIVINATION, SHAPE_UNDEFINED, HIT_ALL, 7, r_none,
     cast_spell_vision);
   */
/* Eighth Level Magic-User Spells */
/*
   spello(SPELL_ANTIPATHY_SYMPATHY, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL,
   600, r_none, cast_spell_antipathy_sympathy);

   spello(SPELL_BIGBY_CLENCHED_FIST, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_CHAR_ROM, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL,
   8, r_none, cast_spell_bigby_clenched_fist);

   spello(SPELL_BINDING, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, 0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_binding);

   spello(SPELL_CLONE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_clone);

   spello(SPELL_DEMAND, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_WORLD, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_demand);

   spello(SPELL_GLASSTEEL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 8, r_none,
   cast_spell_glassteel);

   spello(SPELL_INCENDIARY_CLOUD, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, 0, SCHOOL_ALTERATION, SHAPE_CLOUD, HIT_ALL, 2, r_none,
   cast_spell_incendiary_cloud);

   spello(SPELL_MASS_CHARM, 5, 5, 0, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_ROOM_RANGE,
   0, SCHOOL_ENCHANTMENT, SHAPE_SQUARE, HIT_ALL, 8, r_none, cast_spell_mass_charm);

   spello(SPELL_MAZE, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_CHAR_ROOM,
   0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 3, r_none, cast_spell_maze);

   spello(SPELL_MIND_BLANK, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, 
   TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
   cast_spell_mind_blank);

   spello(SPELL_MONSTER_SUMMONING_VI, 0, 0, 80, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 8,
   r_none, cast_spell_monster_summoning_vi);

   spello(SPELL_OTILUKE_TELEKINETIC_SPHERE, 20, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_SPHERE, HIT_ALL, 4,
   r_none, cast_spell_otiluke_telekinetic_sphere);

   spello(SPELL_OTTO_IRRESISTIBLE_DANCE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_CHAR_ROOM,
   0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, cast_spell_otto_irresistible_dance);

   spello(SPELL_PERMANENCY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
   0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 20, r_none, cast_permanency);

   spello(SPELL_POLYMORPH_ANY_OBJECT, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 
   10, r_none, cast_polymorph_any_object);
 */
  spello(SPELL_POWER_WORD_BLIND, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
	 COM_MATERIAL, TRUE, TAR_CHAR_ROOM | TAR_ROOM_RANGE, 0, SCHOOL_ENCHANTMENT,
	 0, HIT_ALL, 5, r_power_word_blind, cast_spell_power_word_blind);
/*
   spello(SPELL_PRISMATIC_WALL, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 7, r_none,
   cast_spell_prismatic_wall);

   spello(SPELL_SCREEN, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   0, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 100, r_none, cast_spell_screen);

   spello(SPELL_SERTEN_SPELL_IMMUNITY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_serten_spell_immunity);

   spello(SPELL_SINK, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_IGNORE,
   0, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 8, r_none, cast_spell_sink);

   spello(SPELL_SYMBOL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 8, r_none,
   cast_spell_symbol);

   spello(SPELL_TRAP_THE_SOUL, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_CHAR_ROOM, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED,
   HIT_ALL, 10, r_none, cast_spell_trap_the_soul);
 */
/* Ninth Level Magic-User Spells */
/*
   spello(SPELL_ASTRAL_SPELL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, 
   TAR_IGNORE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 9, r_none,
   cast_spell_astral_spell);

   spello(SPELL_BIGBY_CRUSHING_HAND, 5, 5, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_CHAR_ROOM, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL,
   9, r_none, cast_spell_bigby_crushing_hand);

   spello(SPELL_CRYSTALBRITTLE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 9, r_none, 
   cast_spell_crystalbrittle);

   spello(SPELL_ENERGY_DRAIN, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 3, r_none,
   cast_spell_energy_drain);

   spello(SPELL_FORESIGHT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_foresight);

   spello(SPELL_FREEDOM, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_IGNORE, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 9, r_none, 
   cast_spell_freedom);

   spello(SPELL_GATE, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 9, r_none, cast_spell_gate);

   spello(SPELL_IMPRISONMENT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_IGNORE, 0, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 9, r_none, 
   cast_spell_imprisonment);

   spello(SPELL_METEOR_SWARM, 40, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_ROOM_RANGE, 0, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 9, r_none,
   cast_spell_meteor_swarm);

   spello(SPELL_MONSTER_SUMMONING_VII, 0, 0, 90, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, TRUE, TAR_IGNORE, 0, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
   9, r_none, cast_spell_monster_summoning_vii);

   spello(SPELL_MORDENKAINEN_DISJUNCTION, 0, 0, 10, 0, POS_STANDING, COM_VERBAL, FALSE,
   TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 9, r_none, 
   cast_spell_mordenkainen_disjunction);
 */
  spello(SPELL_POWER_WORD_KILL, 5, 2, 10, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_CHAR_ROOM |
  TAR_ROOM_RANGE, 0, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 1, r_none,
	 cast_spell_power_word_kill);
/*
   spello(SPELL_PRISMATIC_SPHERE, 0, 0, 3, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_IGNORE,
   0, SCHOOL_ABJURATION, SHAPE_SPHERE, HIT_ALL, 7, r_none, 
   cast_spell_prismatic_sphere);

   spello(SPELL_SHAPE_CHANGE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 9, r_none, 
   cast_spell_shape_change);

   spello(SPELL_SUCCOR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 14400, r_none, 
   cast_spell_succor);

   spello(SPELL_SUCCOR_REVERSE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 14400, r_none, 
   cast_spell_succor_reverse);

   spello(SPELL_TEMPORAL_REINSTATEMENT, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 9,
   r_none, cast_spell_temporal_reinstatement);

   spello(SPELL_TEMPORAL_STASIS, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM, 0, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 9,
   r_none, cast_spell_temporal_stasis);

   spello(SPELL_TIME_STOP, 0, 0, 5, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_ROOM_RANGE,
   0, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 9, r_none, cast_spell_time_stop);

   spello(SPELL_WEIRD, 30, 0, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   0, SCHOOL_ILLUSION, SHAPE_SPHERE, HIT_ALL, 9, r_none, cast_spell_wierd);

   spello(SPELL_WISH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_IGNORE, 0, SCHOOL_CONJURATION,
   SHAPE_UNDEFINED, HIT_ALL, 10, r_none, cast_spell_wish);
 */
  /* priest spells */

// First Level Cleric Spells 
  /*
     spello(SPELL_ANIMAL_FRIENDSHIP, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
     COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED,
     HIT_ALL, 600, r_none, cast_spell_animal_friendship);
   */
  spello(SPELL_BLESS, 20, 0, 15, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
	 | COM_MATERIAL, FALSE, TAR_ROOM_RANGE | TAR_CHAR_ROOM, SPHERE_ALL, SCHOOL_CONJURATION,
	 SHAPE_SQUARE, HIT_FRIEND, 10, r_bless, cast_spell_bless);
/*
   spello(SPELL_CAUSE_FEAR, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_CHAR_ROOM,
   SPHERE_CHARM, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_cause_fear);
 */
  spello(SPELL_CAUSE_LIGHT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
	 TAR_CHAR_ROOM, SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
	 cast_spell_cause_light);
/*
   spello(SPELL_COMBINE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_ALL, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none, cast_spell_combine);

   spello(SPELL_COMMAND, 30, 0, 0, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_CHAR_ROOM, SPHERE_CHARM,
   SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_command);
 */
  spello(SPELL_CREATE_WATER, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
	 COM_MATERIAL, FALSE, TAR_OBJ_INV, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
	 HIT_ALL, 10, r_none, cast_spell_create_water);

  spello(SPELL_CURE_LIGHT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM, SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_FRIEND, 5, r_cure,
	 cast_spell_cure_light);

  spello(SPELL_CURSE, 20, 0, 15, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC
	 | COM_MATERIAL, TRUE, TAR_ROOM_RANGE | TAR_CHAR_ROOM, SPHERE_ALL, SCHOOL_CONJURATION,
	 SHAPE_SQUARE, HIT_ENEMY, 10, r_curse, cast_spell_curse);
/*
   spello(SPELL_DARKNESS, 120, 0, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_SUN, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 4, r_none, cast_spell_darkness);

   spello(SPELL_DESTROY_WATER, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | 
   COM_MATERIAL, FALSE, TAR_OBJ_INV, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 10, r_none, cast_spell_destroy_water);

   spello(SPELL_DETECT_EVIL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_ALL, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED,
   HIT_ALL, 10, r_none, cast_spell_detect_evil);

   spello(SPELL_DETECT_GOOD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_ALL, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED,
   HIT_ALL, 10, r_none, cast_spell_detect_good);

   spello(SPELL_DETECT_MAGIC, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED,
   HIT_ALL, 10, r_none, cast_spell_detect_magic);
 */
  spello(SPELL_DETECT_POISON, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_DIVINATION,
	 SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 4, r_none, cast_spell_detect_poison);
/*
   spello(SPELL_DETECT_SNARES_AND_PITS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN,
   SHAPE_UNDEFINED, HIT_ALL, 4, r_none, cast_spell_detect_snares_and_pits);

   spello(SPELL_ENDURE_HEAT_ENDURE_COLD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
   FALSE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_endure_heat_endure_cold);

   spello(SPELL_ENTANGLE, 80, 0, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_ROOM_RANGE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_SQUARE, HIT_ALL, 4,
   r_none, cast_spell_entangle);
 */
  spello(SPELL_FAERIE_FIRE, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL, TRUE,
	 TAR_CHAR_ROOM, SPHERE_WEATHER, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ENEMY, 4, r_faerie_fire,
	 cast_spell_faerie_fire);

  spello(SPELL_INVIS_ANIMALS, 0, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_MATERIAL,
	 FALSE, TAR_CHAR_ROOM, SPHERE_ANIMAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_FRIEND, 4,
	 r_invis_animals, cast_spell_invis_animals);

  spello(SPELL_INVIS_UNDEAD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
	 COM_MATERIAL, FALSE, TAR_CHAR_ROOM, SPHERE_NECROMANTIC, SCHOOL_ABJURATION,
      SHAPE_SPHERE, HIT_ALL, 4, r_invis_undead, cast_spell_invis_undead);
/*
   spello(SPELL_LIGHT, 120, 0, 6, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_SUN, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 4, r_none, cast_spell_light);

   spello(SPELL_LOCATE_ANIMALS_OR_PLANTS, 100, 20, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED,
   HIT_ALL, 10, r_none, cast_spell_locate_animals_or_plants);

   spello(SPELL_MAGICAL_STONE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_OBJ_INV, SPHERE_COMBAT, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 4,
   cast_spell_magical_stone);

   spello(SPELL_PASS_WITHOUT_TRACE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_PLANT, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 10,
   r_none, cast_spell_pass_without_trace);

   spello(SPELL_PROTECTION_FROM_EVIL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
   4, r_none, cast_spell_protection_from_evil);

   spello(SPELL_PROTECTION_FROM_GOOD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
   4, r_none, cast_spell_protection_from_good);

   spello(SPELL_PURIFY_FOOD_AND_DRINK, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ALL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_purify_food_and_drink);

   spello(SPELL_PUTREFY_FOOD_AND_DRINK, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ALL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_putrefy_food_and_drink);

   spello(SPELL_REMOVE_FEAR, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
   SPHERE_CHARM, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_remove_fear);

 */
  spello(SPELL_SANCTUARY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
	 TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 4, r_sanctuary,
	 cast_spell_sanctuary);
/*
   spello(SPELL_SHILLELAGH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_OBJ_INV, SPHERE_COMBAT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 2, r_none,
   cast_spell_shillelagh);
 */
// Second Level Cleric Spells

  spello(SPELL_AID, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
	 TAR_CHAR_ROOM, SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 5,
	 r_aid, cast_spell_aid);
/*
   spello(SPELL_AUGURY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 20,
   r_none, cast_spel_augury);

   spello(SPELL_BADBERRY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_badberry);
 */
  spello(SPELL_BARKSKIN, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
	 FALSE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5,
	 r_barkskin, cast_spell_barkskin);
/*
   spello(SPELL_CHANT, 0, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_COMBAT, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 20, r_none, cast_spell_chant);
 */
  spello(SPELL_CHARM_MAMMAL, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
	 TRUE, TAR_CHAR_ROOM, SPHERE_ANIMAL, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 5,
	 r_charm, cast_spell_charm_mammal);
/*
   spello(SPELL_CHILL_METAL, 40, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
   cast_spell_chill_metal);

 */
  spello(SPELL_DETECT_CHARM, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY,
	 SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 10, r_detect_charm,
	 cast_spell_detect_charm);

  spello(SPELL_UNDETECTABLE_CHARM, 1, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
	 SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 10, r_undetectable_charm,
	 cast_spell_undetectable_charm);
/*
   spello(SPELL_DUST_DEVIL, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_ELEMENTAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 20, r_none, 
   cast_spell_dust_devil);

   spello(SPELL_ENTHRALL, 0, 0, 30, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_ROOM_RANGE,
   SPHERE_CHARM, SCHOOL_ENCHANTMENT, SHAPE_SPHERE, HIT_ALL, 10, r_none, cast_spell_enthrall);

   spello(SPELL_FIND_TRAPS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM |
   TAR_SELF_ONLY, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 5, 
   r_none, cast_spell_find_traps);

   spello(SPELL_FIRE_TRAP, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 100, r_none,
   cast_spell_fire_trap);

   spello(SPELL_FLAME_BLADE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, 
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 4,
   r_none, cast_spell_flame_blade);

   spello(SPELL_GOODBERRY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10, r_none,
   cast_spell_goodberry);

   spello(SPELL_HEAT_METAL, 40, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
   cast_spell_heat_metal);

   spello(SPELL_HOLD_PERSON, 120, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, 
   TRUE, TAR_IGNORE, SPHERE_CHARM, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 5,
   r_none, cast_spell_hold_person);

   spello(SPELL_KNOW_ALIGNMENT, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 10,
   r_none, cast_spell_know_alignment);

   spello(SPELL_MESSENGER, 20, 20, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_ANIMAL, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 10, r_none, cast_spell_messenger);

   spello(SPELL_OBSCUREMENT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_WEATHER, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, 
   cast_spell_obscurement);

   spello(SPELL_PRODUCE_FLAME, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, 
   TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
   cast_spell_produce_flame);

   spello(SPELL_RESIST_FIRE_RESIST_COLD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 5, r_none, cast_spell_resist_fire_resist_cold);

   spello(SPELL_SILENCE_15, 120, 0, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, 
   TAR_ROOM_RANGE, SPHERE_GUARDIAN, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 5, r_none,
   cast_spell_silence_15);

   spello(SPELL_SLOW_POISON, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 1,
   r_none, cast_spell_slow_poison);

   spello(SPELL_SNAKE_CHARM, 30, 0, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_ANIMAL, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, 
   cast_spell_snake_charm);

   spello(SPELL_SPEAK_WITH_ANIMALS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_ANIMAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 5, r_none, cast_spell_speak_with_animals);

   spello(SPELL_SPIRITUAL_HAMMER, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_COMBAT, SCHOOL_INVOCATION, SHAPE_UNDEFINED, HIT_ALL, 5,
   r_none, cast_spell_spiritual_hammer);

   spello(SPELL_STRAIGHTEN_WOOD, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, cast_spell_straighten_wood);

   spello(SPELL_TRIP, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE, 
   SPHERE_PLANT, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, cast_spell_trip);

   spello(SPELL_UNDETECTABLE_ALIGNMENT, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 10,
   r_none, cast_spell_undetectable_alignment);

   spello(SPELL_WARP_WOOD, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, cast_spell_warp_wood);

   spello(SPELL_WITHDRAW, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM |
   TAR_SELF_ONLY, SPHERE_PROTECTION, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none,
   cast_spell_withdraw);

   spello(SPELL_WYVERN_WATCH, 30, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, 
   FALSE, TAR_ROOM_RANGE, SPHERE_GUARDIAN, SCHOOL_EVOCATION, SHAPE_SPHERE, HIT_ALL, 5,
   r_none, cast_spell_wyvern_watch);
 */
// Third Level Cleric Spells
  /*
     spello(SPELL_AIR_BREATING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
     TAR_CHAR_ROOM, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 6,
     r_none, cast_spell_air_breathing);

     spello(SPELL_ANIMATE_DEAD, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 10,
     r_none, cast_spell_animate_dead);

     spello(SPELL_BESTOW_CURSE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
     TAR_IGNORE, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 6,
     r_none, cast_spell_bestow_curse);

     spello(SPELL_CALL_LIGHTNING, 360, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, 
     TAR_ROOM_RANGE, SPHERE_WEATHER, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 100, r_none,
     cast_spell_call_lightning);
   */
  spello(SPELL_CAUSE_BLINDNESS, 0, 0, 0, 0, POS_STANDING,
	 COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_CHAR_ROOM, SPHERE_NECROMANTIC, SCHOOL_ABJURATION, SHAPE_UNDEFINED,
	 HIT_ALL, 10, r_cause_blindness, cast_spell_cause_blindness);

  spello(SPELL_CAUSE_DEAFNESS, 0, 0, 0, 0, POS_STANDING,
	 COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_CHAR_ROOM, SPHERE_NECROMANTIC, SCHOOL_ABJURATION, SHAPE_UNDEFINED,
	 HIT_ALL, 10, r_cause_deafness, cast_spell_cause_deafness);

/*
   spello(SPELL_CAUSE_DISEASE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, 
   TAR_CHAR_ROOM, SPHERE_NECROMANTIC, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 10,
   r_none, cast_spell_cause_disease);

   spello(SPELL_CONTINUAL_DARKNESS, 120, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_SUN, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 6, r_none, 
   cast_spell_continual_darkness);

   spello(SPELL_CONTINUAL_LIGHT, 120, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_SUN, SCHOOL_ALTERATION, SHAPE_SPHERE, HIT_ALL, 6, r_none, 
   cast_spell_continual_light);

   spello(SPELL_CREATE_FOOD_AND_WATER, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_CREATION, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100,
   r_none, cast_spell_create_food_and_water);
 */
  spello(SPELL_CURE_BLINDNESS, 0, 0, 0, 0, POS_STANDING,
	 COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM, SPHERE_NECROMANTIC, SCHOOL_ABJURATION, SHAPE_UNDEFINED,
	 HIT_ALL, 10, r_cure_blindness, cast_spell_cure_blindness);

  spello(SPELL_CURE_DEAFNESS, 0, 0, 0, 0, POS_STANDING,
	 COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM, SPHERE_NECROMANTIC, SCHOOL_ABJURATION, SHAPE_UNDEFINED,
	 HIT_ALL, 10, r_cure_deafness, cast_spell_cure_deafness);

/*
   spello(SPELL_CURE_DISEASE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, 
   TAR_CHAR_ROOM, SPHERE_NECROMANTIC, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 10,
   r_none, cast_spell_cure_disease);

   spello(SPELL_DISPEL_MAGIC, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_dispel_magic);

   spello(SPELL_FEIGN_DEATH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_CHAR_ROOM, 
   SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_FRIEND, 1, r_none,
   cast_spell_feign_death);

   spello(SPELL_FLAME_WALK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   5, r_none, cast_spell_flame_walk);

   spello(SPELL_GLYPH_OF_WARDING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_GUARDIAN, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 100,
   r_none, cast_spell_glyph_of_warding);

   spello(SPELL_HOLD_ANIMAL, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_hold_animal);

   spello(SPELL_LOCATE_OBJECT, 60, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 
   100, r_none, cast_spell_locate_object);

   spello(SPELL_MAGICAL_VESTMENT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_PROTECTION, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_magical_vestment);

   spello(SPELL_MELD_INTO_STONE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 6, r_none, cast_spell_meld_into_stone);

   spello(SPELL_NEGATIVE_PLANE_PROTECTION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, 
   FALSE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_negative_plane_protection);

   spello(SPELL_OBSCURE_OBJECT, 60, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 
   100, r_none, cast_spell_obscure_object);

   spello(SPELL_PLANT_GROWTH, 160, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10,
   r_none, cast_spell_plant_growth);

   spello(SPELL_PRAYER, 0, 0, 20, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, SPHERE_COMBAT, SCHOOL_CONJURATION, SHAPE_SPHERE, HIT_ALL, 6,
   r_none, cast_spell_prayer);

   spello(SPELL_PROTECTION_FROM_FIRE, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_protection_from_fire);

   spello(SPELL_PYROTECHNICS, 160, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 6, 
   r_none, cast_spell_pyrotechnics);

   spello(SPELL_REMOVE_CURSE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_remove_curse);

   spello(SPELL_REMOVE_PARALYSIS, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 6, 
   r_none, cast_spell_remove_paralysis);

   spello(SPELL_SNARE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, SPHERE_PLANT, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 300, 
   r_none, cast_spell_snare);

   spello(SPELL_SPEAK_WITH_DEAD, 1, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL,
   100, r_none, cast_spell_speak_with_dead);

   spello(SPELL_SPIKE_GROWTH, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_spike_growth);

   spello(SPELL_STARSHINE, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_SUN, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_starshine);

   spello(SPELL_STONE_SHAPE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_stone_shape);

   spello(SPELL_SUMMON_INSECTS, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, SPHERE_ANIMAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_summon_insects);

   spello(SPELL_TREE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   6, r_none, cast_spell_tree);

   spello(SPELL_WATER_BREATING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_CHAR_ROOM, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_water_breathing);

   spello(SPELL_WATER_WALK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, 
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 
   6, r_none, cast_spell_water_walk);
 */
// Fourth Level Cleric Spells
  /*
     spello(SPELL_ABJURE, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     TRUE, TAR_CHAR_ROOM, SPHERE_SUMMONING, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
     10, r_none, cast_spell_abjure);

     spello(SPELL_ANIMAL_SUMMONING_I, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
     TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 7, 
     r_none, cast_spell_animal_summoning_i);

     spello(SPELL_CALL_WOODLAND_BEINGS, 100, 100, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
     COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_SUMMONING, SCHOOL_CONJURATION, SHAPE_UNDEFINED,
     HIT_ALL, 100, r_none, cast_spell_call_woodland_beings);
   */
  spello(SPELL_CAUSE_SERIOUS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
	 TAR_CHAR_ROOM, SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 7, r_none,
	 cast_spell_cause_serious);
/*
   spello(SPELL_CLOAK_OF_BRAVERY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_CHARM, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_cloak_of_bravery);

   spello(SPELL_CLOAK_OF_FEAR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_CHARM, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_cloak_of_fear);

   spello(SPELL_CONTROL_TEMPERATURE_10, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_WEATHER, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 7, r_none, cast_spell_control_temperature_10);
 */
  spello(SPELL_CURE_SERIOUS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM, SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 7, r_cure,
	 cast_spell_cure_serious);
/*
   spello(SPELL_DETECT_LIE, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
   7, r_none, cast_spell_detect_lie);

   spello(SPELL_DIVINATION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL, 100,
   r_none, cast_spell_divination);

   spello(SPELL_FREE_ACTION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
   TAR_CHAR_ROOM, SPHERE_CHARM, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL, 7, r_none,
   cast_spell_free_action);

   spello(SPELL_GIANT_INSECT, 20, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7,
   r_none, cast_spell_giant_insect);

   spello(SPELL_HALLUCINATORY_FOREST, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_PLANT, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 7, r_none, 
   cast_spell_hallucinatory_forest);

   spello(SPELL_HALLUCINATORY_FOREST_REVERSE, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_PLANT, SCHOOL_ILLUSION, SHAPE_UNDEFINED, HIT_ALL, 7, r_none, 
   cast_spell_hallucinatory_forest_reverse);

   spello(SPELL_HOLD_PLANT, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_IGNORE,
   SPHERE_PLANT, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 7, r_none, 
   cast_spell_hold_plant);

   spello(SPELL_IMBUE_WITH_SPELL_ABILITY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM, SPHERE_CHARM, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED,
   HIT_ALL, 100, r_none, cast_spell_imbue_with_spell_ability);

   spello(SPELL_LOWER_WATER, 120, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_IGNORE, HIT_ALL, 100,
   r_none, cast_spell_lower_water);

   spello(SPELL_NEUTRALIZE_POISON, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 7, r_none,
   cast_spell_neutralize_poison);

   spello(SPELL_PLANT_DOOR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7,
   r_none, cast_spell_plant_door);

   spello(SPELL_PRODUCE_FIRE, 40, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7,
   r_none, cast_spell_produce_fire);

   spello(SPELL_POISON, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_IGNORE, SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 7, r_none,
   cast_spell_poison);

   spello(SPELL_PROTECTION_FROM_EVIL_10, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_SPHERE,
   HIT_ALL, 7, r_none, cast_spell_protection_from_evil_10);

   spello(SPELL_PROTECTION_FROM_GOOD_10, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_SPHERE,
   HIT_ALL, 7, r_none, cast_spell_protection_from_good_10);

   spello(SPELL_PROTECTION_FROM_LIGHTNING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED,
   HIT_ALL, 7, r_none, cast_spell_protection_from_lightning);

   spello(SPELL_QUENCH_FIRE, 40, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7,
   r_none, cast_spell_quench_fire);

   spello(SPELL_RAISE_WATER, 120, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_IGNORE, HIT_ALL, 100,
   r_none, cast_spell_raise_water);

   spello(SPELL_REFECTING_POOL, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
   1200, r_none, cast_spell_reflecting_pool);

   spello(SPELL_REPEL_INSECTS, 0, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_ANIMAL, SCHOOL_ABJURATION, SHAPE_SPHERE,
   HIT_ALL, 10, r_none, cast_spell_repel_insects);

   spello(SPELL_SHRINK_INSECT, 20, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7,
   r_none, cast_spell_shrink_insect);

   spello(SPELL_SPEAK_WITH_PLANTS, 0, 0, 10, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 100, r_none, cast_spell_speak_with_plants);

   spello(SPELL_SPELL_IMMUNITY, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
   10, r_none, cast_spell_spell_immunity);

   spello(SPELL_SNAKES_TO_STICKS, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7,
   r_none, cast_spell_snakes_to_sticks);

   spello(SPELL_STICKS_TO_SNAKES, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7,
   r_none, cast_spell_sticks_to_snakes);

   spello(SPELL_TONGUES, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_DIVINATION, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7, r_none, 
   cast_spell_tongues);

   spello(SPELL_TONGUES_REVERSE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
   SPHERE_DIVINATION, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 7, r_none, 
   cast_spell_tongues_reverse);

   spello(SPELL_UNDETECTABLE_LIE, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_LESSER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
   7, r_none, cast_spell_undetectable_lie);
 */
// Fifth Level Cleric Spells
  /*
     spello(SPELL_AIR_WALK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_CHAR_ROOM, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
     8, r_none, cast_spell_air_walk);

     spello(SPELL_ANIMAL_GROWTH, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 8,
     r_none, cast_spell_animal_growth);

     spello(SPELL_ANIMAL_GROWTH_REVERSE, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 8,
     r_none, cast_spell_animal_growth_reverse);

     spello(SPELL_ANIMAL_SUMMONING_II, 60, 60, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
     TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 8, r_none,
     cast_spell_animal_summoning_ii);

     spello(SPELL_ANTIPLANT_SHELL, 0, 0, 5, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
     TAR_IGNORE, SPHERE_PLANT, SCHOOL_ABJURATION, SHAPE_SPHERE, HIT_ALL, 8, r_none, 
     cast_spell_antiplant_shell);

     spello(SPELL_ATONEMENT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_CHAR_ROOM, SPHERE_ALL, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
     100, r_none, cast_spell_atonement);
   */
  spello(SPELL_CAUSE_CRITIC, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
	 TAR_CHAR_ROOM, SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 8, r_none,
	 cast_spell_cause_critic);
/*
   spello(SPELL_COMMUNE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
   100, r_none, cast_spell_commune);

   spello(SPELL_COMMUNE_WITH_NATURE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
   FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
   100, r_none, cast_spell_commune_with_nature);

   spello(SPELL_CONTROL_WINDS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_WEATHER, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 8,
   r_none, cast_spell_control_winds);

 */
  spello(SPELL_CURE_CRITIC, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
	 TAR_CHAR_ROOM, SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 8, r_cure,
	 cast_spell_cure_critic);
/*
   spello(SPELL_DISPEL_EVIL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
   8, r_none, cast_spell_dispel_evil);

   spello(SPELL_DISPEL_GOOD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_CHAR_ROOM, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
   8, r_none, cast_spell_dispel_good);

   spello(SPELL_FALSE_SEEING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_DIVINATION, SCHOOL_GREATER_DIVIN, HIT_ALL, 8,
   r_none, cast_spell_false_seeing);

   spello(SPELL_FLAME_STRIKE, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_IGNORE, SPHERE_COMBAT, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 8,
   r_none, cast_spell_flame_strike);

   spello(SPELL_INSECT_PLAGUE, 120, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   TRUE, TAR_IGNORE, SPHERE_COMBAT, SCHOOL_CONJURATION, SHAPE_CLOUD, HIT_ALL, 100, 
   r_none, cast_spell_insect_plague);

   spello(SPELL_MAGIC_FONT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_DIVINATION, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
   600, r_none, cast_spell_magic_font);

   spello(SPELL_MOONBEAM, 60, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_SUN, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 7,
   r_none, cast_spell_moonbeam);

   spello(SPELL_PASS_PLANT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 8,
   r_none, cast_spell_pass_plant);

   spello(SPELL_PLANE_SHIFT, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ASTRAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 8,
   r_none, cast_spell_plane_shift);

   spello(SPELL_QUEST, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, 
   FALSE, TAR_CHAR_ROOM, SPHERE_CHARM, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL,
   8, r_none, cast_spell_quest);

   spello(SPELL_RAINBOW, 120, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_WEATHER, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL,
   7, r_none, cast_spell_rainbow);

   spello(SPELL_RAISE_DEAD, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 10,
   r_none, cast_spell_raise_dead);

   spello(SPELL_SLAY_LIVING, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
   TAR_IGNORE, SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 10,
   r_none, cast_spell_slay_living);

   spello(SPELL_SPIKE_STONES, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 6,
   r_none, cast_spell_spike_stones);

   spello(SPELL_TRANSMUTE_MUD_TO_ROCK, 160, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 8, r_none, cast_spell_transmute_mud_to_rock);

   spello(SPELL_TRANSMUTE_ROCK_TO_MUD, 160, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED,
   HIT_ALL, 8, r_none, cast_spell_transmute_rock_to_mud);

   spello(SPELL_TRUE_SEEING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_CHAR_ROOM, SPHERE_DIVINATION, SCHOOL_GREATER_DIVIN, HIT_ALL, 8,
   r_none, cast_spell_true_seeing);

   spello(SPELL_WALL_OF_FIRE, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 8,
   r_none, cast_spell_wall_of_fire);
 */
// Sixth Level Cleric Spells
  /*
     spello(SPELL_AERIAL_SERVANT, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
     TAR_IGNORE, SPHERE_SUMMONING, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 9,
     r_none, cast_spell_conjuration)

     spello(SPELL_ANIMAL_SUMMONING_III, 100, 100, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
     FALSE, TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 9,
     r_none, cast_spell_animal_summoning_iii);

     spello(SPELL_ANIMATE_OBJECT, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
     TAR_IGNORE, SPHERE_CREATION, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 9,
     r_none, cast_spell_animate_object);

     spello(SPELL_ANTIANIMAL_SHELL, 0, 0, 3, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | 
     COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_ABJURATION, SHAPE_SPHERE,
     HIT_ALL, 10, r_none, cast_spell_antianimal_shell);

     spello(SPELL_BLADE_BARRIER, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
     TAR_IGNORE, SPHERE_GUARDIAN, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 9,
     r_none, cast_spell_blade_barrier);

     spello(SPELL_CONJURE_ANIMALS, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
     TAR_IGNORE, SPHERE_SUMMONING, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 9,
     r_none, cast_spell_conjure_animals);

     spello(SPELL_CONJURE_FIRE_ELEMENTAL, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
     FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
     60, r_none, cast_spell_conjure_fire_elemental);

     spello(SPELL_CONJURE_FIRE_ELEMENTAL_REVERSE, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
     FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
     60, r_none, cast_spell_conjure_fire_elemental_reverse);

     spello(SPELL_FIND_THE_PATH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_CHAR_ROOM, SPHERE_DIVINATION, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
     30, r_none, cast_spell_find_the_path);

     spello(SPELL_FIRE_SEEDS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, 
     FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
     10, r_none, cast_spell_fire_seeds);

     spello(SPELL_FORBIDDANCE, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_PROTECTION, SCHOOL_ABJURATION, SHAPE_UNDEFINED, HIT_ALL,
     60, r_none, cast_spell_forbiddance);

     spello(SPELL_LOSE_THE_PATH, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_CHAR_ROOM, SPHERE_DIVINATION, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
     30, r_none, cast_spell_lose_the_path);

     spello(SPELL_HARM, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_CHAR_ROOM,
     SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 10, r_heal, cast_spell_heal);
   */
  spello(SPELL_HEAL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
	 SPHERE_HEALING, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 10, r_heal, cast_spell_heal);
/*
   spello(SPELL_HEROES_FEAST, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_CREATION, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL,
   100, r_none, cast_spell_heroes_feast);

   spello(SPELL_LIVEOAK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL,
   100, r_none, cast_spell_liveoak);

   spello(SPELL_PART_WATER, 20, 20, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   100, r_none, cast_spell_part_water);

   spello(SPELL_SPEAK_WITH_MONSTERS, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
   FALSE, TAR_CHAR_ROOM | TAR_SELF_ONLY, SPHERE_DIVINATION, SCHOOL_GREATER_DIVIN,
   SHAPE_UNDEFINED, HIT_ALL, 9, r_none, cast_spell_speak_with_monsters);

   spello(SPELL_STONE_TELL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
   FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_GREATER_DIVIN, SHAPE_UNDEFINED, HIT_ALL,
   100, r_none, cast_spell_stone_tell);

   spello(SPELL_TRANSMUTE_DUST_TO_WATER, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, 
   HIT_ALL, 8, r_none, cast_spell_transmute_dust_to_water);

   spello(SPELL_TRANSMUTE_WATER_TO_DUST, 60, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
   COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, 
   HIT_ALL, 8, r_none, cast_spell_transmute_water_to_dust);

   spello(SPELL_TRANSPORT_VIA_PLANTS, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, 
   FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
   4, r_none, cast_spell_transport_via_plants);

   spello(SPELL_TURN_WOOD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE,
   TAR_IGNORE, SPHERE_PLANT, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 9,
   r_none, cast_spell_turn_wood);

   spello(SPELL_WALL_OF_THORNS, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
   FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
   9, r_none, cast_spell_wall_of_thorns);

   spello(SPELL_WEATHER_SUMMONING, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
   FALSE, TAR_IGNORE, SPHERE_WEATHER, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
   100, r_none, cast_spell_weather_summoning);
 */
  spello(SPELL_WORD_RECALL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, FALSE, TAR_CHAR_ROOM |
	 TAR_SELF_ONLY, SPHERE_SUMMONING, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none,
	 cast_spell_word_recall);

// Seventh Level Cleric Spells
  /*
     spello(SPELL_ANIMATE_ROCK, 40, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
     10, r_none, cast_spell_animate_rock);

     spello(SPELL_ASTRAL_SPELL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
     SPHERE_ASTRAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 300, r_none, cast_spell_astral_spell);

     spello(SPELL_CHANGESTAFF, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_PLANT, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, r_none,
     cast_spell_changestaff);

     spello(SPELL_CHARIOT_OF_SUSTARRE, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC |
     COM_MATERIAL, FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_EVOCATION, SHAPE_UNDEFINED,
     HIT_ALL, 100, r_none, cast_spell_chariot_of_sustarre);

     spello(SPELL_CONFUSION, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     TRUE, TAR_IGNORE, SPHERE_CHARM, SCHOOL_ENCHANTMENT, SHAPE_UNDEFINED, HIT_ALL, 10,
     r_none, cast_spell_confusion);

     spello(SPELL_CONJURE_EARTH_ELEMENTAL, 40, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
     FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
     100, r_none, cast_spell_conjure_earth_elemental);

     spello(SPELL_CONJURE_EARTH_ELEMENTAL_REVERSE, 40, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC,
     FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL,
     100, r_none, cast_spell_conjure_earth_elemental_reverse);

     spello(SPELL_CONTROL_WEATHER, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_WEATHER, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 100,
     r_none, cast_spell_control_weather);

     spello(SPELL_CREEPING_DOOM, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE,
     TAR_IGNORE, SPHERE_ANIMAL, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 10,
     r_none, cast_spell_creeping_doom);

     spello(SPELL_DESTRUCTION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     TRUE, TAR_IGNORE, SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 100,
     r_none, cast_spell_destruction);

     spello(SPELL_EARTHQUAKE, 120, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL,
     100, r_none, cast_spell_earthquake);

     spello(SPELL_ENERGY_DRAIN, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, TAR_CHAR_ROOM,
     SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 30, r_none, 
     cast_spell_energy_drain);

     spello(SPELL_EXACTION, 10, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_CHARM, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 10,
     r_none, cast_spell_exaction);

     spello(SPELL_FIRE_QUENCH, 160, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, 
     TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 10,
     r_none, cast_spell_fire_quench);

     spello(SPELL_FIRE_STORM, 160, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, TRUE, 
     TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 10,
     r_none, cast_spell_fire_storm);

     spello(SPELL_GATE, 30, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
     SPHERE_SUMMONING, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 5, r_none, 
     cast_spell_gate);

     spello(SPELL_HOLY_WORD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_IGNORE, SPHERE_COMBAT,
     SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_holy_word);

     spello(SPELL_REGENERATE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_CHAR_ROOM, SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL,
     30, r_none, cast_spell_regenerate);

     spello(SPELL_REINCARNATE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_IGNORE,
     SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 100, r_none, 
     cast_spell_reincarnate);

     spello(SPELL_RESTORATION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC, FALSE, TAR_CHAR_ROOM,
     SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 30, r_none, 
     cast_spell_restoration);

     spello(SPELL_RESURRECTION, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL,
     FALSE, TAR_IGNORE, SPHERE_NECROMANTIC, SCHOOL_NECROMANCY, SHAPE_UNDEFINED, HIT_ALL, 100,
     r_none, cast_spell_resurrection);

     spello(SPELL_SUCCOR, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
     TAR_IGNORE, SPHERE_SUMMONING, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 14400,
     r_none, cast_spell_succor);

     spello(SPELL_SUCCOR_REVERSE, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
     TAR_IGNORE, SPHERE_SUMMONING, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 14400,
     r_none, cast_spell_succor_reverse);

     spello(SPELL_SUNRAY, 10, 10, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
     TAR_IGNORE, SPHERE_SUN, SCHOOL_EVOCATION, SHAPE_UNDEFINED, HIT_ALL, 4, r_none,
     cast_spell_sunray);

     spello(SPELL_SYMBOL, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
     TAR_IGNORE, SPHERE_GUARDIAN, SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 3,
     r_none, cast_spell_symbol);

     spello(SPELL_TRANSMUTE_METAL_TO_WOOD, 80, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
     TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10,
     r_none, cast_spell_transmute_metal_to_wood);

     spello(SPELL_WIND_WALK, 0, 0, 0, 0, POS_STANDING, COM_VERBAL | COM_SOMATIC | COM_MATERIAL, FALSE,
     TAR_IGNORE, SPHERE_ELEMENTAL, SCHOOL_ALTERATION, SHAPE_UNDEFINED, HIT_ALL, 10,
     r_none, cast_spell_wind_walk);

     spello(SPELL_UNHOLY_WORD, 0, 0, 0, 0, POS_STANDING, COM_VERBAL, TRUE, TAR_IGNORE, SPHERE_COMBAT,
     SCHOOL_CONJURATION, SHAPE_UNDEFINED, HIT_ALL, 1, r_none, cast_spell_unholy_word);
   */



}


/******* Following are various events ***************/

EVENTFUNC(cast_event)
{
  struct spell_event *se = (struct spell_event *) event_obj;
  int spellnum, found;
  struct char_data *ch = NULL, *tch = NULL;
  struct obj_data *tobj = NULL;
  struct location_data *loc = NULL;

  /* extract all the info from se */
  ch = se->ch;

  spellnum = se->spellnum;
  if (se->target != -1)
    sprintf(arg, "%c%ld", UID_CHAR, se->target);

  found = find_spell_target(ch, arg, spellnum, &tch, &tobj);
  if (found) {
    if (tch) {
      SET_FACING(ch, face_angle(angle_offset(ch, NULL, tch, NULL)));
    }
    if (tobj) {
    }
  }
  if (!found) {
    if (se->in_loc) {
      CREATE(loc, struct location_data, 1);
      loc->real_room = se->target;
      loc->x_cord = se->location.x_cord;
      loc->y_cord = se->location.y_cord;
      loc->z_cord = se->location.z_cord;
      loc->angle = se->location.angle;
      if (!IS_SET(SINFO.targets, TAR_ROOM_WORLD))
	unobstructed_line(ch, NULL, NULL, loc, 0, 0, TRUE);
      found = TRUE;
    }
  }
  strcpy(arg, se->arg);

  /* clear it all */
  GET_ACTION(ch) = NULL;
  REMOVE_BIT(ch->char_specials.saved.affected_by2, AFF2_CASTING);
  FREE(event_obj);

  /* spell is lost, working or not */
  if (GET_SPELL(ch, spellnum) == MEMINFO_READY) {
    SET_SPELL(ch, spellnum, MEMINFO_KNOWN);
  } else if (GET_SPELL(ch, spellnum) > MEMINFO_READY) {
    SET_SPELL(ch, spellnum, (GET_SPELL(ch, spellnum) - 1));
  }
  /* find and make sure the target is still available */
  if (!found) {
    act("Your target is no longer available!", FALSE, ch, NULL, NULL, TO_CHAR);
    if (IS_SET(SINFO.targets, TAR_CHAR_ROOM | TAR_CHAR_WORLD) &&
	!IS_SET(SINFO.targets, TAR_NOT_SELF))
      tch = ch;
    else {
      if (loc)
	FREE(loc);
      return 0;
    }
  }
  if ((GET_POS(ch) < SINFO.min_position) && !IS_TRUSTED(ch)) {
    act("Your concentration was broken!", FALSE, ch, NULL, NULL, TO_CHAR);
    if (loc)
      FREE(loc);
    return 0;
  }
  act("You complete your spell.", FALSE, ch, NULL, NULL, TO_CHAR);
  say_spell(ch, spellnum, tch, tobj);

  call_magic(ch, tch, tobj, spellnum, GET_LEVEL(ch), CAST_SPELL, loc);

  if (loc)
    FREE(loc);
  return 0;
}
